@@ -61,11 +61,15 @@
 *                               type. Moved a few functions from cdfdist.h to
 *                               here.
 *   V3.14 10-Jan-18, M Liu      Added ARM and IA64 openVMS encoding.
+*   V3.15 10-Apr-21, M Liu      Added a few functions to handle UTF-8 encoding
+*                               characters.
 ******************************************************************************/
 
 #if !defined(CDFh_INCLUDEd__)
 #define CDFh_INCLUDEd__
 
+#include <wchar.h>
+
 /******************************************************************************
 * CDF defined types
 ******************************************************************************/
@@ -325,6 +329,8 @@ static double *TT2000NULL = 0;
 #define TT2000_CDF_MAYNEEDUPDATE        ((CDFstatus) 1010)
 #define BLOCKINGFACTOR_TOO_SMALL        ((CDFstatus) 1011)
 #define BLOCKINGFACTOR_TOO_SMALL2       ((CDFstatus) 1012)
+#define STRING_NOT_UTF8_ENCODING        ((CDFstatus) 1013)
+#define BLOCKINGFACTOR_TOO_LARGE        ((CDFstatus) 1014)
 
 #define CDF_OK                          ((CDFstatus) 0)
 
@@ -466,6 +472,7 @@ static double *TT2000NULL = 0;
 #define FUNCTION_NOT_SUPPORTED          ((CDFstatus) (-2236))
 #define TRY_TO_READ_NONSTRING_DATA      ((CDFstatus) (-2237))
 #define BAD_NUM_STRINGS                 ((CDFstatus) (-2238))
+#define CANNOT_CONVERT_WIDECHAR         ((CDFstatus) (-2239))
 
 /******************************************************************************
 * Functions (for INTERNAL interface).
@@ -705,6 +712,20 @@ CDFputAttrEntry(id,2,attrNum,entryNum,dataType,numElems,value)
 #define CDFputAttrzEntry(id,attrNum,entryNum,dataType,numElems,value) \
 CDFputAttrEntry(id,3,attrNum,entryNum,dataType,numElems,value)
 
+#define CDFputAttrStrgEntry(id,attrNum,entryNum,string) \
+CDFputAttrStrEntry(id,1,attrNum,entryNum,string)
+#define CDFputAttrStrrEntry(id,attrNum,entryNum,string) \
+CDFputAttrStrEntry(id,2,attrNum,entryNum,string)
+#define CDFputAttrStrzEntry(id,attrNum,entryNum,string) \
+CDFputAttrStrEntry(id,3,attrNum,entryNum,string)
+
+#define CDFputAttrWStrgEntry(id,attrNum,entryNum,string,locale) \
+CDFputAttrWStrEntry(id,1,attrNum,entryNum,string,locale)
+#define CDFputAttrWStrrEntry(id,attrNum,entryNum,string,locale) \
+CDFputAttrWStrEntry(id,2,attrNum,entryNum,string,locale)
+#define CDFputAttrWStrzEntry(id,attrNum,entryNum,string,locale) \
+CDFputAttrWStrEntry(id,3,attrNum,entryNum,string,locale)
+
 #define CDFattrGet(id,attrNum,entryNum,value) \
 CDFgetAttrEntry(id,0,attrNum,entryNum,value)
 #define CDFgetAttrgEntry(id,attrNum,entryNum,value) \
@@ -714,11 +735,45 @@ CDFgetAttrEntry(id,2,attrNum,entryNum,value)
 #define CDFgetAttrzEntry(id,attrNum,entryNum,value) \
 CDFgetAttrEntry(id,3,attrNum,entryNum,value)
 
+#define CDFreadAttrStrgEntry CDFgetAttrStrgEntry 
+#define CDFreadAttrStrrEntry CDFgetAttrStrrEntry 
+#define CDFreadAttrStrzEntry CDFgetAttrStrzEntry 
+#define CDFreadAttrStrEntry CDFgetAttrStrEntry 
+#define CDFgetAttrStrgEntry(id,attrNum,entryNum,string) \
+CDFgetAttrStrEntry(id,1,attrNum,entryNum,string)
+#define CDFgetAttrStrrEntry(id,attrNum,entryNum,string) \
+CDFgetAttrStrEntry(id,2,attrNum,entryNum,string)
+#define CDFgetAttrStrzEntry(id,attrNum,entryNum,string) \
+CDFgetAttrStrEntry(id,3,attrNum,entryNum,string)
+
+#define CDFgetAttrWStrgEntry(id,attrNum,entryNum,string,locale) \
+CDFgetAttrWStrEntry(id,1,attrNum,entryNum,string,locale)
+#define CDFgetAttrWStrrEntry(id,attrNum,entryNum,string,locale) \
+CDFgetAttrWStrEntry(id,2,attrNum,entryNum,string,locale)
+#define CDFgetAttrWStrzEntry(id,attrNum,entryNum,string,locale) \
+CDFgetAttrWStrEntry(id,3,attrNum,entryNum,string,locale)
+
+#define CDFreadAttrrEntryStrings CDFgetAttrrEntryStrings
+#define CDFreadAttrzEntryStrings CDFgetAttrzEntryStrings
+#define CDFreadAttrEntryStrings CDFgetAttrEntryStrings
 #define CDFgetAttrrEntryStrings(id,attrNum,entryNum,numStrings,strings) \
 CDFgetAttrEntryStrings(id,0,attrNum,entryNum,numStrings,strings)
 #define CDFgetAttrzEntryStrings(id,attrNum,entryNum,numStrings,strings) \
 CDFgetAttrEntryStrings(id,1,attrNum,entryNum,numStrings,strings)
 
+#define CDFreadgAttrEntry(id,attrNum,entryNum,dataType,numElms,value) \
+CDFreadAttrEntry(id,0,attrNum,entryNum,dataType,numElms,value)
+#define CDFreadAttrgEntry(id,attrNum,entryNum,dataType,numElms,value) \
+CDFreadAttrEntry(id,0,attrNum,entryNum,dataType,numElms,value)
+#define CDFreadrAttrEntry(id,attrNum,entryNum,dataType,numElms,value) \
+CDFreadAttrEntry(id,1,attrNum,entryNum,dataType,numElms,value)
+#define CDFreadAttrrEntry(id,attrNum,entryNum,dataType,numElms,value) \
+CDFreadAttrEntry(id,1,attrNum,entryNum,dataType,numElms,value)
+#define CDFreadzAttrEntry(id,attrNum,entryNum,dataType,numElms,value) \
+CDFreadAttrEntry(id,2,attrNum,entryNum,dataType,numElms,value)
+#define CDFreadAttrzEntry(id,attrNum,entryNum,dataType,numElms,value) \
+CDFreadAttrEntry(id,2,attrNum,entryNum,dataType,numElms,value)
+
 #define CDFputAttrrEntryStrings(id,attrNum,entryNum,numStrings,strings) \
 CDFputAttrEntryStrings(id,0,attrNum,entryNum,numStrings,strings)
 #define CDFputAttrzEntryStrings(id,attrNum,entryNum,numStrings,strings) \
@@ -811,140 +866,265 @@ CDFgetAttrEntryNumStrings(id,1,attrNum,entryNum,numStrings)
 #define CDFgetNumrVars(id,numVars) CDFgetNumVars(id,0,numVars)
 #define CDFgetNumzVars(id,numVars) CDFgetNumVars(id,1,numVars)
 
+#define CDFdeleterVar(id,varNum) CDFdeleteVar(id,0,varNum)
 #define CDFdeletezVar(id,varNum) CDFdeleteVar(id,1,varNum)
 
+#define CDFdeleterVarRecords(id,varNum,sRec,eRec) \
+CDFdeleteVarRecords(id,0,varNum,sRec,eRec)
 #define CDFdeletezVarRecords(id,varNum,sRec,eRec) \
 CDFdeleteVarRecords(id,1,varNum,sRec,eRec)
 
+#define CDFdeleterVarRecordsRenumber(id,varNum,sRec,eRec) \
+CDFdeleteVarRecordsRenumber(id,0,varNum,sRec,eRec)
 #define CDFdeletezVarRecordsRenumber(id,varNum,sRec,eRec) \
 CDFdeleteVarRecordsRenumber(id,1,varNum,sRec,eRec)
 
+#define CDFgetrVarName(id,varNum,varName) \
+CDFgetVarName(id,0,varNum,varName)
 #define CDFgetzVarName(id,varNum,varName) \
 CDFgetVarName(id,1,varNum,varName)
 
+#define CDFgetrVarMaxWrittenRecNum(id,varNum,maxRec) \
+CDFgetVarMaxWrittenRecNum(id,0,varNum,maxRec)
 #define CDFgetzVarMaxWrittenRecNum(id,varNum,maxRec) \
 CDFgetVarMaxWrittenRecNum(id,1,varNum,maxRec)
 
+#define CDFgetrVarsMaxWrittenRecNum(id,maxRec) \
+CDFgetVarsMaxWrittenRecNum(id,0,maxRec)
 #define CDFgetzVarsMaxWrittenRecNum(id,maxRec) \
 CDFgetVarsMaxWrittenRecNum(id,1,maxRec)
 
+#define CDFgetrVarMaxAllocRecNum(id,varNum,maxAllocRec) \
+CDFgetVarMaxAllocRecNum(id,0,varNum,maxAllocRec)
 #define CDFgetzVarMaxAllocRecNum(id,varNum,maxAllocRec) \
 CDFgetVarMaxAllocRecNum(id,1,varNum,maxAllocRec)
 
+#define CDFgetrVarDataType(id,varNum,dataType) \
+CDFgetVarDataType(id,0,varNum,dataType)
 #define CDFgetzVarDataType(id,varNum,dataType) \
 CDFgetVarDataType(id,1,varNum,dataType)
 
+#define CDFgetrVarAllocRecords(id,varNum,allocRecs) \
+CDFgetVarAllocRecords(id,0,varNum,allocRecs)
 #define CDFgetzVarAllocRecords(id,varNum,allocRecs) \
 CDFgetVarAllocRecords(id,1,varNum,allocRecs)
+
+#define CDFsetrVarAllocRecords(id,varNum,allocRecs) \
+CDFsetVarAllocRecords(id,0,varNum,allocRecs)
 #define CDFsetzVarAllocRecords(id,varNum,allocRecs) \
 CDFsetVarAllocRecords(id,1,varNum,allocRecs)
 
+#define CDFsetrVarAllocBlockRecords(id,varNum,firstRec,lastRec) \
+CDFsetVarAllocBlockRecords(id,0,varNum,firstRec,lastRec)
 #define CDFsetzVarAllocBlockRecords(id,varNum,firstRec,lastRec) \
 CDFsetVarAllocBlockRecords(id,1,varNum,firstRec,lastRec)
 
+#define CDFgetrVarBlockingFactor(id,varNum,bf) \
+CDFgetVarBlockingFactor(id,0,varNum,bf)
 #define CDFgetzVarBlockingFactor(id,varNum,bf) \
 CDFgetVarBlockingFactor(id,1,varNum,bf)
+
+#define CDFsetrVarBlockingFactor(id,varNum,bf) \
+CDFsetVarBlockingFactor(id,0,varNum,bf)
 #define CDFsetzVarBlockingFactor(id,varNum,bf) \
 CDFsetVarBlockingFactor(id,1,varNum,bf)
 
+#define CDFgetrVarCompression(id,varNum,cType,cParms,cPct) \
+CDFgetVarCompression(id,0,varNum,cType,cParms,cPct)
 #define CDFgetzVarCompression(id,varNum,cType,cParms,cPct) \
 CDFgetVarCompression(id,1,varNum,cType,cParms,cPct)
+
+#define CDFsetrVarCompression(id,varNum,cType,cParms) \
+CDFsetVarCompression(id,0,varNum,cType,cParms)
 #define CDFsetzVarCompression(id,varNum,cType,cParms) \
 CDFsetVarCompression(id,1,varNum,cType,cParms)
 
+#define CDFsetrVarDataSpec(id,varNum,dataType) \
+CDFsetVarDataSpec(id,0,varNum,dataType,(long)-99)
 #define CDFsetzVarDataSpec(id,varNum,dataType) \
 CDFsetVarDataSpec(id,1,varNum,dataType,(long)-99)
 
+#define CDFsetrVarDimVariances(id,varNum,dimVarys) \
+CDFsetVarDimVariances(id,0,varNum,dimVarys)
 #define CDFsetzVarDimVariances(id,varNum,dimVarys) \
 CDFsetVarDimVariances(id,1,varNum,dimVarys)
 
+#define CDFgetrVarDimVariances(id,varNum,dimVarys) \
+CDFgetVarDimVariances(id,0,varNum,dimVarys)
 #define CDFgetzVarDimVariances(id,varNum,dimVarys) \
 CDFgetVarDimVariances(id,1,varNum,dimVarys)
 
+#define CDFgetrVarNumElements(id,varNum,numEles) \
+CDFgetVarNumElements(id,0,varNum,numEles)
 #define CDFgetzVarNumElements(id,varNum,numEles) \
 CDFgetVarNumElements(id,1,varNum,numEles)
 
+#define CDFgetrVarNumRecsWritten(id,varNum,numRecs) \
+CDFgetVarNumRecsWritten(id,0,varNum,numRecs)
 #define CDFgetzVarNumRecsWritten(id,varNum,numRecs) \
 CDFgetVarNumRecsWritten(id,1,varNum,numRecs)
 
+#define CDFsetrVarInitialRecs(id,varNum,initRecs) \
+CDFsetVarInitialRecs(id,0,varNum,initRecs)
 #define CDFsetzVarInitialRecs(id,varNum,initRecs) \
 CDFsetVarInitialRecs(id,1,varNum,initRecs)
 
+#define CDFgetrVarPadValue(id,varNum,pad) \
+CDFgetVarPadValue(id,0,varNum,pad)
 #define CDFgetzVarPadValue(id,varNum,pad) \
 CDFgetVarPadValue(id,1,varNum,pad)
+
+#define CDFsetrVarPadValue(id,varNum,pad) \
+CDFsetVarPadValue(id,0,varNum,pad)
 #define CDFsetzVarPadValue(id,varNum,pad) \
 CDFsetVarPadValue(id,1,varNum,pad)
 
+#define CDFgetrVarRecVariance(id,varNum,recVary) \
+CDFgetVarRecVariance(id,0,varNum,recVary)
 #define CDFgetzVarRecVariance(id,varNum,recVary) \
 CDFgetVarRecVariance(id,1,varNum,recVary)
+
+#define CDFsetrVarRecVariance(id,varNum,recVary) \
+CDFsetVarRecVariance(id,0,varNum,recVary)
 #define CDFsetzVarRecVariance(id,varNum,recVary) \
 CDFsetVarRecVariance(id,1,varNum,recVary)
 
+#define CDFgetrVarSeqData(id,varNum,data) \
+CDFgetVarSeqData(id,0,varNum,data)
 #define CDFgetzVarSeqData(id,varNum,data) \
 CDFgetVarSeqData(id,1,varNum,data)
+
+#define CDFputrVarSeqData(id,varNum,data) \
+CDFputVarSeqData(id,0,varNum,data)
 #define CDFputzVarSeqData(id,varNum,data) \
 CDFputVarSeqData(id,1,varNum,data)
 
+#define CDFgetrVarSparseRecords(id,varNum,sprecs) \
+CDFgetVarSparseRecords(id,0,varNum,sprecs)
 #define CDFgetzVarSparseRecords(id,varNum,sprecs) \
 CDFgetVarSparseRecords(id,1,varNum,sprecs)
+
+#define CDFsetrVarSparseRecords(id,varNum,sprecs) \
+CDFsetVarSparseRecords(id,0,varNum,sprecs)
 #define CDFsetzVarSparseRecords(id,varNum,sprecs) \
 CDFsetVarSparseRecords(id,1,varNum,sprecs)
 
-#define CDFgetzVarsRecordData(id,numVars,varNames,recNum,bufferPtr) \
-CDFgetVarsRecordDatabyNames(id,1,numVars,varNames,recNum,bufferPtr)
-#define CDFputzVarsRecordData(id,numVars,varNames,recNum,bufferPtr) \
-CDFputVarsRecordDatabyNames(id,1,numVars,varNames,recNum,bufferPtr)
+#define CDFgetVarsRecordDatabyVarNames CDFgetVarsRecordDatabyNames
+#define CDFputVarsRecordDatabyVarNums CDFputVarsRecordDatabyNums
 
+#define CDFgetrVarsRecordDatabyNums(id,numVars,varNums,recNum,bufferPtrs) \
+CDFgetVarsRecordDatabyNums(id,0,numVars,varNums,recNum,bufferPtrs)
+#define CDFgetzVarsRecordDatabyNums(id,numVars,varNums,recNum,bufferPtrs) \
+CDFgetVarsRecordDatabyNums(id,1,numVars,varNums,recNum,bufferPtrs)
+
+#define CDFputrVarsRecordDatabyNums(id,numVars,varNums,recNum,bufferPtrs) \
+CDFputVarsRecordDatabyNums(id,0,numVars,varNums,recNum,bufferPtrs)
+#define CDFputzVarsRecordDatabyNums(id,numVars,varNums,recNum,bufferPtrs) \
+CDFputVarsRecordDatabyNums(id,1,numVars,varNums,recNum,bufferPtrs)
+
+#define CDFgetrVarsRecordData(id,numVars,varNames,recNum,bufferPtrs) \
+CDFgetVarsRecordDatabyNames(id,0,numVars,varNames,recNum,bufferPtrs)
+#define CDFgetzVarsRecordData(id,numVars,varNames,recNum,bufferPtrs) \
+CDFgetVarsRecordDatabyNames(id,1,numVars,varNames,recNum,bufferPtrs)
+
+#define CDFputrVarsRecordData(id,numVars,varNames,recNum,bufferPtrs) \
+CDFputVarsRecordDatabyNames(id,0,numVars,varNames,recNum,bufferPtrs)
+#define CDFputzVarsRecordData(id,numVars,varNames,recNum,bufferPtrs) \
+CDFputVarsRecordDatabyNames(id,1,numVars,varNames,recNum,bufferPtrs)
+
+#define CDFgetrVarsRecordDatabyNumbers(id,numVars,varNumbers,recNum,buffer) \
+CDFgetVarsRecordDatabyNumbers(id,0,numVars,varNumbers,recNum,buffer)
 #define CDFgetzVarsRecordDatabyNumbers(id,numVars,varNumbers,recNum,buffer) \
 CDFgetVarsRecordDatabyNumbers(id,1,numVars,varNumbers,recNum,buffer)
+
+#define CDFputrVarsRecordDatabyNumbers(id,numVars,varNumbers,recNum,buffer) \
+CDFputVarsRecordDatabyNumbers(id,0,numVars,varNumbers,recNum,buffer)
 #define CDFputzVarsRecordDatabyNumbers(id,numVars,varNumbers,recNum,buffer) \
 CDFputVarsRecordDatabyNumbers(id,1,numVars,varNumbers,recNum,buffer)
 
+#define CDFgetrVarRecordData(id,varNum,recNum,buffer) \
+CDFgetVarRecordData(id,0,varNum,recNum,buffer)
 #define CDFgetzVarRecordData(id,varNum,recNum,buffer) \
 CDFgetVarRecordData(id,1,varNum,recNum,buffer)
+
+#define CDFputrVarRecordData(id,varNum,recNum,buffer) \
+CDFputVarRecordData(id,0,varNum,recNum,buffer)
 #define CDFputzVarRecordData(id,varNum,recNum,buffer) \
 CDFputVarRecordData(id,1,varNum,recNum,buffer)
 
+#define CDFsetrVarCacheSize(id,varNum,numBuffers) \
+CDFsetVarCacheSize(id,0,varNum,numBuffers)
 #define CDFsetzVarCacheSize(id,varNum,numBuffers) \
 CDFsetVarCacheSize(id,1,varNum,numBuffers)
 
+#define CDFsetrVarsCacheSize(id,numBuffers) \
+CDFsetVarsCacheSize(id,0,numBuffers)
 #define CDFsetzVarsCacheSize(id,numBuffers) \
 CDFsetVarsCacheSize(id,1,numBuffers)
 
+#define CDFgetrVarCacheSize(id,varNum,numBuffers) \
+CDFgetVarCacheSize(id,0,varNum,numBuffers)
 #define CDFgetzVarCacheSize(id,varNum,numBuffers) \
 CDFgetVarCacheSize(id,1,varNum,numBuffers)
 
+#define CDFconfirmrVarExistence(id,varName) \
+CDFconfirmVarExistence(id,0,varName)
 #define CDFconfirmzVarExistence(id,varName) \
 CDFconfirmVarExistence(id,1,varName)
 
+#define CDFconfirmrVarPadValueExistence(id,varNum) \
+CDFconfirmVarPadValueExistence(id,0,varNum)
 #define CDFconfirmzVarPadValueExistence(id,varNum) \
 CDFconfirmVarPadValueExistence(id,1,varNum)
 
+#define CDFgetrVarReservePercent(id,varNum,percent) \
+CDFgetVarReservePercent(id,0,varNum,percent)
 #define CDFgetzVarReservePercent(id,varNum,percent) \
 CDFgetVarReservePercent(id,1,varNum,percent)
 
+#define CDFsetrVarReservePercent(id,varNum,percent) \
+CDFsetVarReservePercent(id,0,varNum,percent)
 #define CDFsetzVarReservePercent(id,varNum,percent) \
 CDFsetVarReservePercent(id,1,varNum,percent)
 
+#define CDFgetrVarSeqPos(id,varNum,recNum,indices) \
+CDFgetVarSeqPos(id,0,varNum,recNum,indices)
 #define CDFgetzVarSeqPos(id,varNum,recNum,indices) \
 CDFgetVarSeqPos(id,1,varNum,recNum,indices)
+
+#define CDFsetrVarSeqPos(id,varNum,recNum,indices) \
+CDFsetVarSeqPos(id,0,varNum,recNum,indices)
 #define CDFsetzVarSeqPos(id,varNum,recNum,indices) \
 CDFsetVarSeqPos(id,1,varNum,recNum,indices)
 
+#define CDFgetzVarAllRecordsbyVarNum CDFgetzVarAllRecordsByVarID
+#define CDFgetrVarAllRecordsByVarID(id,varNum,buffer) \
+CDFgetVarAllRecordsByVarID(id,0,varNum,buffer)
 #define CDFgetzVarAllRecordsByVarID(id,varNum,buffer) \
 CDFgetVarAllRecordsByVarID(id,1,varNum,buffer)
+
+#define CDFputzVarAllRecordsbyVarNum CDFputzVarAllRecordsByVarID
+#define CDFputrVarAllRecordsByVarID(id,varNum,numRecs,buffer) \
+CDFputVarAllRecordsByVarID(id,0,varNum,numRecs,buffer)
 #define CDFputzVarAllRecordsByVarID(id,varNum,numRecs,buffer) \
 CDFputVarAllRecordsByVarID(id,1,varNum,numRecs,buffer)
 
+#define CDFgetrVarRangeRecordsByVarID(id,varNum,startRec,stopRec,buffer) \
+CDFgetVarRangeRecordsByVarID(id,0,varNum,startRec,stopRec,buffer)
 #define CDFgetzVarRangeRecordsByVarID(id,varNum,startRec,stopRec,buffer) \
 CDFgetVarRangeRecordsByVarID(id,1,varNum,startRec,stopRec,buffer)
+
+#define CDFputrVarRangeRecordsByVarID(id,varNum,startRec,stopRec,buffer) \
+CDFputVarRangeRecordsByVarID(id,0,varNum,startRec,stopRec,buffer)
 #define CDFputzVarRangeRecordsByVarID(id,varNum,startRec,stopRec,buffer) \
 CDFputVarRangeRecordsByVarID(id,1,varNum,startRec,stopRec,buffer)
 
-#define CDFinsertzVarRecordsByVarID(id,varNum,startRec,numRecs,buffer) \
-CDFinsertVarRecordsByVarID(id,1,varNum,startRec,numRecs,buffer)
 #define CDFinsertrVarRecordsByVarID(id,varNum,startRec,numRecs,buffer) \
 CDFinsertVarRecordsByVarID(id,0,varNum,startRec,numRecs,buffer)
+#define CDFinsertzVarRecordsByVarID(id,varNum,startRec,numRecs,buffer) \
+CDFinsertVarRecordsByVarID(id,1,varNum,startRec,numRecs,buffer)
+
+#define CDF_Free_Strings CDF_Free_String
 
 /*
  * CLOSE_  *
@@ -1648,6 +1828,14 @@ CDFlib (SELECT_, CDF_, id, \
 
 #define TT2000breakdown breakdownTT2000
 
+/******************************************************************************
+* UTF8 macros define'd
+******************************************************************************/
+#define UTF8StrLength UTF8StrChars
+#define UTF8CharLength UTF8CharBytes
+#define UTF8CharAtLength UTF8CharAtBytes
+#define UTF8StrLengthFromCharAt UTF8StrCharsFromCharAt
+
 /******************************************************************************
 * Function prototypes.
 *     It is assumed that `__cplusplus' is defined for ALL C++ compilers.  If
@@ -1736,9 +1924,25 @@ VISIBLE_PREFIX CDFstatus CDFputAttrEntry PROTOARGs((
   CDFid id, int grzEntry, long attrNum, long entryNum, long dataType,
   long numElems, void *value
 ));
+VISIBLE_PREFIX CDFstatus CDFputAttrStrEntry PROTOARGs((
+  CDFid id, int grzEntry, long attrNum, long entryNum, char *string
+));
 VISIBLE_PREFIX CDFstatus CDFgetAttrEntry PROTOARGs((
   CDFid id, int grzEntry, long attrNum, long entryNum, void *value
 ));
+VISIBLE_PREFIX CDFstatus CDFgetAttrStrEntry PROTOARGs((
+  CDFid id, int grzEntry, long attrNum, long entryNum, char **string
+));
+/* #if defined(WIN32) */
+VISIBLE_PREFIX CDFstatus CDFputAttrWStrEntry PROTOARGs((
+  CDFid id, int grzEntry, long attrNum, long entryNum, wchar_t *wstring,
+  char *locale
+));
+VISIBLE_PREFIX CDFstatus CDFgetAttrWStrEntry PROTOARGs((
+  CDFid id, int grzEntry, long attrNum, long entryNum, wchar_t **wstring,
+  char *locale
+));
+/* #endif */
 VISIBLE_PREFIX CDFstatus CDFgetAttrEntryNumStrings PROTOARGs((
   CDFid id, int rzEntry, long attrNum, long entryNum, 
   long *numStrings
@@ -1752,9 +1956,9 @@ VISIBLE_PREFIX CDFstatus CDFputAttrEntryStrings PROTOARGs((
   long numStrings, char **strings
 ));
 VISIBLE_PREFIX void CDFdataFree PROTOARGs((CDFdata value));
-VISIBLE_PREFIX CDFstatus CDFreadgAttrEntry PROTOARGs((
-  CDFid id, long attrNum, long entryNum, long *dataType, long *numElms,
-  CDFdata *value
+VISIBLE_PREFIX CDFstatus CDFreadAttrEntry PROTOARGs((
+  CDFid id, int grzEntry, long attrNum, long entryNum, long *dataType,
+  long *numElms, CDFdata *value
 ));
 VISIBLE_PREFIX CDFstatus CDFreadzAttrEntry PROTOARGs((
   CDFid id, long attrNum, long entryNum, long *dataType, long *numElms,
@@ -1787,13 +1991,21 @@ VISIBLE_PREFIX CDFstatus CDFgetVarRecordData PROTOARGs((
 VISIBLE_PREFIX CDFstatus CDFputVarRecordData PROTOARGs((
   CDFid id, int zVar, long varNum, long recNum, void *buffer
 ));
+VISIBLE_PREFIX CDFstatus CDFgetVarsRecordDatabyNums PROTOARGs((
+  CDFid id, int zVar, long numVars, long varNums[], long recNum,
+  void *buffers[]
+));
 VISIBLE_PREFIX CDFstatus CDFgetVarsRecordDatabyNames PROTOARGs((
   CDFid id, int zVar, long numVars, char *varNames[], long recNum,
-  void *buffer[]
+  void *buffers[]
+));
+VISIBLE_PREFIX CDFstatus CDFputVarsRecordDatabyNums PROTOARGs((
+  CDFid id, int zVar, long numVars, long varNums[], long recNum,
+  void *buffers[]
 ));
 VISIBLE_PREFIX CDFstatus CDFputVarsRecordDatabyNames PROTOARGs((
   CDFid id, int zVar, long numVars, char *varNames[], long recNum,
-  void *buffer[]
+  void *buffers[]
 ));
 VISIBLE_PREFIX CDFstatus CDFgetVarAllRecordsByVarID PROTOARGs((
   CDFid id, int zVar, long varNum, void *buffer
@@ -1863,6 +2075,9 @@ VISIBLE_PREFIX CDFstatus CDFdeleteRecordsForAll PROTOARGs((
 VISIBLE_PREFIX void CDFsetFileBackward PROTOARGs((
   int flag
 ));
+VISIBLE_PREFIX void CDFsetFileBackward2 PROTOARGs((
+  int flag
+));
 VISIBLE_PREFIX void CDFsetFileBackwardFlag PROTOARGs((
   int flag
 ));
@@ -1994,6 +2209,18 @@ VISIBLE_PREFIX void UnixTimetoEPOCH16 PROTOARGs((
   double *epoch,
   int    numTimes
 ));
+VISIBLE_PREFIX int UTF8StrChars PROTOARGs((const char *, int));
+VISIBLE_PREFIX int UTF8CharBytes PROTOARGs((const char *));
+VISIBLE_PREFIX int UTF8CharAtOffset PROTOARGs((const char *, int));
+VISIBLE_PREFIX char *UTF8CharAtString PROTOARGs((const char *, int));
+VISIBLE_PREFIX int UTF8CharAtBytes PROTOARGs((const char *, int));
+VISIBLE_PREFIX int UTF8StrCharsFromCharAt PROTOARGs((const char *, int));
+VISIBLE_PREFIX int UTF8ByteLengthFromCharAt PROTOARGs((const char *, int));
+VISIBLE_PREFIX int UTF8toUnicode PROTOARGs((int *, const char *, int));
+VISIBLE_PREFIX int UnicodetoUTF8 PROTOARGs((char *, int *, int));
+VISIBLE_PREFIX unsigned char *UTF8Check PROTOARGs((unsigned char *, int, int *));
+VISIBLE_PREFIX int IsUTF8Char(const char *s);
+
 /******************************************************************************
 * A new set of functions to handle CDF_TIME_TT2000 time type.
 ******************************************************************************/
