@@ -30,6 +30,9 @@
 
 #include "cdflib.h"
 #include "cdftools.h"
+#if defined(WIN32)
+#  include <Windows.h>
+#endif
 
 #if defined(mac)
 #  include "cdf2skt.rh"
@@ -65,6 +68,7 @@
 #define ISO8601qual     21
 #define ZISO8601qual    22
 #define ADVFORMATqual   23
+#define NOVALIDATEqual  24
 
 /******************************************************************************
 * Macro constants.
@@ -102,6 +106,12 @@
 ((size_t) strlen(value) < (size_t) strlen(label) ? \
 (strlen(label) - strlen(value)) / (size_t) 2 : 0)
 
+/******************************************************************************
+* Global variables.
+******************************************************************************/
+CDFid id;
+int iso8601;
+Logical toValidate;
 /******************************************************************************
 * Function Prototypes.
 ******************************************************************************/
@@ -113,8 +123,6 @@ Logical CreateSkeletonTable PROTOARGs((int argC, char *argV[]));
 #if defined(__cplusplus)
 }
 #endif
-int iso8601;
-CDFid id;
 Logical SkeletonTableQOPs PROTOARGs((int *argC, char **argV[]));
 Logical WriteHeader PROTOARGs((char *CDFname, int varValues, char *variables));
 Logical WriteGlobalAttr PROTOARGs((void));
