@@ -73,7 +73,9 @@
 
 #if defined(WIN32)
 #  define IBMPC
-#  define win32
+#  if !defined(win32)
+#    define win32
+#  endif
 #  if !defined MSVC67
 #    if (_MSC_VER == 1300) 
 /* VC7, building with pre-VC7 runtime libraries */
@@ -117,7 +119,7 @@
 #endif
 
 #if defined(__ppc__) || (defined(__APPLE__) && !defined(i386) && \
-                         !defined(__x86_64__))
+                         !defined(__x86_64__) && !defined(__arm64__))
 #  define POWERPC
 #  define unix
 #  if defined(__MACH__)  	/* Mac OS X cc (gcc) */
@@ -135,14 +137,19 @@
 #  define unix
 #endif
 
-#if defined(__arm__)
+#if defined(__arm__) || defined(__arm64__) || \
+    (defined(__ARM_ARCH) && __ARM_ARCH > 0)
+/* linux(gcc)-based or clang-based */
 #  define ARM
+#  if !defined(unix)
+#    define unix
+#  endif
 #  if defined(__BYTE_ORDER__)
-#     if __BYTE_ORDER__ == __ORDER_LITTLE_ENDIAN__
-#        define ARM_LITTLE
-#     else
-#        define ARM_BIG
-#     endif
+#    if __BYTE_ORDER__ == __ORDER_LITTLE_ENDIAN__
+#      define ARM_LITTLE
+#    else
+#      define ARM_BIG
+#    endif
 #  endif
 #endif
 
@@ -292,6 +299,7 @@
 #  include <stdio.h>
 #  include <string.h>
 #  include <ctype.h>
+#  include <wctype.h>
 #  include <time.h>
 #  include <math.h>
 #  include <unixio.h>
@@ -303,6 +311,7 @@
 #  include <ssdef.h>
 #  include <stat.h>
 #  include <unistd.h>
+#  include <locale.h>
 #  include <lib$routines.h>
 #  if defined(VARARGS)
 #    include <varargs.h>
@@ -316,6 +325,7 @@
 #  include <stdio.h>
 #  include <string.h>
 #  include <ctype.h>
+#  include <wctype.h>
 #  include <time.h>
 #  include <math.h>
 #  include <pwd.h>
@@ -323,6 +333,8 @@
 #  include <sys/types.h>
 #  include <sys/stat.h>
 #  include <sys/time.h>
+#  include <wchar.h>
+#  include <locale.h>
 #  if defined(VARARGS)
 #    include <varargs.h>
 #  else
@@ -343,12 +355,15 @@
 #endif
 
 #if defined(dos) || defined(win32) || defined(__MINGW32__)
-#  include <stdlib.h>
 #  include <stdio.h>
+#  include <stdlib.h>
 #  include <string.h>
 #  include <ctype.h>
+#  include <wctype.h>
 #  include <time.h>
 #  include <math.h>
+#  include <wchar.h>
+#  include <locale.h>
 #  if defined(win32)
 #    include <io.h>
 #    include <fcntl.h>
@@ -450,7 +465,7 @@ typedef unsigned long uLongx;
 #if defined(alphaosf) || defined(IRIX64bit) || defined(_LP64) || \
     defined(__amd64) || defined(__x86_64__) || defined(__ia64__) || \
     defined(AIX64) || defined(HP64) || defined(__PPC64__) || \
-    defined(__ppc64__)
+    defined(__ppc64__) || defined(__arm64__)
    /***************************************************************************
    * `long' is 64 bits on a DEC Alpha/OSF1, an SGi/IRIX 6.x using the `-64'
    * option (64-bit objects) and a Solaris sparcv9, AMD64 and EM64T/IA64.
@@ -552,7 +567,7 @@ typedef sChar sByte;
 #    if defined(__osf__) || defined(__CYGWIN__) || \
         defined(vms) || defined(sgi) || defined(__MINGW32__) || \
         defined(__FreeBSD__) || defined(macosXintel) || defined(macosXppc) || \
-        defined(__PPC__)
+        defined(__PPC__) || defined(__arm64__)
 #      define FOPEN64 fopen
 #    else
 #      define FOPEN64 fopen64
@@ -1085,6 +1100,8 @@ VISIBLE_PREFIX void ClearBytes PROTOARGs((
   void *buffer, int firstByte, int lastByte
 ));
 VISIBLE_PREFIX char *catchrX PROTOARGs((char *dst, int src, size_t max));
+VISIBLE_PREFIX char *catchrY PROTOARGs((char *dst, char *src, size_t nbytes,
+                                        size_t max));
 VISIBLE_PREFIX char *prependX PROTOARGs((char *dst, char *src, size_t max));
 VISIBLE_PREFIX Logical IsReg PROTOARGs((char *));
 VISIBLE_PREFIX void ExpandPath PROTOARGs((
