@@ -35,6 +35,9 @@
 
 #include "cdflib.h"
 #include "cdftools.h"
+#if defined(WIN32)
+#  include <Windows.h>
+#endif
 
 /******************************************************************************
 * Parameter/qualifier constants.
@@ -56,6 +59,7 @@
 #define DETECTqual       11
 #define EPOCHVVqual      12
 #define ZISO8601qual     13
+#define NOVALIDATEqual   14
 
 #define ALLDUMP           0
 #define DATADUMP          1
