@@ -34,6 +34,9 @@
 
 #include "cdftools.h"
 #include "cdflib.h" 
+#if defined(WIN32)
+#  include <Windows.h>
+#endif
 
 #if defined(mac)
 #  include "skt2cdf.rh"
@@ -71,7 +74,7 @@
 #define MAX_SKTLINE_LEN		132
 #define MAX_MSG_LEN		132
 #define BASE_STREAM_SIZE	132
-#define BASE_TOKEN_SIZE		25
+#define BASE_TOKEN_SIZE		50
 #define BASE_STRING_SIZE	132
 #define BASE_LOCATION_SIZE	20
 #define BASE_EVALUE_SIZE	132
@@ -81,7 +84,7 @@
 ******************************************************************************/
 
 enum searchForENUM { TOKEN, DELIMSTRING, BRACEDchar, BRACEDnonChar,
-		     VALLOCATION, VAREPOCH, VAREPOCH16, VARCHAR };
+		     VALLOCATION, VAREPOCH, VAREPOCH16, VARTT2000, VARCHAR };
 
 enum nextItemENUM { HEADERmark,
 		    CDForOUTPUTmark,
@@ -111,6 +114,9 @@ enum nextItemENUM { HEADERmark,
 		    CDFCHECKSUMmark,
 		    CDFCHECKSUMCOLONmark,
 		    CDFCHECKSUMfield,
+		    CDFLEAPSECONDLASTUPDATEDmark,
+		    CDFLEAPSECONDLASTUPDATEDCOLONmark,
+		    CDFLEAPSECONDLASTUPDATEDfield,
 		    GATTRmark,
 		    GaNAMEfieldOrVATTRmark,
 		    GaNAMEfield,
