@@ -38,11 +38,17 @@
 #if !defined(WINDOZh_INCLUDEd__)
 #define WINDOZh_INCLUDEd__
 
+#define _XOPEN_SOURCE_EXTENDED
+#if defined(__CYGWIN__)
+#  define _XOPEN_SOURCE 700
+#endif
+
 /******************************************************************************
 * Include files.
 ******************************************************************************/
 
 #include "cdftools.h"
+#include <wchar.h>
 
 /******************************************************************************
 * Which type of screen management is being used?
@@ -582,9 +588,9 @@ int paste_virtual_display PROTOARGs((WINDOWid, int, int));
 int put_chars PROTOARGs((WINDOWid, char *, int, int, int, int, int));
 int read_display PROTOARGs((WINDOWid, int, char *));
 #if defined(CURSESui)
-  int read_input PROTOARGs((WINDOWid, int *, int, Logical));
+  int read_input PROTOARGs((WINDOWid, wint_t *, int, Logical));
 #else
-  int read_input PROTOARGs((int *, int, Logical));
+  int read_input PROTOARGs((wint_t *, int, Logical));
 #endif
 int repaint_screen PROTOARGs((void));
 int repaste_virtual_display PROTOARGs((WINDOWid, int, int));
