@@ -39,7 +39,7 @@ static CDFstatus InsertEntry PROTOARGs((
   OFF_T vxrOffset, int atEntryN, int *count
 ));
 static void InsertAtEntry PROTOARGs((
-  Int32 first, Int32 last, OFF_T offset, struct VXRstruct64 *VXR, int atEntryN, 
+  Int32 first, Int32 last, OFF_T offset, struct VXRstruct64 *VXR, int atEntryN,
   Logical *push, Int32 *pushFirst, Int32 *pushLast, OFF_T *pushOffset
 ));
 static CDFstatus AppendEntry PROTOARGs((
@@ -881,7 +881,8 @@ OFF_T *cvvrOffset;
   CVVR.RecordType = CVVR_;
   CVVR.rfuA = (Int32) 0;
   CVVR.cSize = cSize;
-  if (!sX(AllocateIR64(CDF,CVVR.RecordSize,cvvrOffset),&pStatus)) return pStatus;
+  if (!sX(AllocateIR64(CDF,CVVR.RecordSize,cvvrOffset),&pStatus))
+    return pStatus;
   if (!sX(WriteCVVR64(CDF->fp,*cvvrOffset,
 		      CVVR_RECORDx,&CVVR,
 		      CVVR_NULL),&pStatus)) return pStatus;
