@@ -1,4 +1,9 @@
 /******************************************************************************
+* Copyright 1996-2014 United States Government as represented by the
+* Administrator of the National Aeronautics and Space Administration.
+* All Rights Reserved.
+******************************************************************************/
+/******************************************************************************
 *
 *  NSSDC/CDF                                        CDF `check sum' operations.
 *
@@ -7,29 +12,30 @@
 *  Modification history:
 *
 *   V1.0  21-Mar-06, M Liu      Original version.
+*   V2.0  06-Jan-23, M Liu      Increased BLOCKSIZE from 16384 to 1048576 
 ******************************************************************************/
 
 #include "cdflib.h"
 #include "cdflib64.h"
 #include "cdfmd5.h"
 
-#define BUFFSIZE 16384
+#define BLOCKSIZE 1048576
 
 /******************************************************************************
 * Local function prototypes.
 ******************************************************************************/
 CDFstatus AddChecksumMD5 PROTOARGs((vFILE *vFp, long size,
-                                    unsigned char *signature));
+                                    uChar *signature));
 CDFstatus AddChecksumMD5_64 PROTOARGs((vFILE *vFp, OFF_T size,
-                                       unsigned char *signature));
+                                       uChar *signature));
 CDFstatus ComputeChecksumMD5 PROTOARGs((vFILE *vFp, long size,
-                                        unsigned char *signature));
+                                        uChar *signature));
 CDFstatus ComputeChecksumMD5_64 PROTOARGs((vFILE *vFp, OFF_T size,
-                                           unsigned char *signature));
+                                           uChar *signature));
 CDFstatus GetChecksumMD5 PROTOARGs((vFILE *vFp, long size, 
-                                    unsigned char *signature));
+                                    uChar *signature));
 CDFstatus GetChecksumMD5_64 PROTOARGs((vFILE *vFp, OFF_T size,
-                                       unsigned char *signature));
+                                       uChar *signature));
 CDFstatus AddChecksum PROTOARGs((struct CDFstruct *CDF));
 CDFstatus AddChecksum64 PROTOARGs((struct CDFstruct *CDF));
 CDFstatus VerifyChecksum PROTOARGs((struct CDFstruct *CDF));
@@ -42,12 +48,12 @@ CDFstatus VerifyChecksum64 PROTOARGs((struct CDFstruct *CDF));
 CDFstatus AddChecksumMD5 (vFp, size, signature)
 vFILE *vFp;
 long  size;
-unsigned char signature[16];
+uChar signature[16];
 {
   
 #if defined(vms)
   size_t lastBlk;
-  unsigned char buffer[512];
+  uChar buffer[512];
   int i, j, inBlk, cross= 0;
   inBlk = size % 512;
   lastBlk = (size_t) 512 * (size / 512);
@@ -55,7 +61,8 @@ unsigned char signature[16];
     if (fseek(vFp->fp,lastBlk,SEEK_SET) == EOF) {
       return FALSE;
     }
-    for (i = 0; i < 512; i++) buffer[i] = 0;
+    /* for (i = 0; i < 512; i++) buffer[i] = 0; */
+    memset (buffer, 0, (size_t) 512);
     if (fread(buffer,512,1,vFp->fp) != 1) {
       return FALSE;
     }
@@ -75,7 +82,8 @@ unsigned char signature[16];
     }
 
     if (cross > 0) {
-      for (i = 0; i < 512; i++) buffer[i] = 0;
+      /* for (i = 0; i < 512; i++) buffer[i] = 0; */
+      memset (buffer, 0, (size_t) 512); 
       j = 0;
       for (i = cross; i < 16; i++) {
          buffer[j] = signature[i];
@@ -86,8 +94,10 @@ unsigned char signature[16];
       }
     }
   } else { 
-    for (i = 16; i < 512; i++) buffer[i] = 0;
-    for (i = 0; i < 16; i++) buffer[i] = signature[i];
+    /* for (i = 16; i < 512; i++) buffer[i] = 0; */
+    /* for (i = 0; i < 16; i++) buffer[i] = signature[i]; */
+    memcpy (buffer, signature, (size_t) 16);
+    memset (buffer+16, 0, (size_t) 512-16);
     if (fseek(vFp->fp,size,SEEK_SET) == EOF) {
       return FALSE;
     }
@@ -113,7 +123,7 @@ unsigned char signature[16];
 CDFstatus AddChecksumMD5_64 (vFp, size, signature)
 vFILE *vFp;
 OFF_T  size;
-unsigned char signature[16];
+uChar signature[16];
 {    
 
 #if defined(win32) && (_FILE_OFFSET_BITS == 64)
@@ -126,67 +136,73 @@ unsigned char signature[16];
   }
 #else
   if (vFp->fp == NULL) return FALSE;
-#if defined(vms)
-  OFF_T lastBlk;
-  unsigned char buffer[512];
-  int i, j, inBlk, cross= 0;
-  inBlk = size % 512;
-  lastBlk = (OFF_T) 512 * (size / 512);
-  if (inBlk != 0) {
-    if (FSEEK64(vFp->fp,lastBlk,SEEK_SET) == EOF) {
-      vFp->error = TRUE;
-      return FALSE;
-    }
-    for (i = 0; i < 512; i++) buffer[i] = 0;
-    if (FREAD64(buffer,512,1,vFp->fp) != 1) {
-      vFp->error = TRUE;
-      return FALSE;
-    }
-    for (i = 0; i < 16; i++) {
-       j = inBlk + i;
-       if (j > 511) {
-         cross = i;
-         break;
-       }
-       buffer[j] = signature[i];
-    }
-    if (FSEEK64(vFp->fp,lastBlk,SEEK_SET) == EOF) {
-      vFp->error = TRUE;
-      return FALSE;
-    }
-    if (FWRITE64(buffer,512,1,vFp->fp) != 1) {
-      vFp->error = TRUE;
-      return FALSE;
-    }
+  else {
+#   if defined(vms)
+      OFF_T lastBlk;
+      uChar buffer[512];
+      int i, j, inBlk, cross= 0;
+      inBlk = size % 512;
+      lastBlk = (OFF_T) 512 * (size / 512);
+      if (inBlk != 0) {
+        if (FSEEK64(vFp->fp,lastBlk,SEEK_SET) == EOF) {
+          vFp->error = TRUE;
+          return FALSE;
+        }
+        /* for (i = 0; i < 512; i++) buffer[i] = 0; */
+        memset (buffer, 0, (size_t) 512);
+        if (FREAD64(buffer,512,1,vFp->fp) != 1) {
+          vFp->error = TRUE;
+          return FALSE;
+        }
+        for (i = 0; i < 16; i++) {
+          j = inBlk + i;
+          if (j > 511) {
+            cross = i;
+            break;
+          }
+          buffer[j] = signature[i];
+        }
+        if (FSEEK64(vFp->fp,lastBlk,SEEK_SET) == EOF) {
+          vFp->error = TRUE;
+          return FALSE;
+        }
+        if (FWRITE64(buffer,512,1,vFp->fp) != 1) {
+          vFp->error = TRUE;
+          return FALSE;
+        }
 
-    if (cross > 0) {
-      for (i = 0; i < 512; i++) buffer[i] = 0;
-      j = 0;
-      for (i = cross; i < 16; i++) {
-         buffer[j] = signature[i];
-         j++;
+        if (cross > 0) {
+          /* for (i = 0; i < 512; i++) buffer[i] = 0; */
+          memset (buffer, 0, (size_t) 512);
+          j = 0;
+          for (i = cross; i < 16; i++) {
+            buffer[j] = signature[i];
+            j++;
+          }
+          if (FWRITE64(buffer,512,1,vFp->fp) != 1) {
+            vFp->error = TRUE;
+            return FALSE;
+          }
+        }
+      } else {
+        /* for (i = 16; i < 512; i++) buffer[i] = 0; */
+        /* for (i = 0; i < 16; i++) buffer[i] = signature[i]; */
+        memcpy (buffer, signature, (size_t) 16);
+        memset (buffer+16, 0, (size_t) 512-16);
+        if (FSEEK64(vFp->fp,size,SEEK_SET) == EOF) {
+          vFp->error = TRUE;
+          return FALSE;
+        }
+        if (FWRITE64(buffer,512,1,vFp->fp) != 1) {
+          vFp->error = TRUE;
+          return FALSE;
+        }
       }
-      if (FWRITE64(buffer,512,1,vFp->fp) != 1) {
-        vFp->error = TRUE;
-        return FALSE;
-      }
-    }
-  } else {
-    for (i = 16; i < 512; i++) buffer[i] = 0;
-    for (i = 0; i < 16; i++) buffer[i] = signature[i];
-    if (FSEEK64(vFp->fp,size,SEEK_SET) == EOF) {
-      vFp->error = TRUE;
-      return FALSE;
-    }
-    if (FWRITE64(buffer,512,1,vFp->fp) != 1) {
-      vFp->error = TRUE;
-      return FALSE;
-    }
+#   else
+      if (FSEEK64(vFp->fp,(OFF_T) size,vSEEK_SET) == EOF) return FALSE;
+      if (FWRITE64(signature,1,16,vFp->fp) != 16) return FALSE;
+#   endif
   }
-#else
-  if (FSEEK64(vFp->fp,(OFF_T) size,vSEEK_SET) == EOF) return FALSE;
-  if (FWRITE64(signature,1,16,vFp->fp) != 16) return FALSE;
-#endif
   if (FLUSH64(vFp->fp) == EOF) {
     vFp->error = TRUE;
     return FALSE;
@@ -202,22 +218,29 @@ unsigned char signature[16];
 CDFstatus ComputeChecksumMD5 (vFp, size, signature)
 vFILE *vFp;
 long  size;
-unsigned char  signature[16];
+uChar  signature[16];
 {
-  unsigned char buffer[BUFFSIZE];
+  uChar *buffer;
   struct MD5Context md5c;
   int jj = 0, readBytes;
 
   MD5Init(&md5c);
   if (fseek(vFp->fp, 0, vSEEK_SET) == EOF) return FALSE;
+  buffer = (uChar *) malloc (BLOCKSIZE + 72);
+  if (!buffer) return FALSE; 
   while (jj < size) {
-     if (jj+BUFFSIZE < size) readBytes = BUFFSIZE;
+     if (jj+BLOCKSIZE < size) readBytes = BLOCKSIZE;
      else readBytes = size - jj;
-     if (fread(buffer,1,readBytes,vFp->fp) != (size_t) readBytes) return FALSE;
+     if (fread(buffer,1,readBytes,vFp->fp) != (size_t) readBytes) {
+	free (buffer);
+	return FALSE;
+     }
      MD5Update(&md5c, buffer, (unsigned) readBytes);
      jj = jj + readBytes;
   }
-  MD5Final(signature, &md5c);
+  MD5FinalZ(signature, &md5c);
+  free (buffer);
+  buffer = NULL;
   return TRUE;
 
 }
@@ -229,9 +252,9 @@ unsigned char  signature[16];
 CDFstatus ComputeChecksumMD5_64 (vFp, size, signature)
 vFILE *vFp;
 OFF_T  size;
-unsigned char  signature[16];
+uChar  signature[16];
 {
-  unsigned char buffer[BUFFSIZE];
+  uChar *buffer;
   struct MD5Context md5c;
   OFF_T jj = 0; 
   int readBytes;
@@ -242,19 +265,25 @@ unsigned char  signature[16];
 #else
      if (FSEEK64(vFp->fp,(OFF_T) 0,vSEEK_SET) == EOF) return FALSE;
 #endif
+  buffer = (uChar *) malloc (BLOCKSIZE + 72);
+  if (!buffer) return FALSE;
   while (jj < size) {
-     if (jj+BUFFSIZE < size) readBytes = BUFFSIZE;
+     if (jj+BLOCKSIZE < size) readBytes = BLOCKSIZE;
      else readBytes = (int) (size - jj);
 #if defined(win32) && (_FILE_OFFSET_BITS == 64)
-     if (FREAD64(vFp->fh,buffer,(unsigned int) readBytes) != (int) readBytes)
+     if (FREAD64(vFp->fh,buffer,(unsigned int) readBytes) != (int) readBytes) {
+	free (buffer);
         return FALSE;
+     }
 #else
      if (FREAD64(buffer,1,readBytes,vFp->fp) != readBytes) return FALSE;
 #endif
      MD5Update(&md5c, buffer, (unsigned) readBytes);
      jj = jj + readBytes;
   }
-  MD5Final(signature, &md5c);
+  MD5FinalZ(signature, &md5c);
+  free (buffer);
+  buffer = NULL;
   return TRUE;
 }
 
@@ -265,7 +294,7 @@ unsigned char  signature[16];
 CDFstatus GetChecksumMD5 (vFp, size, signature)
 vFILE *vFp;
 long  size;
-unsigned char signature[16];
+uChar signature[16];
 { 
 
   if (fseek(vFp->fp,size,vSEEK_SET) == EOF) return FALSE;
@@ -280,7 +309,7 @@ unsigned char signature[16];
 CDFstatus GetChecksumMD5_64 (vFp, size, signature)
 vFILE *vFp;
 OFF_T  size;
-unsigned char *signature;
+uChar *signature;
 {
 
 #if defined(win32) && (_FILE_OFFSET_BITS == 64)
@@ -319,7 +348,7 @@ struct CDFstruct *CDF;
   Int32 CDRflags;
   long CCRsize, CPRsize, usedSize; 
   CDFstatus pStatus = CDF_OK;
-  unsigned char signature[16], csig[16];
+  uChar signature[16], csig[16];
 
   if (!sX(ReadCDR(CDF->fp,V2_CDR_OFFSET,
                   CDR_FLAGS,&CDRflags,
@@ -379,7 +408,7 @@ struct CDFstruct *CDF;
   Int32 CDRflags;
   OFF_T CCRsize, CPRsize, usedSize;
   CDFstatus pStatus = CDF_OK;
-  unsigned char signature[16], csig[16];
+  uChar signature[16], csig[16];
 
   if (!sX(ReadCDR64(CDF->fp,V3_CDR_OFFSET64,
                     CDR_FLAGS,&CDRflags,
@@ -450,7 +479,7 @@ struct CDFstruct *CDF;
   Int32 CDRflags;
   long CCRsize, CPRsize, usedSize; 
   CDFstatus pStatus = CDF_OK;
-  unsigned char signature[16];
+  uChar signature[16];
 
   if (!sX(ReadCDR(CDF->fp,V2_CDR_OFFSET,
                   CDR_FLAGS,&CDRflags,
@@ -509,7 +538,7 @@ struct CDFstruct *CDF;
   Int32 CDRflags;
   OFF_T CCRsize, CPRsize, usedSize; 
   CDFstatus pStatus = CDF_OK;
-  unsigned char signature[16];
+  uChar signature[16];
 
   if (!sX(ReadCDR64(CDF->fp,V3_CDR_OFFSET64,
                     CDR_FLAGS,&CDRflags,
