@@ -1,4 +1,9 @@
 /******************************************************************************
+* Copyright 1996-2014 United States Government as represented by the
+* Administrator of the National Aeronautics and Space Administration.
+* All Rights Reserved.
+******************************************************************************/
+/******************************************************************************
 *
 *  NSSDC/CDF                                           CDF `close' operations.
 *
@@ -41,6 +46,7 @@ struct CurStruct *Cur;
     case CDF_:
     case CDFwithSTATS_: {
       struct CDFstruct *CDF; vSTATS vStats;
+      long version, release, increment;
       Logical statistics = (Va->item == CDFwithSTATS_);
       vSTATS *vStatsDotP = BOO(statistics,va_arg(Va->ap,vSTATS *),NULL);
       vSTATS *vStatsStageP = BOO(statistics,va_arg(Va->ap,vSTATS *),NULL);
@@ -67,13 +73,15 @@ struct CurStruct *Cur;
 	KillAbortedCDF (CDF, Cur);
 	break;
       }
+      CDFgetVersion ((CDFid) CDF, &version, &release, &increment);
       /************************************************************************
       * Get the Vstream statistics for the dotCDF file(s) so far.
       ************************************************************************/
       AddTOvStats (vStatsDotP, &CDF->dotCDFvStats);
       AddTOvStats (vStatsDotP, &CDF->uDotCDFvStats);
       /************************************************************************
-      * If the CDF is open read/write, update the dotCDF file.
+      * If the CDF is open read/write, update the dotCDF file to flush out
+      * the (compressed) data in the staging area..
       ************************************************************************/
       if (CDF->status == READ_WRITE) {
 	if (!sX(UpdateDotCDF64(CDF),&pStatus)) {
@@ -144,7 +152,7 @@ struct CurStruct *Cur;
 	    return CDF_CREATE_ERROR;
 	  }
 	  /********************************************************************
-	  * Write the compressed dotCDF file.
+	  * Write the compressed data into new dotCDF file.
 	  ********************************************************************/
 	  if (!sX(WriteCompressedCDF64(CDF,&CPR,notEMPTY),&pStatus)) {
 	    AbortAccess64 (CDF, noUPDATE, noDELETE);
@@ -217,6 +225,7 @@ struct CurStruct *Cur;
         /**********************************************************************
 	* Not compressed - first close the dotCDF file.
 	**********************************************************************/
+        if (CDF->readOnly) ResetReadOnlyState64(CDF);
 	if (!CLOSEv64(CDF->dotFp,CDF,&vStats)) {
 	  CDF->dotFp = NULL;
 	  AbortAccess64 (CDF, noUPDATE, noDELETE);
@@ -269,12 +278,13 @@ struct CurStruct *Cur;
 	DisplayVs (getenv("VSTREAM_STATS"), "Compress", &vStats);
 #endif
       }
+      if (!PriorTo ("3.5.0", (Int32)version, (Int32)release, (Int32)increment))
+        ReduceOpenCDFsCount();
       /************************************************************************
       * Free memory used and clear current CDF.
       ************************************************************************/
       FreeCDFid (CDF, FALSE);
       Cur->cdf = NULL;
-      ReduceOpenCDFsCount();
       break;
     }
     /**************************************************************************
