@@ -192,13 +192,6 @@ switch (Va->item) {
     SelectCDF (Cur->cdf, CDF)
     if (BADzOP(CDF,!zOp)) return ILLEGAL_IN_zMODE;
     if (!CURRENTvarSELECTED(CDF,zOp)) return NO_VAR_SELECTED;
-/*
-    if (!sX(LocateCurrentVar(CDF,zOp,&offset,&zVar,NULL),&pStatus)) {
-      AbortAccess (CDF, UPDATE, noDELETE);
-      return pStatus;
-    }
-*/
-
     zVar = CurrentVarMode(CDF,zOp);
     if (zModeON(CDF) || zVar) offset = CDF->CURzVarOffset;
     else offset = CDF->CURrVarOffset;
@@ -475,8 +468,8 @@ switch (Va->item) {
     int entryType = E3p(Va->item,gENTRY_EXISTENCE_,
 				 rENTRY_EXISTENCE_,
 				 zENTRY_EXISTENCE_);
-    struct CDFstruct *CDF; Logical zEntry;
-    long entryNum = va_arg (Va->ap, long), entryN;
+    struct CDFstruct *CDF; Logical zEntry = FALSE;
+    long entryNum = va_arg (Va->ap, long), entryN = -1L;
     SelectCDF (Cur->cdf, CDF)
     if (!CURRENTattrSELECTED(CDF)) return NO_ATTR_SELECTED;
     if (!sX(CheckEntryOp(CDF,entryType),&pStatus)) return pStatus;
