@@ -129,13 +129,15 @@ switch (Va->item) {
     /**************************************************************************
     * Allocate/initialize CDF structure.
     **************************************************************************/
-    CDF = (struct CDFstruct *) cdf_AllocateMemory ((size_t)sizeof(struct CDFstruct), NULL);
+    CDF = (struct CDFstruct *) 
+		cdf_AllocateMemory ((size_t)sizeof(struct CDFstruct), NULL);
     if (CDF == NULL) {
       V_close (dotFp, NULL, NULL);
       CDFdeleteFile (CDFnameX);
       return BAD_MALLOC;
     }
-    CDF->CDFname = (char *) cdf_AllocateMemory ((size_t)strlen(CDFnameTx) + 1, NULL);
+    CDF->CDFname = (char *) cdf_AllocateMemory ((size_t)strlen(CDFnameTx) + 1,
+						 NULL);
     if (CDF->CDFname == NULL) {
       V_close (dotFp, NULL, NULL);
       CDFdeleteFile (CDFnameX);
@@ -188,6 +190,9 @@ switch (Va->item) {
     CDF->workingCacheSize = BOO(CDF->singleFile,NUMcacheSINGLE,NUMcacheMULTI);
     CDF->compressCacheSize = NUMcacheCOMPRESS;
     CDF->checksum = BOO(CDFgetChecksumEnvVar()<=0,NONE_CHECKSUM,MD5_CHECKSUM);
+    CDF->keptVarNum = -1;
+    CDF->keptVarzOp = FALSE;
+
     InitCURobjectsStates (CDF);
     AddTOvStats (&CDF->dotCDFvStats, NULL);
     AddTOvStats (&CDF->uDotCDFvStats, NULL);
@@ -196,6 +201,7 @@ switch (Va->item) {
     **************************************************************************/
     if (!CACHEv(CDF->fp,CDF->workingCacheSize)) {
       AbortAccess (CDF, noUPDATE, CDFDELETE);
+      cdf_FreeMemory (CDF->CDFname, NULL);
       cdf_FreeMemory (CDF, NULL);
       return BAD_CACHE_SIZE;
     }
@@ -204,11 +210,13 @@ switch (Va->item) {
     **************************************************************************/
     if (!Write32(CDF->fp,(Int32 *)&magicNumber1)) {
       AbortAccess (CDF, noUPDATE, CDFDELETE);
+      cdf_FreeMemory (CDF->CDFname, NULL);
       cdf_FreeMemory (CDF, NULL);
       return CDF_WRITE_ERROR;
     }
     if (!Write32(CDF->fp,(Int32 *)&magicNumber2u)) {
       AbortAccess (CDF, noUPDATE, CDFDELETE);
+      cdf_FreeMemory (CDF->CDFname, NULL);
       cdf_FreeMemory (CDF, NULL);
       return CDF_WRITE_ERROR;
     }
@@ -241,6 +249,7 @@ switch (Va->item) {
 		     CDR_RECORD,&CDR,copyRight,
 		     CDR_NULL),&pStatus)) {
       AbortAccess (CDF, noUPDATE, CDFDELETE);
+      cdf_FreeMemory (CDF->CDFname, NULL);
       cdf_FreeMemory (CDF, NULL);
       return pStatus;
     }
@@ -270,6 +279,7 @@ switch (Va->item) {
 		     GDR_RECORD,&GDR,
 		     GDR_NULL),&pStatus)) {
       AbortAccess (CDF, noUPDATE, CDFDELETE);
+      cdf_FreeMemory (CDF->CDFname, NULL);
       cdf_FreeMemory (CDF, NULL);
       return pStatus;
     }
@@ -442,6 +452,7 @@ switch (Va->item) {
     if (!sX(AllocateIR(CDF,VDR.RecordSize+padSize,&offset),&pStatus)) {
       CDFdeleteFile (pathnameX);
       AbortAccess (CDF, UPDATE, noDELETE);
+      cdf_FreeMemory (pad, NULL);
       return pStatus;
     }
     DefaultPadValuePre350(dataType, numElements, pad);
@@ -452,6 +463,7 @@ switch (Va->item) {
 		     VDR_NULL),&pStatus)) {
       CDFdeleteFile (pathnameX);
       AbortAccess (CDF, UPDATE, noDELETE);
+      cdf_FreeMemory (pad, NULL);
       return pStatus;
     }
     cdf_FreeMemory (pad, NULL);
