@@ -132,13 +132,15 @@ switch (Va->item) {
     /**************************************************************************
     * Allocate/initialize CDF structure.
     **************************************************************************/
-    CDF = (struct CDFstruct *) cdf_AllocateMemory ((size_t)sizeof(struct CDFstruct), NULL);
+    CDF = (struct CDFstruct *) 
+		cdf_AllocateMemory ((size_t)sizeof(struct CDFstruct), NULL);
     if (CDF == NULL) {
       V_close64 (dotFp, NULL, NULL);
       CDFdeleteFile (CDFnameX);
       return BAD_MALLOC;
     }
-    CDF->CDFname = (char *) cdf_AllocateMemory ((size_t)strlen(CDFnameT) + 1, NULL);
+    CDF->CDFname = (char *) cdf_AllocateMemory ((size_t)strlen(CDFnameT) + 1,
+						 NULL);
     if (CDF->CDFname == NULL) {
       V_close64 (dotFp, NULL, NULL);
       CDFdeleteFile (CDFnameX);
@@ -150,7 +152,7 @@ switch (Va->item) {
     CDF->magic = VALIDid_MAGIC_NUMBER;
     CDF->largeFile = TRUE;
     CDF->CDRoffset64 = (OFF_T) V3_CDR_OFFSET64;
-    CDF->GDRoffset64 = (OFF_T) NO_OFFSET64; /* Reset below when GDR is written. */
+    CDF->GDRoffset64 = (OFF_T) NO_OFFSET64; /* Reset when GDR is written. */
     CDF->readOnly = FALSE;
     CDF->zMode = zMODEoff;
     CDF->decoding = HOST_DECODING;
@@ -192,6 +194,9 @@ switch (Va->item) {
     CDF->compressCacheSize = NUMcacheCOMPRESS;
     CDF->checksum = BOO(CDFgetChecksumEnvVar()<=0,NONE_CHECKSUM,MD5_CHECKSUM);
     CDF->leapSecondUpdated = 1;
+    CDF->keptVarNum = -1;
+    CDF->keptVarzOp = FALSE;
+
     InitCURobjectsStates (CDF);
     AddTOvStats (&CDF->dotCDFvStats, NULL);
     AddTOvStats (&CDF->uDotCDFvStats, NULL);
@@ -200,6 +205,7 @@ switch (Va->item) {
     **************************************************************************/
     if (!CACHEv64(CDF->fp,CDF->workingCacheSize)) {
       AbortAccess64 (CDF, noUPDATE, CDFDELETE);
+      cdf_FreeMemory (CDF->CDFname, NULL);
       cdf_FreeMemory (CDF, NULL);
       return BAD_CACHE_SIZE;
     }
@@ -208,11 +214,13 @@ switch (Va->item) {
     **************************************************************************/
     if (!Write32_64(CDF->fp,(Int32 *)&magicNumber1)) {
       AbortAccess64 (CDF, noUPDATE, CDFDELETE);
+      cdf_FreeMemory (CDF->CDFname, NULL);
       cdf_FreeMemory (CDF, NULL);
       return CDF_WRITE_ERROR;
     }
     if (!Write32_64(CDF->fp,(Int32 *)&magicNumber2u)) {
       AbortAccess64 (CDF, noUPDATE, CDFDELETE);
+      cdf_FreeMemory (CDF->CDFname, NULL);
       cdf_FreeMemory (CDF, NULL);
       return CDF_WRITE_ERROR;
     }
@@ -239,6 +247,7 @@ switch (Va->item) {
 		       CDR_RECORD,&CDR,copyRight,
 		       CDR_NULL),&pStatus)) {
       AbortAccess64 (CDF, noUPDATE, CDFDELETE);
+      cdf_FreeMemory (CDF->CDFname, NULL);
       cdf_FreeMemory (CDF, NULL);
       return pStatus;
     }
@@ -246,7 +255,7 @@ switch (Va->item) {
     * Write GDR.
     **************************************************************************/
     CDF->GDRoffset64 = CDR.GDRoffset;
-    GDR.RecordSize = GDR_BASE_SIZE64 + (OFF_T) ((numDims * sizeof(Int32)));
+    GDR.RecordSize = GDR_BASE_SIZE64 + (OFF_T) (numDims * 4);
     GDR.RecordType = GDR_;
     GDR.rVDRhead = (OFF_T) 0;
     GDR.zVDRhead = (OFF_T) 0;
@@ -273,6 +282,7 @@ switch (Va->item) {
 		       GDR_RECORD,&GDR,
 		       GDR_NULL),&pStatus)) {
       AbortAccess64 (CDF, noUPDATE, CDFDELETE);
+      cdf_FreeMemory (CDF->CDFname, NULL);
       cdf_FreeMemory (CDF, NULL);
       return pStatus;
     }
@@ -415,12 +425,11 @@ switch (Va->item) {
     /**************************************************************************
     * Write rVDR/zVDR.
     **************************************************************************/
-    if (zOp) ttmmpp = zNumDims * sizeof(Int32);
+    if (zOp) ttmmpp = zNumDims * 4;
     else ttmmpp = 0;
     VDR.RecordSize = BOO(zOp,zVDR_BASE_SIZE64,rVDR_BASE_SIZE64) +
 		     ttmmpp +                                     /*DimSizes.*/
-		     BOO(zOp,zNumDims,
-			     CDF->rNumDims)*sizeof(Int32);	  /*DimVarys.*/
+		     BOO(zOp,zNumDims, CDF->rNumDims)*4;	  /*DimVarys.*/
     VDR.RecordType = BOO(zOp,zVDR_,rVDR_);
     VDR.VDRnext = (OFF_T) 0;
     VDR.DataType = dataType;
@@ -453,6 +462,7 @@ switch (Va->item) {
     if (!sX(AllocateIR64(CDF,VDR.RecordSize+padSize,&offset),&pStatus)) {
       CDFdeleteFile (pathnameX);
       AbortAccess64 (CDF, UPDATE, noDELETE);
+      cdf_FreeMemory (pad, NULL);
       return pStatus;
     }
     DefaultPadValue(dataType, numElements, pad);
@@ -463,6 +473,7 @@ switch (Va->item) {
 		       VDR_NULL),&pStatus)) {
       CDFdeleteFile (pathnameX);
       AbortAccess64 (CDF, UPDATE, noDELETE);
+      cdf_FreeMemory (pad, NULL);
       return pStatus;
     }
     cdf_FreeMemory (pad, NULL);
