@@ -115,7 +115,8 @@ switch (Va->item) {
       CDF->stage.fp = NULL;
     }
     if (CDF->compressFp != NULL) {
-      if (!DELETEv64(CDF->compressFp,NULL)) sX (SCRATCH_DELETE_ERROR, &pStatus);
+      if (!DELETEv64(CDF->compressFp,NULL)) sX (SCRATCH_DELETE_ERROR,
+                     &pStatus);
       CDF->compressFp = NULL;
     }
     if (!PriorTo ("3.5.0", (Int32)version, (Int32)release, (Int32)increment))
@@ -145,14 +146,6 @@ switch (Va->item) {
     * Switch to write access.
     **************************************************************************/
     if (!WriteAccess64(CDF,FALSE,&pStatus)) return pStatus;
-
-/*
-    if (!sX(LocateCurrentVar64(CDF,zOp,&vOffset,&zVar,NULL),&pStatus)) {
-      AbortAccess64 (CDF, UPDATE, noDELETE);
-      return pStatus;
-    }
-*/
-
     zVar = CurrentVarMode(CDF,zOp);
     if (zModeON(CDF) || zVar) vOffset = CDF->CURzVarOffset64;
     else vOffset = CDF->CURrVarOffset64;
@@ -320,6 +313,8 @@ switch (Va->item) {
       CDF->CURrVarNum = RESERVED_VARNUM;
       CDF->CURrVarOffset64 = (OFF_T) 0;
     }
+    CDF->keptVarzOp = zOp;
+    CDF->keptVarNum = -1;
     /**************************************************************************
     * Delete the associated attribute entries and renumber the entries that
     * are associated with the variables that followed the variable that was
@@ -1075,7 +1070,8 @@ Int32 *deletedTo;
   * ...and start there.
   ****************************************************************************/
   if (!sX(DeleteRecords_r_64(CDF,Var,vxrOffset,firstRec,
-			     lastRec,deletedTo,&total),&pStatus)) return pStatus;
+			     lastRec,deletedTo,&total),&pStatus))
+    return pStatus;
   /****************************************************************************
   * If the top level of VXR(s) was totally deleted, set the VXR head/tail to
   * ZERO_OFFSET.
@@ -1094,8 +1090,8 @@ Int32 *deletedTo;
 * DeleteRecords_r_64.
 ******************************************************************************/
 
-static CDFstatus DeleteRecords_r_64 (CDF, Var, firstVXRoffset, firstRec, lastRec,
-				     deletedTo, total)
+static CDFstatus DeleteRecords_r_64 (CDF, Var, firstVXRoffset, firstRec,
+				     lastRec, deletedTo, total)
 struct CDFstruct *CDF;
 struct VarStruct *Var;
 OFF_T firstVXRoffset;
@@ -1354,7 +1350,7 @@ OFF_T irSize;
       /************************************************************************
       * Compress the block of remaining records...
       ************************************************************************/
-      if (!sX(Compress64(CDF->stage.fp,offset2,
+      if (!sX(Compress64(CDF,CDF->stage.fp,offset2,
 		         nBytes2,SCRATCH_READ_ERROR,
 		         Var->cType,Var->cParms,
 		         CDF->compressFp,(OFF_T) ZERO_OFFSET64,
@@ -1558,7 +1554,7 @@ OFF_T irSize;
       /************************************************************************
       * Compress the first block of remaining records...
       ************************************************************************/
-      if (!sX(Compress64(CDF->stage.fp,
+      if (!sX(Compress64(CDF,CDF->stage.fp,
 		         Var->stage.areaOffset64,
 		         nBytes1,SCRATCH_READ_ERROR,
 		         Var->cType,Var->cParms,
@@ -1581,7 +1577,7 @@ OFF_T irSize;
       /************************************************************************
       * Compress the second block of remaining records...
       ************************************************************************/
-      if (!sX(Compress64(CDF->stage.fp,offset2,
+      if (!sX(Compress64(CDF,CDF->stage.fp,offset2,
 		         nBytes2,SCRATCH_READ_ERROR,
 		         Var->cType,Var->cParms,
 		         CDF->compressFp,(OFF_T) ZERO_OFFSET64,
@@ -1689,7 +1685,7 @@ OFF_T irSize;
       /************************************************************************
       * Compress the block of remaining records...
       ************************************************************************/
-      if (!sX(Compress64(CDF->stage.fp,
+      if (!sX(Compress64(CDF,CDF->stage.fp,
 		         Var->stage.areaOffset64,
 		         nBytes1,SCRATCH_READ_ERROR,
 		         Var->cType,Var->cParms,
@@ -1721,8 +1717,8 @@ OFF_T irSize;
 * DeleteVXRentry64.
 ******************************************************************************/
 
-static CDFstatus DeleteVXRentry64 (CDF, firstVXRoffset, delVXRoffset, delEntryN,
-				   total)
+static CDFstatus DeleteVXRentry64 (CDF, firstVXRoffset, delVXRoffset,
+				   delEntryN, total)
 struct CDFstruct *CDF;
 OFF_T firstVXRoffset;
 OFF_T delVXRoffset;
@@ -1814,8 +1810,8 @@ Logical *total;
 * InsertIndexEntry64.
 ******************************************************************************/
 
-static CDFstatus InsertIndexEntry64 (CDF, vxrOffset, entryN, after, first, last,
-				     offset)
+static CDFstatus InsertIndexEntry64 (CDF, vxrOffset, entryN, after, first,
+				     last, offset)
 struct CDFstruct *CDF;
 OFF_T vxrOffset;
 int entryN;
@@ -1937,7 +1933,8 @@ OFF_T offset;
 * UpdateIndexEntries_r_64.
 ******************************************************************************/
 
-static CDFstatus UpdateIndexEntries_r_64 (fp, vxrOffset, aboveRecord, recordCount)
+static CDFstatus UpdateIndexEntries_r_64 (fp, vxrOffset, aboveRecord,
+					  recordCount)
 vFILE *fp;
 OFF_T vxrOffset;
 Int32 aboveRecord;
