@@ -1,4 +1,9 @@
 /******************************************************************************
+* Copyright 1996-2014 United States Government as represented by the
+* Administrator of the National Aeronautics and Space Administration.
+* All Rights Reserved.
+******************************************************************************/
+/******************************************************************************
 *
 *  NSSDC/CDF				 CDF Standard Interface (I) /FORTRAN.
 *
@@ -31,6 +36,7 @@
 *                               CDF_getzVarsRecordData, CDF_putrVarsRecordData,
 *                               CDF_putzVarsRecordData to use the number of
 *                               passed variables to allocate buffers.
+*   V3.3  10-Jan-09, M Liu      Added CDF_set_Validate and CDF_get_Validate.
 *
 ******************************************************************************/
 
@@ -946,16 +952,16 @@ Fif_ENTRYPOINT(cdf_inquire_attr__,
 	       CDF_INQUIRE_ATTR)
 (id, attr_num, attr_name, attr_scope, max_gentry, max_rentry, max_zentry, 
  status Fif_GHOSTARG(len))
-Int32	  *id;		   	/* In: CDF identifier. */
-Int32	  *attr_num;	  	/* In: Attribute number. */
-void	  *attr_name;      	/* Out: Attribute name. */
-Int32	  *attr_scope;	   	/* Out: Attribute scope. */
-Int32     *max_gentry;          /* Out: Maximum gEntry number if global attribute. */
-Int32     *max_rentry;          /* Out: Maximum rEntry number if variable attribute. */
-Int32	  *max_zentry;	   	/* Out: Maximum zEntry number if variable attribute. */
-Int32	  *status;		/* Out: CDF status code. */
-Fif_GHOSTDEF(len)		/* Invisible length of "attr_name"
-				   (generated by FORTRAN compiler). */
+Int32	  *id;		/* In: CDF identifier. */
+Int32	  *attr_num;	/* In: Attribute number. */
+void	  *attr_name;   /* Out: Attribute name. */
+Int32	  *attr_scope;	/* Out: Attribute scope. */
+Int32     *max_gentry;  /* Out: Maximum gEntry number if global attribute. */
+Int32     *max_rentry;  /* Out: Maximum rEntry number if variable attribute. */
+Int32	  *max_zentry;	/* Out: Maximum zEntry number if variable attribute. */
+Int32	  *status;	/* Out: CDF status code. */
+Fif_GHOSTDEF(len)	/* Invisible length of "attr_name"
+			   (generated by FORTRAN compiler). */
 {
   char attrNameT[CDF_ATTR_NAME_LEN256];
   long maxgEntryT = -1, maxrEntryT = -1, maxzEntryT = -1, attrScopeT;
@@ -1380,7 +1386,7 @@ Int32	   *status;		/* Out: CDF status code. */
 }
 
 /******************************************************************************
-*  CDF_attr_entry.
+*  CDF_attr_get.
 ******************************************************************************/
 
 VISIBLE_PREFIX
@@ -1398,7 +1404,7 @@ void	  *value;	   	/* Out: Value.
 					      type. */
 Int32	  *status;		/* Out: CDF status code. */
 {
-  long dataType, scope;
+  long dataType, numElems, scope;
   long attrNumT = (long) (*attr_num - 1);
   long entryNumT = (long) (*entry_num - 1);
 
@@ -1413,6 +1419,9 @@ Int32	  *status;		/* Out: CDF status code. */
 			    GET_, BOO(GLOBALscope(scope),
 				      gENTRY_DATATYPE_,
 				      rENTRY_DATATYPE_), &dataType,
+			          BOO(GLOBALscope(scope),
+				      gENTRY_NUMELEMS_,
+				      rENTRY_NUMELEMS_), &numElems,
 			    NULL_);
   if (StatusBAD(*status)) return;
 
@@ -1772,7 +1781,8 @@ Fif_GHOSTDEF(len)		/* Invisible length of "var_name"
 						     &ssh),
 #endif
 #if defined(Fif_GHOSTLEN)
-				       NULterminate(var_name,Fif_GHOSTUSE(len),
+				       NULterminate(var_name,
+                                                    Fif_GHOSTUSE(len),
 						    &ssh),
 #endif
 #if defined(Fif_NOLEN)
@@ -1794,7 +1804,8 @@ Fif_GHOSTDEF(len)		/* Invisible length of "var_name"
                                                        &ssh),
 #endif
 #if defined(Fif_GHOSTLEN)
-                                         NULterminate(var_name,Fif_GHOSTUSE(len),
+                                         NULterminate(var_name,
+                                                      Fif_GHOSTUSE(len),
                                                       &ssh),
 #endif
 #if defined(Fif_NOLEN)
@@ -2038,7 +2049,6 @@ Fif_GHOSTDEF(len)		/* Invisible length of "var_name"
 			    GET_, rVARs_NUMDIMS_, &numDims,
 			    NULL_);
   if (StatusBAD(*status)) {
-    free (varNameT);
     return;
   }
 
@@ -2050,7 +2060,6 @@ Fif_GHOSTDEF(len)		/* Invisible length of "var_name"
 				  rVAR_DIMVARYS_, dimVarysT,
 			    NULL_);
   if (StatusBAD(*status)) {
-    free (varNameT);
     return;
   }
 
@@ -2746,7 +2755,7 @@ Int32     *status;              /* Out: CDF status code. */
     return;
   }
 
-  varNumsT = cdf_AllocateMemory (*num_vars*sizeof(long), NULL);
+  varNumsT = cdf_AllocateMemory ((size_t)*num_vars*sizeof(long), NULL);
   if (varNumsT == NULL) {
     *status = BAD_MALLOC;
     return;
@@ -2796,7 +2805,7 @@ Int32     *status;              /* Out: CDF status code. */
     return;
   }
 
-  varNumsT = cdf_AllocateMemory (*num_vars*sizeof(long), NULL);
+  varNumsT = cdf_AllocateMemory ((size_t)*num_vars*sizeof(long), NULL);
   if (varNumsT == NULL) {
     *status = BAD_MALLOC;
     return;
@@ -2847,7 +2856,7 @@ Int32     *status;              /* Out: CDF status code. */
     return;
   }
 
-  varNumsT = cdf_AllocateMemory (*num_vars*sizeof(long), NULL);
+  varNumsT = cdf_AllocateMemory ((size_t)*num_vars*sizeof(long), NULL);
   if (varNumsT == NULL) {
     *status = BAD_MALLOC;
     return;
@@ -2896,7 +2905,7 @@ Int32     *status;              /* Out: CDF status code. */
     return;
   }
 
-  varNumsT = cdf_AllocateMemory (*num_vars*sizeof(long), NULL);
+  varNumsT = cdf_AllocateMemory ((size_t)*num_vars*sizeof(long), NULL);
   if (varNumsT == NULL) {
     *status = BAD_MALLOC;
     return;
@@ -2944,7 +2953,7 @@ Int32     *status;              /* Out: CDF status code. */
                             NULL_);
   if (StatusBAD(*status)) return;
 
-  varNumsT = cdf_AllocateMemory (*num_vars*sizeof(long), NULL);
+  varNumsT = cdf_AllocateMemory ((size_t)*num_vars*sizeof(long), NULL);
   if (varNumsT == NULL) {
     *status = BAD_MALLOC;
     return;
@@ -2994,7 +3003,7 @@ Int32     *status;              /* Out: CDF status code. */
                             NULL_);
   if (StatusBAD(*status)) return;
 
-  varNumsT = cdf_AllocateMemory (*num_vars*sizeof(long), NULL);
+  varNumsT = cdf_AllocateMemory ((size_t)*num_vars*sizeof(long), NULL);
   if (varNumsT == NULL) {
     *status = BAD_MALLOC;
     return;
@@ -3046,3 +3055,40 @@ Fif_ENTRYPOINT(cdf_get_filebackward__,cdf_get_filebackward_,
   else return (Int32) 1;
 }
 
+/******************************************************************************
+*  CDF_set_Validate.
+******************************************************************************/
+
+VISIBLE_PREFIX
+Fif_PREFIXa
+void
+Fif_PREFIXb
+Fif_ENTRYPOINT(cdf_set_validate__,cdf_set_validate_,
+               cdf_set_validate, CDF_SET_VALIDATE)
+(flag)
+Int32     *flag;                /* In: Flag to set the file to old version. */
+{
+  int fg = (int) *flag;
+  CDFsetValidate(fg);
+  return;
+}
+
+/******************************************************************************
+*  CDF_get_validate.
+******************************************************************************/
+
+VISIBLE_PREFIX
+Fif_PREFIXa
+Int32
+Fif_PREFIXb
+Fif_ENTRYPOINT(cdf_get_validate__,cdf_get_validate_,
+               cdf_get_validate, CDF_GET_VALIDATE)
+()
+{
+
+  int flag;
+  flag = CDFgetValidate();
+  if (flag == 0) return (Int32) 0;
+  else return (Int32) 1;
+}
+
