@@ -68,7 +68,6 @@ switch (Va->item) {
 #if defined(vms) || defined(dos)
     MakeUpperString (CDFpathT);
 #endif
-/*  if (!ValidCDFname(CDFpathT)) return BAD_CDF_NAME; */
     if (!sX(FindCDF(CDFpathT,&no_append,
 		    &upper_case_ext,
 		    &version_numbers),&pStatus)) return pStatus;
@@ -774,7 +773,7 @@ switch (Va->item) {
       }
       if (vType == SPARSE_RECORDS_) {
           if (Var != NULL) {
-            if (Var->stage.areaOffset64 != (OFF_T) NO_OFFSET64 &&
+            if (Var->stage.areaOffset != (Int32) NO_OFFSET &&
                 Var->stage.firstRec != NO_RECORD) {
               Int32 tmp = (Var->stage.lastRec > maxRec ? maxRec :
                            Var->stage.lastRec);
@@ -784,9 +783,9 @@ switch (Va->item) {
       } else if (vType == COMPRESSED_ ||
                  vType == SPARSE_COMPRESSED_RECORDS_) {
         if (Var != NULL) {
-          if (Var->stage.areaOffset64 != (OFF_T) NO_OFFSET64 &&
+          if (Var->stage.areaOffset != (Int32) NO_OFFSET &&
               Var->stage.firstRec != NO_RECORD &&
-              Var->stage.dotOffset64 == (OFF_T) NO_OFFSET64) {
+              Var->stage.dotOffset == (Int32) NO_OFFSET) {
             Int32 tmp = (Var->stage.lastRec > maxRec ? maxRec :
                          Var->stage.lastRec);
             lastAllocatedRecN += (tmp - Var->stage.firstRec + 1);
@@ -846,7 +845,8 @@ switch (Va->item) {
 	if (Var != NULL) {
 	  if (Var->stage.areaOffset != NO_OFFSET &&
 	      Var->stage.firstRec != NO_RECORD) {
-	    Int32 tmp = (Var->stage.lastRec > tMaxRec ? tMaxRec : Var->stage.lastRec);
+	    Int32 tmp = (Var->stage.lastRec > tMaxRec ? tMaxRec :
+                                                        Var->stage.lastRec);
 	    nRecords += (tmp - Var->stage.firstRec + 1);
 	  }
 	}
@@ -860,7 +860,8 @@ switch (Va->item) {
 	  if (Var->stage.areaOffset != NO_OFFSET &&
 	      Var->stage.firstRec != NO_RECORD &&
 	      Var->stage.dotOffset == NO_OFFSET) {
-            Int32 tmp = (Var->stage.lastRec > tMaxRec ? tMaxRec : Var->stage.lastRec);
+            Int32 tmp = (Var->stage.lastRec > tMaxRec ? tMaxRec :
+                                                        Var->stage.lastRec);
 	    nRecords += (tmp - Var->stage.firstRec + 1);
 	  }
 	}
@@ -983,6 +984,7 @@ switch (Va->item) {
     struct CDFstruct *CDF; Int32 offset, nextRec;
     Int32 startRec = (Int32) va_arg (Va->ap, long);
     long *fromRec = va_arg (Va->ap, long *);
+    Int32 flags, sRecords;
     SelectCDF (Cur->cdf, CDF)
     if (BADzOP(CDF,!zOp)) return ILLEGAL_IN_zMODE;
     if (!CURRENTvarSELECTED(CDF,zOp)) return NO_VAR_SELECTED;
@@ -992,6 +994,34 @@ switch (Va->item) {
     if (zModeON(CDF) || zVar) offset = CDF->CURzVarOffset;
     else offset = CDF->CURrVarOffset;
 
+    if (!CDF->readOnly) {
+      if (!sX(ReadVDR(CDF,CDF->fp,offset,zVar,
+		      VDR_FLAGS,&flags,
+		      VDR_sRECORDS,&sRecords,
+		      VDR_NULL),&pStatus)) {
+        AbortAccess (CDF, UPDATE, noDELETE);
+        return pStatus;
+      }
+      /************************************************************************
+      * If the compressed or sparse records, flush out the variable if its
+      * VXR is not yet created.
+      ************************************************************************/
+      if (VARcompressionBITset(flags) || sRecords != NO_SPARSERECORDS) {
+	struct VarStruct *Var;
+	if (zModeON(CDF))
+	  if (CDF->CURzVarNum < CDF->NrVars)
+	    Var = CDF->rVars[(int)CDF->CURzVarNum];
+	  else
+	    Var = CDF->zVars[(int)(CDF->CURzVarNum - CDF->NrVars)];
+	else
+	  Var = BOO(zOp,CDF->zVars[(int)CDF->CURzVarNum],
+			CDF->rVars[(int)CDF->CURrVarNum]);
+	if (Var != NULL) {
+	  if (!sX(FlushStage(CDF,Var),&pStatus)) return pStatus;
+        }
+      }
+    }
+
     if (!sX(NextRecord(CDF,offset,zVar,startRec,&nextRec,&found),&pStatus)) {
       AbortAccess (CDF, UPDATE, noDELETE);
       return pStatus;
@@ -1011,6 +1041,7 @@ switch (Va->item) {
     struct CDFstruct *CDF; Int32 vdrOffset, lastRec;
     Int32 startRec = (Int32) va_arg (Va->ap, long);
     long *toRec = va_arg (Va->ap, long *);
+    Int32 flags, sRecords;
     SelectCDF (Cur->cdf, CDF)
     if (BADzOP(CDF,!zOp)) return ILLEGAL_IN_zMODE;
     if (!CURRENTvarSELECTED(CDF,zOp)) return NO_VAR_SELECTED;
@@ -1020,6 +1051,34 @@ switch (Va->item) {
     if (zModeON(CDF) || zVar) vdrOffset = CDF->CURzVarOffset;
     else vdrOffset = CDF->CURrVarOffset;
 
+    if (!CDF->readOnly) {
+      if (!sX(ReadVDR(CDF,CDF->fp,vdrOffset,zVar,
+		      VDR_FLAGS,&flags,
+		      VDR_sRECORDS,&sRecords,
+		      VDR_NULL),&pStatus)) {
+	AbortAccess (CDF, UPDATE, noDELETE);
+	return pStatus;
+      }
+      /************************************************************************
+      * If the compressed or sparse records, flush out the variable in case
+      * its VXR is not yet created.
+      ************************************************************************/
+      if (VARcompressionBITset(flags) || sRecords != NO_SPARSERECORDS) {
+	struct VarStruct *Var;
+	if (zModeON(CDF))
+	  if (CDF->CURzVarNum < CDF->NrVars)
+	    Var = CDF->rVars[(int)CDF->CURzVarNum];
+	  else
+	    Var = CDF->zVars[(int)(CDF->CURzVarNum - CDF->NrVars)];
+	else
+	  Var = BOO(zOp,CDF->zVars[(int)CDF->CURzVarNum],
+			CDF->rVars[(int)CDF->CURrVarNum]);
+	if (Var != NULL) {
+	  if (!sX(FlushStage(CDF,Var),&pStatus)) return pStatus;
+	}
+      }
+    }
+
     /**************************************************************************
     * Determine the last record in the block containing the starting record.
     **************************************************************************/
@@ -1339,7 +1398,6 @@ switch (Va->item) {
     if (!sX(ReadVDR(CDF,CDF->fp,offset,zVar,
 		    VDR_FLAGS,&flags,
 		    VDR_DATATYPE,&dataType,
-		    VDR_NUMELEMS,&numElems,
 		    VDR_NULL),&pStatus)) {
       AbortAccess (CDF, UPDATE, noDELETE);
       return pStatus;
@@ -1351,8 +1409,7 @@ switch (Va->item) {
         of the pad value is greater than 16, it causes segmentation fault
         for the IDL CDF interface routine.
     ***********************************************************************/
-    if (STRINGdataType(dataType)) 
-        numElems = 1;
+    numElems = 1;
 
     if (PADvalueBITset(flags)) {
       if (!sX(ReadVDR(CDF,CDF->fp,offset,zVar,
@@ -1370,12 +1427,6 @@ switch (Va->item) {
     }
     else {
       DefaultPadValuePre350 (dataType, numElems, padValue);
-      if (!sX(ConvertBuffer(HostEncoding(),CDF->decoding,
-    		            CDF->negToPosFp0,dataType,
-			    numElems,padValue,padValue),&pStatus)) {
-	    AbortAccess (CDF, UPDATE, noDELETE);
-	return pStatus;
-      }
       sX (NO_PADVALUE_SPECIFIED, &pStatus);
     }
     break;
@@ -1408,6 +1459,20 @@ switch (Va->item) {
 				rd->dimIndices,
 				Var->dimVarys,
 				Var->nPhyDimValues) * Var->NvalueBytes;
+    if (zOp != CDF->keptVarzOp || Var->varN != CDF->keptVarNum) {
+      /* Not reading the same variable... clearing the caches if the    */
+      /* remaining caches can't hold the new compressed/sparse data to  */
+      /* avoid filling up all caches and then a temporary file becomes  */
+      /* necessary.             */
+      if (CDF->stage.fp != NULL && (COMPRESSED(Var) || SPARSErecords(Var)) &&
+	  CheckForOverCaches(CDF, Var)) {
+	if (!sX(FlushAllVarStages(CDF,Var),&pStatus)) return pStatus;
+	V_freecaches (CDF->stage.fp);
+	V_stagereset (CDF);
+      }
+      CDF->keptVarzOp = zOp;
+      CDF->keptVarNum = Var->varN;
+    }
     if (!sX(ReadVarValues(CDF,Var,phyRecNum,
 			  offset,INT32_ONE,value),&pStatus)) {
       AbortAccess (CDF, UPDATE, noDELETE);
@@ -1456,6 +1521,16 @@ switch (Va->item) {
 	return pStatus;
       }
     }
+    if (zOp != CDF->keptVarzOp || Var->varN != CDF->keptVarNum) {
+      if (CDF->stage.fp != NULL && (COMPRESSED(Var) || SPARSErecords(Var)) &&
+	  CheckForOverCaches(CDF, Var)) {
+	if (!sX(FlushAllVarStages(CDF,Var),&pStatus)) return pStatus;
+	V_freecaches (CDF->stage.fp);
+	V_stagereset (CDF);
+      }
+      CDF->keptVarzOp = zOp;
+      CDF->keptVarNum = Var->varN;
+    }
     if (!sX(HyperRead(CDF,Var,rd,buffer),&pStatus)) {
       AbortAccess (CDF, UPDATE, noDELETE);
       return pStatus;
@@ -1487,6 +1562,16 @@ switch (Va->item) {
     recNum = Var->seqValueOffset / Var->NphyRecValues;
     if (recNum > Var->maxRec) return END_OF_VAR;
     offset = (Var->seqValueOffset % Var->NphyRecValues) * Var->NvalueBytes;
+    if (zOp != CDF->keptVarzOp || Var->varN != CDF->keptVarNum) {
+      if (CDF->stage.fp != NULL && (COMPRESSED(Var) || SPARSErecords(Var)) &&
+	  CheckForOverCaches(CDF, Var)) {
+	if (!sX(FlushAllVarStages(CDF,Var),&pStatus)) return pStatus;
+	V_freecaches (CDF->stage.fp);
+	V_stagereset (CDF);
+      }
+      CDF->keptVarzOp = zOp;
+      CDF->keptVarNum = Var->varN;
+    }
     if (!sX(ReadVarValues(CDF,Var,recNum,offset,INT32_ONE,value),&pStatus)) {
       AbortAccess (CDF, UPDATE, noDELETE);
       return pStatus;
@@ -1543,6 +1628,16 @@ switch (Va->item) {
 	   return pStatus;
 	 }
        }
+       if (zOp != CDF->keptVarzOp || Var->varN != CDF->keptVarNum) {
+	 if (CDF->stage.fp != NULL && (COMPRESSED(Var) || SPARSErecords(Var))
+	     && CheckForOverCaches(CDF, Var)) {
+	   if (!sX(FlushAllVarStages(CDF,Var),&pStatus)) return pStatus;
+	   V_freecaches (CDF->stage.fp);
+	   V_stagereset (CDF);
+         }
+         CDF->keptVarzOp = zOp;
+         CDF->keptVarNum = Var->varN;
+       } 
        recNum = BOO(Var->recVary,BOO(zOp,Var->zRD.recNumber,
 					 CDF->rRD.recNumber),0);
        if (!sX(ReadVarValues(CDF,Var,recNum,INT32_ZERO,
@@ -1796,6 +1891,10 @@ switch (Va->item) {
       AbortAccess (CDF, UPDATE, noDELETE);
       return pStatus;
     }
+/*
+    if (STRINGdataType(dataType))
+      *((char *)value+(int)numElems) = '\0';
+*/
     if (!sX(ConvertBuffer(CDF->encoding,CDF->decoding,
 			  CDF->negToPosFp0,dataType,
 			  numElems,value,value),&pStatus)) {
@@ -1804,6 +1903,75 @@ switch (Va->item) {
     }
     break;
   }
+  /****************************************************************************
+   * rENTRY_NUMSTRINGS_/zENTRY_NUMSTRINGS_
+   ***************************************************************************/
+  case rENTRY_NUMSTRINGS_:
+  case zENTRY_NUMSTRINGS_: {
+    Logical dataOp = (Va->item == rENTRY_NUMSTRINGS_);
+    struct CDFstruct *CDF;
+    long *value = va_arg (Va->ap, long *);
+    Int32 tValue;
+    Int32 eOffset;
+    SelectCDF (Cur->cdf, CDF)
+    if (!CURRENTattrSELECTED(CDF)) return NO_ATTR_SELECTED;
+    if (BOO(dataOp, CDF->CURgrEntryNum, CDF->CURzEntryNum) ==
+        RESERVED_ENTRYNUM) return NO_ENTRY_SELECTED;
+    if (!sX(CheckEntryOp(CDF,Va->item),&pStatus)) return pStatus;
+    eOffset = BOO(dataOp, CDF->CURgrEntryOffset,
+                          CDF->CURzEntryOffset);
+    if (eOffset == (Int32) RESERVED_ENTRYOFFSET) return NO_SUCH_ENTRY;
+    if (!sX(ReadAEDR(CDF->fp,eOffset,
+                     AEDR_NUMSTRINGS,&tValue,
+                     AEDR_NULL),&pStatus)) {
+      AbortAccess (CDF, UPDATE, noDELETE);
+      return pStatus;
+    }
+    *value = (long) BOO((tValue>0), tValue, 0);
+    break;
+  }
+  /****************************************************************************
+   * rENTRY_STRINGSDATA_/zENTRY_STRINGSDATA_, 
+   ***************************************************************************/
+  case rENTRY_STRINGSDATA_:
+  case zENTRY_STRINGSDATA_: {
+    Logical zOp = (Va->item == zENTRY_STRINGSDATA_);
+    struct CDFstruct *CDF;
+    Int32 dataType, numElems;
+    Int32 offset;
+    long *numStrings = va_arg (Va->ap, long *);
+    char ***strings = va_arg (Va->ap, char ***);
+    char *tmp;
+    SelectCDF (Cur->cdf, CDF)
+    if (!CURRENTattrSELECTED(CDF)) return NO_ATTR_SELECTED;
+    if ((!zOp?CDF->CURgrEntryNum:CDF->CURzEntryNum) == RESERVED_ENTRYNUM)
+      return NO_ENTRY_SELECTED;
+    offset = (!zOp?CDF->CURgrEntryOffset:CDF->CURzEntryOffset);
+    if (offset == (Int32) RESERVED_ENTRYOFFSET) return NO_SUCH_ENTRY;
+    if (!sX(ReadAEDR(CDF->fp,offset,
+                     AEDR_DATATYPE,&dataType,
+                     AEDR_NUMELEMS,&numElems,
+                     AEDR_NULL),&pStatus)) {
+      AbortAccess (CDF, UPDATE, noDELETE);
+      return pStatus;
+    }
+    if (!STRINGdataType(dataType)) return TRY_TO_READ_NONSTRING_DATA;
+    tmp = (char *) malloc((size_t)numElems+1);
+    if (!sX(ReadAEDR(CDF->fp,offset,
+                     AEDR_NUMSTRINGS,numStrings,
+                     AEDR_VALUE,tmp,
+                     AEDR_NULL),&pStatus)) {
+      AbortAccess (CDF, UPDATE, noDELETE);
+      return pStatus;
+    }
+    *(tmp+numElems) = '\0';
+    /* Pre-3.7.0, the field was used and filled with 0. */
+    if (*numStrings < 1) *numStrings = 1;
+    StringBreakIntoStrings (tmp, numElems, *numStrings, strings);
+    free (tmp);
+    tmp = NULL;
+    break;
+  }
   /****************************************************************************
   * Unknown item, must be the next function.
   ****************************************************************************/
@@ -1814,3 +1982,4 @@ switch (Va->item) {
 }
 return pStatus;
 }
+
