@@ -78,7 +78,6 @@ switch (Va->item) {
 #if defined(vms) || defined(dos)
     MakeUpperString (CDFpathT);
 #endif
-/*  if (!ValidCDFname(CDFpathT)) return BAD_CDF_NAME; */
     if (!sX(FindCDF(CDFpathT,&no_append,
 		    &upper_case_ext,
 		    &version_numbers),&pStatus)) return pStatus;
@@ -944,7 +943,8 @@ switch (Va->item) {
 	if (Var != NULL) {
 	  if (Var->stage.areaOffset64 != (OFF_T) NO_OFFSET64 &&
 	      Var->stage.firstRec != NO_RECORD) {
-	    Int32 tmp = (Var->stage.lastRec > tMaxRec ? tMaxRec : Var->stage.lastRec);
+	    Int32 tmp = (Var->stage.lastRec > tMaxRec ? tMaxRec :
+                                                        Var->stage.lastRec);
 	    nRecords += (tmp - Var->stage.firstRec + 1);
 	  }
 	}
@@ -958,7 +958,8 @@ switch (Va->item) {
 	  if (Var->stage.areaOffset64 != (OFF_T) NO_OFFSET64 &&
 	      Var->stage.firstRec != NO_RECORD &&
 	      Var->stage.dotOffset64 == (OFF_T) NO_OFFSET64) {
-            Int32 tmp = (Var->stage.lastRec > tMaxRec ? tMaxRec : Var->stage.lastRec);
+            Int32 tmp = (Var->stage.lastRec > tMaxRec ? tMaxRec :
+                                                        Var->stage.lastRec);
 	    nRecords += (tmp - Var->stage.firstRec + 1);
 	  }
 	}
@@ -1080,6 +1081,7 @@ switch (Va->item) {
     struct CDFstruct *CDF; Int32 nextRec; OFF_T offset;
     Int32 startRec = (Int32) va_arg (Va->ap, long);
     long *fromRec = va_arg (Va->ap, long *);
+    Int32 flags, sRecords;
     SelectCDF (Cur->cdf, CDF)
     if (BADzOP(CDF,!zOp)) return ILLEGAL_IN_zMODE;
     if (!CURRENTvarSELECTED(CDF,zOp)) return NO_VAR_SELECTED;
@@ -1089,6 +1091,34 @@ switch (Va->item) {
     if (zModeON(CDF) || zVar) offset = CDF->CURzVarOffset64;
     else offset = CDF->CURrVarOffset64;
 
+    if (!CDF->readOnly) {
+      if (!sX(ReadVDR64(CDF,CDF->fp,offset,zVar,
+                        VDR_FLAGS,&flags,
+                        VDR_sRECORDS,&sRecords,
+                        VDR_NULL),&pStatus)) {
+        AbortAccess64 (CDF, UPDATE, noDELETE);
+        return pStatus;
+      }
+      /************************************************************************
+      * If the compressed or sparse records, flush out the variable if its
+      * VXR is not yet created.
+      ************************************************************************/
+      if (VARcompressionBITset(flags) || sRecords != NO_SPARSERECORDS) {
+        struct VarStruct *Var;
+        if (zModeON(CDF))
+          if (CDF->CURzVarNum < CDF->NrVars)
+            Var = CDF->rVars[(int)CDF->CURzVarNum];
+          else
+            Var = CDF->zVars[(int)(CDF->CURzVarNum - CDF->NrVars)];
+        else
+          Var = BOO(zOp,CDF->zVars[(int)CDF->CURzVarNum],
+                        CDF->rVars[(int)CDF->CURrVarNum]);
+        if (Var != NULL) {
+          if (!sX(FlushStage64(CDF,Var),&pStatus)) return pStatus;
+        }
+      }
+    }
+
     if (!sX(NextRecord64(CDF,offset,zVar,startRec,&nextRec,&found),&pStatus)) {
       AbortAccess64 (CDF, UPDATE, noDELETE);
       return pStatus;
@@ -1108,6 +1138,7 @@ switch (Va->item) {
     struct CDFstruct *CDF; Int32 lastRec; OFF_T vdrOffset;
     Int32 startRec = (Int32) va_arg (Va->ap, long);
     long *toRec = va_arg (Va->ap, long *);
+    Int32 flags, sRecords;
     SelectCDF (Cur->cdf, CDF)
     if (BADzOP(CDF,!zOp)) return ILLEGAL_IN_zMODE;
     if (!CURRENTvarSELECTED(CDF,zOp)) return NO_VAR_SELECTED;
@@ -1117,6 +1148,34 @@ switch (Va->item) {
     if (zModeON(CDF) || zVar) vdrOffset = CDF->CURzVarOffset64;
     else vdrOffset = CDF->CURrVarOffset64;
 
+    if (!CDF->readOnly) {
+      if (!sX(ReadVDR64(CDF,CDF->fp,vdrOffset,zVar,
+                        VDR_FLAGS,&flags,
+                        VDR_sRECORDS,&sRecords,
+                        VDR_NULL),&pStatus)) {
+        AbortAccess64 (CDF, UPDATE, noDELETE);
+        return pStatus;
+      }
+      /************************************************************************
+      * If the compressed or sparse records, flush out the variable in case
+      * its VXR is not yet created.
+      ************************************************************************/
+      if (VARcompressionBITset(flags) || sRecords != NO_SPARSERECORDS) {
+        struct VarStruct *Var;
+        if (zModeON(CDF))
+          if (CDF->CURzVarNum < CDF->NrVars)
+            Var = CDF->rVars[(int)CDF->CURzVarNum];
+          else
+            Var = CDF->zVars[(int)(CDF->CURzVarNum - CDF->NrVars)];
+        else
+          Var = BOO(zOp,CDF->zVars[(int)CDF->CURzVarNum],
+                        CDF->rVars[(int)CDF->CURrVarNum]);
+        if (Var != NULL) {
+          if (!sX(FlushStage64(CDF,Var),&pStatus)) return pStatus;
+        }
+      }
+    }
+
     /**************************************************************************
     * Determine the last record in the block containing the starting record.
     **************************************************************************/
@@ -1497,6 +1556,21 @@ switch (Va->item) {
 	return pStatus;
       }
     }
+    if (zOp != CDF->keptVarzOp || Var->varN != CDF->keptVarNum) {
+      /* Not reading the same variable... clearing the caches if the    */
+      /* remaining caches can't hold the new compressed/sparse data to  */
+      /* avoid filling up all caches and then a temporary file becomes  */
+      /* necessary.             */
+      if (CDF->stage.fp != NULL && (COMPRESSED(Var) || SPARSErecords(Var)) &&
+	  Var->vType == COMPRESSED_ &&
+	  CheckForOverCaches64(CDF, Var)) {
+	if (!sX(FlushAllVarStages64(CDF,Var),&pStatus)) return pStatus;
+	V_freecaches64 (CDF->stage.fp);
+	V_stagereset (CDF);
+      }
+      CDF->keptVarzOp = zOp;
+      CDF->keptVarNum = Var->varN;
+    }
     phyRecNum = BOO(Var->recVary,rd->recNumber,0);
     offset = IndicesValueOffset(Var->numDims,
 				rd->dimIndices,
@@ -1550,6 +1624,16 @@ switch (Va->item) {
 	return pStatus;
       }
     }
+    if (zOp != CDF->keptVarzOp || Var->varN != CDF->keptVarNum) {
+      if (CDF->stage.fp != NULL && (COMPRESSED(Var) || SPARSErecords(Var)) &&
+	  CheckForOverCaches64(CDF, Var)) {
+	if (!sX(FlushAllVarStages64(CDF,Var),&pStatus)) return pStatus;
+	V_freecaches64 (CDF->stage.fp);
+	V_stagereset (CDF);
+      }
+      CDF->keptVarzOp = zOp;
+      CDF->keptVarNum = Var->varN;
+    }
     if (!sX(HyperRead64(CDF,Var,rd,buffer),&pStatus)) {
       AbortAccess64 (CDF, UPDATE, noDELETE);
       return pStatus;
@@ -1580,6 +1664,17 @@ switch (Va->item) {
     }
     recNum = (Int32) (Var->seqValueOffset64 / (OFF_T) Var->NphyRecValues);
     if (recNum > Var->maxRec) return END_OF_VAR;
+    if (zOp != CDF->keptVarzOp || Var->varN != CDF->keptVarNum) {
+      if (CDF->stage.fp != NULL && (COMPRESSED(Var) || SPARSErecords(Var)) &&
+	  CheckForOverCaches64(CDF, Var)) {
+	if (!sX(FlushAllVarStages64(CDF,Var),&pStatus)) return pStatus;
+	V_freecaches64 (CDF->stage.fp);
+	V_stagereset (CDF);
+      }
+      CDF->keptVarzOp = zOp;
+      CDF->keptVarNum = Var->varN;
+    }
+
     offset = (Int32) (Var->seqValueOffset64 % (OFF_T) Var->NphyRecValues) * 
 		     Var->NvalueBytes;
     if (!sX(ReadVarValues64(CDF,Var,recNum,offset,INT32_ONE,value),&pStatus)) {
@@ -1640,6 +1735,17 @@ switch (Va->item) {
        }
        recNum = BOO(Var->recVary,BOO(zOp,Var->zRD.recNumber,
 					 CDF->rRD.recNumber),0);
+       if (zOp != CDF->keptVarzOp || Var->varN != CDF->keptVarNum) {
+	 if (CDF->stage.fp != NULL && (COMPRESSED(Var) || SPARSErecords(Var))
+	     && CheckForOverCaches64(CDF, Var)) {
+	   if (!sX(FlushAllVarStages64(CDF,Var),&pStatus)) return pStatus;
+	   V_freecaches64 (CDF->stage.fp);
+	   V_stagereset (CDF);
+	 }
+	 CDF->keptVarzOp = zOp;
+	 CDF->keptVarNum = Var->varN;
+       }
+
        if (!sX(ReadVarValues64(CDF,Var,recNum,INT32_ZERO,
 			       Var->NphyRecValues,tBuffer),&pStatus)) {
 	 AbortAccess64 (CDF, UPDATE, noDELETE);
@@ -1904,10 +2010,10 @@ switch (Va->item) {
     void *value = va_arg (Va->ap, void *);
     SelectCDF (Cur->cdf, CDF)
     if (!CURRENTattrSELECTED64(CDF)) return NO_ATTR_SELECTED;
-    if (E3(entryType,
-	   CDF->CURgrEntryNum,
-	   CDF->CURgrEntryNum,
-	   CDF->CURzEntryNum) == RESERVED_ENTRYNUM) return NO_ENTRY_SELECTED;
+    if (E3(entryType,CDF->CURgrEntryNum,
+		     CDF->CURgrEntryNum,
+		     CDF->CURzEntryNum) == RESERVED_ENTRYNUM)
+      return NO_ENTRY_SELECTED;
     if (!sX(CheckEntryOp64(CDF,entryType),&pStatus)) return pStatus;
     offset = E3(entryType,CDF->CURgrEntryOffset64,
 			  CDF->CURgrEntryOffset64,
@@ -1921,6 +2027,10 @@ switch (Va->item) {
       AbortAccess64 (CDF, UPDATE, noDELETE);
       return pStatus;
     }
+/*
+    if (STRINGdataType(dataType))
+      *((char *)value+(int)numElems) = '\0';
+*/
     if (!sX(ConvertBuffer(CDF->encoding,CDF->decoding,
 			  CDF->negToPosFp0,dataType,
 			  numElems,value,value),&pStatus)) {
@@ -1968,6 +2078,7 @@ switch (Va->item) {
     if (*numStrings < 1) *numStrings = 1;
     StringBreakIntoStrings (tmp, numElems, *numStrings, strings);
     free (tmp);
+    tmp = NULL;
     break;
   }
   /****************************************************************************
