@@ -1,4 +1,9 @@
 /******************************************************************************
+* Copyright 1996-2014 United States Government as represented by the
+* Administrator of the National Aeronautics and Space Administration.
+* All Rights Reserved.
+******************************************************************************/
+/******************************************************************************
 *
 *  NSSDC/CDF                                  CDF library hyper functions.
 *
@@ -19,7 +24,7 @@
 *				reading/writing data values of a non-full 
 *				physical record, a temp. buffer is used to cut
 *				down the physical I/Os.  
-*				
+*
 ******************************************************************************/
 
 #include "cdflib.h"
@@ -37,15 +42,15 @@ void *buffer;
   Int32 nHypRecValues, nHypRecBytes, nHypDimValues[CDF_MAX_DIMS];
   Logical fullPhyRec, fullPhyDim[CDF_MAX_DIMS];
   Int32 firstHypRec, lastHypRec, firstPhyRec;
-  Int32 recNum, nValues, nRecords, phyRecN, i; Byte *tBuffer, *phyBuffer;
+  Int32 recNum, nValues, nRecords, phyRecN, i; Byte1 *tBuffer, *phyBuffer;
   CDFstatus pStatus = CDF_OK;
   int dimN, dimNt;
-  int firstDim;                 /* Based on majority, the "highest" dimension
+  int firstDim = -1;            /* Based on majority, the "highest" dimension
 				   (changes the slowest in memory/on disk). */
-  int doneDim;                  /* When cycling through the dimensions, the
+  int doneDim = -1;             /* When cycling through the dimensions, the
 				   dimension at which to stop (the dimension
 				   beyond the last). */
-  int dimIncr;                  /* How to cycle through the dimensions.  Incr-
+  int dimIncr = 0;              /* How to cycle through the dimensions.  Incr-
 				   ementing (+1) for row-major, decrementing
 				   (-1) for column-major. */
   /****************************************************************************
@@ -139,7 +144,8 @@ void *buffer;
        * and extract values from the buffer. Otherwise, get one value at a
        * time from the file
        *******************************************************************/
-       phyBuffer = (Byte *) cdf_AllocateMemory((size_t) Var->NphyRecBytes,NULL);
+       phyBuffer = (Byte1 *) cdf_AllocateMemory((size_t) Var->NphyRecBytes,
+						 NULL);
        if (phyBuffer != NULL) {
          if (!sX(ReadVarValues(CDF,Var,phyRecN,INT32_ZERO,Var->NphyRecValues,
                                phyBuffer),&pStatus)) {
@@ -191,7 +197,7 @@ void *phyBuffer;
 struct CDFstruct *CDF;
 struct VarStruct *Var;
 {
-  Int32 tOffset; Int32 nValues, i; Byte *tBuffer; CDFstatus pStatus = CDF_OK;
+  Int32 tOffset; Int32 nValues, i; Byte1 *tBuffer; CDFstatus pStatus = CDF_OK;
   /****************************************************************************
   * What to do depends on the number of dimensions.  Note that this function
   * should never be called with zero dimensions.
@@ -216,7 +222,7 @@ struct VarStruct *Var;
 		 		  tOffset,counts[0],
 				  buffer),&pStatus)) return pStatus; 
 	  } else {
-	    memmove (buffer, (size_t) tOffset + (Byte *) phyBuffer, 
+	    memmove (buffer, (size_t) tOffset + (Byte1 *) phyBuffer, 
 		     (size_t) (Var->NvalueBytes * counts[0]));
 	  }
 	}
@@ -233,7 +239,7 @@ struct VarStruct *Var;
 				      tOffset,INT32_ONE,
 				      tBuffer),&pStatus)) return pStatus; 
 	     } else {
-                memmove (tBuffer, (size_t) tOffset+(Byte *) phyBuffer, 
+                memmove (tBuffer, (size_t) tOffset+(Byte1 *) phyBuffer, 
 			 (size_t) Var->NvalueBytes);
 	     }
 	     tOffset += (intervals[0] * Var->NvalueBytes);
@@ -250,11 +256,11 @@ struct VarStruct *Var;
 	  if (!sX(ReadVarValues(CDF,Var,recNum,offset,
 			        INT32_ONE,buffer),&pStatus)) return pStatus;
 	} else {
-	  memmove(buffer, (size_t) offset + (Byte *) phyBuffer, 
+	  memmove(buffer, (size_t) offset + (Byte1 *) phyBuffer, 
 		  (size_t) Var->NvalueBytes);
 	}
 	if (counts[0] > 1) {
-	  for (i = 1, tBuffer = (Byte *) buffer + (size_t) Var->NvalueBytes;
+	  for (i = 1, tBuffer = (Byte1 *) buffer + (size_t) Var->NvalueBytes;
 	       i < counts[0]; i++, tBuffer += (size_t) Var->NvalueBytes) {
 	     memmove (tBuffer, buffer, (size_t) Var->NvalueBytes);
 	  }
@@ -284,7 +290,7 @@ struct VarStruct *Var;
 				  tOffset,nValues,
 				  buffer),&pStatus)) return pStatus;
 	  } else {
-	    memmove(buffer, (size_t) tOffset + (Byte *) phyBuffer, 
+	    memmove(buffer, (size_t) tOffset + (Byte1 *) phyBuffer, 
 		    (size_t) (Var->NvalueBytes * nValues));
 	  }
 	}
@@ -298,7 +304,7 @@ struct VarStruct *Var;
 				       tOffset,nPhyDimValues[firstDim],
 				       tBuffer),&pStatus)) return pStatus; 
 	       } else {
-		 memmove(tBuffer, (size_t) tOffset + (Byte *) phyBuffer, 
+		 memmove(tBuffer, (size_t) tOffset + (Byte1 *) phyBuffer, 
 			 (size_t) (Var->NvalueBytes * nPhyDimValues[firstDim]));
 	       }
 	     }
@@ -334,7 +340,7 @@ struct VarStruct *Var;
 				  buffer),&pStatus)) return pStatus;
 	  } else {
  
-	    memmove(buffer, (size_t) offset + (Byte *) phyBuffer, 
+	    memmove(buffer, (size_t) offset + (Byte1 *) phyBuffer, 
 		    (size_t) (Var->NvalueBytes * nPhyDimValues[firstDim]));
 	  }
 	}
@@ -352,7 +358,7 @@ struct VarStruct *Var;
 		 	       CDF,Var),&pStatus)) return pStatus;
 	}
 	if (counts[firstDim] > 1) {
-	  for (i = 1, tBuffer = (Byte *) buffer + (size_t) nHypDimBytes;
+	  for (i = 1, tBuffer = (Byte1 *) buffer + (size_t) nHypDimBytes;
 	       i < counts[firstDim]; i++, tBuffer += (size_t) nHypDimBytes) {
 	     memmove (tBuffer, buffer, (size_t) nHypDimBytes);
 	  }
@@ -376,14 +382,14 @@ void *buffer;
 {
   Int32 nHypRecValues, nHypRecBytes, nHypDimValues[CDF_MAX_DIMS];
   Logical fullPhyRec, fullPhyDim[CDF_MAX_DIMS];
-  Int32 firstPhyRec, phyRecN, nValues; Byte *tBuffer; int dimN; Int32 i;
-  CDFstatus pStatus = CDF_OK; Byte *phyBuffer;
-  int firstDim;                 /* Based on majority, the "highest" dimension
+  Int32 firstPhyRec, phyRecN, nValues; Byte1 *tBuffer; int dimN; Int32 i;
+  CDFstatus pStatus = CDF_OK; Byte1 *phyBuffer;
+  int firstDim = -1;            /* Based on majority, the "highest" dimension
 				   (changes the slowest in memory/on disk). */
-  int doneDim;                  /* When cycling through the dimensions, the
+  int doneDim = -1;             /* When cycling through the dimensions, the
 				   dimension at which to stop (the dimension
 				   beyond the last). */
-  int dimIncr;                  /* How to cycle through the dimensions.  Incr-
+  int dimIncr = 0;              /* How to cycle through the dimensions.  Incr-
 				   ementing (+1) for row-major, decrementing
 				   (-1) for column-major. */
   /****************************************************************************
@@ -443,7 +449,7 @@ void *buffer;
       nValues = rd->recCount * Var->NphyRecValues;
     }
     else {
-      tBuffer = (Byte *) buffer +
+      tBuffer = (Byte1 *) buffer +
 		(size_t) (Var->NphyRecBytes * (rd->recCount-1));
       nValues = Var->NphyRecValues;
     }
@@ -493,7 +499,8 @@ void *buffer;
          * with values from the buffer. Otherwise, write one value at a
          * time to the file
          *********************************************************************/
-         phyBuffer = (Byte *) cdf_AllocateMemory((size_t) Var->NphyRecBytes,NULL);
+         phyBuffer = (Byte1 *) cdf_AllocateMemory((size_t) Var->NphyRecBytes,
+						  NULL);
 	 if (phyBuffer != NULL) {
 	    if (!sX(ReadVarValues(CDF,Var,phyRecN,INT32_ZERO,
                                  Var->NphyRecValues,
@@ -502,11 +509,10 @@ void *buffer;
 		cdf_FreeMemory(phyBuffer,NULL);
 		return pStatus;
 	      }
-	      else { /* Don't want it to stop. */
-		pStatus = CDF_OK;
-	      }
 	    }
 	 }
+	 if (pStatus == VIRTUAL_RECORD_DATA) /* Don't want it to stop. */
+	   pStatus = CDF_OK;
 	 if (!sX(HyperWriteDim(Var->numDims,Var->dimSizes,
 			       Var->dimVarys,rd->dimIndices,
 			       rd->dimCounts,rd->dimIntervals,
@@ -539,8 +545,8 @@ void *buffer;
     * record in the buffer will be written (as if the records before it had
     * been written but then overwritten by the last one).
     **************************************************************************/
-    tBuffer = (Byte *) buffer + (size_t) (nHypRecBytes * (rd->recCount - 1));
-    phyBuffer = (Byte *) cdf_AllocateMemory((size_t) Var->NphyRecBytes,NULL);
+    tBuffer = (Byte1 *) buffer + (size_t) (nHypRecBytes * (rd->recCount - 1));
+    phyBuffer = (Byte1 *) cdf_AllocateMemory((size_t) Var->NphyRecBytes, NULL);
     if (phyBuffer != NULL) {
        if (!sX(ReadVarValues(CDF,Var,phyRecN,INT32_ZERO,
                              Var->NphyRecValues, 
@@ -549,11 +555,10 @@ void *buffer;
 	   cdf_FreeMemory(phyBuffer,NULL);
 	   return pStatus;
 	 }
-	 else { /* Don't want it to stop */
-	   pStatus = CDF_OK;
-	 }
        } 
     }
+    if (pStatus == VIRTUAL_RECORD_DATA) /* Don't want it to stop */
+      pStatus = CDF_OK;
     if (!sX(HyperWriteDim(Var->numDims,Var->dimSizes,Var->dimVarys,
 			  rd->dimIndices,rd->dimCounts,
 			  rd->dimIntervals,nHypDimValues,
@@ -598,14 +603,14 @@ Int32 *nPhyDimValues;
 Logical *fullPhyDim;
 int firstDim;
 int dimIncr;
-Int32 recNum;            /* Record number for this write. */
+Int32 recNum;           /* Record number for this write. */
 Int32 offset;           /* Byte offset within record at which to begin. */
 void *buffer;
 void *phyBuffer;
 struct CDFstruct *CDF;
 struct VarStruct *Var;
 {
-  Int32 tOffset; Int32 i, nValues; Byte *tBuffer;
+  Int32 tOffset; Int32 i, nValues; Byte1 *tBuffer;
   CDFstatus pStatus = CDF_OK;
   /****************************************************************************
   * What to do depends on the number of dimensions.  Note that this function
@@ -631,7 +636,7 @@ struct VarStruct *Var;
 				   tOffset,counts[0],
 				   buffer),&pStatus)) return pStatus;
 	    } else {
-	      memmove ((size_t) tOffset + (Byte *) phyBuffer, buffer,
+	      memmove ((size_t) tOffset + (Byte1 *) phyBuffer, buffer,
                        (size_t) (Var->NvalueBytes * counts[0]));
           }
 	}
@@ -648,7 +653,7 @@ struct VarStruct *Var;
 				      tOffset,INT32_ONE,
 				      tBuffer),&pStatus)) return pStatus;
 	     } else {
-               memmove ((size_t) tOffset+(Byte *) phyBuffer, tBuffer,
+               memmove ((size_t) tOffset+(Byte1 *) phyBuffer, tBuffer,
                         (size_t) Var->NvalueBytes);
              }
 	     tOffset += (intervals[0] * Var->NvalueBytes);
@@ -662,13 +667,13 @@ struct VarStruct *Var;
 	* If the count is greater than one, skip to the last value in the
 	* buffer (this is an unlikely situation).
 	**********************************************************************/
-	tBuffer = (Byte *) buffer +
+	tBuffer = (Byte1 *) buffer +
 		  (size_t) (Var->NvalueBytes * (counts[0] - 1));
 	if (phyBuffer == NULL) {
 	  if (!sX(WriteVarValues(CDF,Var,recNum,offset,
 				 INT32_ONE,tBuffer),&pStatus)) return pStatus;
         } else {
-          memmove((size_t) offset + (Byte *) phyBuffer, tBuffer,
+          memmove((size_t) offset + (Byte1 *) phyBuffer, tBuffer,
                   (size_t) Var->NvalueBytes);
         }
       }
@@ -696,7 +701,7 @@ struct VarStruct *Var;
 				   tOffset,nValues,
 				   buffer),&pStatus)) return pStatus;
           } else {
-            memmove((size_t) tOffset + (Byte *) phyBuffer, buffer, 
+            memmove((size_t) tOffset + (Byte1 *) phyBuffer, buffer, 
                     (size_t) (Var->NvalueBytes * nValues));
           }
 	}
@@ -710,8 +715,9 @@ struct VarStruct *Var;
 				        nPhyDimValues[firstDim],
 				        tBuffer),&pStatus)) return pStatus;
                } else {
-                 memmove((size_t) tOffset + (Byte *) phyBuffer, tBuffer,
-                         (size_t) (Var->NvalueBytes * nPhyDimValues[firstDim]));
+                 memmove((size_t) tOffset + (Byte1 *) phyBuffer, tBuffer,
+                         (size_t) (Var->NvalueBytes *
+                                   nPhyDimValues[firstDim]));
                }
 	     }
 	     else {
@@ -740,7 +746,7 @@ struct VarStruct *Var;
 	* The first dimension's variance is FALSE, skip to the last subarray
 	* and write the single physical subarray.
 	**********************************************************************/
-	tBuffer = (Byte *) buffer +
+	tBuffer = (Byte1 *) buffer +
 		  (size_t) (nHypDimBytes * (counts[firstDim] - 1));
 	if (fullPhyDim[nextDim]) {
 	  if (phyBuffer == NULL) {
@@ -748,7 +754,7 @@ struct VarStruct *Var;
 				   nPhyDimValues[firstDim],
 				   tBuffer),&pStatus)) return pStatus;
 	  } else {
-            memmove((size_t) offset + (Byte *) phyBuffer, tBuffer,
+            memmove((size_t) offset + (Byte1 *) phyBuffer, tBuffer,
                     (size_t) (Var->NvalueBytes * nPhyDimValues[firstDim]));
           }
 	}
