@@ -265,6 +265,65 @@ struct CDFstruct *CDF;
   return pStatus;
 }
 
+/******************************************************************************
+* FlushAllVarStages.
+******************************************************************************/
+
+VISIBLE_PREFIX CDFstatus FlushAllVarStages (CDF, Vare)
+struct CDFstruct *CDF;
+struct VarStruct *Vare;
+{
+  CDFstatus pStatus = CDF_OK; int varN; struct VarStruct *Var; Logical zVar;
+  /**************************************************************************
+  * Update r/zVariables depending on the variable type...
+  **************************************************************************/
+  for (zVar = 0; zVar <= 1; zVar++) {
+    if (BOO(zVar,CDF->zVars,CDF->rVars) != NULL) {
+      Int32 nVars = BOO(zVar,CDF->NzVars,CDF->NrVars);
+      for (varN = 0; varN < nVars; varN++) {
+         Var = BOO(zVar,CDF->zVars[varN],CDF->rVars[varN]);
+         if (Var == NULL || (Var->vType != COMPRESSED_ &&
+                             Var->vType != SPARSE_RECORDS_)) continue;
+         if (Vare != NULL && (Var->varN == Vare->varN &&
+                              Var->zVar == Vare->zVar)) continue;
+         if (!sX(FlushStage(CDF,Var),&pStatus)) break;
+         Var->stage.areaOffset = NO_OFFSET;
+         Var->stage.firstRec = NO_RECORD;
+         Var->stage.lastRec = NO_RECORD;
+         Var->stage.dotOffset = NO_OFFSET;
+         Var->stage.modified = FALSE;
+      }
+    }
+  }
+  return pStatus;
+}
+
+/******************************************************************************
+* CheckForOverCaches - checks if the size of the variable's data from a
+* blocking factor for compressed type or a single record for non-compressed 
+* type will cause CDF's remaining staging caches to be over the limit, which
+* would cause a temporary file(s) being created.
+******************************************************************************/
+
+VISIBLE_PREFIX Logical CheckForOverCaches (CDF, Var)
+struct CDFstruct *CDF;
+struct VarStruct *Var;
+{
+  Int32 numCaches, remaining, numRecs;
+  Int32 totalBytes;
+  if (COMPRESSED(Var)) numRecs = Var->blockingFactor;
+  else if (SPARSErecords(Var)) numRecs = 1;
+  else numRecs = 1;
+  totalBytes = Var->NphyRecBytes * numRecs;
+  numCaches = (Int32) (totalBytes / MIN_BLOCKING_BYTES_compressed);
+  remaining = (Int32) (totalBytes % MIN_BLOCKING_BYTES_compressed);
+  if (remaining > 0) ++numCaches;
+  if ((CDF->stage.fp->nBuffers + numCaches) > CDF->stage.fp->maxBuffers)
+    return TRUE;
+  else
+    return FALSE;
+}
+
 /******************************************************************************
 * CloseVarFiles.
 *
@@ -2233,6 +2292,7 @@ char *delimiter;
     tmp3 = strstr(tmp2, delimiter);
   }
   free (tmp);
+  tmp = NULL;
   return count;
 }
 
@@ -2247,8 +2307,14 @@ long numStrings;
 char **strings;
 {
   int ix;
-  for (ix = 0; ix < (int) numStrings; ++ix) free(strings[ix]);
-  if (strings != NULL) free(strings);
+  for (ix = 0; ix < (int) numStrings; ++ix) {
+    free (strings[ix]);
+    strings[ix] = NULL;
+  }
+  if (strings != NULL) {
+    free (strings);
+    strings = NULL;
+  }
 }
 
 /******************************************************************************
@@ -2309,6 +2375,7 @@ int   numElems;
   }
   return;
 }
+
 /******************************************************************************
 * FillNulsToString.
 * This function scans for Nul in a string buffer. If found, it fills Nul(s)
@@ -2334,6 +2401,7 @@ int   numElems;
   }
   return;
 }
+
 /******************************************************************************
 * DisplayVs.
 ******************************************************************************/
