@@ -48,8 +48,10 @@ STATICforIDL CDFstatus CDFope (Va, Cur)
 struct VAstruct *Va;
 struct CurStruct *Cur;
 {
-CDFstatus pStatus = CDF_OK;
+CDFstatus pStatus = CDF_OK, status;
+Logical compressedCDF;
 
+compressedCDF = FALSE;
 switch (Va->item) {
   /****************************************************************************
   * CDF_, open an existing CDF.
@@ -64,7 +66,6 @@ switch (Va->item) {
     Int32 magicNumber1, magicNumber2, CDRflags, version, release, increment;
     Logical largeFile;
     int  validate;
-
     /**************************************************************************
     * Get arguments for this operation/item.
     **************************************************************************/
@@ -85,12 +86,16 @@ switch (Va->item) {
     MakeUpperString (CDFnameT);
 #endif
     RemoveCDFFileExtension(CDFnameT, CDFnameTx);
-/*  if (!ValidCDFname(CDFnameTx)) return BAD_CDF_NAME; */
     if (!sX(FindCDF(CDFnameTx,&no_append,
 		    &upper_case_ext,
 		    &version_numbers),&pStatus)) return pStatus;
     /**************************************************************************
     * Open CDF file.
+    * -- file pointed by dotFp
+    * -- if uncompressed.. set the number of caches to NUMcacheUNKNOWN
+    *                      pointed by dotFp
+    * -- if   compressed.. set the number of caches to NUMcacheCOMPRESS
+    *                      pointed by uDotFp
     **************************************************************************/
     BuildFilePath (CDFt, CDFnameTx, no_append, upper_case_ext, version_numbers,
 		   INT32_ZERO, CDFnameX);
@@ -139,7 +144,8 @@ switch (Va->item) {
       case V2magicNUMBER_1:
       case V3magicNUMBER_1:
 	switch (magicNumber2) {
-	  case V2magicNUMBER_2u: /* Same as V3magicNUMBER_2u */
+	  case V2magicNUMBER_2u: /* (Same as V3magicNUMBER_2u) - uncompressed */
+	    compressedCDF = FALSE;    
 	    if (magicNumber1 == V2magicNUMBER_1) { /* 32-offset */
 	      if (!CACHEv(dotFp,NUMcacheUNKNOWN)) {
 	        V_close (dotFp, NULL, NULL);
@@ -153,35 +159,13 @@ switch (Va->item) {
 	    }
 	    uDotFp = NULL;
 	    break;
-	  case V2magicNUMBER_2c:
+	  case V2magicNUMBER_2c: /* (Same as V3magicNUMBER_2c) - compressed */
+	    compressedCDF = TRUE;
 	    uDotFp = V_scratch (ScratchDirectory(NULL), "cdf");
 	    if (uDotFp == NULL) {
 	      V_close (dotFp, NULL, NULL);
 	      return CDF_CREATE_ERROR;
 	    }
-	    if (magicNumber1 == V2magicNUMBER_1) {
-	      if (!CACHEv(uDotFp,NUMcacheUNKNOWN)) {
-	        V_close (dotFp, NULL, NULL);
-	        V_delete (uDotFp, NULL);
-	        return BAD_CACHE_SIZE;
-	      } 
-              if (!sX(DecompressCDF(dotFp,uDotFp),&pStatus)) {
-                V_close (dotFp, NULL, NULL);
-                V_delete (uDotFp, NULL);
-                return pStatus;
-              }
-	    } else {
-              if (!CACHEv64(uDotFp,NUMcacheUNKNOWN)) {
-                V_close64 (dotFp, NULL, NULL);
-                V_delete64 (uDotFp, NULL);
-                return BAD_CACHE_SIZE;
-              }
-	      if (!sX(DecompressCDF64(dotFp,uDotFp),&pStatus)) {
-	        V_close64 (dotFp, NULL, NULL);
-	        V_delete64 (uDotFp, NULL);
-	        return pStatus;
-	      }
-	    }
 	    break;
 	  default:
 	    if (!largeFile)
@@ -204,6 +188,8 @@ switch (Va->item) {
     }
     /**************************************************************************
     * Allocate and begin initializing CDF structure.
+    * -- CDF->fp points to either dotFp (uncompressed)
+    *                          or uDotFp (compressed)
     **************************************************************************/
     CDF = (struct CDFstruct *) cdf_AllocateMemory (
 					(size_t)sizeof(struct CDFstruct), NULL);
@@ -232,6 +218,46 @@ switch (Va->item) {
     }
     else
       strcpyX (CDF->CDFname, CDFnameTx, 0);
+/*
+    CDF->tmpptr = NULL;
+    CDF->tmpptrSize64 = 0;
+    CDF->tmpptrSize = 0;
+*/
+    if (compressedCDF) {
+      if (magicNumber1 == V2magicNUMBER_1) {
+	int ccrLen, remaining;
+	if (!Read32(dotFp,&ccrLen)) {
+	  V_close (dotFp, NULL, NULL);
+	  return CDF_READ_ERROR;
+	}
+	if (!CACHEv(uDotFp,NUMcacheCOMPRESS)) {
+	  V_close (dotFp, NULL, NULL);
+	  V_delete (uDotFp, NULL);
+	  return BAD_CACHE_SIZE;
+	} 
+	if (!sX(DecompressCDF(CDF,dotFp,uDotFp),&pStatus)) {
+	  V_close (dotFp, NULL, NULL);
+	  V_delete (uDotFp, NULL);
+	  return pStatus;
+	}
+      } else {
+	OFF_T ccrLen; int remaining;
+	if (!Read64_64(dotFp,&ccrLen)) {
+	  V_close64 (dotFp, NULL, NULL);
+	  return CDF_READ_ERROR;
+	}
+        if (!CACHEv64(uDotFp,NUMcacheCOMPRESS)) {
+          V_close64 (dotFp, NULL, NULL);
+          V_delete64 (uDotFp, NULL);
+          return BAD_CACHE_SIZE;
+        }
+	if (!sX(DecompressCDF64(CDF,dotFp,uDotFp),&pStatus)) {
+	  V_close64 (dotFp, NULL, NULL);
+	  V_delete64 (uDotFp, NULL);
+	  return pStatus;
+	}
+      }
+    }
     CDF->magic = VALIDid_MAGIC_NUMBER;
     CDF->largeFile = largeFile;
     CDF->dotFp = dotFp;
@@ -264,6 +290,11 @@ switch (Va->item) {
     CDF->CURzVarOffset64 = (OFF_T) 0;
     CDF->CURrVarOffset = 0;
     CDF->CURrVarOffset64 = (OFF_T) 0;
+    CDF->leapSecondUpdated = 0;
+/*
+    CDF->tmpptrSize64 = (OFF_T) 0;
+    CDF->tmpptrSize = 0;
+*/
     /**************************************************************************
     * Read necessary fields from the CDR and GDR.
     **************************************************************************/
@@ -321,7 +352,6 @@ switch (Va->item) {
         *id = (CDFid) NULL;
         return pStatus;
       }
-
       /************************************************************************
        * Set state info for use in ReadGDR - required by changes to support
        * READONLY performance enhancement.
@@ -353,7 +383,12 @@ switch (Va->item) {
     CDF->badEOF = PriorTo("2.1",version,release,increment);
     CDF->badTerminatingOffsets = PriorTo("2.1",version,release,increment);
     CDF->assumedScopes = PriorTo("2.5",version,release,increment);
-    CDF->workingCacheSize = BOO(CDF->singleFile,NUMcacheSINGLE,NUMcacheMULTI);
+    CDF->workingCacheSize = BOO(CDF->singleFile,
+				BOO(compressedCDF,NUMcacheCOMPRESS,
+				    NUMcacheSINGLE),
+				NUMcacheMULTI);
+    CDF->keptVarNum = -1;
+    CDF->keptVarzOp = FALSE;
     /************************************************************************
      * Validate the CDF to make sure that certain data fields are normal by
      * doing some sanity checks.
@@ -375,7 +410,8 @@ switch (Va->item) {
           return pStatus;
         }   
       } else {
-        pStatus = ValidateCDF64(CDF,CDF->fp,CDF->CDRoffset64,CDF->eof64, debug);
+        pStatus = ValidateCDF64(CDF,CDF->fp,CDF->CDRoffset64,CDF->eof64,
+                                debug);
         if (pStatus != CDF_OK) {
           AbortAccess64 (CDF, noUPDATE, noDELETE);
           cdf_FreeMemory (CDF, NULL);
@@ -384,7 +420,6 @@ switch (Va->item) {
           if (pStatus == CORRUPTED_V3_CDF) {
             char vri[6];
             long libver, librel, libinc;
-            CDFstatus status;
             snprintf (vri, (size_t) 6, "%1.1d.%1.1d.%1.1d", (int) version,
                       (int) release, (int) increment);
             status = CDFlib (GET_, LIB_VERSION_, &libver,
@@ -431,7 +466,7 @@ switch (Va->item) {
     }
     /**************************************************************************
     * Set the cache size for the "working" dotCDF file based on the format
-    * of the CDF.
+    * of the CDF and compression.
     **************************************************************************/
     if (!largeFile) {
       if (!CACHEv(CDF->fp,CDF->workingCacheSize)) {
@@ -603,6 +638,7 @@ static CDFstatus ValidateCDFTT2000 (CDFid id, int zVars, int rVars)
     status =  CDFgetVarAllRecordsByVarID (id, 1, (long) i, buffer);
     if (status < CDF_OK) {
       free (buffer);
+      buffer = NULL;
       return status;
     } 
     for (j = 0; j < count; ++j) {
@@ -610,6 +646,7 @@ static CDFstatus ValidateCDFTT2000 (CDFid id, int zVars, int rVars)
         tt2000 = *(((long long *)buffer)+j);
     }
     free (buffer);
+    buffer = NULL;
   }
   status = CDFlib(GET_, rVARs_NUMDIMS_, &numDims,
                         rVARs_DIMSIZES_, dimSizes,
@@ -636,6 +673,7 @@ static CDFstatus ValidateCDFTT2000 (CDFid id, int zVars, int rVars)
     status =  CDFgetVarAllRecordsByVarID (id, 0, (long) i, buffer);
     if (status < CDF_OK) {
       free (buffer);
+      buffer = NULL;
       return status;
     }
     for (j = 0; j < count; ++j) {
@@ -643,6 +681,7 @@ static CDFstatus ValidateCDFTT2000 (CDFid id, int zVars, int rVars)
         tt2000 = *(((long long *)buffer)+j);
     }
     free (buffer);
+    buffer = NULL;
   }
   if (tt2000 != DEFAULT_TT2000_PADVALUE) {
     breakdownTT2000 (tt2000, &yy, &mm, &dd, &hh, &mn, &ss, &ml, &ms, &ns);
