@@ -482,6 +482,9 @@ switch (Va->item) {
     Logical zOp = (Va->item == zVAR_BLOCKINGFACTOR_);
     struct CDFstruct *CDF; struct VarStruct *Var;
     Int32 blocking = (Int32) va_arg (Va->ap, long), flags;
+    Int32 recSize, numDims, dimSizes[CDF_MAX_DIMS], dimVarys[CDF_MAX_DIMS],
+          numElems, dataType;
+    int i;
     SelectCDF (Cur->cdf, CDF)
     if (BADzOP(CDF,!zOp)) return ILLEGAL_IN_zMODE;
     if (!CURRENTvarSELECTED(CDF,zOp)) return NO_VAR_SELECTED;
@@ -521,13 +524,61 @@ switch (Va->item) {
     **************************************************************************/
     if (!sX(ReadVDR(CDF,CDF->fp,Var->VDRoffset,Var->zVar,
 		    VDR_FLAGS,&flags,
+		    VDR_DIMVARYS,&dimVarys,
+		    VDR_DATATYPE,&dataType,
+		    VDR_NUMELEMS,&numElems,
 		    VDR_NULL),&pStatus)) {
       AbortAccess (CDF, UPDATE, noDELETE);
       return pStatus;
     }
+    if (zOp) {
+      if (!sX(ReadVDR(CDF,CDF->fp,Var->VDRoffset64,Var->zVar,
+		      VDR_zNUMDIMS,&numDims,
+		      VDR_zDIMSIZES,dimSizes,
+		      VDR_NULL),&pStatus)) {
+        AbortAccess (CDF, UPDATE, noDELETE);
+        return pStatus;
+      }
+    } else {
+      if (!sX(ReadGDR(CDF->fp,CDF->GDRoffset64,
+		      GDR_rNUMDIMS,&numDims,
+		      GDR_rDIMSIZES,dimSizes,
+		      GDR_NULL),&pStatus)) {
+        AbortAccess (CDF, UPDATE, noDELETE);
+        return pStatus;
+      }
+    }
     if (!RECvaryBITset(flags) && blocking > 1) {
       blocking = 1;
-      sX (FORCED_PARAMETER, &pStatus);
+/*      sX (FORCED_PARAMETER, &pStatus); */
+    }
+    if (VARcompressionBITset(flags) && RECvaryBITset(flags) && (blocking != 0))
+    {
+      long total;
+      recSize = numElems;
+      recSize *= (Int32) CDFelemSize(dataType);
+      for (i = 0; i < (int) numDims; ++i)
+        if (dimVarys[i]) recSize *= dimSizes[i];
+      total = (long) blocking * recSize;
+      if (total < MIN_BLOCKING_BYTES_compressed) {
+      }
+      if (total > (long) NUMcacheSTAGE*MIN_BLOCKING_BYTES_compressed) {
+	int numCaches, iy;
+	numCaches = (int) ((total-1) / MIN_BLOCKING_BYTES_compressed) + 1;
+	iy = 0;
+	while (numCaches > NUMcacheSTAGE) {
+	  blocking = blocking / 2;
+	  if (blocking == 0 ) {
+	    blocking = 1;
+	    iy = 1; 
+	    break;
+	  }
+	  total = (long) recSize * blocking;
+	  numCaches = (int) ((total-1) / MIN_BLOCKING_BYTES_compressed) + 1;
+	}
+	if (iy == 0)
+	  sX (BLOCKINGFACTOR_TOO_LARGE, &pStatus);
+      }
     }
     if (!sX(WriteVDR(CDF,CDF->fp,Var->VDRoffset,Var->zVar,
 		     VDR_BLOCKING,&blocking,
@@ -711,6 +762,21 @@ switch (Va->item) {
     /**************************************************************************
     * Write value.
     **************************************************************************/
+    if (zOp != CDF->keptVarzOp || Var->varN != CDF->keptVarNum) {
+      /* Not reading the same variable... clearing the caches if the    */
+      /* remaining caches can't hold the new compressed/sparse data to  */
+      /* avoid filling up all caches and then a temporary file becomes  */
+      /* necessary.             */
+      if (CDF->stage.fp != NULL && (COMPRESSED(Var) || SPARSErecords(Var)) &&
+	  CheckForOverCaches(CDF, Var)) {
+	if (!sX(FlushAllVarStages(CDF,Var),&pStatus)) return pStatus;
+	V_freecaches (CDF->stage.fp);
+	V_stagereset (CDF);
+      }
+      CDF->keptVarzOp = zOp;
+      CDF->keptVarNum = Var->varN;
+    }
+
     rd = BOO(zModeON(CDF),&(Var->zRD),BOO(zOp,&(Var->zRD),&(CDF->rRD)));
     phyRecNum = (Var->recVary ? rd->recNumber : 0);
     offset = IndicesValueOffset(Var->numDims,
@@ -722,6 +788,9 @@ switch (Va->item) {
       AbortAccess (CDF, UPDATE, noDELETE);
       return pStatus;
     }
+    if (Var->vType == COMPRESSED_) {
+      if (!sX(FlushStage(CDF,Var),&pStatus)) return pStatus;
+    }
     /**************************************************************************
     * Update last access time.
     **************************************************************************/
@@ -791,10 +860,29 @@ switch (Va->item) {
     /**************************************************************************
     * Write values.
     **************************************************************************/
+    /**************************************************************************
+    * Write values.
+    * Delete the staging vFile to free up the caches so the new write can
+    * start from the scratch, if the variable is chaned.
+    **************************************************************************/
+    if (zOp != CDF->keptVarzOp || Var->varN != CDF->keptVarNum) {
+      if (CDF->stage.fp != NULL && (COMPRESSED(Var) || SPARSErecords(Var)) &&
+	  CheckForOverCaches(CDF, Var)) {
+	if (!sX(FlushAllVarStages(CDF,Var),&pStatus)) return pStatus;
+	V_freecaches (CDF->stage.fp);
+	V_stagereset (CDF);
+      }
+      CDF->keptVarzOp = zOp;
+      CDF->keptVarNum = Var->varN;
+    }
+
     if (!sX(HyperWrite(CDF,Var,rd,buffer),&pStatus)) {
       AbortAccess (CDF, UPDATE, noDELETE);
       return pStatus;
     }
+    if (Var->vType == COMPRESSED_) {
+      if (!sX(FlushStage(CDF,Var),&pStatus)) return pStatus;
+    }
     /**************************************************************************
     * Update last access time.
     **************************************************************************/
@@ -851,11 +939,25 @@ switch (Va->item) {
     * Write value.
     **************************************************************************/
     offset = (Var->seqValueOffset % Var->NphyRecValues) * Var->NvalueBytes;
+    if (zOp != CDF->keptVarzOp || Var->varN != CDF->keptVarNum) {
+      if (CDF->stage.fp != NULL && (COMPRESSED(Var) || SPARSErecords(Var)) &&
+	  CheckForOverCaches(CDF, Var)) {
+	if (!sX(FlushAllVarStages(CDF,Var),&pStatus)) return pStatus;
+	V_freecaches (CDF->stage.fp);
+	V_stagereset (CDF);
+      }
+      CDF->keptVarzOp = zOp;
+      CDF->keptVarNum = Var->varN;
+    }
+
     if (!sX(WriteVarValues(CDF,Var,phyRecNum,
 			   offset,INT32_ONE,value),&pStatus)) {
       AbortAccess (CDF, UPDATE, noDELETE);
       return pStatus;
     }
+    if (Var->vType == COMPRESSED_) {
+      if (!sX(FlushStage(CDF,Var),&pStatus)) return pStatus;
+    }
     /**************************************************************************
     * Update last access time and current sequential position.
     **************************************************************************/
@@ -938,6 +1040,17 @@ switch (Va->item) {
        /***********************************************************************
        * Write variable value(s).
        ***********************************************************************/
+       if (zOp != CDF->keptVarzOp || Var->varN != CDF->keptVarNum) {
+	 if (CDF->stage.fp != NULL && (COMPRESSED(Var) || SPARSErecords(Var))
+	     && CheckForOverCaches(CDF, Var)) {
+	   if (!sX(FlushAllVarStages(CDF,Var),&pStatus)) return pStatus;
+	   V_freecaches (CDF->stage.fp);
+	   V_stagereset (CDF);
+	 }
+	 CDF->keptVarzOp = zOp;
+	 CDF->keptVarNum = Var->varN;
+       }
+
        recNum = BOO(Var->recVary,BOO(zOp,Var->zRD.recNumber,
 					 CDF->rRD.recNumber),0);
        if (!sX(WriteVarValues(CDF,Var,recNum,INT32_ZERO,
@@ -945,6 +1058,9 @@ switch (Va->item) {
 	 AbortAccess (CDF, UPDATE, noDELETE);
 	 return pStatus;
        }
+       if (Var->vType == COMPRESSED_) {
+	 if (!sX(FlushStage(CDF,Var),&pStatus)) return pStatus;
+       }
        /***********************************************************************
        * Increment buffer pointer and update time of last access for variable.
        ***********************************************************************/
@@ -966,7 +1082,9 @@ switch (Va->item) {
   case ATTR_SCOPE_:
   case gENTRY_DATA_:
   case rENTRY_DATA_:
+  case rENTRY_STRINGSDATA_:
   case zENTRY_DATA_:
+  case zENTRY_STRINGSDATA_:
   case gENTRY_DATASPEC_:
   case rENTRY_DATASPEC_:
   case zENTRY_DATASPEC_:
@@ -988,3 +1106,4 @@ switch (Va->item) {
 }
 return pStatus;
 }
+
