@@ -177,7 +177,6 @@ switch (Va->item) {
   * CDF_LEAPSECONDLASTUPDATED, 
   ****************************************************************************/
   case CDF_LEAPSECONDLASTUPDATED_: {
-/*    long lastUpdated = va_arg (Va->ap, long);*/       /* Last updated day. */
     break;
   }
   /****************************************************************************
@@ -337,7 +336,7 @@ switch (Va->item) {
 	  return CDF_CREATE_ERROR;
 	}
 	pCount = CompressionParmsCount ((Int32) cType);
-	CPR.RecordSize = CPR_BASE_SIZE + (pCount * sizeof(Int32));
+	CPR.RecordSize = CPR_BASE_SIZE + (pCount * 4);
 	CPR.RecordType = CPR_;
 	CPR.rfuA = 0;
 	CPR.cType = (Int32) cType;
@@ -367,6 +366,8 @@ switch (Va->item) {
     Int32 VDRoffset, maxAllocated, pCount = 0; int p;
     struct CDFstruct *CDF; struct VarStruct *Var;
     struct VDRstruct VDR; struct CPRstruct CPR;
+    Int32 numDims, dimSizes[CDF_MAX_DIMS], recSize;
+    int i;
     long cType = va_arg (Va->ap, long);         /* Compression type. */
     long *cParms = va_arg (Va->ap, long *);     /* Compression parameters. */
     /**************************************************************************
@@ -434,6 +435,47 @@ switch (Va->item) {
       if (cType != NO_COMPRESSION) return CANNOT_COMPRESS;
     }
     /**************************************************************************
+    * If the existing blocking factor is too small/large, then give warning.
+    **************************************************************************/
+    if (!zOp) {
+      if (!sX(ReadGDR(CDF->fp,CDF->GDRoffset64,
+		      GDR_rNUMDIMS,&numDims,
+		      GDR_rDIMSIZES,dimSizes,
+		      GDR_NULL),&pStatus)) {
+	AbortAccess (CDF, UPDATE, noDELETE);
+	return pStatus;
+      }
+    }
+    if (RECvaryBITset(VDR.Flags) && (VDR.blockingFactor != 0)) {
+      long total;
+      recSize = VDR.NumElems;
+      recSize *= (Int32) CDFelemSize(VDR.DataType);
+      for (i = 0; i < (int) (zOp?VDR.zNumDims:numDims); ++i)
+	if (VDR.DimVarys[i]) recSize *= (zOp?VDR.zDimSizes[i]:dimSizes[i]);
+      total = (long) recSize * VDR.blockingFactor;
+      if (total < (long) MIN_BLOCKING_BYTES_compressed) { 
+      }
+      if (total > (long) NUMcacheSTAGE * MIN_BLOCKING_BYTES_compressed) {
+	int numCaches, iy; Int32 blocking;
+	numCaches = (int) ((total-1) / MIN_BLOCKING_BYTES_compressed) + 1;
+	blocking = VDR.blockingFactor;
+	iy = 0;
+	while (numCaches > NUMcacheSTAGE) {
+	  blocking = blocking / 2;
+	  if (blocking == 0) {
+	    blocking = 1;
+	    iy = 1;
+	    break;
+	  }
+	  total = (long) recSize * blocking;
+	  numCaches = (int) ((total-1) / MIN_BLOCKING_BYTES_compressed) + 1;
+	}
+	if (iy == 0) 
+	  sX (BLOCKINGFACTOR_TOO_LARGE, &pStatus);
+	VDR.blockingFactor = blocking;
+      }
+    }
+    /**************************************************************************
     * Disable current compression (if any).
     **************************************************************************/
     if (VARcompressionBITset(VDR.Flags)) {
@@ -462,7 +504,7 @@ switch (Va->item) {
       case GZIP_COMPRESSION:
 	pCount = CompressionParmsCount ((Int32) cType);
 	SetBit32 (&VDR.Flags, VDR_COMPRESSION_BIT);
-	CPR.RecordSize = CPR_BASE_SIZE + (pCount * sizeof(Int32));
+	CPR.RecordSize = CPR_BASE_SIZE + (pCount * 4);
 	CPR.RecordType = CPR_;
 	CPR.cType = (Int32) cType;
 	CPR.rfuA = 0;
@@ -841,7 +883,11 @@ switch (Va->item) {
 	   CDF->CURzEntryNum) == RESERVED_ENTRYNUM) return NO_ENTRY_SELECTED;
     if (!ValidDataType((Int32)dataType) || InValidDataType((Int32)dataType))
       return BAD_DATA_TYPE;
-    if (numElems < 1) return BAD_NUM_ELEMS;
+    if (!STRINGdataType(dataType) && numElems < 1) return BAD_NUM_ELEMS;
+    if (STRINGdataType(dataType) && numElems < 1) {
+      if ((*((unsigned char *)value)) == '\0') numElems = 1;
+      else return BAD_NUM_ELEMS;
+    }
     if (!sX(ReadADR(CDF->fp,CDF->CURattrOffset,
 		    ADR_RECORD,&ADR,
 		    ADR_NULL),&pStatus)) {
@@ -850,6 +896,13 @@ switch (Va->item) {
     }
     if (!sX(CheckEntryOp(CDF,entryType),&pStatus)) return pStatus;
     if (!WriteAccess(CDF,FALSE,&pStatus)) return pStatus;
+    if (STRINGdataType(dataType)) {
+      int cnt;
+/*
+      if (UTF8Check((unsigned char *)value, (int) numElems, &cnt) != NULL)
+	pStatus = STRING_NOT_UTF8_ENCODING;
+*/
+    }
     nBytesNew = (int) (CDFelemSize(dataType) * numElems);
     eOffset = E3(entryType,CDF->CURgrEntryOffset,
 			   CDF->CURgrEntryOffset,
@@ -1056,7 +1109,7 @@ switch (Va->item) {
   }
   /****************************************************************************
    * rENTRY_STRINGSDATA_/zENTRY_STRINGSDATA_, 
-   ****************************************************************************/
+   ***************************************************************************/
   case rENTRY_STRINGSDATA_:
   case zENTRY_STRINGSDATA_: {
     Logical zOp = (Va->item == zENTRY_STRINGSDATA_);
@@ -1101,7 +1154,7 @@ switch (Va->item) {
     if (eOffset != (Int32) RESERVED_ENTRYOFFSET) {
       /************************************************************************
        * The entry already exists.
-       ************************************************************************/
+       ***********************************************************************/
       struct AEDRstruct AEDR; int nBytesCur;
       if (!sX(ReadAEDR(CDF->fp,eOffset,
                        AEDR_RECORD,&AEDR,NULL,
@@ -1177,7 +1230,7 @@ switch (Va->item) {
     else {
       /************************************************************************
        * The entry does not exist.
-       ************************************************************************/
+       ***********************************************************************/
       struct AEDRstruct AEDR;
       Int32 lastOffset;
       zEntry = BOO(!zOp,FALSE, BOO(zModeON(CDF),
@@ -1268,6 +1321,7 @@ switch (Va->item) {
       }
     }
     free (tmpStr);
+    tmpStr = NULL;
     break;
   }
 }
