@@ -115,7 +115,7 @@ vFILE *fp;
 Int32 offset;
 Int32 *irType;
 {
-  long irTypeOffset = offset + sizeof(Int32);
+  long irTypeOffset = offset + 4;
   if (!SEEKv(fp,irTypeOffset,vSEEK_SET)) return CRE;
   if (!Read32(fp,irType)) return CRE;
   return CDF_OK;
@@ -780,10 +780,11 @@ va_dcl
       }
       else if (fp->CURzEntrySel)
       {
-          if (fp->ADRList[fp->CurADRIndex]->zAEDRList[fp->CurAEDRIndex] != NULL)
+          if (fp->ADRList[fp->CurADRIndex]->zAEDRList[fp->CurAEDRIndex] != 
+              NULL)
           {
               CurAEDR = 
-                      fp->ADRList[fp->CurADRIndex]->zAEDRList[fp->CurAEDRIndex];
+                   fp->ADRList[fp->CurADRIndex]->zAEDRList[fp->CurAEDRIndex];
           }
           else
           {
@@ -796,7 +797,7 @@ va_dcl
               NULL)
           {
               CurAEDR = 
-                     fp->ADRList[fp->CurADRIndex]->grAEDRList[fp->CurAEDRIndex];
+                  fp->ADRList[fp->CurADRIndex]->grAEDRList[fp->CurAEDRIndex];
           }
           else
           {
@@ -805,9 +806,9 @@ va_dcl
       };  
   }; 
   /***************************************************************************
-  * For whatever AEDR data is selected, if READONLYon, read the data from memory
-  * using CurADRIndex and CurAEDRIndex to select the correct AEDR from the 
-  * AEDRList. Otherwise, read the data from the file/cache using the file 
+  * For whatever AEDR data is selected, if READONLYon, read the data from 
+  * memory using CurADRIndex and CurAEDRIndex to select the correct AEDR from 
+  * the AEDRList. Otherwise, read the data from the file/cache using the file 
   * offset.
   ***************************************************************************/ 
   for (;;) {
@@ -1122,7 +1123,7 @@ va_dcl
 	   if (!sX(ReadVDR(CDF,fp,offset,zVar,
 			   VDR_zNUMDIMS,&nDims,
 			   VDR_NULL),&pStatus)) return pStatus;
-	   tOffset = offset + zVDR_DIMVARYS_OFFSETb + (nDims*sizeof(Int32)) +
+	   tOffset = offset + zVDR_DIMVARYS_OFFSETb + (nDims*4) +
 		     BOO(CDF->wastedSpace,VDR_WASTED_SIZE,0);
 	 }
 	 else {
@@ -1155,8 +1156,7 @@ va_dcl
 			   VDR_zNUMDIMS,&zNumDims,
 			   VDR_NULL),&pStatus)) return pStatus;
 	   tOffset = offset + zVDR_PADVALUE_OFFSETb +
-		     (zNumDims*sizeof(Int32)) +
-		     (zNumDims*sizeof(Int32)) +
+		     (zNumDims*4) + (zNumDims*4) +
 		     BOO(CDF->wastedSpace,VDR_WASTED_SIZE,0);
 	 }
 	 else {
@@ -1168,8 +1168,7 @@ va_dcl
 			   GDR_rNUMDIMS,&rNumDims,
 			   GDR_NULL),&pStatus)) return pStatus;
 	   tOffset = offset + rVDR_PADVALUE_OFFSETb +
-		     (rNumDims*sizeof(Int32)) +
-		     BOO(CDF->wastedSpace,VDR_WASTED_SIZE,0);
+		     (rNumDims*4) + BOO(CDF->wastedSpace,VDR_WASTED_SIZE,0);
 	 }
 	 if (!SEEKv(fp,tOffset,vSEEK_SET)) return CRE;
 	 nBytes = (size_t) (CDFelemSize(dataType) * numElems);
@@ -1269,8 +1268,8 @@ va_dcl
 			 VXR_NULL),&pStatus)) return pStatus;
 	 switch (field) {
 	   case VXR_FIRSTREC: break;
-	   case VXR_LASTREC: tOffset += nEntries * sizeof(Int32); break;
-	   case VXR_OFFSET: tOffset += 2 * nEntries * sizeof(Int32); break;
+	   case VXR_LASTREC: tOffset += nEntries * 4; break;
+	   case VXR_OFFSET: tOffset += 2 * nEntries * 4; break;
 	   default: return CDF_INTERNAL_ERROR;
 	 }
 	 if (!SEEKv(fp,tOffset,vSEEK_SET)) return CRE;
@@ -1649,7 +1648,8 @@ STATICforIDL CDFstatus ReadADRList (vFILE *fp)
   if (fp->GDR->NumAttr > 0 && fp->ADRList == NULL) return BAD_MALLOC;
   for (i = 0; i < fp->GDR->NumAttr; i++)
   {
-      fp->ADRList[i] = cdf_AllocateMemory((size_t)sizeof(struct ADRstruct), NULL);
+      fp->ADRList[i] = cdf_AllocateMemory((size_t)sizeof(struct ADRstruct),
+                                          NULL);
       if (fp->ADRList[i] == NULL) return BAD_MALLOC;
   };
   /***************************************************************************
@@ -1696,9 +1696,9 @@ STATICforIDL CDFstatus ReadADRList (vFILE *fp)
 /******************************************************************************
 * ReadAEDRList.
 * Reads the set of entrys (zENTRYs or grENTRYs) associated with an attribute
-* into a list indexed by the entry number. If there is no entry for a particular
-* index (i.e., there is no entry for the associated variable/attribute
-* pair), the list entry is NULL.
+* into a list indexed by the entry number. If there is no entry for a
+* particular index (i.e., there is no entry for the associated
+* variable/attribute pair), the list entry is NULL.
 ******************************************************************************/
 STATICforIDL CDFstatus ReadAEDRList (vFILE *fp, 
                                      struct AEDRstructExt ***AEDRList, 
@@ -1736,7 +1736,8 @@ STATICforIDL CDFstatus ReadAEDRList (vFILE *fp,
   ***************************************************************************/
   while (NxtAEDR != 0 && iEntries != NumEntries)
   {
-      TempAEDR = cdf_AllocateMemory((size_t)sizeof(struct AEDRstructExt), NULL);
+      TempAEDR = cdf_AllocateMemory((size_t)sizeof(struct AEDRstructExt), 
+                                    NULL);
       if (TempAEDR == NULL) return BAD_MALLOC;
       if (!SEEKv(fp,NxtAEDR,vSEEK_SET)) return CRE;
       if (!Read32(fp,&(TempAEDR->AEDR.RecordSize))) return CRE;
