@@ -29,9 +29,9 @@
 *                               eliminate the potential buffer overflow.
 *   V3.3  28-Jul-08, M Liu      Modified to remove checks for wasted space as 
 *                               it is handled by cdfread.c for V2 CDF.
-*   V3.3a 15-Sep-08, M Liu      Modified ReadAEDRList64 to pass in the number of
-*                               entries to set up the list, not just the last
-*                               entry number. 
+*   V3.3a 15-Sep-08, M Liu      Modified ReadAEDRList64 to pass in the number
+*                               of entries to set up the list, not just the
+*                               last entry number. 
 *   V3.4  12-Aug-10, M Liu      Replaced strcpy by strcpyX in ReadVDR. 
 *
 ******************************************************************************/
@@ -142,7 +142,7 @@ int count;
 
 /******************************************************************************
 * ReadIrSize64.
-*   The size is always in the first 4-byte field.
+*   The size is always in the first 8-byte field.
 ******************************************************************************/
 
 STATICforIDL CDFstatus ReadIrSize64 (fp, offset, irSize)
@@ -210,8 +210,8 @@ va_dcl
 	 if (!Read64_64(fp,&(CDR->RecordSize))) return CRE;
 	 if (!Read32_64(fp,&(CDR->RecordType))) return CRE;
 	 if (!Read64_64(fp,&(CDR->GDRoffset))) return CRE;
-         /* group read for Version, Release, Encoding, Flags, rfuA, rfuB,
-            Increment, rfuD, rfuE -- all 4-byte big-endian. */
+         /* Group read for Version, Release, Encoding, Flags, rfuA, rfuB,
+            Increment, rfuD, rfuE -- 9 4-byte big-endians. */
          if (!Read32s_64(fp,&(CDR->Version),9)) return CRE;
          /*
   	   if (!Read32_64(fp,&(CDR->Version))) return CRE;
@@ -242,8 +242,12 @@ va_dcl
        case CDR_GDROFFSET: {
 	 OFF_T *buffer = va_arg (ap, OFF_T *); OFF_T tOffset = offset;
 	 switch (field) {
-	   case CDR_RECORDSIZE: tOffset += (OFF_T) CDR_RECORDSIZE_OFFSET64; break;
-	   case CDR_GDROFFSET: tOffset += (OFF_T) CDR_GDROFFSET_OFFSET64; break;
+	   case CDR_RECORDSIZE:
+	     tOffset += (OFF_T) CDR_RECORDSIZE_OFFSET64;
+	     break;
+	   case CDR_GDROFFSET:
+	     tOffset += (OFF_T) CDR_GDROFFSET_OFFSET64;
+	     break;
 	   default: return CDF_INTERNAL_ERROR;
 	 }
          if (!SEEKv64(fp,tOffset,vSEEK_SET)) return CRE;
@@ -254,12 +258,16 @@ va_dcl
 	 Int32 *buffer = va_arg (ap, Int32 *);
 	 OFF_T tOffset = offset;
 	 switch (field) {
-	   case CDR_RECORDTYPE: tOffset += (OFF_T) CDR_RECORDTYPE_OFFSET64; break;
+	   case CDR_RECORDTYPE:
+	     tOffset += (OFF_T) CDR_RECORDTYPE_OFFSET64;
+	     break;
 	   case CDR_VERSION: tOffset += (OFF_T) CDR_VERSION_OFFSET64; break;
 	   case CDR_RELEASE: tOffset += (OFF_T) CDR_RELEASE_OFFSET64; break;
 	   case CDR_ENCODING: tOffset += (OFF_T) CDR_ENCODING_OFFSET64; break;
 	   case CDR_FLAGS: tOffset += (OFF_T) CDR_FLAGS_OFFSET64; break;
-	   case CDR_INCREMENT: tOffset += (OFF_T) CDR_INCREMENT_OFFSET64; break;
+	   case CDR_INCREMENT:
+	     tOffset += (OFF_T) CDR_INCREMENT_OFFSET64;
+	     break;
 	   default: return CDF_INTERNAL_ERROR;
 	 }
 	 if (!SEEKv64(fp,tOffset,vSEEK_SET)) return CRE;
@@ -307,13 +315,14 @@ va_dcl
   ***************************************************************************/
   if (read_only_mode == READONLYon && fp->GDR64 == NULL)
   {
-      fp->GDR64 = cdf_AllocateMemory((size_t)sizeof(struct GDRstruct64), NULL);
+      fp->GDR64 = cdf_AllocateMemory((size_t)sizeof(struct GDRstruct64),
+                                     NULL);
       if (fp->GDR64 == NULL) return CRE;
       if (!SEEKv64(fp,offset,vSEEK_SET)) return CRE;
       if (!Read64_64(fp,&(fp->GDR64->RecordSize))) return CRE;
       if (!Read32_64(fp,&(fp->GDR64->RecordType))) return CRE;
-      /* group read for rVDRhead, zVDRhead, ADRhead, eof -- all 8-byte
-         big-endian */
+      /* Group read for rVDRhead, zVDRhead, ADRhead, eof -- 4 8-byte
+         big-endians */
       if (!Read64s_64(fp,&(fp->GDR64->rVDRhead),4)) return CRE;
       /*
         if (!Read64_64(fp,&(fp->GDR64->rVDRhead))) return CRE;
@@ -321,8 +330,8 @@ va_dcl
         if (!Read64_64(fp,&(fp->GDR64->ADRhead))) return CRE;
         if (!Read64_64(fp,&(fp->GDR64->eof))) return CRE;
       */
-      /* group read for NrVars, NumAttr, rMaxRec, rNumDims, NzVars -- all
-         4-byte big-endian */
+      /* Group read for NrVars, NumAttr, rMaxRec, rNumDims, NzVars -- 5
+         4-byte big-endians */
       if (!Read32s_64(fp,&(fp->GDR64->NrVars),5)) return CRE;
       /*
         if (!Read32_64(fp,&(fp->GDR64->NrVars))) return CRE;
@@ -332,8 +341,8 @@ va_dcl
         if (!Read32_64(fp,&(fp->GDR64->NzVars))) return CRE;
       */
       if (!Read64_64(fp,&(fp->GDR64->UIRhead))) return CRE;
-      /* group read for rfuC, LeapSecondLastUpdated, rfuE -- all 4-byte
-         big-endian */
+      /* Group read for rfuC, LeapSecondLastUpdated, rfuE -- 3 4-byte
+         big-endians */
       if (!Read32s_64(fp,&(fp->GDR64->rfuC),3)) return CRE;
       /*
         if (!Read32_64(fp,&(fp->GDR64->rfuC))) return CRE;
@@ -364,6 +373,8 @@ va_dcl
 	     if (!SEEKv64(fp,offset,vSEEK_SET)) return CRE;
 	     if (!Read64_64(fp,&(GDR->RecordSize))) return CRE;
 	     if (!Read32_64(fp,&(GDR->RecordType))) return CRE;
+	     /* Group read for rVDRhead, zVDRhead, ADRhead, eof -- 4 8-byte
+		big-endians. */
              if (!Read64s_64(fp,&(GDR->rVDRhead),4)) return CRE;
              /*
 	       if (!Read64_64(fp,&(GDR->rVDRhead))) return CRE;
@@ -371,6 +382,8 @@ va_dcl
 	       if (!Read64_64(fp,&(GDR->ADRhead))) return CRE;
 	       if (!Read64_64(fp,&(GDR->eof))) return CRE;
              */
+	     /* Group read for NrVars, NumAttr, rMaxRec, rNumDims, NzVars --
+		5 4-byte big-endians. */
              if (!Read32s_64(fp,&(GDR->NrVars),5)) return CRE;
              /*
 	       if (!Read32_64(fp,&(GDR->NrVars))) return CRE;
@@ -380,6 +393,8 @@ va_dcl
 	       if (!Read32_64(fp,&(GDR->NzVars))) return CRE;
              */
 	     if (!Read64_64(fp,&(GDR->UIRhead))) return CRE;
+	     /* Group read for rfuC, LeapSecondLastUpdated, rfuE -- 3 4-byte
+		big-endians. */
              if (!Read32s_64(fp,&(GDR->rfuC),3)) return CRE;
              /*
 	       if (!Read32_64(fp,&(GDR->rfuC))) return CRE;
@@ -650,11 +665,11 @@ va_dcl
   {
       return NO_ATTR_SELECTED;
   };
-  /***************************************************************************
+  /****************************************************************************
   * For whatever ADR data is selected, if READONLYon, read the data from memory
   * using CurADRIndex to select the correct ADR from the ADRList. Otherwise,
   * read the data from the file/cache using the file offset.
-  ***************************************************************************/
+  ****************************************************************************/
   for (;;) {
      int field = va_arg (ap, int);
      switch (field) {
@@ -668,11 +683,14 @@ va_dcl
 	     if (!SEEKv64(fp,offset,vSEEK_SET)) return CRE;
              if (!Read64_64(fp,&(ADR->RecordSize))) return CRE;
              if (!Read32_64(fp,&(ADR->RecordType))) return CRE;
+	     /* Group read for ADRnext, AgrEDRhead -- 2 8-byte big-endians. */
              if (!Read64s_64(fp,&(ADR->ADRnext),2)) return CRE;
              /*
                if (!Read64_64(fp,&(ADR->ADRnext))) return CRE;
                if (!Read64_64(fp,&(ADR->AgrEDRhead))) return CRE;
              */
+	     /* Group read for Scope, Num, NgrEntries, MAXgrEntry, rfuA --
+		5 4-byte big-endians. */
              if (!Read32s_64(fp,&(ADR->Scope),5)) return CRE;
              /*
                if (!Read32_64(fp,&(ADR->Scope))) return CRE;
@@ -682,6 +700,8 @@ va_dcl
                if (!Read32_64(fp,&(ADR->rfuA))) return CRE;
              */
              if (!Read64_64(fp,&(ADR->AzEDRhead))) return CRE;
+	     /* Group read for NzEntries, MAXzEntry, rfuE -- 3 4-byte
+		big-endians. */
              if (!Read32s_64(fp,&(ADR->NzEntries),3)) return CRE;
              /*
                if (!Read32_64(fp,&(ADR->NzEntries))) return CRE;
@@ -880,7 +900,7 @@ va_dcl
 #endif
   pStatus = CDFlib(CONFIRM_, CDF_READONLY_MODE_, &read_only_mode, NULL_);
   if (pStatus != CDF_OK) return pStatus;
-  /***************************************************************************
+  /****************************************************************************
   * If READONLYon and either an attribute or an entry have not been selected,
   * return the appropriate status. If they have been selected but there is no
   * entry for the selected attribute, return BAD_ENTRY_NUM.  Note that for each
@@ -889,7 +909,7 @@ va_dcl
   * attribute, the entry list has a NULL pointer for that entry. Note also that
   * the reading of ADRs and AEDRs into memory is triggered by the 1st time an
   * ADR (or AEDR) is selected.
-  ***************************************************************************/
+  ***********************************************************************.****/
   if (read_only_mode == READONLYon)
   {
       if (fp->CurADRIndex == RESERVED_ENTRYNUM)
@@ -902,11 +922,11 @@ va_dcl
       }
       else if (fp->CURzEntrySel)
       {
-          if (fp->ADRList64[fp->CurADRIndex]->zAEDRList64[fp->CurAEDRIndex] 
-              != NULL)
+          if (fp->ADRList64[fp->CurADRIndex]->zAEDRList64[fp->CurAEDRIndex] != 
+              NULL)
           {
               CurAEDR =
-                  fp->ADRList64[fp->CurADRIndex]->zAEDRList64[fp->CurAEDRIndex];
+                fp->ADRList64[fp->CurADRIndex]->zAEDRList64[fp->CurAEDRIndex];
           }
           else
           {
@@ -919,7 +939,7 @@ va_dcl
               NULL)
           {
               CurAEDR =
-                 fp->ADRList64[fp->CurADRIndex]->grAEDRList64[fp->CurAEDRIndex];
+                fp->ADRList64[fp->CurADRIndex]->grAEDRList64[fp->CurAEDRIndex];
           }
           else
           {
@@ -928,9 +948,9 @@ va_dcl
       };
   };
   /***************************************************************************
-  * For whatever AEDR data is selected, if READONLYon, read the data from memory
-  * using CurADRIndex and CurAEDRIndex to select the correct AEDR from the
-  * AEDRList. Otherwise, read the data from the file/cache using the file
+  * For whatever AEDR data is selected, if READONLYon, read the data from
+  * memory using CurADRIndex and CurAEDRIndex to select the correct AEDR from
+  * the AEDRList. Otherwise, read the data from the file/cache using the file
   * offset.
   ***************************************************************************/
   for (;;) {
@@ -948,13 +968,15 @@ va_dcl
              if (!Read64_64(fp,&(AEDR->RecordSize))) return CRE;
              if (!Read32_64(fp,&(AEDR->RecordType))) return CRE;
              if (!Read64_64(fp,&(AEDR->AEDRnext))) return CRE;
+	     /* Group read for AttrNum, DataType, Num, NumElems, NumStrings,
+		rfuB, rfuC, rfuD, rfuE -- 9 4-byte big-endians. */
              if (!Read32s_64(fp,&(AEDR->AttrNum),9)) return CRE;
              /*
                if (!Read32_64(fp,&(AEDR->AttrNum))) return CRE;
                if (!Read32_64(fp,&(AEDR->DataType))) return CRE;
                if (!Read32_64(fp,&(AEDR->Num))) return CRE;
                if (!Read32_64(fp,&(AEDR->NumElems))) return CRE;
-               if (!Read32_64(fp,&(AEDR->AEDR.NumStrings))) return CRE;
+               if (!Read32_64(fp,&(AEDR->NumStrings))) return CRE;
                if (!Read32_64(fp,&(AEDR->rfuB))) return CRE;
                if (!Read32_64(fp,&(AEDR->rfuC))) return CRE;
                if (!Read32_64(fp,&(AEDR->rfuD))) return CRE;
@@ -1169,16 +1191,20 @@ va_dcl
 	 if (!Read64_64(fp,&(VDR->RecordSize))) return CRE;
 	 if (!Read32_64(fp,&(VDR->RecordType))) return CRE;
 	 if (!Read64_64(fp,&(VDR->VDRnext))) return CRE;
+	 /* Group read for DataType, MaxRec -- 2 4-byte big-endians. */
          if (!Read32s_64(fp,&(VDR->DataType),2)) return CRE;
          /*
 	   if (!Read32_64(fp,&(VDR->DataType))) return CRE;
 	   if (!Read32_64(fp,&(VDR->MaxRec))) return CRE;
          */
+	 /* Group read for VXRhead, VXRtail -- 2 8-byte big-endians. */
          if (!Read64s_64(fp,&(VDR->VXRhead),2)) return CRE;
          /*
 	   if (!Read64_64(fp,&(VDR->VXRhead))) return CRE;
 	   if (!Read64_64(fp,&(VDR->VXRtail))) return CRE;
          */
+	 /* Group read for Flags, sRecords, rfuB, rfuC, rfuF, NumElems,
+	    Num -- 7 4-byte big-endians. */
          if (!Read32s_64(fp,&(VDR->Flags),7)) return CRE;
          /*
 	   if (!Read32_64(fp,&(VDR->Flags))) return CRE;
@@ -1261,8 +1287,7 @@ va_dcl
 	   if (!sX(ReadVDR64(CDF,fp,offset,zVar,
 			    VDR_zNUMDIMS,&nDims,
 			    VDR_NULL),&pStatus)) return pStatus;
-	   tOffset = offset + (OFF_T) (zVDR_DIMVARYS_OFFSETb64 +
-				       (nDims*sizeof(Int32)));
+	   tOffset = offset + (OFF_T) (zVDR_DIMVARYS_OFFSETb64 + nDims*4);
 	 }
 	 else {
 	   OFF_T GDRoffset;
@@ -1293,8 +1318,7 @@ va_dcl
 			     VDR_zNUMDIMS,&zNumDims,
 			     VDR_NULL),&pStatus)) return pStatus;
 	   tOffset = offset + (OFF_T) (zVDR_PADVALUE_OFFSETb64 +
-		     (zNumDims*sizeof(Int32)) +
-		     (zNumDims*sizeof(Int32)));
+		     (zNumDims*4) + (zNumDims*4));
 	 }
 	 else {
 	   Int32 rNumDims;
@@ -1306,7 +1330,7 @@ va_dcl
 			     GDR_rNUMDIMS,&rNumDims,
 			     GDR_NULL),&pStatus)) return pStatus;
 	   tOffset = offset + (OFF_T) (rVDR_PADVALUE_OFFSETb64 +
-		     (rNumDims*sizeof(Int32)));
+		     (rNumDims*4));
 	 }
 	 if (!SEEKv64(fp,tOffset,vSEEK_SET)) return CRE;
 	 nBytes = (size_t) (CDFelemSize(dataType) * numElems);
@@ -1400,11 +1424,12 @@ va_dcl
 	 if (!Read64_64(fp,&(VXR->RecordSize))) return CRE;
 	 if (!Read32_64(fp,&(VXR->RecordType))) return CRE;
 	 if (!Read64_64(fp,&(VXR->VXRnext))) return CRE;
-         /*
-         if (!Read32s_64(fp,&(VXR->Nentries),2)) return CRE;
-         */
+	 /* Group read for Nentries, NusedEntries -- 2 4-byte big-endians. */
+	 if (!Read32s_64(fp,&(VXR->Nentries),2)) return CRE;
+/*
 	 if (!Read32_64(fp,&(VXR->Nentries))) return CRE;
 	 if (!Read32_64(fp,&(VXR->NusedEntries))) return CRE;
+*/
          if ((int)VXR->Nentries < 0 || (int)VXR->Nentries > MAX_VXR_ENTRIES)
            return CV3C;
 	 if (!Read32s_64(fp,VXR->First,(int)VXR->Nentries)) return CRE;
@@ -1421,7 +1446,7 @@ va_dcl
 			   VXR_NULL),&pStatus)) return pStatus;
 	 switch (field) {
 	   case VXR_FIRSTREC: break;
-	   case VXR_LASTREC: tOffset += (OFF_T) nEntries * sizeof(Int32); break;
+	   case VXR_LASTREC: tOffset += (OFF_T) nEntries * 4; break;
 	   default: return CDF_INTERNAL_ERROR;
 	 }
 	 if (!SEEKv64(fp,tOffset,vSEEK_SET)) return CRE;
@@ -1436,7 +1461,7 @@ va_dcl
          if (!sX(ReadVXR64(fp,offset,
                            VXR_NENTRIES,&nEntries,
                            VXR_NULL),&pStatus)) return pStatus;
-         tOffset += (OFF_T) (2 * nEntries * sizeof(Int32)); 
+         tOffset += (OFF_T) (2 * nEntries * 4); 
          if (!SEEKv64(fp,tOffset,vSEEK_SET)) return CRE;
          if ((int)nEntries < 0 || nEntries > MAX_VXR_ENTRIES)
            return CV3C;
@@ -1655,7 +1680,9 @@ va_dcl
 	   case CCR_RECORDSIZE:
              tOffset += (OFF_T) CCR_RECORDSIZE_OFFSET64;
              break;
-	   case CCR_CPROFFSET: tOffset += (OFF_T) CCR_CPROFFSET_OFFSET64; break;
+	   case CCR_CPROFFSET:
+	     tOffset += (OFF_T) CCR_CPROFFSET_OFFSET64;
+	     break;
 	   case CCR_USIZE: tOffset += (OFF_T) CCR_USIZE_OFFSET64; break;
 	   default: return CDF_INTERNAL_ERROR;
 	 }
@@ -1697,6 +1724,8 @@ va_dcl
 	 struct CPRstruct64 *CPR = va_arg (ap, struct CPRstruct64 *); int i;
 	 if (!SEEKv64(fp,offset,vSEEK_SET)) return CRE;
 	 if (!Read64_64(fp,&(CPR->RecordSize))) return CRE;
+	 /* Group read for RecordType, cType, rfuA, pCount -- 4 4-byte
+	    big-endians. */
          if (!Read32s_64(fp,&(CPR->RecordType),4)) return CRE;
          /*
 	   if (!Read32_64(fp,&(CPR->RecordType))) return CRE;
@@ -1771,6 +1800,8 @@ va_dcl
 	 struct SPRstruct64 *SPR = va_arg (ap, struct SPRstruct64 *); int i;
 	 if (!SEEKv64(fp,offset,vSEEK_SET)) return CRE;
 	 if (!Read64_64(fp,&(SPR->RecordSize))) return CRE;
+	 /* Group read for RecordType, sArraysType, rfuA, pCount -- 4 4-byte
+	    big-endians. */
          if (!Read32s_64(fp,&(SPR->RecordType),4)) return CRE;
          /*
 	   if (!Read32_64(fp,&(SPR->RecordType))) return CRE;
@@ -1845,6 +1876,7 @@ va_dcl
 	 struct CVVRstruct64 *CVVR = va_arg (ap, struct CVVRstruct64 *);
 	 if (!SEEKv64(fp,offset,vSEEK_SET)) return CRE;
 	 if (!Read64_64(fp,&(CVVR->RecordSize))) return CRE;
+	 /* Group read for RecordType, rfuA -- 2 4-byte big-endians. */
          if (!Read32s_64(fp,&(CVVR->RecordType),2)) return CRE;
          /*
 	   if (!Read32_64(fp,&(CVVR->RecordType))) return CRE;
@@ -1895,9 +1927,9 @@ STATICforIDL CDFstatus ReadADRList64 (vFILE *fp)
   Int32 i;
   OFF_T temp;
 
-  /**************************************************************************
+  /***************************************************************************
   * Allocate memory for the list of ADRs 
-  ***************************************************************************/
+  ****************************************************************************/
   if (fp->GDR64->NumAttr > 0)
   fp->ADRList64 = cdf_AllocateMemory((size_t)fp->GDR64->NumAttr * 
                                      sizeof(struct ADRstruct64**), NULL);
@@ -1922,6 +1954,8 @@ STATICforIDL CDFstatus ReadADRList64 (vFILE *fp)
       fp->ADRList64[i]->ADRnext = 0;
       if (!Read64_64(fp,&temp)) return CRE;
       fp->ADRList64[i]->AgrEDRhead = temp;
+      /* Group read for Scope, Num, NgrEntries, MAXgrEntry, rfuA -- 5 4-byte
+	 big-endians. */
       if (!Read32s_64(fp,&(fp->ADRList64[i]->Scope),5)) return CRE;
       /*
       if (!Read32_64(fp,&(fp->ADRList64[i]->Scope))) return CRE;
@@ -1932,6 +1966,7 @@ STATICforIDL CDFstatus ReadADRList64 (vFILE *fp)
       */
       if (!Read64_64(fp,&temp)) return CRE;
       fp->ADRList64[i]->AzEDRhead = temp;
+      /* Group read for NzEntries, MAXzEntry, rfuE -- 3 4-byte big-endians. */
       if (!Read32s_64(fp,&(fp->ADRList64[i]->NzEntries),3)) return CRE;
       /*
         if (!Read32_64(fp,&(fp->ADRList64[i]->NzEntries))) return CRE;
@@ -1962,9 +1997,9 @@ STATICforIDL CDFstatus ReadADRList64 (vFILE *fp)
 /******************************************************************************
 * ReadAEDRList64.
 * Reads the set of entrys (zENTRYs or grENTRYs) associated with an attribute
-* into a list indexed by the entry number. If there is no entry for a particular
-* index (i.e., there is no entry for the associated variable/attribute
-* pair), the list entry is NULL.
+* into a list indexed by the entry number. If there is no entry for a
+* particular index (i.e., there is no entry for the associated
+* variable/attribute pair), the list entry is NULL.
 ******************************************************************************/
 STATICforIDL CDFstatus ReadAEDRList64 (vFILE *fp, 
                                        struct AEDRstructExt64 ***AEDRList, 
@@ -2013,6 +2048,8 @@ STATICforIDL CDFstatus ReadAEDRList64 (vFILE *fp,
       if (!Read64_64(fp,&temp)) return CRE;
       NxtAEDR = temp;
       TempAEDR->AEDR.AEDRnext = 0;
+      /* Group read for AttrNum, DataType, Num, NumElems, NumStrings, rfuB,
+	 rfuC, rfuD, rfuE -- 9 4-byte big-endians. */
       if (!Read32s_64(fp,&(TempAEDR->AEDR.AttrNum),9)) return CRE;
       /*
         if (!Read32_64(fp,&(TempAEDR->AEDR.AttrNum))) return CRE;
