@@ -43,8 +43,7 @@ Common Data Format (CDF)\012\
 https://cdf.gsfc.nasa.gov\012\
 Space Physics Data Facility\012\
 NASA/Goddard Space Flight Center\012\
-Greenbelt, Maryland 20771 USA\012\
-(User support: gsfc-cdf-support@lists.nasa.gov)\012", CDF_COPYRIGHT_LEN);
+Greenbelt, Maryland 20771 USA\012", CDF_COPYRIGHT_LEN);
   return;
 }
 
@@ -571,7 +570,8 @@ break;
 
 case NOT_A_CDF_OR_NOT_SUPPORTED:
 strcpyX (textPtr,
-"NOT_A_CDF_OR_NOT_SUPPORTED: Named CDF is corrupted or not supported by the current library version.",
+"NOT_A_CDF_OR_NOT_SUPPORTED: Named CDF is corrupted or not supported by "
+"the current library version.",
 CDF_STATUSTEXT_LEN);
 break;
 
@@ -745,13 +745,15 @@ break;
 
 case TT2000_TIME_ERROR:
 strcpyX (textPtr,
-"TT2000_TIME_ERROR: Error handling the date/time for TT2000 or a TT2000 epoch.",
+"TT2000_TIME_ERROR: Error handling the date/time for TT2000 or a TT2000 "
+"epoch.",
 CDF_STATUSTEXT_LEN);
 break;
 
 case UNABLE_TO_PROCESS_CDF:
 strcpyX (textPtr,
-"UNABLE_TO_PROCESS_CDF: One of data fields is invalid... Upgrade the library might fix the problem.",
+"UNABLE_TO_PROCESS_CDF: One of data fields is invalid... Upgrade the library "
+"might fix the problem.",
 CDF_STATUSTEXT_LEN);
 break;
 
@@ -769,55 +771,74 @@ break;
 
 case ILLEGAL_EPOCH_FIELD:
 strcpyX (textPtr,
-"ILLEGAL_EPOCH_FIELD: One or more of the date/time fields is out of valid range.",
+"ILLEGAL_EPOCH_FIELD: One or more of the date/time fields is out of valid "
+"range.",
 CDF_STATUSTEXT_LEN);
 break;
 
 case CANNOT_INSERT_RECORDS:
 strcpyX (textPtr,
-"CANNOT_INSERT_RECORDS: Cannot insert records to a variable with sparserecords.",
+"CANNOT_INSERT_RECORDS: Cannot insert records to a variable with "
+"sparserecords.",
 CDF_STATUSTEXT_LEN);
 break;
 
 case TT2000_CDF_MAYNEEDUPDATE:
 strcpyX (textPtr,
-"TT2000_CDF_MAYNEEDUPDATE: CDF version or its file-based template may need to upgrade to handle TT2000 data properly (even its value may be valid).",
+"TT2000_CDF_MAYNEEDUPDATE: CDF version or its file-based template may need to "
+"upgrade to handle TT2000 data properly (even its value may be valid).",
 CDF_STATUSTEXT_LEN);
 break;
 
 case TT2000_USED_OUTDATED_TABLE:
 strcpyX (textPtr,
-"TT2000_USED_OUTDATED_TABLE: A TT2000 data is either invalid (made with an oudated leap second table) or trying to use an outdated leap second table.",
+"TT2000_USED_OUTDATED_TABLE: A TT2000 data is either invalid (made with an "
+"oudated leap second table) or trying to use an outdated leap second table.",
 CDF_STATUSTEXT_LEN);
 break;
 
 case BADDATE_LEAPSECOND_UPDATED:
 strcpyX (textPtr,
-"BADDATE_LEAPSECOND_UPDATED: The last leap second updated date is not valid (not in the leap second table).",
+"BADDATE_LEAPSECOND_UPDATED: The last leap second updated date is not valid "
+"(not in the leap second table).",
 CDF_STATUSTEXT_LEN);
 break;
 
 case FUNCTION_NOT_SUPPORTED:
 strcpyX (textPtr,
-"FUNCTION_NOT_SUPPORTED: This function is not supported. Likely, it is trying to access an rVariable.",
+"FUNCTION_NOT_SUPPORTED: This function is not supported. Likely, it is trying "
+"to access an rVariable.",
 CDF_STATUSTEXT_LEN);
 break;
 
 case BLOCKINGFACTOR_TOO_SMALL:
 strcpyX (textPtr,
-"BLOCKINGFACTOR_TOO_SMALL: The blocking factor set for the compressed variable is smaller than the default. It is changed but may make data access ineffective.",
+"BLOCKINGFACTOR_TOO_SMALL: The blocking factor set for the compressed "
+"variable is smaller than the default. It is changed but may make data access "
+"inefficient.",
 CDF_STATUSTEXT_LEN);
 break;
 
 case BLOCKINGFACTOR_TOO_SMALL2:
 strcpyX (textPtr,
-"BLOCKINGFACTOR_TOO_SMALL2: The variable's blocking factor is smaller than the default if it is compressed. The compression is set but may make data access ineffective.",
+"BLOCKINGFACTOR_TOO_SMALL2: The variable's blocking factor is smaller than "
+"the default if it is compressed. The compression is set but may make data "
+"access inefficient.",
+CDF_STATUSTEXT_LEN);
+break;
+
+case BLOCKINGFACTOR_TOO_LARGE:
+strcpyX (textPtr,
+"BLOCKINGFACTOR_TOO_LARGE: The intended blocking factor is too large. To avoid"
+"a temporary file(s) to be created for compression, a smaller value is "
+"assigned.",
 CDF_STATUSTEXT_LEN);
 break;
 
 case TRY_TO_READ_NONSTRING_DATA:
 strcpyX (textPtr,
-"TRY_TO_READ_NONSTRING_DATA: Wrong function as the attribute entry data is non-string.",
+"TRY_TO_READ_NONSTRING_DATA: Wrong function as the attribute entry data is "
+"non-string.",
 CDF_STATUSTEXT_LEN);
 break;
 
@@ -827,6 +848,19 @@ strcpyX (textPtr,
 CDF_STATUSTEXT_LEN);
 break;
 
+case STRING_NOT_UTF8_ENCODING:
+strcpyX (textPtr,
+"STRING_NOT_UTF8_ENCODING: String not in UTF8 encoding form.",
+CDF_STATUSTEXT_LEN);
+break;
+
+case CANNOT_CONVERT_WIDECHAR:
+strcpyX (textPtr,
+"CANNOT_CONVERT_WIDECHAR: Can not convert between wide-character string and "
+"multi-byte character string. Locale might not be properly set.",
+CDF_STATUSTEXT_LEN);
+break;
+
 default:
 snprintf (textPtr, (size_t) CDF_STATUSTEXT_LEN+1,
           "Text not found - unknown CDF status code (%ld).", status);
