@@ -3,7 +3,7 @@
 * Administrator of the National Aeronautics and Space Administration.
 * All Rights Reserved.
 ******************************************************************************/
-/****************************************************************************** 
+/******************************************************************************
 *
 *  SPDF/CDF                       CDF_TIME_TT2000 utility routines for C.
 *
@@ -251,7 +251,8 @@ long long *nanoSecSinceJ2000;
 }
 
 /******************************************************************************
-* CDF_TT2000_from_UTC_EPOCH_f (FORTRAN equivalent of CDF_TT2000_from_UTC_EPOCH).
+* CDF_TT2000_from_UTC_EPOCH_f (FORTRAN equivalent of 
+* CDF_TT2000_from_UTC_EPOCH).
 ******************************************************************************/
 
 VISIBLE_PREFIX 
@@ -360,20 +361,66 @@ Fif_ENTRYPOINT(encode_tt2000__,
                encode_tt2000_,
                encode_tt2000,
                ENCODE_TT2000)
-(nanoSecSinceJ2000, format, string Fif_GHOSTARG(string_len))
+(nanoSecSinceJ2000, style, string Fif_GHOSTARG(string_len))
 long long *nanoSecSinceJ2000;
-int *format;
+int  *style;
 void *string;
 Fif_GHOSTDEF(string_len)
 {
   char tString[TT2000_0_STRING_LEN+1];
-  int len = -1;
-  if (*format == 3) len = TT2000_3_STRING_LEN;
-  else if (*format == 0) len = TT2000_0_STRING_LEN;
-  else if (*format == 1) len = TT2000_1_STRING_LEN;
-  else if (*format == 2) len = TT2000_2_STRING_LEN;
+  int  len = -1;
+  int  style2 = *style;
+  if (style2 < 0 || style2 > 4) style2 = 3;
+  if (style2 == 3) len = TT2000_3_STRING_LEN;
+  else if (style2 == 0) len = TT2000_0_STRING_LEN;
+  else if (style2 == 1) len = TT2000_1_STRING_LEN;
+  else if (style2 == 2) len = TT2000_2_STRING_LEN;
+  else if (style2 == 4) len = TT2000_4_STRING_LEN;
+  if (len != -1) { 
+    CDF_TT2000_to_UTC_string (*nanoSecSinceJ2000, tString, style2);
+#if defined(Fif_GHOSTLEN)
+    CtoFORTstring (tString, string, Fif_GHOSTUSE(string_len));
+#else
+    CtoFORTstring (tString, string, len);
+#endif
+  } else {
+#if defined(Fif_GHOSTLEN)
+    CtoFORTstring (" ", string, Fif_GHOSTUSE(string_len));
+#else
+    CtoFORTstring (" ", string, 1);
+#endif
+  }
+}
+
+/******************************************************************************
+* toencode_TT2000 (aka encode_TT2000).
+******************************************************************************/
+
+VISIBLE_PREFIX
+Fif_PREFIXa
+void
+Fif_PREFIXb
+Fif_ENTRYPOINT(toencode_tt2000__,
+               toencode_tt2000_,
+               toencode_tt2000,
+               TOENCODE_TT2000)
+(nanoSecSinceJ2000, style, string Fif_GHOSTARG(string_len))
+long long *nanoSecSinceJ2000;
+int  *style;
+void *string;
+Fif_GHOSTDEF(string_len)
+{
+  char tString[TT2000_0_STRING_LEN+1];
+  int  len = -1;
+  int  style2 = *style;
+  if (style2 < 0 || style2 > 4) style2 = 3;
+  encodeTT2000 (*nanoSecSinceJ2000, tString, style2);
+  if (style2 == 3) len = TT2000_3_STRING_LEN;
+  else if (style2 == 0) len = TT2000_0_STRING_LEN;
+  else if (style2 == 1) len = TT2000_1_STRING_LEN;
+  else if (style2 == 2) len = TT2000_2_STRING_LEN;
+  else if (style2 == 4) len = TT2000_4_STRING_LEN;
   if (len != -1) { 
-    CDF_TT2000_to_UTC_string (*nanoSecSinceJ2000, tString, *format);
 #if defined(Fif_GHOSTLEN)
     CtoFORTstring (tString, string, Fif_GHOSTUSE(string_len));
 #else
@@ -400,20 +447,23 @@ Fif_ENTRYPOINT(cdf_tt2000_to_utc_string_f__,
                cdf_tt2000_to_utc_string_f_,
                cdf_tt2000_to_utc_string_f,
                CDF_TT2000_TO_UTC_STRING_F)
-(nanoSecSinceJ2000, format, string Fif_GHOSTARG(string_len))
+(nanoSecSinceJ2000, style, string Fif_GHOSTARG(string_len))
 long long *nanoSecSinceJ2000;
-int *format;
+int  *style;
 void *string;
 Fif_GHOSTDEF(string_len)
 {
   char tString[TT2000_0_STRING_LEN+1];
   int len = -1;
-  if (*format == 3) len = TT2000_3_STRING_LEN;
-  else if (*format == 0) len = TT2000_0_STRING_LEN;
-  else if (*format == 1) len = TT2000_1_STRING_LEN;
-  else if (*format == 2) len = TT2000_2_STRING_LEN;
+  int style2 = *style;
+  if (style2 < 0 || style2 > 4) style2 = 3;
+  if (style2 == 3) len = TT2000_3_STRING_LEN;
+  else if (style2 == 0) len = TT2000_0_STRING_LEN;
+  else if (style2 == 1) len = TT2000_1_STRING_LEN;
+  else if (style2 == 2) len = TT2000_2_STRING_LEN;
+  else if (style2 == 4) len = TT2000_4_STRING_LEN;
   if (len != -1) { 
-    CDF_TT2000_to_UTC_string (*nanoSecSinceJ2000, tString, *format);
+    CDF_TT2000_to_UTC_string (*nanoSecSinceJ2000, tString, style2);
 #if defined(Fif_GHOSTLEN)
     CtoFORTstring (tString, string, Fif_GHOSTUSE(string_len));
 #else
@@ -446,17 +496,57 @@ long long *tt2000;
 Fif_GHOSTDEF(string_len)
 {
   struct STRINGstruct *ssh = NULL;
-  size_t len = strlen(string)+1;
 #if defined(Fif_DESCR)
-  *tt2000 = CDF_TT2000_from_UTC_string (DESCRtoREFnul(string,len,&ssh));
+  *tt2000 = CDF_TT2000_from_UTC_string (DESCRtoREFnul(string,
+                                        TT2000_0_STRING_LEN,&ssh));
 #endif
 #if defined(Fif_GHOSTLEN)
-  *tt2000 = CDF_TT2000_from_UTC_string (NULterminate(string,Fif_GHOSTUSE(string_len),&ssh));
+  *tt2000 = CDF_TT2000_from_UTC_string (NULterminate(string,
+                                                     Fif_GHOSTUSE(string_len),
+                                                     &ssh));
 #endif
 #if defined(Fif_NOLEN)
-  *tt2000 = CDF_TT2000_from_UTC_string (FindEndNUL(string,len,&ssh));
+  *tt2000 = CDF_TT2000_from_UTC_string (FindEndNUL(string,
+                                        TT2000_0_STRING_LEN,&ssh));
 #endif
   FreeStrings (ssh);
+  return;
+}
+
+/******************************************************************************
+* toparse_TT2000.
+******************************************************************************/
+
+VISIBLE_PREFIX
+Fif_PREFIXa
+void
+Fif_PREFIXb
+Fif_ENTRYPOINT(toparse_tt2000__,
+               toparse_tt2000_,
+               toparse_tt2000,
+               TOPARSE_TT2000)
+(string, tt2000 Fif_GHOSTARG(string_len))
+char *string;
+long long *tt2000;
+Fif_GHOSTDEF(string_len)
+{
+  struct STRINGstruct *ssh = NULL;
+#if defined(Fif_DESCR)
+  *tt2000 = CDF_TT2000_from_UTC_string (DESCRtoREFnul(string,
+                                                      TT2000_0_STRING_LEN,
+                                                      &ssh));
+#endif
+#if defined(Fif_GHOSTLEN)
+  *tt2000 = CDF_TT2000_from_UTC_string (NULterminate(string,
+                                                     Fif_GHOSTUSE(string_len),
+                                                     &ssh));
+#endif
+#if defined(Fif_NOLEN)
+  *tt2000 = CDF_TT2000_from_UTC_string (FindEndNUL(string,TT2000_0_STRING_LEN,
+                                                   &ssh));
+#endif
+  FreeStrings (ssh);
+  return;
 }
 
 /******************************************************************************
@@ -478,17 +568,22 @@ long long *tt2000;
 Fif_GHOSTDEF(string_len)
 {
   struct STRINGstruct *ssh = NULL;
-  size_t len = strlen(string)+1;
 #if defined(Fif_DESCR)
-  *tt2000 = CDF_TT2000_from_UTC_string (DESCRtoREFnul(string,len,&ssh));
+  *tt2000 = CDF_TT2000_from_UTC_string (DESCRtoREFnul(string,
+                                                      TT2000_3_STRING_LEN,
+                                                      &ssh));
 #endif
 #if defined(Fif_GHOSTLEN)
-  *tt2000 = CDF_TT2000_from_UTC_string (NULterminate(string,Fif_GHOSTUSE(string_len),&ssh));
+  *tt2000 = CDF_TT2000_from_UTC_string (NULterminate(string,
+                                                     Fif_GHOSTUSE(string_len),
+                                                     &ssh));
 #endif
 #if defined(Fif_NOLEN)
-  *tt2000 = CDF_TT2000_from_UTC_string (FindEndNUL(string,len,&ssh));
+  *tt2000 = CDF_TT2000_from_UTC_string (FindEndNUL(string,TT2000_3_STRING_LEN,
+                                                   &ssh));
 #endif
   FreeStrings (ssh);
+  return;
 }
 
 /******************************************************************************
@@ -624,4 +719,44 @@ double *tableo;
   cdf_FreeMemory (table, NULL);
 }
 
+/******************************************************************************
+* TT2000_to_UnixTime (FORTRAN equivalent of TT2000toUnixTime).
+******************************************************************************/
+
+VISIBLE_PREFIX
+Fif_PREFIXa
+void
+Fif_PREFIXb
+Fif_ENTRYPOINT(tt2000_to_unixtime__,
+               tt2000_to_unixtime_,
+               tt2000_to_unixtime,
+               TT2000_TO_UNIXTIME)
+(tt2000s, unixTimes, num)
+long long *tt2000s;
+double  *unixTimes;
+int     *num;
+{
+  TT2000toUnixTime (tt2000s, unixTimes, *num);
+}
+
+/******************************************************************************
+* UnixTime_to_TT2000 (FORTRAN equivalent of UnixTimetoTT2000).
+******************************************************************************/
+
+VISIBLE_PREFIX
+Fif_PREFIXa
+void
+Fif_PREFIXb
+Fif_ENTRYPOINT(unixtime_to_tt2000__,
+               unixtime_to_tt2000_,
+               unixtime_to_tt2000,
+               UNIXTIME_TO_TT2000)
+(unixTimes, tt2000s, num)
+double  *unixTimes;
+long long *tt2000s;
+int  *num;
+{
+  UnixTimetoTT2000 (unixTimes, tt2000s, *num);
+}
+
 
