@@ -97,7 +97,7 @@ static CDFstatus QuitCDF (char *why, OFF_T offset, int isize, int num,
                     (long) *(int *)value2);
           strcatX (text, text2, 150);
         } else if (isize == 8) { /* Each one is a 8-byte long long type. */
-                                 /* Each long long is up to 19 digits + sign. */
+                                 /* Each long long is up to 19 digits + sign.*/
           char text2[2 * 20 + 6 + 1];
           text2[0] = (char) NUL;
 #if !defined(win32) && !defined(__MINGW32__)
@@ -117,7 +117,8 @@ static CDFstatus QuitCDF (char *why, OFF_T offset, int isize, int num,
           text2 = (char *) cdf_AllocateMemory (len, NULL);
           if (text2 != NULL) {
             text2[0] = (char) NUL;
-            snprintf (text2, len, "(%s vs %s)", (char *)value1, (char *)value2);
+            snprintf (text2, len, "(%s vs %s)", (char *)value1,
+                      (char *)value2);
             strcatX (text, text2, 150);
             cdf_FreeMemory (text2, NULL);
           }
@@ -427,7 +428,8 @@ static CDFstatus ValidateGDR (vFILE *fp, OFF_T offset, Logical debug)
     return QuitCDF ("CDF(GDR): NrVars or their link is invalid ", offset,
                     4, 1, &(GDR.NrVars), 0, debug);
   if (GDR.NumAttr < 0 || (GDR.NumAttr > 0 && GDR.ADRhead < 1)) 
-    return QuitCDF ("CDF(GDR): # of attributes or their link is invalid ", offset,
+    return QuitCDF ("CDF(GDR): # of attributes or their link is invalid ", 
+                    offset,
                     4, 1, &(GDR.NumAttr), 0, debug);
   if (GDR.rMaxRec < -1) 
     return QuitCDF ("CDF(GDR): max rVars record is invalid ", offset,
@@ -497,8 +499,8 @@ static CDFstatus ValidateADR (struct CDFstruct *CDF, vFILE *fp, OFF_T offset,
     return QuitCDF ("CDF(ADR): global attribute zEntry is invalid ", offset,
                     4, 1, &(ADR.NzEntries), 0, debug);
   if ((ADR.Scope == GLOBAL_SCOPE) && (ADR.AzEDRhead > 0))
-    return QuitCDF ("CDF(ADR): global attribute zEntry link is invalid ", offset,
-                    8, 1, &(ADR.AzEDRhead), 0, debug);
+    return QuitCDF ("CDF(ADR): global attribute zEntry link is invalid ",
+                    offset, 8, 1, &(ADR.AzEDRhead), 0, debug);
 /*
   if ((ADR.Scope == VARIABLE_SCOPE) && (ADR.MAXgrEntry > -1) &&
       (GDR.NrVars < 1))
@@ -693,13 +695,15 @@ static CDFstatus ValidateVXR (vFILE *fp, OFF_T offset, Int32 lastRec,
                     4, 2, &(VXR.NusedEntries), &(VXR.Nentries), debug);
   if (VXR.VXRnext > 0) {
     if (VXR.Last[VXR.NusedEntries-1] > lastRec)
-      return QuitCDF ("CDF(VXR): a variable last record does not match in a Variable Index Record: ",
+      return QuitCDF ("CDF(VXR): a variable last record does not match in a "
+                      "Variable Index Record: ",
                       offset, 4, 2, &(VXR.Last[VXR.NusedEntries-1]), &lastRec,
                       debug);
     status = ValidateVXR (fp, VXR.VXRnext, lastRec, debug);
     if (status != CDF_OK) return status;
   } else if (VXR.VXRnext < 0) {
-      return QuitCDF ("CDF(VXR): a link offset to next record is negative in a Variable Index Record: ",
+      return QuitCDF ("CDF(VXR): a link offset to next record is negative in "
+                      "a Variable Index Record: ",
                       offset, 8, 1, &(VXR.VXRnext), 0, debug);
   }
 
@@ -846,8 +850,8 @@ static CDFstatus ValidateCPR (vFILE *fp, OFF_T offset, Logical debug)
     return QuitCDF ("CDF(CPR): reocrd size is invalid ", offset,
                     8, 1, &(CPR.RecordSize), 0, debug);
   if (CPR.pCount != 1) 
-    return QuitCDF ("CDF(CPR): compression parameter count is invalid ", offset,
-                    4, 1, &(CPR.pCount), 0, debug);
+    return QuitCDF ("CDF(CPR): compression parameter count is invalid ",
+                    offset, 4, 1, &(CPR.pCount), 0, debug);
   cType = (long) CPR.cType;
   cParms[0] = (long) CPR.cParms[0];
   if (ValidateCompression64(cType, cParms) != CDF_OK) 
@@ -882,8 +886,8 @@ static CDFstatus ValidateSPR (vFILE *fp, OFF_T offset, Logical debug)
     return QuitCDF ("CDF(SPR): record size is invalid ", offset,
                     8, 1, &(SPR.RecordSize), 0, debug);
   if (SPR.pCount < 1 || SPR.pCount > CDF_MAX_PARMS) 
-    return QuitCDF ("CDF(SPR): sparseness parameter count is invalid ", offset,
-                    4, 1, &(SPR.pCount), 0, debug);
+    return QuitCDF ("CDF(SPR): sparseness parameter count is invalid ",
+                    offset, 4, 1, &(SPR.pCount), 0, debug);
   return CDF_OK;
 }
 
@@ -910,8 +914,8 @@ static CDFstatus ValidateCVVR (vFILE *fp, OFF_T offset, Logical debug)
     return QuitCDF ("CDF(CVVR): record size is invalid ", offset,
                     8, 1, &(CVVR.RecordSize), 0, debug);
   if (CVVR.cSize < 0) 
-    return QuitCDF ("CDF(CVVR): uncompressed records size is invalid ", offset,
-                    8, 1, &(CVVR.cSize), 0, debug);
+    return QuitCDF ("CDF(CVVR): uncompressed records size is invalid ",
+                    offset, 8, 1, &(CVVR.cSize), 0, debug);
   return CDF_OK;
 }
 
@@ -982,9 +986,9 @@ static CDFstatus ValidateAttributeLinks (struct CDFstruct *CDF, vFILE *fp,
 ******************************************************************************/
 
 static CDFstatus ValidateAttributeEntryLink (struct CDFstruct *CDF, vFILE *fp, 
-                                      Int32 num, Logical zEntry, OFF_T EDRhead,
-                                             Int32 numEntries, Int32 maxEntry, 
-                                             Logical debug)
+                                             Int32 num, Logical zEntry,
+                                             OFF_T EDRhead, Int32 numEntries,
+                                             Int32 maxEntry, Logical debug)
 {
   CDFstatus status;
   OFF_T offset, nextAEDR;
@@ -1028,7 +1032,8 @@ static CDFstatus ValidateAttributeEntryLink (struct CDFstruct *CDF, vFILE *fp,
   }
   if (lastNum != maxEntry) {
     cdf_FreeMemory (visits, NULL);
-    return QuitCDF ("CDF: last entry # is not the max entry # in an attr link: ",
+    return QuitCDF ("CDF: last entry # is not the max entry # in an attr "
+                    "link: ",
                     (OFF_T) -1, 4, 2, &lastNum, &maxEntry, debug);
   }
   cdf_FreeMemory (visits, NULL);
@@ -1104,18 +1109,18 @@ static CDFstatus ValidateVariableLinks (struct CDFstruct *CDF, vFILE *fp,
     if (status == CDF_OK) {
       if (num >= numVars) { 
         if (zVar)
-          return QuitCDF ("CDF(VDR): a zVar unreachable in the variable chain: ",
-                          offset, 4, 1, &num, 0, debug);
+          return QuitCDF ("CDF(VDR): a zVar unreachable in the variable "
+                          "chain: ", offset, 4, 1, &num, 0, debug);
         else
-          return QuitCDF ("CDF(VDR): a rVar unreachable in the variable chain: ",
-                          offset, 4, 1, &num, 0, debug);
+          return QuitCDF ("CDF(VDR): a rVar unreachable in the variable "
+                          "chain: ", offset, 4, 1, &num, 0, debug);
       } else {
         if (zVar) 
-          return QuitCDF ("CDF(VDR): a zVar is repeated in the variable chain: ",
-                          offset, 4, 1, &num, 0, debug);
+          return QuitCDF ("CDF(VDR): a zVar is repeated in the variable "
+                          "chain: ", offset, 4, 1, &num, 0, debug);
         else
-          return QuitCDF ("CDF(VDR): a rVar unreachable in the variable chain: ",
-                          offset, 4, 1, &num, 0, debug);
+          return QuitCDF ("CDF(VDR): a rVar unreachable in the variable "
+                          "chain: ", offset, 4, 1, &num, 0, debug);
       }
     }
   }
