@@ -107,7 +107,7 @@ vFILE *fp;
 Int32 offset;
 Int32 *irType;
 {
-  long irTypeOffset = offset + sizeof(Int32);
+  long irTypeOffset = offset + 4;
   if (!SEEKv(fp,irTypeOffset,vSEEK_SET)) return CWE;
   if (!Write32(fp,irType)) return CWE;
   return CDF_OK;
@@ -394,26 +394,25 @@ va_dcl
 	 if (!Write32(fp,&(AEDR->rfuC))) return CWE;
 	 if (!Write32(fp,&(AEDR->rfuD))) return CWE;
 	 if (!Write32(fp,&(AEDR->rfuE))) return CWE;
-         if (value != NULL) {
+	 if (value != NULL) {
 	   if (STRINGdataType(AEDR->DataType)) {
-	     size_t len; char *newValue = NULL;
-	     len = strlen ((char *)value);
-	     if ((int)len < (int) AEDR->NumElems) {
-	       int ix;
-	       newValue = (char *) cdf_AllocateMemory ((size_t) AEDR->NumElems,
-						       NULL);
-	       memcpy (newValue, (char *)value, len);
-	       for (ix = (int) len; ix < (int) AEDR->NumElems; ++ix)
-	         *(newValue+ix) = (char) '\0';
-	     }
+	     char *newValue; size_t len;
+	     newValue = (char *) cdf_AllocateMemory (sizeof(char) *
+      						     AEDR->NumElems+1, NULL);
+	     memset ((void *)newValue, '\0', (size_t)AEDR->NumElems+1);
+	     len = strlen((char *)value);
+	     memcpy ((void *)newValue, (void *)value,
+		     (size_t) MINIMUM(AEDR->NumElems,len));
 	     if (!sX(WriteBuffer(CDF,fp,AEDR->DataType,AEDR->NumElems,
-				 (newValue==NULL?value:(void *)newValue)),
-		     &pStatus))
+				 (void *)newValue),&pStatus)) {
+	       cdf_FreeMemory (newValue, NULL);
 	       return pStatus;
-	     if (newValue != NULL) cdf_FreeMemory (newValue, NULL);
+	     }
+	     cdf_FreeMemory (newValue, NULL);
 	   } else
 	     if (!sX(WriteBuffer(CDF,fp,AEDR->DataType,
-				 AEDR->NumElems,value),&pStatus)) return pStatus;
+				 AEDR->NumElems,value),&pStatus))
+	       return pStatus;
 	 }
 	 break;
        }
@@ -427,20 +426,19 @@ va_dcl
 	 tOffset = offset + AEDR_VALUE_OFFSET;
 	 if (!SEEKv(fp,tOffset,vSEEK_SET)) return CWE;
 	 if (STRINGdataType(dataType)) {
-	   size_t len; char *newValue = NULL;
-	   len = strlen ((char *)value);
-	   if ((int)len < (int) numElems) {
-	     int ix;
-	     newValue = (char *) cdf_AllocateMemory ((size_t) numElems, NULL);
-	     memcpy (newValue, (char *)value, len);
-	     for (ix = (int) len; ix < (int) numElems; ++ix)
-	        *(newValue+ix) = (char) '\0';
-	   }
-	   if (!sX(WriteBuffer(CDF,fp,dataType,numElems,
-			       (newValue==NULL?value:(void *)newValue)),
-		   &pStatus))
+	   char *newValue; size_t len;
+	   newValue = (char *) cdf_AllocateMemory (sizeof(char) *
+						   numElems, NULL);
+	   memset ((void *)newValue, '\0', (size_t)numElems);
+           len = strlen((char *)value);
+           memcpy ((void *)newValue, (void *)value,
+                   (size_t)MINIMUM(numElems,len));
+	   if (!sX(WriteBuffer(CDF,fp,dataType,numElems,(void *)newValue),
+		   &pStatus)) {
+	     cdf_FreeMemory (newValue, NULL);
 	     return pStatus;
-	   if (newValue != NULL) cdf_FreeMemory (newValue, NULL);
+	   } else
+	     cdf_FreeMemory (newValue, NULL);
 	 } else
 	   if (!sX(WriteBuffer(CDF,fp,dataType,
 			       numElems,value),&pStatus)) return pStatus;
@@ -540,24 +538,21 @@ va_dcl
 	 if (!Write32s(fp,VDR->DimVarys,(int)nDims)) return CWE;
 	 if (PADvalueBITset(VDR->Flags) && padValue != NULL) {
 	   if (STRINGdataType(VDR->DataType)) {
-	     size_t len; char *newPad = NULL;
-	     len = strlen ((char *)padValue);
-	     if ((int)len < (int) VDR->NumElems) {
-	       int ix;
-	       newPad = (char *) cdf_AllocateMemory ((size_t) VDR->NumElems,
-						     NULL);
-	       memcpy (newPad, (char *)padValue, len);
-	       for (ix = (int) len; ix < (int) VDR->NumElems; ++ix)
-		 *(newPad+ix) = (char) '\0';
-	     }
+	     char *newPad; size_t len;
+	     newPad = (char *) cdf_AllocateMemory (sizeof(char) *
+						   VDR->NumElems, NULL);
+	     memset ((void *)newPad, '\0', (size_t)VDR->NumElems);
+             len = (int) strlen((char *)padValue);
+	     memcpy ((void *)newPad, (void *)padValue,
+		     (size_t)MINIMUM(VDR->NumElems,len));
 	     if (!sX(WriteBuffer(CDF,fp,VDR->DataType,VDR->NumElems,
-				 (newPad==NULL?padValue:(void *)newPad)),
-		     &pStatus))
+				 (void *)newPad), &pStatus)) {
+	       cdf_FreeMemory (newPad, NULL);
 	       return pStatus;
-	     if (newPad != NULL) cdf_FreeMemory (newPad, NULL);
+	     } else
+	       cdf_FreeMemory (newPad, NULL);
 	   } else
-	     if (!sX(WriteBuffer(CDF,fp,VDR->DataType,
-			         VDR->NumElems,
+	     if (!sX(WriteBuffer(CDF,fp,VDR->DataType,VDR->NumElems,
 			         padValue),&pStatus)) return pStatus;
 	 }
 	 break;
@@ -597,7 +592,7 @@ va_dcl
 	   if (!sX(ReadVDR(CDF,fp,offset,zVar,
 			   VDR_zNUMDIMS,&nDims,
 			   VDR_NULL),&pStatus)) return pStatus;
-	   tOffset = offset + zVDR_DIMVARYS_OFFSETb + (nDims * sizeof(Int32));
+	   tOffset = offset + zVDR_DIMVARYS_OFFSETb + (nDims * 4);
 	 }
 	 else {
 	   Int32 GDRoffset;
@@ -626,7 +621,7 @@ va_dcl
 			   VDR_zNUMDIMS,&zNumDims,
 			   VDR_NULL),&pStatus)) return pStatus;
 	   tOffset = offset + zVDR_PADVALUE_OFFSETb +
-		     (zNumDims * sizeof(Int32)) + (zNumDims * sizeof(Int32));
+		     (zNumDims * 4) + (zNumDims * 4);
 	 }
 	 else {
 	   Int32 rNumDims, GDRoffset;
@@ -636,27 +631,26 @@ va_dcl
 	   if (!sX(ReadGDR(fp,GDRoffset,
 			   GDR_rNUMDIMS,&rNumDims,
 			   GDR_NULL),&pStatus)) return pStatus;
-	   tOffset = offset + rVDR_PADVALUE_OFFSETb + (rNumDims*sizeof(Int32));
+	   tOffset = offset + rVDR_PADVALUE_OFFSETb + (rNumDims*4);
 	 }
 	 if (!SEEKv(fp,tOffset,vSEEK_SET)) return CWE;
 	 if (STRINGdataType(dataType)) {
-	   size_t len; char *newPad = NULL;
-	   len = strlen ((char *)padValue);
-	   if ((int)len < (int) numElems) {
-	     int ix;
-	     newPad = (char *) cdf_AllocateMemory ((size_t) numElems, NULL);
-	     memcpy (newPad, (char *)padValue, len);
-	     for (ix = (int) len; ix < (int) numElems; ++ix)
-	        *(newPad+ix) = (char) '\0';
-	   }
-	   if (!sX(WriteBuffer(CDF,fp,dataType,numElems,
-			       (newPad==NULL?padValue:(void *)newPad)),
-		   &pStatus))
+	   char *newPad; size_t len;
+	   newPad = (char *) cdf_AllocateMemory (sizeof(char) *
+						 numElems, NULL);
+	   memset ((void *)newPad, '\0', (size_t)numElems);
+	   len = strlen((char *)padValue);
+	   memcpy ((void *)newPad, (void *)padValue,
+		   (size_t)MINIMUM(numElems,len));
+	   if (!sX(WriteBuffer(CDF,fp,dataType,numElems,(void *)newPad),
+		   &pStatus)) {
+	     cdf_FreeMemory (newPad, NULL);
 	     return pStatus;
-	   if (newPad != NULL) cdf_FreeMemory (newPad, NULL);
+	   } else
+	     cdf_FreeMemory (newPad, NULL);
 	 } else
-	   if (!sX(WriteBuffer(CDF,fp,dataType,
-			       numElems,padValue),&pStatus)) return pStatus;
+	   if (!sX(WriteBuffer(CDF,fp,dataType,numElems,padValue),
+		   &pStatus)) return pStatus;
 	 break;
        }
        default: {
@@ -734,8 +728,8 @@ va_dcl
 			 VXR_NULL),&pStatus)) return pStatus;
 	 switch (field) {
 	   case VXR_FIRSTREC: break;
-	   case VXR_LASTREC: tOffset += nEntries * sizeof(Int32); break;
-	   case VXR_OFFSET: tOffset += 2 * nEntries * sizeof(Int32); break;
+	   case VXR_LASTREC: tOffset += nEntries * 4; break;
+	   case VXR_OFFSET: tOffset += 2 * nEntries * 4; break;
 	   default: return CDF_INTERNAL_ERROR;
 	 }
 	 if (!SEEKv(fp,tOffset,vSEEK_SET)) return CWE;
