@@ -226,8 +226,12 @@ va_dcl
        case CDR_GDROFFSET: {
          OFF_T *buffer = va_arg (ap, OFF_T *); OFF_T tOffset = offset;
 	 switch (field) {
-	   case CDR_RECORDSIZE: tOffset += (OFF_T) CDR_RECORDSIZE_OFFSET64; break;
-	   case CDR_GDROFFSET: tOffset += (OFF_T) CDR_GDROFFSET_OFFSET64; break;
+	   case CDR_RECORDSIZE:
+	     tOffset += (OFF_T) CDR_RECORDSIZE_OFFSET64; 
+	     break;
+	   case CDR_GDROFFSET:
+	     tOffset += (OFF_T) CDR_GDROFFSET_OFFSET64;
+	     break;
 	   default: return CDF_INTERNAL_ERROR;
 	 }
 	 if (!SEEKv64(fp,tOffset,vSEEK_SET)) return CWE;
@@ -237,14 +241,16 @@ va_dcl
        default: {
 	 Int32 *buffer = va_arg (ap, Int32 *); OFF_T tOffset = offset;
 	 switch (field) {
-/*	   case CDR_RECORDSIZE: tOffset += (OFF_T) CDR_RECORDSIZE_OFFSET64; break; */
-	   case CDR_RECORDTYPE: tOffset += (OFF_T) CDR_RECORDTYPE_OFFSET64; break;
-/*	   case CDR_GDROFFSET: tOffset += (OFF_T) CDR_GDROFFSET_OFFSET64; break; */
+	   case CDR_RECORDTYPE:
+	     tOffset += (OFF_T) CDR_RECORDTYPE_OFFSET64;
+	     break;
 	   case CDR_VERSION: tOffset += (OFF_T) CDR_VERSION_OFFSET64; break;
 	   case CDR_RELEASE: tOffset += (OFF_T) CDR_RELEASE_OFFSET64; break;
 	   case CDR_ENCODING: tOffset += (OFF_T) CDR_ENCODING_OFFSET64; break;
 	   case CDR_FLAGS: tOffset += (OFF_T) CDR_FLAGS_OFFSET64; break;
-	   case CDR_INCREMENT: tOffset += (OFF_T) CDR_INCREMENT_OFFSET64; break;
+	   case CDR_INCREMENT:
+	     tOffset += (OFF_T) CDR_INCREMENT_OFFSET64;
+	     break;
 	   default: return CDF_INTERNAL_ERROR;
 	 }
 	 if (!SEEKv64(fp,tOffset,vSEEK_SET)) return CWE;
@@ -335,7 +341,9 @@ va_dcl
        case GDR_UIRHEAD: {
          OFF_T *buffer = va_arg (ap, OFF_T *); OFF_T tOffset = offset;
          switch (field) {
-           case GDR_RECORDSIZE: tOffset += (OFF_T) GDR_RECORDSIZE_OFFSET64; break;
+           case GDR_RECORDSIZE:
+	     tOffset += (OFF_T) GDR_RECORDSIZE_OFFSET64;
+	     break;
            case GDR_rVDRHEAD: tOffset += (OFF_T) GDR_rVDRHEAD_OFFSET64; break;
            case GDR_zVDRHEAD: tOffset += (OFF_T) GDR_zVDRHEAD_OFFSET64; break;
            case GDR_ADRHEAD: tOffset += (OFF_T) GDR_ADRHEAD_OFFSET64; break;
@@ -350,19 +358,17 @@ va_dcl
        default: {
 	 Int32 *buffer = va_arg (ap, Int32 *); OFF_T tOffset = offset;
 	 switch (field) {
-/*	   case GDR_RECORDSIZE: tOffset += (OFF_T) GDR_RECORDSIZE_OFFSET64; break; */
-	   case GDR_RECORDTYPE: tOffset += (OFF_T) GDR_RECORDTYPE_OFFSET64; break;
-/*	   case GDR_rVDRHEAD: tOffset += (OFF_T) GDR_rVDRHEAD_OFFSET64; break; */
-/*	   case GDR_zVDRHEAD: tOffset += (OFF_T) GDR_zVDRHEAD_OFFSET64; break; */
-/*	   case GDR_ADRHEAD: tOffset += (OFF_T) GDR_ADRHEAD_OFFSET64; break; */
-/*	   case GDR_EOF: tOffset += (OFF_T) GDR_EOF_OFFSET64; break; */
+	   case GDR_RECORDTYPE:
+	     tOffset += (OFF_T) GDR_RECORDTYPE_OFFSET64;
+	     break;
 	   case GDR_NrVARS: tOffset += (OFF_T) GDR_NrVARS_OFFSET64; break;
 	   case GDR_NUMATTR: tOffset += (OFF_T) GDR_NUMATTR_OFFSET64; break;
 	   case GDR_rMAXREC: tOffset += (OFF_T) GDR_rMAXREC_OFFSET64; break;
 	   case GDR_rNUMDIMS: tOffset += (OFF_T) GDR_rNUMDIMS_OFFSET64; break;
 	   case GDR_NzVARS: tOffset += (OFF_T) GDR_NzVARS_OFFSET64; break;
-	   case GDR_LEAPSECONDLASTUPDATED: tOffset += (OFF_T) GDR_LEAPSECONDLASTUPDATED_OFFSET64; break;
-/*	   case GDR_UIRHEAD: tOffset += (OFF_T) GDR_UIRHEAD_OFFSET64; break; */
+	   case GDR_LEAPSECONDLASTUPDATED:
+	     tOffset += (OFF_T) GDR_LEAPSECONDLASTUPDATED_OFFSET64;
+	     break;
 	   default: return CDF_INTERNAL_ERROR;
 	 }
 	 if (!SEEKv64(fp,tOffset,vSEEK_SET)) return CWE;
@@ -445,10 +451,16 @@ va_dcl
        case ADR_AzEDRHEAD: {
          OFF_T *buffer = va_arg (ap, OFF_T *); OFF_T tOffset = offset;
          switch (field) {
-           case ADR_RECORDSIZE: tOffset += (OFF_T) ADR_RECORDSIZE_OFFSET64; break;
+           case ADR_RECORDSIZE:
+	     tOffset += (OFF_T) ADR_RECORDSIZE_OFFSET64;
+	     break;
            case ADR_ADRNEXT: tOffset += (OFF_T) ADR_ADRNEXT_OFFSET64; break;
-           case ADR_AgrEDRHEAD: tOffset += (OFF_T) ADR_AgrEDRHEAD_OFFSET64; break;
-           case ADR_AzEDRHEAD: tOffset += (OFF_T) ADR_AzEDRHEAD_OFFSET64; break;
+           case ADR_AgrEDRHEAD:
+	     tOffset += (OFF_T) ADR_AgrEDRHEAD_OFFSET64;
+	     break;
+           case ADR_AzEDRHEAD:
+	     tOffset += (OFF_T) ADR_AzEDRHEAD_OFFSET64;
+	     break;
            default: return CDF_INTERNAL_ERROR;
          }
          if (!SEEKv64(fp,tOffset,vSEEK_SET)) return CWE;
@@ -458,17 +470,23 @@ va_dcl
        default: {
 	 Int32 *buffer = va_arg (ap, Int32 *); OFF_T tOffset = offset;
 	 switch (field) {
-/*	   case ADR_RECORDSIZE: tOffset += (OFF_T) ADR_RECORDSIZE_OFFSET64; break; */
-	   case ADR_RECORDTYPE: tOffset += (OFF_T) ADR_RECORDTYPE_OFFSET64; break;
-/*	   case ADR_ADRNEXT: tOffset += (OFF_T) ADR_ADRNEXT_OFFSET64; break; */
-/*	   case ADR_AgrEDRHEAD: tOffset += (OFF_T) ADR_AgrEDRHEAD_OFFSET64; break; */
+	   case ADR_RECORDTYPE:
+	     tOffset += (OFF_T) ADR_RECORDTYPE_OFFSET64;
+	     break;
 	   case ADR_SCOPE: tOffset += (OFF_T) ADR_SCOPE_OFFSET64; break;
 	   case ADR_NUM: tOffset += (OFF_T) ADR_NUM_OFFSET64; break;
-	   case ADR_NgrENTRIES: tOffset += (OFF_T) ADR_NgrENTRIES_OFFSET64; break;
-	   case ADR_MAXgrENTRY: tOffset += (OFF_T) ADR_MAXgrENTRY_OFFSET64; break;
-/*	   case ADR_AzEDRHEAD: tOffset += (OFF_T) ADR_AzEDRHEAD_OFFSET64; break; */
-	   case ADR_NzENTRIES: tOffset += (OFF_T) ADR_NzENTRIES_OFFSET64; break;
-	   case ADR_MAXzENTRY: tOffset += (OFF_T) ADR_MAXzENTRY_OFFSET64; break;
+	   case ADR_NgrENTRIES:
+	     tOffset += (OFF_T) ADR_NgrENTRIES_OFFSET64;
+	     break;
+	   case ADR_MAXgrENTRY:
+	     tOffset += (OFF_T) ADR_MAXgrENTRY_OFFSET64;
+	     break;
+	   case ADR_NzENTRIES:
+	     tOffset += (OFF_T) ADR_NzENTRIES_OFFSET64;
+	     break;
+	   case ADR_MAXzENTRY:
+	     tOffset += (OFF_T) ADR_MAXzENTRY_OFFSET64;
+	     break;
 	   default: return CDF_INTERNAL_ERROR;
 	 }
 	 if (!SEEKv64(fp,tOffset,vSEEK_SET)) return CWE;
@@ -532,25 +550,23 @@ va_dcl
 	 if (!Write32_64(fp,&(AEDR->rfuE))) return CWE;
 */
 	 if (value != NULL) {
-           if (STRINGdataType(AEDR->DataType)) {
-             size_t len; char *newValue = NULL;
-             len = strlen ((char *)value);
-             if ((int)len < AEDR->NumElems) {
-               int ix;
-               newValue = (char *) cdf_AllocateMemory ((size_t) AEDR->NumElems,
-						       NULL);
-               memcpy (newValue, (char *)value, len);
-               for (ix = (int) len; ix < AEDR->NumElems; ++ix)
-                  *(newValue+ix) = (char) '\0';
-             }
+	   if (STRINGdataType(AEDR->DataType)) {
+	     char *newValue; size_t len;
+	     newValue = (char *) cdf_AllocateMemory (sizeof(char) *
+	    					     (AEDR->NumElems), NULL);
+	     memset ((void *)newValue, '\0', (size_t)AEDR->NumElems);
+	     len = strlen((char *)value);
+	     memcpy ((void *)newValue, (void *)value,
+		     (size_t) MINIMUM(AEDR->NumElems,len));
 	     if (!sX(WriteBuffer64(CDF,fp,AEDR->DataType,AEDR->NumElems,
-				   (newValue==NULL?value:(void *)newValue)),
-		     &pStatus))
+				   (void *)newValue),&pStatus)) {
+	       cdf_FreeMemory (newValue, NULL);
 	       return pStatus;
-	     if (newValue != NULL) cdf_FreeMemory (newValue, NULL);
+	     }
+	     cdf_FreeMemory (newValue, NULL);
            } else
-	     if (!sX(WriteBuffer64(CDF,fp,AEDR->DataType,
-				   AEDR->NumElems,value),&pStatus))
+	     if (!sX(WriteBuffer64(CDF,fp,AEDR->DataType,AEDR->NumElems,
+				   value),&pStatus))
 	       return pStatus;
 	 }
 	 break;
@@ -564,32 +580,35 @@ va_dcl
 			    AEDR_NULL),&pStatus)) return pStatus;
 	 tOffset = offset + AEDR_VALUE_OFFSET64;
 	 if (!SEEKv64(fp,tOffset,vSEEK_SET)) return CWE;
-         if (STRINGdataType(dataType)) {
-           size_t len; char *newValue = NULL;
-           len = strlen ((char *)value);
-           if ((int)len < (int)numElems) {
-             int ix;
-             newValue = (char *) cdf_AllocateMemory ((size_t) numElems, NULL);
-             memcpy (newValue, (char *)value, len);
-             for (ix = (int) len; ix < (int)numElems; ++ix)
-                *(newValue+ix) = (char) '\0';
-           }
-	   if (!sX(WriteBuffer64(CDF,fp,dataType,numElems,
-				 (newValue==NULL?value:(void *)newValue)),
-		   &pStatus))
-	     return pStatus;
-	   if (newValue != NULL) cdf_FreeMemory (newValue, NULL);
+	 if (STRINGdataType(dataType)) {
+	   char *newValue; size_t len;
+	   newValue = (char *) cdf_AllocateMemory (sizeof(char) *
+	    					   numElems, NULL);
+	   memset ((void *)newValue, '\0', (size_t)numElems);
+	   len = strlen ((char *)value);
+           memcpy ((void *)newValue, (void *)value,
+                   (size_t)MINIMUM(numElems,len));
+	   if (!sX(WriteBuffer64(CDF,fp,dataType,numElems,(void *)newValue),
+		   &pStatus)) {
+	      cdf_FreeMemory (newValue, NULL);
+	      return pStatus;
+	   } else
+	     cdf_FreeMemory (newValue, NULL);
          } else
-	   if (!sX(WriteBuffer64(CDF,fp,dataType,
-				 numElems,value),&pStatus)) return pStatus;
+	     if (!sX(WriteBuffer64(CDF,fp,dataType,numElems,value),
+		     &pStatus)) return pStatus;
 	 break;
        }
        case AEDR_RECORDSIZE:
        case AEDR_AEDRNEXT: {
          OFF_T *buffer = va_arg (ap, OFF_T *); OFF_T tOffset = offset;
          switch (field) {
-           case AEDR_RECORDSIZE: tOffset += (OFF_T) AEDR_RECORDSIZE_OFFSET64; break;
-           case AEDR_AEDRNEXT: tOffset += (OFF_T) AEDR_AEDRNEXT_OFFSET64; break;
+           case AEDR_RECORDSIZE:
+	     tOffset += (OFF_T) AEDR_RECORDSIZE_OFFSET64;
+	     break;
+           case AEDR_AEDRNEXT:
+	     tOffset += (OFF_T) AEDR_AEDRNEXT_OFFSET64;
+	     break;
            default: return CDF_INTERNAL_ERROR;
          }
          if (!SEEKv64(fp,tOffset,vSEEK_SET)) return CWE;
@@ -599,14 +618,20 @@ va_dcl
        default: {
 	 Int32 *buffer = va_arg (ap, Int32 *); OFF_T tOffset = offset;
 	 switch (field) {
-/*	   case AEDR_RECORDSIZE: tOffset += (OFF_T) AEDR_RECORDSIZE_OFFSET64; break; */
-	   case AEDR_RECORDTYPE: tOffset += (OFF_T) AEDR_RECORDTYPE_OFFSET64; break;
-/*	   case AEDR_AEDRNEXT: tOffset += (OFF_T) AEDR_AEDRNEXT_OFFSET64; break; */
+	   case AEDR_RECORDTYPE:
+	     tOffset += (OFF_T) AEDR_RECORDTYPE_OFFSET64;
+	     break;
 	   case AEDR_ATTRNUM: tOffset += (OFF_T) AEDR_ATTRNUM_OFFSET64; break;
-	   case AEDR_DATATYPE: tOffset += (OFF_T) AEDR_DATATYPE_OFFSET64; break;
+	   case AEDR_DATATYPE:
+	     tOffset += (OFF_T) AEDR_DATATYPE_OFFSET64;
+	     break;
 	   case AEDR_NUM: tOffset += (OFF_T) AEDR_NUM_OFFSET64; break;
-	   case AEDR_NUMELEMS: tOffset += (OFF_T) AEDR_NUMELEMS_OFFSET64; break;
-	   case AEDR_NUMSTRINGS: tOffset += (OFF_T) AEDR_NUMSTRINGS_OFFSET64; break;
+	   case AEDR_NUMELEMS:
+	     tOffset += (OFF_T) AEDR_NUMELEMS_OFFSET64;
+	     break;
+	   case AEDR_NUMSTRINGS:
+	     tOffset += (OFF_T) AEDR_NUMSTRINGS_OFFSET64;
+	     break;
 	   default: return CDF_INTERNAL_ERROR;
 	 }
 	 if (!SEEKv64(fp,tOffset,vSEEK_SET)) return CWE;
@@ -703,24 +728,21 @@ va_dcl
 	 if (!Write32s_64(fp,VDR->DimVarys,(int)nDims)) return CWE;
 	 if (PADvalueBITset(VDR->Flags) && padValue != NULL) {
 	   if (STRINGdataType(VDR->DataType)) {
-	     size_t len; char *newPad = NULL;
-	     len = strlen ((char *)padValue);
-	     if ((int)len < (int)VDR->NumElems) {
-	       int ix;
-	       newPad = (char *) cdf_AllocateMemory ((size_t) VDR->NumElems,
-						     NULL);
-	       memcpy (newPad, (char *)padValue, len);
-	       for (ix = (int) len; ix < (int)VDR->NumElems; ++ix)
-	          *(newPad+ix) = (char) '\0';
-	     }
+	     char *newPad, *tmp; int len;
+	     newPad = (char *) cdf_AllocateMemory (sizeof(char) *
+	    					   VDR->NumElems, NULL);
+	     memset ((void *)newPad, '\0', (size_t)VDR->NumElems);
+	     len = (int) strlen((char *)padValue);
+	     memcpy ((void *)newPad, (void *)padValue,
+		     (size_t)MINIMUM(VDR->NumElems,len));
 	     if (!sX(WriteBuffer64(CDF,fp,VDR->DataType,VDR->NumElems,
-				   (newPad==NULL?padValue:(void *)newPad)),
-	             &pStatus))
+				   (void *)newPad), &pStatus)) {
+	       cdf_FreeMemory (newPad, NULL);
 	       return pStatus;
-	     if (newPad != NULL) cdf_FreeMemory (newPad, NULL);
+	     } else
+	       cdf_FreeMemory (newPad, NULL);
 	   } else
-	     if (!sX(WriteBuffer64(CDF,fp,VDR->DataType,
-				   VDR->NumElems,
+	     if (!sX(WriteBuffer64(CDF,fp,VDR->DataType,VDR->NumElems,
 				   padValue),&pStatus)) return pStatus;
 	 }
 	 break;
@@ -805,23 +827,22 @@ va_dcl
 	 }
 	 if (!SEEKv64(fp,tOffset,vSEEK_SET)) return CWE;
 	 if (STRINGdataType(dataType)) {
-	   size_t len; char *newPad = NULL;
-	   len = strlen ((char *)padValue);
-	   if ((int)len < (int)numElems) {
-             int ix;
-	     newPad = (char *) cdf_AllocateMemory ((size_t) numElems, NULL);
-	     memcpy (newPad, (char *)padValue, len);
-	     for (ix = (int) len; ix < (int)numElems; ++ix)
-		*(newPad+ix) = (char) '\0';
-	   }
-	   if (!sX(WriteBuffer64(CDF,fp,dataType,numElems,
-				 (newPad==NULL?padValue:(void *)newPad)),
-		   &pStatus))
+	   char *newPad; int len;
+	   newPad = (char *) cdf_AllocateMemory (sizeof(char) *
+						 numElems, NULL);
+	   memset ((void *)newPad, '\0', (size_t)numElems);
+	   len = (int) strlen((char *)padValue);
+	   memcpy ((void *)newPad, (void *)padValue,
+		   (size_t)MINIMUM(numElems,len));
+	   if (!sX(WriteBuffer64(CDF,fp,dataType,numElems,(void *)newPad),
+		   &pStatus)) {
+	     cdf_FreeMemory (newPad, NULL);
 	     return pStatus;
-	   if (newPad != NULL) cdf_FreeMemory (newPad, NULL);
+	   } else
+	     cdf_FreeMemory (newPad, NULL);
 	 } else
-	   if (!sX(WriteBuffer64(CDF,fp,dataType,
-				 numElems,padValue),&pStatus)) return pStatus;
+	   if (!sX(WriteBuffer64(CDF,fp,dataType,numElems,padValue),
+		   &pStatus)) return pStatus;
 	 break;
        }
        case VDR_RECORDSIZE:
@@ -831,7 +852,9 @@ va_dcl
        case VDR_CPRorSPR: {
          OFF_T *buffer = va_arg (ap, OFF_T *); OFF_T tOffset = offset;
          switch (field) {
-           case VDR_RECORDSIZE: tOffset += (OFF_T) VDR_RECORDSIZE_OFFSET64; break;
+           case VDR_RECORDSIZE:
+	     tOffset += (OFF_T) VDR_RECORDSIZE_OFFSET64;
+	     break;
            case VDR_VDRNEXT: tOffset += (OFF_T) VDR_VDRNEXT_OFFSET64; break;
            case VDR_VXRHEAD: tOffset += (OFF_T) VDR_VXRHEAD_OFFSET64; break;
            case VDR_VXRTAIL: tOffset += (OFF_T) VDR_VXRTAIL_OFFSET64; break;
@@ -845,18 +868,15 @@ va_dcl
        default: {
 	 Int32 *buffer = va_arg (ap, Int32 *); OFF_T tOffset = offset;
 	 switch (field) {
-/*	   case VDR_RECORDSIZE: tOffset += (OFF_T) VDR_RECORDSIZE_OFFSET64; break; */
-	   case VDR_RECORDTYPE: tOffset += (OFF_T) VDR_RECORDTYPE_OFFSET64; break;
-/*	   case VDR_VDRNEXT: tOffset += (OFF_T) VDR_VDRNEXT_OFFSET64; break; */
+	   case VDR_RECORDTYPE:
+	     tOffset += (OFF_T) VDR_RECORDTYPE_OFFSET64;
+	     break;
 	   case VDR_DATATYPE: tOffset += (OFF_T) VDR_DATATYPE_OFFSET64; break;
 	   case VDR_MAXREC: tOffset += (OFF_T) VDR_MAXREC_OFFSET64; break;
-/*	   case VDR_VXRHEAD: tOffset += (OFF_T) VDR_VXRHEAD_OFFSET64; break; */
-/*	   case VDR_VXRTAIL: tOffset += (OFF_T) VDR_VXRTAIL_OFFSET64; break; */
 	   case VDR_FLAGS: tOffset += (OFF_T) VDR_FLAGS_OFFSET64; break;
 	   case VDR_sRECORDS: tOffset += (OFF_T) VDR_sRECORDS_OFFSET64; break;
 	   case VDR_NUMELEMS: tOffset += (OFF_T) VDR_NUMELEMS_OFFSET64; break;
 	   case VDR_NUM: tOffset += (OFF_T) VDR_NUM_OFFSET64; break;
-/*	   case VDR_CPRorSPR: tOffset += (OFF_T) VDR_CPRorSPR_OFFSET64; break; */
 	   case VDR_BLOCKING: tOffset += (OFF_T) VDR_BLOCKING_OFFSET64; break;
 	   default: return CDF_INTERNAL_ERROR;
 	 }
@@ -942,7 +962,9 @@ va_dcl
        case VXR_VXRNEXT: {
          OFF_T *buffer = va_arg (ap, OFF_T *); OFF_T tOffset = offset;
          switch (field) {
-           case VXR_RECORDSIZE: tOffset += (OFF_T) VXR_RECORDSIZE_OFFSET64; break; 
+           case VXR_RECORDSIZE:
+	     tOffset += (OFF_T) VXR_RECORDSIZE_OFFSET64;
+	     break; 
            case VXR_VXRNEXT: tOffset += (OFF_T) VXR_VXRNEXT_OFFSET64; break; 
            default: return CDF_INTERNAL_ERROR;
          }
@@ -953,11 +975,13 @@ va_dcl
        default: {
 	 Int32 *buffer = va_arg (ap, Int32 *); OFF_T tOffset = offset;
 	 switch (field) {
-/*	   case VXR_RECORDSIZE: tOffset += (OFF_T) VXR_RECORDSIZE_OFFSET64; break; */
-	   case VXR_RECORDTYPE: tOffset += (OFF_T) VXR_RECORDTYPE_OFFSET64; break;
-/*	   case VXR_VXRNEXT: tOffset += (OFF_T) VXR_VXRNEXT_OFFSET64; break; */
+	   case VXR_RECORDTYPE:
+	     tOffset += (OFF_T) VXR_RECORDTYPE_OFFSET64;
+	     break;
 	   case VXR_NENTRIES: tOffset += (OFF_T) VXR_NENTRIES_OFFSET64; break;
-	   case VXR_NUSEDENTRIES: tOffset += (OFF_T) VXR_NUSEDENTRIES_OFFSET64; break;
+	   case VXR_NUSEDENTRIES:
+	     tOffset += (OFF_T) VXR_NUSEDENTRIES_OFFSET64;
+	     break;
 	   default: return CDF_INTERNAL_ERROR;
 	 }
 	 if (!SEEKv64(fp,tOffset,vSEEK_SET)) return CWE;
@@ -1011,8 +1035,9 @@ va_dcl
        default: {
 	 Int32 *buffer = va_arg (ap, Int32 *); OFF_T tOffset = offset;
 	 switch (field) {
-/*	   case VVR_RECORDSIZE: tOffset += (OFF_T) VVR_RECORDSIZE_OFFSET64; break; */
-	   case VVR_RECORDTYPE: tOffset += (OFF_T) VVR_RECORDTYPE_OFFSET64; break;
+	   case VVR_RECORDTYPE:
+	     tOffset += (OFF_T) VVR_RECORDTYPE_OFFSET64;
+	     break;
 	   default: return CDF_INTERNAL_ERROR;
 	 }
 	 if (!SEEKv64(fp,tOffset,vSEEK_SET)) return CWE;
@@ -1071,8 +1096,9 @@ va_dcl
        default: {
 	 OFF_T *buffer = va_arg (ap, OFF_T *); OFF_T tOffset = offset;
 	 switch (field) {
-	   case UIR_RECORDSIZE: tOffset += (OFF_T) UIR_RECORDSIZE_OFFSET64; break;
-/*	   case UIR_RECORDTYPE: tOffset += (OFF_T) UIR_RECORDTYPE_OFFSET64; break; */
+	   case UIR_RECORDSIZE:
+	     tOffset += (OFF_T) UIR_RECORDSIZE_OFFSET64;
+	     break;
 	   case UIR_NEXTUIR: tOffset += (OFF_T) UIR_NEXTUIR_OFFSET64; break;
 	   case UIR_PREVUIR: tOffset += (OFF_T) UIR_PREVUIR_OFFSET64; break;
 	   default: return CDF_INTERNAL_ERROR;
@@ -1128,7 +1154,9 @@ va_dcl
        case CCR_RFUa: {
          Int32 *buffer = va_arg (ap, Int32 *); OFF_T tOffset = offset;
          switch (field) {
-           case CCR_RECORDTYPE: tOffset += (OFF_T) CCR_RECORDTYPE_OFFSET64; break;
+           case CCR_RECORDTYPE:
+	     tOffset += (OFF_T) CCR_RECORDTYPE_OFFSET64;
+	     break;
            case CCR_RFUa: tOffset += (OFF_T) CCR_RFUa_OFFSET64; break;
            default: return CDF_INTERNAL_ERROR;
          }
@@ -1139,11 +1167,13 @@ va_dcl
        default: {
 	 OFF_T *buffer = va_arg (ap, OFF_T *); OFF_T tOffset = offset;
 	 switch (field) {
-	   case CCR_RECORDSIZE: tOffset += (OFF_T) CCR_RECORDSIZE_OFFSET64; break;
-/*	   case CCR_RECORDTYPE: tOffset += (OFF_T) CCR_RECORDTYPE_OFFSET64; break; */
-	   case CCR_CPROFFSET: tOffset += (OFF_T) CCR_CPROFFSET_OFFSET64; break;
+	   case CCR_RECORDSIZE:
+	     tOffset += (OFF_T) CCR_RECORDSIZE_OFFSET64;
+	     break;
+	   case CCR_CPROFFSET:
+	     tOffset += (OFF_T) CCR_CPROFFSET_OFFSET64;
+	     break;
 	   case CCR_USIZE: tOffset += (OFF_T) CCR_USIZE_OFFSET64; break;
-/*	   case CCR_RFUa: tOffset += (OFF_T) CCR_RFUa_OFFSET64; break; */
 	   default: return CDF_INTERNAL_ERROR;
 	 }
 	 if (!SEEKv64(fp,tOffset,vSEEK_SET)) return CWE;
@@ -1208,8 +1238,9 @@ va_dcl
        default: {
 	 Int32 *buffer = va_arg (ap, Int32 *); OFF_T tOffset = offset;
 	 switch (field) {
-/*	   case CPR_RECORDSIZE: tOffset += (OFF_T) CPR_RECORDSIZE_OFFSET64; break; */
-	   case CPR_RECORDTYPE: tOffset += (OFF_T) CPR_RECORDTYPE_OFFSET64; break;
+	   case CPR_RECORDTYPE:
+	     tOffset += (OFF_T) CPR_RECORDTYPE_OFFSET64;
+	     break;
 	   case CPR_CTYPE: tOffset += (OFF_T) CPR_CTYPE_OFFSET64; break;
 	   case CPR_RFUa: tOffset += (OFF_T) CPR_RFUa_OFFSET64; break;
 	   case CPR_PCOUNT: tOffset += (OFF_T) CPR_PCOUNT_OFFSET64; break;
@@ -1282,8 +1313,9 @@ va_dcl
        default: {
 	 Int32 *buffer = va_arg (ap, Int32 *); OFF_T tOffset = offset;
 	 switch (field) {
-/*	   case SPR_RECORDSIZE: tOffset += (OFF_T) SPR_RECORDSIZE_OFFSET64; break; */
-	   case SPR_RECORDTYPE: tOffset += (OFF_T) SPR_RECORDTYPE_OFFSET64; break;
+	   case SPR_RECORDTYPE:
+	     tOffset += (OFF_T) SPR_RECORDTYPE_OFFSET64;
+	     break;
 	   case SPR_STYPE: tOffset += (OFF_T) SPR_STYPE_OFFSET64; break;
 	   case SPR_RFUa: tOffset += (OFF_T) SPR_RFUa_OFFSET64; break;
 	   case SPR_PCOUNT: tOffset += (OFF_T) SPR_PCOUNT_OFFSET64; break;
@@ -1345,7 +1377,9 @@ va_dcl
        case CVVR_CSIZE: {
          OFF_T *buffer = va_arg (ap, OFF_T *); OFF_T tOffset = offset;
          switch (field) {
-           case CVVR_RECORDSIZE: tOffset += (OFF_T) CVVR_RECORDSIZE_OFFSET64; break;
+           case CVVR_RECORDSIZE:
+	     tOffset += (OFF_T) CVVR_RECORDSIZE_OFFSET64;
+	     break;
            case CVVR_CSIZE: tOffset += (OFF_T) CVVR_CSIZE_OFFSET64; break;
            default: return CDF_INTERNAL_ERROR;
          }
@@ -1356,10 +1390,10 @@ va_dcl
        default: {
 	 Int32 *buffer = va_arg (ap, Int32 *); OFF_T tOffset = offset;
 	 switch (field) {
-/*	   case CVVR_RECORDSIZE: tOffset += (OFF_T) CVVR_RECORDSIZE_OFFSET64; break; */
-	   case CVVR_RECORDTYPE: tOffset += (OFF_T) CVVR_RECORDTYPE_OFFSET64; break;
+	   case CVVR_RECORDTYPE:
+	     tOffset += (OFF_T) CVVR_RECORDTYPE_OFFSET64;
+	     break;
 	   case CVVR_RFUa: tOffset += (OFF_T) CVVR_RFUa_OFFSET64; break;
-/*	   case CVVR_CSIZE: tOffset += (OFF_T) CVVR_CSIZE_OFFSET64; break; */
 	   default: return CDF_INTERNAL_ERROR;
 	 }
 	 if (!SEEKv64(fp,tOffset,vSEEK_SET)) return CWE;
