@@ -3,7 +3,7 @@
 * Administrator of the National Aeronautics and Space Administration.
 * All Rights Reserved.
 ******************************************************************************/
-/****************************************************************************** 
+/******************************************************************************
 *
 *  NSSDC/CDF                       EPOCH utility routines for C applications.
 *
@@ -189,6 +189,7 @@ char *inString;
      epoch = ILLEGAL_EPOCH_VALUE;
   }
   free (tmpPtr);
+  tmpPtr = NULL;
   return epoch;
 }
 
@@ -324,6 +325,7 @@ double epoch16[2];
       *(tmpx+(len-1)) = '\0';
       dummy = parseEPOCH16_4 (tmpx, epoch16);
       free (tmpx);
+      tmpx = NULL;
     } else {
       if (len == (EPOCH16_3_STRING_LEN - 1))
         dummy = parseEPOCH16_3 (tmpPtr, epoch16);
@@ -347,6 +349,7 @@ double epoch16[2];
     dummy = ILLEGAL_EPOCH_VALUE;
   }
   free (tmpPtr);
+  tmpPtr = NULL;
   return dummy;
 }
 
@@ -406,14 +409,15 @@ double epoch[2];
   if (nsec < 0 || nsec > 999) return ILLEGAL_EPOCH_VALUE;
   if (psec < 0 || psec > 999) return ILLEGAL_EPOCH_VALUE;
   if (year == 9999 && month == 12 && day == 31 && hour == 23 && minute == 59 &&
-      second == 59 && msec == 999 && usec == 999 && nsec == 999 && psec == 999) {
+      second == 59 && msec == 999 && usec == 999 && nsec == 999 &&
+      psec == 999) {
     epoch[0] = -1.0E31;
     epoch[1] = -1.0E31;
     return 0.0;
   }
   epoch[0] = mmm / (double) 1000.0;
-  epoch[1] = msec * pow(10.0, 9.0) + usec * pow(10.0, 6.0) + nsec * pow(10.0, 3.0) + 
-	     psec;
+  epoch[1] = msec * pow(10.0, 9.0) + usec * pow(10.0, 6.0) + 
+             nsec * pow(10.0, 3.0) + psec;
   return (double) 0.0;
 }
 
@@ -668,7 +672,8 @@ double epoch[2];
     return ILLEGAL_EPOCH_VALUE;
   }
   if (year == 9999 && month == 12 && day == 31 && hour == 23 && minute == 59 &&
-      second == 59 && msec == 999 && usec == 999 && nsec == 999 && psec == 999) {
+      second == 59 && msec == 999 && usec == 999 && nsec == 999 &&
+      psec == 999) {
     epoch[0] = -1.0E31;
     epoch[1] = -1.0E31;
     return (double) 0.0;
@@ -755,7 +760,8 @@ double epoch[2];
     return ILLEGAL_EPOCH_VALUE;
   }
   if (year == 9999 && month == 12 && day == 31 && hour == 23 && minute == 59 &&
-      second == 59 && msec == 999 && usec == 999 && nsec == 999 && psec == 999) {
+      second == 59 && msec == 999 && usec == 999 && nsec == 999 &&
+      psec == 999) {
     epoch[0] = -1.0E31;
     epoch[1] = -1.0E31;
     return (double) 0.0;
@@ -1060,7 +1066,7 @@ char epString[EPOCH1_STRING_LEN+1];
   return;
 }
 
-/******************************************************************************   
+/******************************************************************************
 * encodeEPOCH16_1.
 * This function is an extension of encodeEPOCH1. It is used to handle the time
 * that may contain as small as picoseconds. 
@@ -1130,7 +1136,7 @@ char epString[EPOCH2_STRING_LEN+1];
   return;
 }
 
-/******************************************************************************   
+/******************************************************************************
 * encodeEPOCH16_2.
 * This function is an extension of encodeEPOCH2. It is used to handle the time
 * that may contain as small as picoseconds. 
@@ -1193,7 +1199,7 @@ char epString[EPOCH3_STRING_LEN+1];
   return;
 }
 
-/****************************************************************************** 
+/******************************************************************************
 * encodeEPOCH16_3.
 * This function is an extension of encodeEPOCH3. It is used to handle the time
 * that may contain as small as picoseconds. 
@@ -1903,7 +1909,8 @@ long year, month, day, hour, minute, second, msec;
     msecInDay = msec;
   }
   else {
-    msecInDay = (3600000L * hour) + (60000L * minute) + (1000L * second) + msec;
+    msecInDay = (3600000L * hour) + (60000L * minute) + (1000L * second) +
+                msec;
   }
   /****************************************************************************
   * Return the milliseconds since 0 A.D.
