@@ -1,4 +1,9 @@
 /******************************************************************************
+* Copyright 1996-2014 United States Government as represented by the
+* Administrator of the National Aeronautics and Space Administration.
+* All Rights Reserved.
+******************************************************************************/
+/******************************************************************************
 *
 *  NSSDC/CDF                 EPOCH utility routines for Fortran applications.
 *
@@ -11,6 +16,8 @@
 *   V1.2   9-Sep-96, J Love	CDF V2.6.
 *   V2.6  29-Jan-04, M Liu	Added a new set of CDF_EPOCH16 functions for 
 *                               handling fraction of a second up to picosecond.
+*   V3.7  29-Sep-18, M Liu	Added toEncode_ and toParse_ for CDF_EPOCH and
+*                               CDF_EPOCH16 data types.
 *
 ******************************************************************************/
 
@@ -26,7 +33,7 @@ void
 Fif_PREFIXb
 Fif_ENTRYPOINT(parse_epoch__,parse_epoch_,parse_epoch,PARSE_EPOCH)
 (string, epoch Fif_GHOSTARG(string_len))
-void *string;
+void   *string;
 double *epoch;
 Fif_GHOSTDEF(string_len)
 {
@@ -44,6 +51,34 @@ Fif_GHOSTDEF(string_len)
   return;
 }
 
+/******************************************************************************
+* toParse_EPOCH (FORTRAN equivalent of toParseEPOCH).
+******************************************************************************/
+
+VISIBLE_PREFIX
+Fif_PREFIXa
+void
+Fif_PREFIXb
+Fif_ENTRYPOINT(toparse_epoch__,toparse_epoch_,toparse_epoch,TOPARSE_EPOCH)
+(string, epoch Fif_GHOSTARG(string_len))
+void   *string;
+double *epoch;
+Fif_GHOSTDEF(string_len)
+{
+  struct STRINGstruct *ssh = NULL;
+#if defined(Fif_DESCR)
+  *epoch = toParseEPOCH(DESCRtoREFnul(string,EPOCH_STRING_LEN,&ssh));
+#endif
+#if defined(Fif_GHOSTLEN)
+  *epoch = toParseEPOCH(NULterminate(string,Fif_GHOSTUSE(string_len),&ssh));
+#endif
+#if defined(Fif_NOLEN)
+  *epoch = toParseEPOCH(FindEndNUL(string,EPOCH_STRING_LEN,&ssh));
+#endif
+  FreeStrings (ssh);
+  return;
+}
+
 /******************************************************************************
 * parse_EPOCH1 (FORTRAN equivalent of parseEPOCH1).
 ******************************************************************************/
@@ -54,7 +89,7 @@ void
 Fif_PREFIXb
 Fif_ENTRYPOINT(parse_epoch1__,parse_epoch1_,parse_epoch1,PARSE_EPOCH1)
 (string, epoch Fif_GHOSTARG(string_len))
-void *string;
+void   *string;
 double *epoch;
 Fif_GHOSTDEF(string_len)
 {
@@ -82,7 +117,7 @@ void
 Fif_PREFIXb
 Fif_ENTRYPOINT(parse_epoch2__,parse_epoch2_,parse_epoch2,PARSE_EPOCH2)
 (string, epoch Fif_GHOSTARG(string_len))
-void *string;
+void   *string;
 double *epoch;
 Fif_GHOSTDEF(string_len)
 {
@@ -110,7 +145,7 @@ void
 Fif_PREFIXb
 Fif_ENTRYPOINT(parse_epoch3__,parse_epoch3_,parse_epoch3,PARSE_EPOCH3)
 (string, epoch Fif_GHOSTARG(string_len))
-void *string;
+void   *string;
 double *epoch;
 Fif_GHOSTDEF(string_len)
 {
@@ -138,7 +173,7 @@ void
 Fif_PREFIXb
 Fif_ENTRYPOINT(parse_epoch4__,parse_epoch4_,parse_epoch4,PARSE_EPOCH4)
 (string, epoch Fif_GHOSTARG(string_len))
-void *string;
+void   *string;
 double *epoch;
 Fif_GHOSTDEF(string_len)
 {
@@ -167,7 +202,7 @@ Fif_PREFIXb
 Fif_ENTRYPOINT(encode_epoch__,encode_epoch_,encode_epoch,ENCODE_EPOCH)
 (epoch, string Fif_GHOSTARG(string_len))
 double *epoch;
-void *string;
+void   *string;
 Fif_GHOSTDEF(string_len)
 {
   char tString[EPOCH_STRING_LEN+1];
@@ -180,6 +215,41 @@ Fif_GHOSTDEF(string_len)
   return;
 }
 
+/******************************************************************************
+* toEncode_EPOCH (FORTRAN equivalent of toEncodeEPOCH).
+******************************************************************************/
+
+VISIBLE_PREFIX
+Fif_PREFIXa
+void
+Fif_PREFIXb
+Fif_ENTRYPOINT(toencode_epoch__,toencode_epoch_,toencode_epoch,TOENCODE_EPOCH)
+(epoch, style, string Fif_GHOSTARG(string_len))
+double *epoch;
+int    *style;
+void   *string;
+Fif_GHOSTDEF(string_len)
+{
+  char tString[EPOCH_STRING_LEN+1];
+  int  style2 = *style;
+  if (style2 < 0 || style2 > 4) style2 = 4;
+  if (style2 == 0) encodeEPOCH (*epoch, tString);
+  else if (style2 == 1) encodeEPOCH1 (*epoch, tString);
+  else if (style2 == 2) encodeEPOCH2 (*epoch, tString);
+  else if (style2 == 3) encodeEPOCH3 (*epoch, tString);
+  else if (style2 == 4) encodeEPOCH4 (*epoch, tString);
+#if defined(Fif_GHOSTLEN)
+  CtoFORTstring (tString, string, Fif_GHOSTUSE(string_len));
+#else
+  if (style2 == 0) CtoFORTstring (tString, string, EPOCH_STRING_LEN);
+  else if (style2 == 1) CtoFORTstring (tString, string, EPOCH1_STRING_LEN);
+  else if (style2 == 2) CtoFORTstring (tString, string, EPOCH2_STRING_LEN);
+  else if (style2 == 3) CtoFORTstring (tString, string, EPOCH3_STRING_LEN);
+  else if (style2 == 4) CtoFORTstring (tString, string, EPOCH4_STRING_LEN);
+#endif
+  return;
+}
+
 /******************************************************************************
 * encode_EPOCH1 (FORTRAN equivalent of encodeEPOCH1).
 ******************************************************************************/
@@ -191,7 +261,7 @@ Fif_PREFIXb
 Fif_ENTRYPOINT(encode_epoch1__,encode_epoch1_,encode_epoch1,ENCODE_EPOCH1)
 (epoch, string Fif_GHOSTARG(string_len))
 double *epoch;
-void *string;
+void   *string;
 Fif_GHOSTDEF(string_len)
 {
   char tString[EPOCH1_STRING_LEN+1];
@@ -215,7 +285,7 @@ Fif_PREFIXb
 Fif_ENTRYPOINT(encode_epoch2__,encode_epoch2_,encode_epoch2,ENCODE_EPOCH2)
 (epoch, string Fif_GHOSTARG(string_len))
 double *epoch;
-void *string;
+void   *string;
 Fif_GHOSTDEF(string_len)
 {
   char tString[EPOCH2_STRING_LEN+1];
@@ -239,7 +309,7 @@ Fif_PREFIXb
 Fif_ENTRYPOINT(encode_epoch3__,encode_epoch3_,encode_epoch3,ENCODE_EPOCH3)
 (epoch, string Fif_GHOSTARG(string_len))
 double *epoch;
-void *string;
+void   *string;
 Fif_GHOSTDEF(string_len)
 {
   char tString[EPOCH3_STRING_LEN+1];
@@ -263,7 +333,7 @@ Fif_PREFIXb
 Fif_ENTRYPOINT(encode_epoch4__,encode_epoch4_,encode_epoch4,ENCODE_EPOCH4)
 (epoch, string Fif_GHOSTARG(string_len))
 double *epoch;
-void *string;
+void   *string;
 Fif_GHOSTDEF(string_len)
 {
   char tString[EPOCH4_STRING_LEN+1];
@@ -287,8 +357,8 @@ Fif_PREFIXb
 Fif_ENTRYPOINT(encode_epochx__,encode_epochx_,encode_epochx,ENCODE_EPOCHX)
 (epoch, format, string Fif_GHOSTARG(format_len) Fif_GHOSTARG(string_len))
 double *epoch;
-void *format;
-void *string;
+void   *format;
+void   *string;
 Fif_GHOSTDEF(format_len)
 Fif_GHOSTDEF(string_len)
 {
@@ -326,13 +396,13 @@ Fif_ENTRYPOINT(epoch_breakdown__,
 	       EPOCH_BREAKDOWN)
 (epoch, year, month, day, hour, minute, second, msec)
 double *epoch;
-Int32 *year;
-Int32 *month;
-Int32 *day;
-Int32 *hour;
-Int32 *minute;
-Int32 *second;
-Int32 *msec;
+Int32  *year;
+Int32  *month;
+Int32  *day;
+Int32  *hour;
+Int32  *minute;
+Int32  *second;
+Int32  *msec;
 {
   double tEpoch = *epoch;
   long tYear, tMonth, tDay, tHour, tMinute, tSecond, tMsec;
@@ -358,13 +428,13 @@ void
 Fif_PREFIXb
 Fif_ENTRYPOINT(compute_epoch__,compute_epoch_,compute_epoch,COMPUTE_EPOCH)
 (year, month, day, hour, minute, second, msec, epoch)
-Int32 *year;
-Int32 *month;
-Int32 *day;
-Int32 *hour;
-Int32 *minute;
-Int32 *second;
-Int32 *msec;
+Int32  *year;
+Int32  *month;
+Int32  *day;
+Int32  *hour;
+Int32  *minute;
+Int32  *second;
+Int32  *msec;
 double *epoch;
 {
   *epoch = computeEPOCH ((long) *year, (long) *month, (long) *day,
@@ -384,7 +454,7 @@ Fif_PREFIXb
 Fif_ENTRYPOINT(parse_epoch16__,parse_epoch16_,parse_epoch16,
                PARSE_EPOCH16)
 (string, epoch Fif_GHOSTARG(string_len))
-void *string;
+void   *string;
 double *epoch;
 Fif_GHOSTDEF(string_len)
 {
@@ -406,6 +476,37 @@ Fif_GHOSTDEF(string_len)
   return;
 }
 
+/******************************************************************************
+* toParse_EPOCH16 (FORTRAN equivalent of toParseEPOCH16).
+******************************************************************************/
+
+VISIBLE_PREFIX
+Fif_PREFIXa
+void
+Fif_PREFIXb
+Fif_ENTRYPOINT(toparse_epoch16__,toparse_epoch16_,toparse_epoch16,
+               TOPARSE_EPOCH16)
+(string, epoch Fif_GHOSTARG(string_len))
+void   *string;
+double *epoch;
+Fif_GHOSTDEF(string_len)
+{
+  struct STRINGstruct *ssh = NULL;
+  double mmm;
+#if defined(Fif_DESCR)
+  mmm = toParseEPOCH16(DESCRtoREFnul(string,EPOCH16_STRING_LEN,&ssh), epoch);
+#endif
+#if defined(Fif_GHOSTLEN)
+  mmm = toParseEPOCH16(NULterminate(string,Fif_GHOSTUSE(string_len),&ssh), 
+		       epoch);
+#endif
+#if defined(Fif_NOLEN)
+  mmm = toParseEPOCH16(FindEndNUL(string,EPOCH16_STRING_LEN,&ssh), epoch);
+#endif
+  FreeStrings (ssh);
+  return;
+}
+
 /******************************************************************************
 * parse_EPOCH16_1 (FORTRAN equivalent of parseEPOCH16_1).
 ******************************************************************************/
@@ -417,23 +518,21 @@ Fif_PREFIXb
 Fif_ENTRYPOINT(parse_epoch16_1__,parse_epoch16_1_,parse_epoch16_1,
 	       PARSE_EPOCH16_1)
 (string, epoch Fif_GHOSTARG(string_len))
-void *string;
+void   *string;
 double *epoch;
 Fif_GHOSTDEF(string_len)
 {
   struct STRINGstruct *ssh = NULL;
   double mmm;
 #if defined(Fif_DESCR)
-  mmm = parseEPOCH16_1(DESCRtoREFnul(string,EPOCH16_1_STRING_LEN,&ssh), 
-		          epoch);
+  mmm = parseEPOCH16_1(DESCRtoREFnul(string,EPOCH16_1_STRING_LEN,&ssh), epoch);
 #endif
 #if defined(Fif_GHOSTLEN)
   mmm = parseEPOCH16_1(NULterminate(string,Fif_GHOSTUSE(string_len),&ssh), 
-		          epoch);
+		       epoch);
 #endif
 #if defined(Fif_NOLEN)
-  mmm = parseEPOCH16_1(FindEndNUL(string,EPOCH16_1_STRING_LEN,&ssh), 
-		          epoch);
+  mmm = parseEPOCH16_1(FindEndNUL(string,EPOCH16_1_STRING_LEN,&ssh), epoch);
 #endif
   FreeStrings (ssh);
   return;
@@ -450,23 +549,21 @@ Fif_PREFIXb
 Fif_ENTRYPOINT(parse_epoch16_2__,parse_epoch16_2_,parse_epoch16_2,
 	       PARSE_EPOCH16_2)
 (string, epoch Fif_GHOSTARG(string_len))
-void *string;
+void   *string;
 double *epoch;
 Fif_GHOSTDEF(string_len)
 {
   struct STRINGstruct *ssh = NULL;
   double mmm;
 #if defined(Fif_DESCR)
-  mmm = parseEPOCH16_2(DESCRtoREFnul(string,EPOCH16_2_STRING_LEN,&ssh), 
-		          epoch);
+  mmm = parseEPOCH16_2(DESCRtoREFnul(string,EPOCH16_2_STRING_LEN,&ssh), epoch);
 #endif
 #if defined(Fif_GHOSTLEN)
   mmm = parseEPOCH16_2(NULterminate(string,Fif_GHOSTUSE(string_len),&ssh), 
-		          epoch);
+		       epoch);
 #endif
 #if defined(Fif_NOLEN)
-  mmm = parseEPOCH16_2(FindEndNUL(string,EPOCH16_2_STRING_LEN,&ssh), 
-		          epoch);
+  mmm = parseEPOCH16_2(FindEndNUL(string,EPOCH16_2_STRING_LEN,&ssh), epoch);
 #endif
   FreeStrings (ssh);
   return;
@@ -483,23 +580,21 @@ Fif_PREFIXb
 Fif_ENTRYPOINT(parse_epoch16_3__,parse_epoch16_3_,parse_epoch16_3,
 	       PARSE_EPOCH16_3)
 (string, epoch Fif_GHOSTARG(string_len))
-void *string;
+void   *string;
 double *epoch;
 Fif_GHOSTDEF(string_len)
 {
   struct STRINGstruct *ssh = NULL;
   double mmm;
 #if defined(Fif_DESCR)
-  mmm = parseEPOCH16_3(DESCRtoREFnul(string,EPOCH16_3_STRING_LEN,&ssh), 
-		          epoch);
+  mmm = parseEPOCH16_3(DESCRtoREFnul(string,EPOCH16_3_STRING_LEN,&ssh), epoch);
 #endif
 #if defined(Fif_GHOSTLEN)
   mmm = parseEPOCH16_3(NULterminate(string,Fif_GHOSTUSE(string_len),&ssh), 
-		          epoch);
+		       epoch);
 #endif
 #if defined(Fif_NOLEN)
-  mmm = parseEPOCH16_3(FindEndNUL(string,EPOCH16_3_STRING_LEN,&ssh), 
-		          epoch);
+  mmm = parseEPOCH16_3(FindEndNUL(string,EPOCH16_3_STRING_LEN,&ssh), epoch);
 #endif
   FreeStrings (ssh);
   return;
@@ -516,23 +611,21 @@ Fif_PREFIXb
 Fif_ENTRYPOINT(parse_epoch16_4__,parse_epoch16_4_,parse_epoch16_4,
 	       PARSE_EPOCH16_4)
 (string, epoch Fif_GHOSTARG(string_len))
-void *string;
+void   *string;
 double *epoch;
 Fif_GHOSTDEF(string_len)
 {
   struct STRINGstruct *ssh = NULL;
   double mmm;
 #if defined(Fif_DESCR)
-  mmm = parseEPOCH16_4(DESCRtoREFnul(string,EPOCH16_4_STRING_LEN,&ssh), 
-		          epoch);
+  mmm = parseEPOCH16_4(DESCRtoREFnul(string,EPOCH16_4_STRING_LEN,&ssh), epoch);
 #endif
 #if defined(Fif_GHOSTLEN)
   mmm = parseEPOCH16_4(NULterminate(string,Fif_GHOSTUSE(string_len),&ssh), 
-		          epoch);
+		       epoch);
 #endif
 #if defined(Fif_NOLEN)
-  mmm = parseEPOCH16_4(FindEndNUL(string,EPOCH16_4_STRING_LEN,&ssh), 
-		          epoch);
+  mmm = parseEPOCH16_4(FindEndNUL(string,EPOCH16_4_STRING_LEN,&ssh), epoch);
 #endif
   FreeStrings (ssh);
   return;
@@ -550,7 +643,7 @@ Fif_ENTRYPOINT(encode_epoch16__,encode_epoch16_,encode_epoch16,
 	       ENCODE_EPOCH16)
 (epoch, string Fif_GHOSTARG(string_len))
 double *epoch;
-void *string;
+void   *string;
 Fif_GHOSTDEF(string_len)
 {
   char tString[EPOCH16_STRING_LEN+1];
@@ -563,6 +656,42 @@ Fif_GHOSTDEF(string_len)
   return;
 }
 
+/******************************************************************************
+* toencode_EPOCH16 (FORTRAN equivalent of toEncodeEPOCH16).
+******************************************************************************/
+
+VISIBLE_PREFIX
+Fif_PREFIXa
+void
+Fif_PREFIXb
+Fif_ENTRYPOINT(toencode_epoch16__,toencode_epoch16_,toencode_epoch16,
+	       toENCODE_EPOCH16)
+(epoch, style, string Fif_GHOSTARG(string_len))
+double *epoch;
+int    *style;
+void   *string;
+Fif_GHOSTDEF(string_len)
+{
+  char tString[EPOCH16_STRING_LEN+1];
+  int  style2 = *style;
+  if (style2 < 0 || style2 > 4) style2 = 4;
+  if (style2 == 0) encodeEPOCH16 (epoch, tString);
+  else if (style2 == 1) encodeEPOCH16_1 (epoch, tString);
+  else if (style2 == 2) encodeEPOCH16_2 (epoch, tString);
+  else if (style2 == 3) encodeEPOCH16_3 (epoch, tString);
+  else if (style2 == 4) encodeEPOCH16_4 (epoch, tString);
+#if defined(Fif_GHOSTLEN)
+  CtoFORTstring (tString, string, Fif_GHOSTUSE(string_len));
+#else
+  if (style2 == 0) CtoFORTstring (tString, string, EPOCH16_STRING_LEN);
+  if (style2 == 1) CtoFORTstring (tString, string, EPOCH16_1_STRING_LEN);
+  if (style2 == 2) CtoFORTstring (tString, string, EPOCH16_2_STRING_LEN);
+  if (style2 == 3) CtoFORTstring (tString, string, EPOCH16_3_STRING_LEN);
+  if (style2 == 4) CtoFORTstring (tString, string, EPOCH16_4_STRING_LEN);
+#endif
+  return;
+}
+
 /******************************************************************************
 * encode_EPOCH16_1 (FORTRAN equivalent of encodeEPOCH16_1).
 ******************************************************************************/
@@ -575,7 +704,7 @@ Fif_ENTRYPOINT(encode_epoch16_1__,encode_epoch16_1_,encode_epoch16_1,
 	       ENCODE_EPOCH16_1)
 (epoch, string Fif_GHOSTARG(string_len))
 double *epoch;
-void *string;
+void   *string;
 Fif_GHOSTDEF(string_len)
 {
   char tString[EPOCH16_1_STRING_LEN+1];
@@ -600,7 +729,7 @@ Fif_ENTRYPOINT(encode_epoch16_2__,encode_epoch16_2_,encode_epoch16_2,
 	       ENCODE_EPOCH16_2)
 (epoch, string Fif_GHOSTARG(string_len))
 double *epoch;
-void *string;
+void   *string;
 Fif_GHOSTDEF(string_len)
 {
   char tString[EPOCH16_2_STRING_LEN+1];
@@ -625,7 +754,7 @@ Fif_ENTRYPOINT(encode_epoch16_3__,encode_epoch16_3_,encode_epoch16_3,
 	       ENCODE_EPOCH16_3)
 (epoch, string Fif_GHOSTARG(string_len))
 double *epoch;
-void *string;
+void   *string;
 Fif_GHOSTDEF(string_len)
 {
   char tString[EPOCH16_3_STRING_LEN+1];
@@ -650,7 +779,7 @@ Fif_ENTRYPOINT(encode_epoch16_4__,encode_epoch16_4_,encode_epoch16_4,
 	       ENCODE_EPOCH16_4)
 (epoch, string Fif_GHOSTARG(string_len))
 double *epoch;
-void *string;
+void   *string;
 Fif_GHOSTDEF(string_len)
 {
   char tString[EPOCH16_4_STRING_LEN+1];
@@ -675,8 +804,8 @@ Fif_ENTRYPOINT(encode_epoch16_x__,encode_epoch16_x_,encode_epoch16_x,
 	       ENCODE_EPOCH16_X)
 (epoch, format, string Fif_GHOSTARG(format_len) Fif_GHOSTARG(string_len))
 double *epoch;
-void *format;
-void *string;
+void   *format;
+void   *string;
 Fif_GHOSTDEF(format_len)
 Fif_GHOSTDEF(string_len)
 {
@@ -716,19 +845,20 @@ Fif_ENTRYPOINT(epoch16_breakdown__,
 	       EPOCH16_BREAKDOWN)
 (epoch, year, month, day, hour, minute, second, msec, usec, nsec, psec)
 double *epoch;
-Int32 *year;
-Int32 *month;
-Int32 *day;
-Int32 *hour;
-Int32 *minute;
-Int32 *second;
-Int32 *msec;
-Int32 *usec;
-Int32 *nsec;
-Int32 *psec;
+Int32  *year;
+Int32  *month;
+Int32  *day;
+Int32  *hour;
+Int32  *minute;
+Int32  *second;
+Int32  *msec;
+Int32  *usec;
+Int32  *nsec;
+Int32  *psec;
 {
   double tEpoch[2];
-  long tYear, tMonth, tDay, tHour, tMinute, tSecond, tMsec, tUsec, tNsec, tPsec;
+  long tYear, tMonth, tDay, tHour, tMinute, tSecond, tMsec, tUsec, tNsec,
+       tPsec;
   tEpoch[0] = *(double *) epoch;
   tEpoch[1] = *((double *)epoch + 1);
   EPOCH16breakdown (tEpoch, &tYear, &tMonth, &tDay, &tHour, &tMinute, 
@@ -757,16 +887,16 @@ Fif_PREFIXb
 Fif_ENTRYPOINT(compute_epoch16__,compute_epoch16_,compute_epoch16,
 	       COMPUTE_EPOCH16)
 (year, month, day, hour, minute, second, msec, usec, nsec, psec, epoch)
-Int32 *year;
-Int32 *month;
-Int32 *day;
-Int32 *hour;
-Int32 *minute;
-Int32 *second;
-Int32 *msec;
-Int32 *usec;
-Int32 *nsec; 
-Int32 *psec;
+Int32  *year;
+Int32  *month;
+Int32  *day;
+Int32  *hour;
+Int32  *minute;
+Int32  *second;
+Int32  *msec;
+Int32  *usec;
+Int32  *nsec; 
+Int32  *psec;
 double *epoch;
 {
   double mmm;
@@ -776,3 +906,84 @@ double *epoch;
 			    (long) *psec, epoch);
   return;
 }
+
+/******************************************************************************
+* EPOCH_to_UnixTime (FORTRAN equivalent of EPOCHtoUnixTime).
+******************************************************************************/
+
+VISIBLE_PREFIX
+Fif_PREFIXa
+void
+Fif_PREFIXb
+Fif_ENTRYPOINT(epoch_to_unixtime__,
+               epoch_to_unixtime_,
+               epoch_to_unixtime,
+               EPOCH_TO_UNIXTIME)
+(epochs, unixTimes, num)
+double *epochs;
+double *unixTimes;
+int  *num;
+{
+  EPOCHtoUnixTime (epochs, unixTimes, *num);
+}
+
+/******************************************************************************
+* UnixTime_to_EPOCH (FORTRAN equivalent of UnixTimetoEPOCH).
+******************************************************************************/
+
+VISIBLE_PREFIX
+Fif_PREFIXa
+void
+Fif_PREFIXb
+Fif_ENTRYPOINT(unixtime_to_epoch__,
+               unixtime_to_epoch_,
+               unixtime_to_epoch,
+               UNIXTIME_TO_EPOCH)
+(unixTimes, epochs, num)
+double *unixTimes;
+double *epochs;
+int  *num;
+{
+  UnixTimetoEPOCH (unixTimes, epochs, *num);
+}
+
+/******************************************************************************
+* EPOCH16_to_UnixTime (FORTRAN equivalent of EPOCH16toUnixTime).
+******************************************************************************/
+
+VISIBLE_PREFIX
+Fif_PREFIXa
+void
+Fif_PREFIXb
+Fif_ENTRYPOINT(epoch16_to_unixtime__,
+               epoch16_to_unixtime_,
+               epoch16_to_unixtime,
+               EPOCH16_TO_UNIXTIME)
+(epochs, unixTimes, num)
+double *epochs;
+double *unixTimes;
+int  *num;
+{
+  EPOCH16toUnixTime (epochs, unixTimes, *num);
+}
+
+/******************************************************************************
+* UnixTime_to_EPOCH16 (FORTRAN equivalent of UnixTimetoEPOCH16).
+******************************************************************************/
+
+VISIBLE_PREFIX
+Fif_PREFIXa
+void
+Fif_PREFIXb
+Fif_ENTRYPOINT(unixtime_to_epoch16__,
+               unixtime_to_epoch16_,
+               unixtime_to_epoch16,
+               UNIXTIME_TO_EPOCH16)
+(unixTimes, epochs, num)
+double *unixTimes;
+double *epochs;
+int  *num;
+{
+  UnixTimetoEPOCH16 (unixTimes, epochs, *num);
+}
+
