@@ -75,7 +75,8 @@
 #endif
 
 #define LASTphyBLOCKn(vFp) \
-BOO(vFp->phyLength64 == 0,NO_BLOCK,(long) ((vFp->phyLength64-1)/nCACHE_BUFFER_BYTEs))
+BOO(vFp->phyLength64 == 0,NO_BLOCK, \
+                          (long) ((vFp->phyLength64-1)/nCACHE_BUFFER_BYTEs))
 
 #if defined(MICROSOFTC_700) && INCLUDEvMEMORY
 #define CACHEbufferREADfrom(cache) \
@@ -109,7 +110,7 @@ static Logical vRead PROTOARGs((
 static Logical vWrite PROTOARGs((
   OFF_T offset, void *buffer, size_t nBytes, vFILE *vFp
 ));
-static vCACHE *AllocateBuffer PROTOARGs((vFILE *vFp));
+static vCACHE *AllocateBuffer PROTOARGs((vFILE *vFp, Logical *new));
 static vCACHE *PageIn PROTOARGs((vFILE *vFp, long blockN));
 static Logical WriteBlockFromCache PROTOARGs((
   vFILE *vFp, vCACHE *cache, size_t Nbytes
@@ -158,34 +159,42 @@ char *a_mode;
 
 /******************************************************************************
 * FindCache. If it is found, then move it to the head.
+*          
 ******************************************************************************/
 
 static vCACHE *FindCache (vFp, blockN)
 vFILE *vFp;
 long blockN;
 {
-  vCACHE *cache = vFp->cacheHead;
-  while (cache != NULL) {
-    if (cache->blockN == blockN) {
-      if (cache != vFp->cacheHead) {
-        if (cache == vFp->cacheTail) {
-          cache->prev->next = NULL;
-          vFp->cacheTail = cache->prev;
+  if (vFp->scratch && vFp->fp == NULL && vFp->cacheNumInd != NULL) {
+    if (vFp->cacheNumInd[(int)blockN] == 1) {
+      return vFp->cacheLocPtr[(int)blockN];
+    } else 
+      return NULL;
+  } else {
+    vCACHE *cache = vFp->cacheHead;
+    while (cache != NULL) {
+      if (cache->blockN == blockN) {
+	if (cache != vFp->cacheHead) {
+	  if (cache == vFp->cacheTail) {
+	    cache->prev->next = NULL;
+	    vFp->cacheTail = cache->prev;
+	  }
+	  else {
+	    cache->next->prev = cache->prev;
+	    cache->prev->next = cache->next;
+	  }
+	  vFp->cacheHead->prev = cache;
+	  cache->next = vFp->cacheHead;
+	  vFp->cacheHead = cache;
+	  cache->prev = NULL;
         }
-        else {
-          cache->next->prev = cache->prev;
-          cache->prev->next = cache->next;
-        }
-        vFp->cacheHead->prev = cache;
-        cache->next = vFp->cacheHead;
-        vFp->cacheHead = cache;
-        cache->prev = NULL;
+        return cache;
       }
-      return cache;
+      cache = cache->next;
     }
-    cache = cache->next;
+    return NULL;
   }
-  return NULL;
 }
 
 /******************************************************************************
@@ -216,6 +225,8 @@ vCACHE *firstCache;     /* Pointer to the first cache structure to flush. */
 {
   vCACHE *cache; long nBytes; OFF_T tmpBytes;
   OFF_T tmp; 
+int ix;
+ix = 0;
   for (cache = firstCache; cache != NULL; cache = cache->next) {
      if (cache->modified) {
 #if defined(vms)
@@ -250,6 +261,7 @@ vCACHE *firstCache;     /* Pointer to the first cache structure to free. */
       FreeVMemory ((MemHandle) cache->ptr);
     else
 #endif
+if (cache->ptr != NULL)
       cdf_FreeMemory (cache->ptr, NULL);
     cdf_FreeMemory (cache, NULL);
     cache = nextCache;
@@ -261,11 +273,12 @@ vCACHE *firstCache;     /* Pointer to the first cache structure to free. */
 * AllocateBuffer.
 * Allocate a cache structure to use.  It may be necessary to page out a block
 * to the file.  Returns a pointer to the allocated cache structure (or NULL if
-* an error occurred).
+* an error occurred). New cache is placed at the head of the link-list.
 ******************************************************************************/
 
-static vCACHE *AllocateBuffer (vFp)
+static vCACHE *AllocateBuffer (vFp, new)
 vFILE *vFp;
+Logical *new;
 {
   vCACHE *cache; long nBytes; OFF_T tmpBytes;
 #if !defined(vms)
@@ -273,12 +286,14 @@ vFILE *vFp;
 #endif
   /****************************************************************************
   * Check if a new cache structure can be allocated.  If the allocation(s)
-  * fail, process as if the maximum number of cache buffers has already been
+  * fails, process as if the maximum number of cache buffers has already been
   * reached.
   ****************************************************************************/
-  if (vFp->nBuffers < vFp->maxBuffers) {
+  *new = FALSE;
+  if ((vFp->maxBuffers == -1) || vFp->nBuffers < vFp->maxBuffers) {
     cache = (vCACHE *) cdf_AllocateMemory ((size_t)sizeof(vCACHE), NULL);
     if (cache != NULL) {
+      *new = TRUE;
 #if defined(MICROSOFTC_700) && INCLUDEvMEMORY
       if (useVmem)
 	cache->ptr = (void *) AllocateVMemory (nCACHE_BUFFER_BYTEs);
@@ -442,8 +457,13 @@ long blockN;
 {
   long nBytes; vCACHE *cache; void *buffer;
   OFF_T offset, tmpBytes;
-  cache = AllocateBuffer (vFp);
+  Logical new;
+  cache = AllocateBuffer (vFp, &new);
   if (cache == NULL) return NULL;
+  if (new && vFp->scratch && vFp->fp == NULL) {
+    vFp->cacheNumInd[blockN] = 1;
+    vFp->cacheLocPtr[blockN] = cache;
+  }
   offset = ((OFF_T) blockN) * nCACHE_BUFFER_BYTEs;
   tmpBytes = vFp->phyLength64 - offset;
   if (tmpBytes > nCACHE_BUFFER_BYTEs)
@@ -543,7 +563,9 @@ vFILE *vFp;
         return TRUE;
 #else
      if (FSEEK64(vFp->fp,(OFF_T) offset,vSEEK_SET) == EOF) return FALSE;
-     if (FREAD64(buffer,nBytes,1,vFp->fp) == 1) return TRUE;
+     if (FREAD64(buffer,nBytes,1,vFp->fp) == 1) {
+       return TRUE;
+     }
 #endif
   }
   return FALSE;
@@ -600,16 +622,18 @@ vFILE *vFp;
     if (vFp->path != NULL && strlen(vFp->path) != 0) { 
       strcpyX (tmpPath, vFp->path, 0);
       AppendToDir (tmpPath, "");
-#     if defined(unix) || defined(vms)
+#     if defined(unix) || defined(vms) || defined(__CYGWIN__)
         strcatX (tmpPath, TmpFileTemplate, 0);
         fileid = mkstemp(tmpPath);
-        if (fileid != -1) fp = fdopen(fileid, "w+");
-        else {
+        if (fileid != -1) {
+          fp = fdopen(fileid, "w+");
+        } else {
           for (i = 1; i <= MAX_TMP; i++) {
             strcpyX (tmpPath, vFp->path, 0);
             AppendToDir (tmpPath, "");
             snprintf (EofS(tmpPath), (size_t) pathLength + 1 - strlen(tmpPath),
-                      "TMP%08ld.%s", (long)(rand()%100000000), vFp->scratchExt);
+                      "TMP%08ld.%s", (long)(rand()%100000000),
+                      vFp->scratchExt);
             if (!IsReg(tmpPath)) {
               fp = CDFOpenFile (tmpPath, WRITE_PLUS_a_mode);
               if (fp != NULL) break;
@@ -628,8 +652,10 @@ vFILE *vFp;
               int i1, i2;
               for (i = 1; i <= MAX_TMP; i++) {
                 fh = -1;
-                i1 = (int) (((double)rand() / (RAND_MAX + 1)) * 25) + (int) 'A';
-                i2 = (int) (((double)rand() / (RAND_MAX + 1)) * 25) + (int) 'A';
+                i1 = (int) (((double)rand() / (RAND_MAX + 1)) * 25) +
+                     (int) 'A';
+                i2 = (int) (((double)rand() / (RAND_MAX + 1)) * 25) +
+                     (int) 'A';
                 strcpyX (tmpPath, vFp->path, 0);
                 AppendToDir (tmpPath, "");
                 snprintf (EofS(tmpPath), (size_t) pathLength + 1 -
@@ -644,10 +670,11 @@ vFILE *vFp;
           }
 #       else
           for (i = 1; i <= MAX_TMP; i++) {
-            strcpy (tmpPath, vFp->path);
+            strcpyX (tmpPath, vFp->path, (size_t)pathLength);
             AppendToDir (tmpPath, "");
             snprintf (EofS(tmpPath), (size_t) pathLength + 1 - strlen(tmpPath),
-                      "TMP%08ld.%s", (long)(rand()%100000000), vFp->scratchExt);
+                      "TMP%08ld.%s", (long)(rand()%100000000),
+                      vFp->scratchExt);
             if (!IsReg(tmpPath)) fp = CDFOpenFile (tmpPath, WRITE_PLUS_a_mode);
             if (fp != NULL) break;
           }
@@ -655,31 +682,33 @@ vFILE *vFp;
 #     endif
     } else {
       char *tmpFolder = NULL;
-      strcpy (tmpPath, "");
+      strcpyX (tmpPath, "", (size_t)pathLength);
 #     if defined(unix) || defined(vms)
         tmpFolder = (char *) cdf_AllocateMemory ((size_t)pathLength + 1, NULL);
         if (tmpFolder == NULL) return FALSE;
         writable = access(".", W_OK);
         if (writable == 0) {
-          strcpy (tmpPath, TmpFileTemplate);
-          strcpy (tmpFolder, "");
+          strcpyX (tmpPath, TmpFileTemplate, (size_t)pathLength);
+          strcpyX (tmpFolder, "", (size_t)pathLength);
         } else {
 #         if defined(unix)
-            strcpy (tmpPath, "/tmp/");
-            strcpy (tmpFolder, tmpPath);
+            strcpyX (tmpPath, "/tmp/", (size_t)pathLength);
+            strcpyX (tmpFolder, tmpPath, (size_t)pathLength);
 #         else
-            strcpy (tmpPath, "SYS$SCRATCH:");
-            strcpy (tmpFolder, tmpPath);
+            strcpyX (tmpPath, "SYS$SCRATCH:", (size_t)pathLength);
+            strcpyX (tmpFolder, tmpPath, (size_t)pathLength);
 #         endif
           strcatX (tmpPath, TmpFileTemplate, 0);
         }
         fileid = mkstemp(tmpPath);
-        if (fileid != -1) fp = fdopen(fileid, "w+");
-        else {
+        if (fileid != -1) {
+          fp = fdopen(fileid, "w+");
+        } else {
           for (i = 1; i <= MAX_TMP; i++) {
-            strcpy (tmpPath, tmpFolder);
+            strcpyX (tmpPath, tmpFolder, (size_t)pathLength);
             snprintf (EofS(tmpPath), (size_t) pathLength + 1 - strlen(tmpPath),
-                      "TMP%08ld.%s", (long)(rand()%100000000), vFp->scratchExt);
+                      "TMP%08ld.%s", (long)(rand()%100000000),
+                      vFp->scratchExt);
             if (!IsReg(tmpPath)) {
               fp = CDFOpenFile (tmpPath, WRITE_PLUS_a_mode);
               if (fp != NULL) break;
@@ -699,9 +728,11 @@ vFILE *vFp;
               int i1, i2;
               for (i = 1; i <= MAX_TMP; i++) {
                 fh = -1;
-                strcpy (tmpPath, "");
-                i1 = (int) (((double)rand() / (RAND_MAX + 1)) * 25) + (int) 'A';
-                i2 = (int) (((double)rand() / (RAND_MAX + 1)) * 25) + (int) 'A';
+                strcpyX (tmpPath, "", (size_t)pathLength);
+                i1 = (int) (((double)rand() / (RAND_MAX + 1)) * 25) +
+                     (int) 'A';
+                i2 = (int) (((double)rand() / (RAND_MAX + 1)) * 25) +
+                     (int) 'A';
                 snprintf (EofS(tmpPath), (size_t) pathLength + 1 -
                                          strlen(tmpPath),
                           "TMP%c%c%5.5d.%s", i1, i2, rand(), vFp->scratchExt);
@@ -717,7 +748,9 @@ vFILE *vFp;
 #       else
           snprintf (EofS(tmpPath), (size_t) pathLength + 1 - strlen(tmpPath),
                     "TMP%08ld.%s", (long)(rand()%100000000), vFp->scratchExt);
-          if (!IsReg(tmpPath)) fp = CDFOpenFile (tmpPath, WRITE_PLUS_a_mode);
+          if (!IsReg(tmpPath)) {
+	    fp = CDFOpenFile (tmpPath, WRITE_PLUS_a_mode);
+	  }
 #       endif
 #     endif
     }
@@ -811,7 +844,7 @@ vFILE *vFp;             /* Pointer to vFILE structure. */
 }
 
 /******************************************************************************
-* V_read64.
+* V_read64. Read data from cache(s), starting from offset64, into buffer
 ******************************************************************************/
 
 VISIBLE_PREFIX size_t V_read64 (buffer, item_size, n_items, vFp)
@@ -862,6 +895,15 @@ vFILE *vFp;             /* Pointer to vFILE structure. */
   (vFp->nV_reads)++;
   /****************************************************************************
   * Read from first block...
+  *    Calling FindCache to see if the blockN in cache link
+  *            if found, use the cache
+  *            if not found, calling PageIn (calling AllocateBuffer to get a
+  *            cache)
+  *               1. if the max cache number NOT reached, allocate a new cache
+  *               2. otherwise, from cache link's tail to find the un-modifed
+  *                  or use the tail cache from cache link (write cache out)
+  *               3. move the cache to head
+  *            locate the buffer point for moving the data to 
   ****************************************************************************/
   firstBlockN = (long) (vFp->offset64 / (OFF_T) nCACHE_BUFFER_BYTEs);
   bufferOffset = (int) ((OFF_T) vFp->offset64 % nCACHE_BUFFER_BYTEs);
@@ -938,7 +980,7 @@ vFILE *vFp;             /* Pointer to vFILE structure. */
 }
 
 /******************************************************************************
-* V_write64.
+* V_write64. -- write data to cache, which will be marked as modified
 ******************************************************************************/
 
 VISIBLE_PREFIX size_t V_write64 (buffer, item_size, n_items, vFp)
@@ -958,6 +1000,7 @@ vFILE *vFp;             /* Pointer to vFILE structure. */
   vCACHE *cache;        /* Pointer to cache structure. */
   Byte1 *cBuffer;       /* Pointer to cache buffer. */
   size_t nBytesInBlock;	/* Number of bytes to the end of the block. */
+  Logical new;
   /****************************************************************************
   * Validate write.
   ****************************************************************************/
@@ -977,6 +1020,15 @@ vFILE *vFp;             /* Pointer to vFILE structure. */
   * Write to first block...
   * Note that if this is a scratch file, the first block is always placed in
   * the cache (even if a full block).
+  * Calling FindCache to see if the blockN in cache link
+  *         =========
+  *    if found, use the cache
+  *    if not found, calling PageIn
+  *       1. if the max cache number NOT reached - allocate a new cache
+  *       2. otherwise, from cache's head to find the un-modifed or use the 
+  *          tail from modified cache link - move the cache to head
+  *    locate the buffer point in cache for moving the data from buffer to
+  *
   ****************************************************************************/
   firstBlockN = (long) (vFp->offset64 / (OFF_T) nCACHE_BUFFER_BYTEs); 
   bufferOffset = (int) ((OFF_T) vFp->offset64 % nCACHE_BUFFER_BYTEs);
@@ -994,11 +1046,15 @@ vFILE *vFp;             /* Pointer to vFILE structure. */
 	}
       }
       else {
-	cache = AllocateBuffer (vFp);
+	cache = AllocateBuffer (vFp, &new);
 	if (cache == NULL) {
 	  vFp->error = TRUE;
 	  return 0;
 	}
+	if (new && vFp->scratch && vFp->fp == NULL) {
+	  vFp->cacheNumInd[firstBlockN] = 1;
+	  vFp->cacheLocPtr[firstBlockN] = cache;
+	}
 	cache->blockN = firstBlockN;
 #if CLEAR_BYTES
 	cBuffer = CACHEbufferWRITEto (cache);
@@ -1065,11 +1121,15 @@ vFILE *vFp;             /* Pointer to vFILE structure. */
 	   }
 	 }
 	 else {
-	   cache = AllocateBuffer (vFp);
+	   cache = AllocateBuffer (vFp, &new);
 	   if (cache == NULL) {
 	     vFp->error = TRUE;
 	     return 0;
 	   }
+	   if (new && vFp->scratch && vFp->fp == NULL) {
+	     vFp->cacheNumInd[blockN] = 1;
+	     vFp->cacheLocPtr[blockN] = cache;
+	   }
 	   cache->blockN = blockN;
 #if CLEAR_BYTES
 	   cBuffer = CACHEbufferWRITEto (cache);
@@ -1127,6 +1187,7 @@ char *a_mode;           /* Access mode. */
 #if defined(vms)
   struct STAT st;       /* Status block from `stat'. */
 #endif
+int jj;
   /****************************************************************************
   * Open the file.
   ****************************************************************************/
@@ -1202,7 +1263,8 @@ char *a_mode;           /* Access mode. */
 #else
   vFp->fp = fp;
 #endif
-  vFp->path = (char *) cdf_AllocateMemory ((size_t)strlen(file_spec) + 1, NULL);
+  vFp->path = (char *) cdf_AllocateMemory ((size_t)strlen(file_spec) + 1,
+                                           NULL);
   if (vFp->path == NULL) {
     cdf_FreeMemory (vFp, NULL);
     FCLOSE64 (fp);
@@ -1227,6 +1289,15 @@ char *a_mode;           /* Access mode. */
   vFp->GDR64 = NULL;
   vFp->ADRList = NULL;
   vFp->ADRList64 = NULL;
+  vFp->cacheNumInd = NULL;
+  vFp->cacheLocPtr = NULL;
+  vFp->cacheNumInd = (long *) cdf_AllocateMemory ((size_t)sizeof(long) *
+                                                  NUMcacheSINGLE, NULL);
+  vFp->cacheLocPtr = (vCACHE **) cdf_AllocateMemory ((size_t)sizeof(vCACHE *) *
+                                                     NUMcacheSINGLE, NULL);
+  memset (vFp->cacheNumInd, 0, NUMcacheSINGLE * sizeof(long));
+  memset (vFp->cacheLocPtr, 0, NUMcacheSINGLE * sizeof(vCACHE *));
+
   /****************************************************************************
   * Determine length of file and set current offset.
   ****************************************************************************/
@@ -1302,23 +1373,22 @@ int maxBuffers;         /* New maximum number of cache buffers. */
   if (vFp == NULL) return EOF;
   if (vFp->magic_number != VSTREAM_MAGIC_NUMBER) return EOF;
   if (vFp->error) return EOF;
-  if (maxBuffers < 1) return EOF;
-  if (maxBuffers > vFp->maxBuffers) {
+  if ((vFp->maxBuffers != -1) && maxBuffers < 1) return EOF;
+  if ((vFp->maxBuffers != -1) && maxBuffers > vFp->maxBuffers) {
     /**************************************************************************
     * The number of cache buffers is increasing.
     **************************************************************************/
     vFp->maxBuffers = maxBuffers;
   }
   else {
-    if (maxBuffers < vFp->maxBuffers) {
+    if ((vFp->maxBuffers != -1) && maxBuffers < vFp->maxBuffers) {
       /************************************************************************
       * The number of cache buffers is decreasing - flush to disk and free
       * the buffers which are going away.
       ************************************************************************/
       vCACHE *cache; int count;
       if (vFp->nBuffers > maxBuffers) {
-        for (count = 1,
-             cache = vFp->cacheHead;
+        for (count = 1, cache = vFp->cacheHead;
              count < maxBuffers; count++) cache = cache->next;
         if (!FlushCache64(vFp,cache->next)) {
           vFp->error = TRUE;
@@ -1335,6 +1405,17 @@ int maxBuffers;         /* New maximum number of cache buffers. */
   return 0;
 }
 
+/******************************************************************************
+* V_freecaches64.
+* Free the caches from head to tail.
+******************************************************************************/
+
+VISIBLE_PREFIX void V_freecaches64 (vFp)
+vFILE *vFp;
+{
+   FreeCache(vFp->cacheHead);
+}
+
 /******************************************************************************
 * V_flush64.
 * Flush the file to disk.
@@ -1353,29 +1434,31 @@ vFILE *vFp;             /* Pointer to vFILE structure. */
   * Flush cache buffers.  If this is a scratch file, this will cause the file
   * to be created if it hasn't been already (unless nothing has been written).
   ****************************************************************************/
-  if (!FlushCache64(vFp,vFp->cacheHead)) {
-    vFp->error = TRUE;
-    return EOF;
-  }
-  /****************************************************************************
-  * Flush file.  Note that the file will not be flushed if this is a scratch
-  * file to which nothing has been written.
-  ****************************************************************************/
+  if (vFp->fp != NULL) { /* xxxxxxxxxxxxxxxxxxxxxxxxxx */
+    if (!FlushCache64(vFp,vFp->cacheHead)) {
+      vFp->error = TRUE;
+      return EOF;
+    }
+    /**************************************************************************
+    * Flush file.  Note that the file will not be flushed if this is a scratch
+    * file to which nothing has been written.
+    **************************************************************************/
 #if defined(win32) && (_FILE_OFFSET_BITS == 64)
-  if (vFp->fh != 0) {
-    if (FLUSH64(vFp->fh) == EOF) {
-      vFp->error = TRUE;
-      return EOF;
+    if (vFp->fh != 0) {
+      if (FLUSH64(vFp->fh) == EOF) {
+        vFp->error = TRUE;
+        return EOF;
+      }
     }
-  }
 #else
-  if (vFp->fp != NULL) {
-    if (FLUSH64(vFp->fp) == EOF) {
-      vFp->error = TRUE;
-      return EOF;
+    if (vFp->fp != NULL) {
+      if (FLUSH64(vFp->fp) == EOF) {
+        vFp->error = TRUE;
+        return EOF;
+      }
     }
-  }
 #endif
+  } /* xxxxxxxxxxxxxxxxxxxxxxxxxx */
   /****************************************************************************
   * Return success.
   ****************************************************************************/
@@ -1441,6 +1524,8 @@ vSTATS *vStats;         /* Pointer to statistics structure. */
   * Deallocate cache and vFILE structure.
   ****************************************************************************/
   FreeCache (vFp->cacheHead);
+  if (vFp->cacheNumInd != NULL) cdf_FreeMemory (vFp->cacheNumInd, NULL);
+  if (vFp->cacheLocPtr != NULL) cdf_FreeMemory (vFp->cacheLocPtr, NULL);
   cdf_FreeMemory (vFp->path, NULL);
   cdf_FreeMemory (vFp, NULL);
   /****************************************************************************
@@ -1528,6 +1613,10 @@ vSTATS *vStats;         /* Pointer to statistics structure. */
   * Deallocate cache and vFILE structure.
   ****************************************************************************/
   FreeCache (vFp->cacheHead);
+  if (vFp->cacheNumInd != NULL)
+    cdf_FreeMemory (vFp->cacheNumInd, NULL);
+  if (vFp->cacheLocPtr != NULL)
+    cdf_FreeMemory (vFp->cacheLocPtr, NULL);
   cdf_FreeMemory (vFp->path, NULL);
   cdf_FreeMemory (vFp, NULL);
   /****************************************************************************
