@@ -57,6 +57,9 @@ Logical useFormat;
 Logical report[3];
 int advformat, metaformat, varformat, precision;
 Logical displayStats;
+#ifdef WIN32
+  static unsigned int oldCodePage = 0;
+#endif
 
 /******************************************************************************
 * Main.
@@ -71,10 +74,8 @@ MAIN {
 #else
   success = CreateSkeletonTable (argc, argv);
 #endif
-#if defined(DEBUG)
-  if (cdf_FreeMemory(NULL,FatalError) > 0) DisplayWarning ("Abandoned buffers.");
-#else
-  cdf_FreeMemory (NULL, FatalError);
+#ifdef WIN32
+  if (oldCodePage != 0) SetConsoleOutputCP(oldCodePage);
 #endif
   return BOO(success,EXIT_SUCCESS_,EXIT_FAILURE_);
 }
@@ -103,17 +104,16 @@ char *argV[];
     "skeleton", "nonrv", "nrvfile", "nrvtable", "log", "nolog", "zmode",
     "neg2posfp0", "noneg2posfp0", "format", "noformat", "report", "screen",
     "noscreen", "cache", "values", "page", "nopage", "about", "statistics",
-    "nostatistics", "iso8601", "ziso8601", "advformat", NULL };
+    "nostatistics", "iso8601", "ziso8601", "advformat", "novalidate", NULL };
   static int optRequired[] = {
     TRUE,  FALSE, FALSE, FALSE, FALSE, FALSE, TRUE,
     FALSE, FALSE, FALSE, FALSE, TRUE,  FALSE,
     FALSE, TRUE,  TRUE,  FALSE, FALSE, FALSE, FALSE,
-    FALSE, FALSE, FALSE, TRUE, 0
+    FALSE, FALSE, FALSE, TRUE, FALSE, 0
   };
   static char *reportTokens[] = { "errors", "warnings", "informationals" };
   static char *formatTokens[] = { "best", "no", "metaonly", "dataonly", "all"};
 
-char ADVF[9];
   /****************************************************************************
   * Parse qualifiers/options/parameters (QOP).
   ****************************************************************************/
@@ -247,6 +247,13 @@ char ADVF[9];
       } else
         iso8601 = 0; /* DEFAULTstyleCDF2SKT; */
       /************************************************************************
+      * Check for `novalidate' qualifier.
+      ************************************************************************/
+      if (qop->qualEntered[NOVALIDATEqual]) {
+        toValidate = FALSE;
+      } else
+        toValidate = TRUE;
+      /************************************************************************
       * Check for `report' qualifier.  If absent, use defaults.
       ************************************************************************/
       if (qop->qualEntered[REPORTqual]) {
@@ -318,7 +325,8 @@ char ADVF[9];
       if (qop->qualEntered[ADVFORMATqual]) {
         int ij;
         for (ij = 0; ij < 5; ++ij) {
-          if (strcmpIgCase(qop->qualOpt[ADVFORMATqual], formatTokens[ij]) == 1) {
+          if (strcmpIgCase(qop->qualOpt[ADVFORMATqual],
+			   formatTokens[ij]) == 1) {
             advformat = ij;
             break;
           }
@@ -341,6 +349,15 @@ char ADVF[9];
   }
 
   /* CDFsetValidate (VALIDATEFILEon); */
+
+#if !defined(WIN32)
+  setlocale(LC_CTYPE, "en_US.UTF-8");
+#else
+  setlocale(LC_CTYPE, ".UTF8");
+  oldCodePage = GetConsoleOutputCP();
+  SetConsoleOutputCP(65001);
+#endif
+
   /****************************************************************************
   * Create skeleton table.
   ****************************************************************************/
@@ -353,6 +370,8 @@ char ADVF[9];
 
   TimeStamp (creationStamp);
 
+  if (!toValidate) CDFsetValidate (VALIDATEFILEoff);
+ 
   if (outToScreen)
     SKTfp = stdout;
   else {
@@ -383,6 +402,8 @@ char ADVF[9];
 
   WriteOut (SKTfp, "! Skeleton table for the \"");
   WriteOut (SKTfp, CDFname);
+  if (!EndsWithIgCase(CDFpath, ".cdf"))
+    WriteOut (SKTfp, ".cdf");
   WriteOut (SKTfp, "\" CDF.");
 
   WriteOut (SKTfp, "\n! Generated: ");
@@ -394,7 +415,6 @@ char ADVF[9];
 
   ExpandPath (CDFpath, CDFpathX);
   status = CDFlib (OPEN_, CDF_, CDFpathX, &id,
-/*		   SELECT_, CDF_READONLY_MODE_, READONLYon, */
 		   SELECT_, CDF_zMODE_, zMode,
 			    CDF_NEGtoPOSfp0_MODE_, BOO(negToPosFp0,
 			  			       NEGtoPOSfp0on,
@@ -536,6 +556,8 @@ char *variables;
   WriteOut (SKTfp, "\n");
   WriteOut (SKTfp, "\n                       CDF NAME: ");
   WriteOut (SKTfp, CDFname);
+  if (!EndsWithIgCase(CDFname, ".cdf"))
+    WriteOut (SKTfp, ".cdf");
 
   WriteOut (SKTfp, "\n                  DATA ENCODING: ");
   WriteOut (SKTfp, EncodingToken(encoding));
@@ -588,7 +610,6 @@ char *variables;
                    NULL_);
   if (!StatusHandlerC2S(status)) return FALSE;
 
-/* if (compress != NO_COMPRESSION) */
   WriteOutFP (SKTfp, "\n! CDF_COMPRESSION: %s",
               CompressionToken(compress, compParms));
   WriteOutFP (SKTfp, "\n! (Valid compression: None, GZIP.1-9, RLE.0, HUFF.0, AHUFF.0)");
@@ -597,7 +618,6 @@ char *variables;
                    NULL_);
   if (!StatusHandlerC2S(status)) return FALSE;
 
-/* if (checkSum != NO_CHECKSUM) */
   WriteOutFP (SKTfp, "\n! CDF_CHECKSUM: %s", ChecksumToken(checkSum)); 
   WriteOutFP (SKTfp, "\n! (Valid checksum: None, MD5)");
 
@@ -661,7 +681,6 @@ Logical WriteGlobalAttr ()
          status = CDFlib (GET_, ATTR_MAXgENTRY_, &maxEntry,
 		          NULL_);
          if (!StatusHandlerC2S(status)) return FALSE;
-
          /*********************************************************************
          * If first global scope attribute found, print field headings.
          *********************************************************************/
@@ -685,7 +704,6 @@ Logical WriteGlobalAttr ()
 	   ccc += ENTRY_NUM_COL - ccc;
 	   ccc += WriteOut (SKTfp, "------      ----       -----");
          }
-
          /*********************************************************************
          * Write name.
          *********************************************************************/
@@ -695,7 +713,6 @@ Logical WriteGlobalAttr ()
          snprintf (tempS, (size_t) sizeof(tempS),
 	           "  %c%s%c", delim, attrName, delim);
          ccc = WriteOut (SKTfp, tempS);
-
          /*********************************************************************
          * Write each entry.
          *********************************************************************/
@@ -712,12 +729,22 @@ Logical WriteGlobalAttr ()
 	      entryCount++;
 
 	      value = cdf_AllocateMemory ((size_t) (CDFelemSize(dataType) *
-					  numElements), FatalError);
+					  numElements+1), FatalError);
 
 	      status = CDFlib (GET_, gENTRY_DATA_, value,
 			       NULL_);
 	      if (!StatusHandlerC2S(status)) return FALSE;
 
+	      if (STRINGdataType(dataType)) {
+	        int ii;
+		*(((char *)value)+numElements) = '\0';
+	        /* Not allow LF and CR */
+	        for (ii = 0; ii < (int) numElements; ++ii) { 
+	          if (((int)*(((char *)value)+ii) == (int) '\n') ||
+	              ((int)*(((char *)value)+ii) == (int) '\r'))
+	  	    memcpy (((char *)value)+ii, " ", 1);
+	        }
+	      }
 	      if (entryCount == 1) {
 	        if (ccc > ENTRY_NUM_COL - 1) {
 		  WriteOut (SKTfp, "\n");
@@ -731,27 +758,16 @@ Logical WriteGlobalAttr ()
 
 	      Ncharacters (SKTfp, ENTRY_NUM_COL - ccc, (int) ' ');
 	      ccc += ENTRY_NUM_COL - ccc;
-
 	      snprintf (tempS, (size_t) sizeof(tempS),
 		        "%*ld:    %s   ", ENTRY_NUM_WIDTH, entryN + 1,
 		        DataTypePaddedString(dataType));
 	      ccc += WriteOut (SKTfp, tempS);
-
-	      if (STRINGdataType (dataType)) { /* not allow LF and CR */
-	        int ii;
-	        for (ii = 0; ii < (int) numElements; ++ii) { 
-	          if (((int)*(((char *)value)+ii) == (int) '\n') ||
-	              ((int)*(((char *)value)+ii) == (int) '\r'))
-	  	    memcpy (((char *)value)+ii, " ", 1);
-	        }
-	      }
-
 	      ccc += WriteOut (SKTfp, "{ ");
-	      WriteEntryValueSKT (SKTfp, dataType, numElements, value, ccc,
+	      WriteEntryValueSKT (SKTfp, dataType, numElements,
+				  value, ccc,
 	  		          MAX_COL_TO_USE - 4, iso8601, NULL);
 						/* -4 for possible " } ." */
 	      ccc += WriteOut (SKTfp, " }");
-
 	      cdf_FreeMemory (value, FatalError);	    
 	    }
 	    CHECKforABORTso
@@ -910,7 +926,7 @@ Logical zVar;
   long sarray[CDF_MAX_PARMS];
   char *format;
   int  howto;
-
+  int  varNameLen;
   format = NULL;
   sarray[0] = 0;
   status = CDFlib (SELECT_, VAR(zVar), varN,
@@ -939,14 +955,15 @@ Logical zVar;
 
   pad = cdf_AllocateMemory ((size_t) ((varNumElements + 
                                        (STRINGdataType(varDataType)? 1 : 0)) *
-                                      CDFelemSize(varDataType)),
+                                      CDFelemSize(varDataType)), 
                             FatalError);
   status = CDFlib (GET_, VAR_PADVALUE(zVar), pad,
                    NULL_);
   if (!StatusHandlerC2S(status)) return FALSE;
-  if (status != NO_PADVALUE_SPECIFIED)
+  if (status != NO_PADVALUE_SPECIFIED) {
     padValue = 1;
-  else
+    if (STRINGdataType(varDataType)) *((char *)pad+varNumElements) = '\0';
+  } else
     padValue = 0;
 
   /***************************************************************************
@@ -1041,15 +1058,43 @@ Logical zVar;
 
   /***************************************************************************
   * Write variable definition line.
+  * Variable name might break into multiple lines if its length is more than
+  * 74 characters. In this case, an extra "-" is appended to the end for
+  * an indication of continuation. 
   ***************************************************************************/
 
   WriteOut (SKTfp, "\n");
 
   delim = PickDelimiter (varName, strlen(varName));
   WriteOut (SKTfp, "\n");
-  snprintf (tempS, (size_t) sizeof(tempS), "  %c%s%c", delim, varName, delim);
-  ccc = WriteOut (SKTfp, tempS);
-
+  varNameLen = (int) strlen(varName);
+  if (varNameLen < 75) {
+    snprintf (tempS, (size_t) sizeof(tempS), "  %c%s%c", delim, varName, delim);
+    ccc = WriteOut (SKTfp, tempS);
+  } else {
+    int len, ix;
+    char namePart[73];
+    len = varNameLen;
+    ix = 0;
+    while (len >= 72) {
+      strncpy(namePart, varName+ix, 72);
+      namePart[72] = (char ) '\0';
+      snprintf (tempS, (size_t) sizeof(tempS), "  %c%s%c %c", delim, namePart,
+                delim,(len==72?' ':'-'));
+      ccc = WriteOut (SKTfp, tempS);
+      WriteOut (SKTfp, "\n");
+      ix += 72;
+      len -= 72;
+    }
+    if (len > 0) {
+      strcpyX(namePart, varName+ix, 72);
+      snprintf (tempS, (size_t) sizeof(tempS), "  %c%s%c", delim, namePart,
+                delim);
+      ccc = WriteOut (SKTfp, tempS);
+      WriteOut (SKTfp, "\n");
+      ccc = 0;
+    }
+  }
   if (varDataType != CDF_TIME_TT2000) ddd = 0;
   else ddd = 4;
 
@@ -1164,13 +1209,8 @@ Logical zVar;
       *(((char *) pad)+0) = (char) DEFAULT_CHAR_PADVALUE;
       *(((char *) pad)+1) = (char) '\0';
     } else {
-      long numBytes = (long) UTF8StrLength ((unsigned char *)pad,
-                                            (int)varNumElements);
+      int ix, len;
       iy = -1;
-      if (numBytes == varNumElements) {
-        for (ix = 0; ix < (int) varNumElements; ++ix)
-          if (!Printable(*(((char *) pad)+ix))) *(((char *) pad)+ix) = '.';
-      }
       for (ix = (varNumElements-1); ix >= 0; --ix) {
         if (*(((char *) pad)+ix) != ' ') {
           iy = ix;
@@ -1271,14 +1311,22 @@ Logical zVar;
 
 	 WriteOut (SKTfp, "\n");
        }
-
        value = cdf_AllocateMemory ((size_t) (entryNumElements *
-				             CDFelemSize(entryDataType)),
+				             CDFelemSize(entryDataType)+1), 
 				   FatalError);
 
        status = CDFlib (GET_, BOO(zVar,zENTRY_DATA_,rENTRY_DATA_), value,
 		        NULL_);
        if (!StatusHandlerC2S(status)) return FALSE;
+       if (STRINGdataType(entryDataType)) {
+         int ii;
+	 *(((char *)value)+entryNumElements) = '\0';
+         for (ii = 0; ii < (int) entryNumElements; ++ii) {
+           if (((int)*(((char *)value)+ii) == (int) '\n') ||
+               ((int)*(((char *)value)+ii) == (int) '\r'))
+             memcpy (((char *)value)+ii, " ", 1);
+         }
+       }
 
        delim = PickDelimiter (attrName, strlen(attrName));
        WriteOut (SKTfp, "\n");
@@ -1300,19 +1348,11 @@ Logical zVar;
        ccc += WriteOut (SKTfp, DataTypePaddedString(entryDataType));
        ccc += WriteOut (SKTfp, "   ");
 
-       if (STRINGdataType (entryDataType)) { /* not allow LF and CR */
-         int ii;
-         for (ii = 0; ii < (int) entryNumElements; ++ii) {
-           if (((int)*(((char *)value)+ii) == (int) '\n') ||
-               ((int)*(((char *)value)+ii) == (int) '\r'))
-             memcpy (((char *)value)+ii, " ", 1);
-         }
-       }
        ccc += WriteOut (SKTfp, "{ ");
        if (metaformat == 0)
          WriteEntryValue (SKTfp, entryDataType, entryNumElements,
-                          value, ccc, MAX_COL_TO_USE - 4, iso8601,
-                          NULL);
+                          value, ccc,
+			  MAX_COL_TO_USE - 4, iso8601, NULL);
        else {
          howto = 1;
          if (advformat == 0) {
@@ -1327,7 +1367,8 @@ Logical zVar;
            }
          }
          WriteEntryValueSKT (SKTfp, entryDataType, entryNumElements,
-			     value, ccc, MAX_COL_TO_USE - 4, iso8601,
+			     value, ccc,
+			     MAX_COL_TO_USE - 4, iso8601,
 			     (howto==1?format:NULL));
        }
        ccc += WriteOut (SKTfp, " }");
@@ -1524,8 +1565,9 @@ char *format;
         strcatX (tempS, "] = ",MAX_LINE_LEN);
 	ccc += WriteOut (SKTfp, tempS);
         if (STRINGdataType(dataType)) {
+          long maxLen = (long) strlen (value);
           ccc += WriteOut (fp, "{ ");
-          WriteStringValue (fp, numElements, value, ccc,
+          WriteStringValue (fp, MINIMUM(numElements,maxLen), value, ccc,
 		            MAX_COL_TO_USE - 2);    /* -2 for " }" */
           WriteOut (fp, " }");
         }
@@ -1980,7 +2022,8 @@ char **argV[];
       case HELPin: {
         int n;
         *argC = 1;
-        *argV = (char **) cdf_AllocateMemory ((size_t)*argC * sizeof(char *), FatalError);
+        *argV = (char **) cdf_AllocateMemory ((size_t)*argC * sizeof(char *),
+					      FatalError);
 	n = 0;
 	MAKEstrARGv (argV, n, pgmName)
 #ifdef __MWERKS__
