@@ -39,7 +39,11 @@ void *GetVirtualVarRecords (long, Logical);
 /******************************************************************************
 * Global variables.
 ******************************************************************************/
+#ifdef WIN32
+  static unsigned int oldCodePage = 0;
+#endif
 
+FILE *fp;
 /******************************************************************************
 * Main.
 ******************************************************************************/
@@ -49,10 +53,8 @@ MAIN {
   Logical success = TRUE;
   strcpyX (pgmName, "CDFdump", MAX_PROGRAM_NAME_LEN);
   success = DumpCDF (argc, argv);
-#if defined(DEBUG)
-  if (cdf_FreeMemory(NULL,FatalError) > 0) DisplayWarning ("Abandoned buffers.");
-#else
-  cdf_FreeMemory (NULL, FatalError);
+#ifdef WIN32
+  if (oldCodePage != 0) SetConsoleOutputCP(oldCodePage);
 #endif
   return BOO(success,EXIT_SUCCESS_,EXIT_FAILURE_);
 }
@@ -67,7 +69,6 @@ int  argC;
 char *argV[];
 {
 
-CDFid id;
 CDFstatus status;
 long numDims, dimSizes[CDF_MAX_DIMS], dimIntervals[CDF_MAX_DIMS], 
      dimIndices[CDF_MAX_DIMS], dimCounts[CDF_MAX_DIMS];
@@ -89,36 +90,37 @@ char tText[256];
 char shortStr1[20], shortStr2[20], *prtFormat = NULL;
 void *value, *working = NULL;
 void *value2 = NULL;
+unsigned char *checkPtr = NULL;
 Logical  toCol2Row = FALSE;
 Byte1 **handles[1], *buffer = NULL;
 char **vars = NULL;
 int  numVars = 0;
 size_t nValueBytes[1];
 size_t offset, nBytes;
-struct HyperStruct hyperX;
+struct HyperStruct hyperSt;
 struct GroupStruct groups;
 Logical Z, pad, ifound;
 int  varNum, i, ii, j, k, ir, imore, iskip, icount = 0, ij, precision;
 long tmpLong[1] = {0};
 Logical qopError = FALSE;
 Logical detect = FALSE;
-FILE *fp;
 QOP *qop;
 Logical virtual = FALSE;
+Logical toValidate;
 static char *validQuals[] = {
   "dump", "output", "format", "noformat", "significant", "vars",
   "about", "header", "noheader", "recordrange", "col2row", "detect",
-  "epoch_virtual", NULL 
+  "epoch_virtual", "ziso8601", "novalidate", NULL 
 };
 static int optRequired[] = {
   TRUE, TRUE, FALSE, FALSE, TRUE, TRUE,
-  FALSE, FALSE, FALSE, TRUE, FALSE, FALSE, FALSE,
+  FALSE, FALSE, FALSE, TRUE, FALSE, FALSE, FALSE, TRUE, FALSE,
   0
 };
-int numValidDumps = 5;
+int numValidDumps = 6;
 static char *validDumps[] = {
-  "all", "data", "metadata", "global", "variable" };
-
+  "all", "data", "nodata", "metadata", "global", "variable" };
+char *locale;
 
 /******************************************************************************
 * Parse qualifiers/options/parameters (QOP).
@@ -151,9 +153,15 @@ switch (argC) {
         }
       }
       if (!Z) {
-        DisplayError ("Invalid dump option.");
-        qopError = TRUE;
-        return TRUE;
+        if (strcmpIgCase(dumpOption, "meta") == 1) {
+          dump = 3;
+          Z = TRUE;
+	}
+        if (!Z) {
+          DisplayError ("Invalid dump option.");
+          qopError = TRUE;
+          return TRUE;
+        }
       }
     } else
       MakeNUL(dumpOption);
@@ -224,7 +232,7 @@ switch (argC) {
         vars = (char **) cdf_AllocateMemory ((size_t)sizeof(char *) * numVars,
                                              FatalError);
         for (i = 0; i < numVars; i++) 
-          vars[i] = (char *) cdf_AllocateMemory ((size_t)CDF_VAR_NAME_LEN256+1, 
+          vars[i] = (char *) cdf_AllocateMemory ((size_t)CDF_VAR_NAME_LEN256+1,
                                                  FatalError);
         ParseStringForTokens (',', varsOption, vars);
       }   
@@ -252,6 +260,20 @@ switch (argC) {
     } else
       epochVV = FALSE;
     /**************************************************************************
+    * Check for the `ziso8601' qualifier.
+    **************************************************************************/
+    if (qop->qualEntered[ZISO8601qual]) {
+      ziso8601 = TRUE;
+    } else
+      ziso8601 = FALSE;
+    /**************************************************************************
+    * Check for the `novalidate' qualifier.
+    **************************************************************************/
+    if (qop->qualEntered[NOVALIDATEqual]) {
+      toValidate = FALSE;
+    } else
+      toValidate = TRUE;
+    /**************************************************************************
     * Get CDF pathname.
     **************************************************************************/
     if (qop->Nparms < 1) {
@@ -262,7 +284,6 @@ switch (argC) {
       strcpyX (CDFpath, qop->parms[CDFPATHparm], DU_MAX_PATH_LEN);
       ParsePath (CDFpath, dir, CDFname);
     }
-
     if (!NULstring(outputFile)) {
       if (strncmpIgCase(outputFile, "source", 6) != 0) {
         int loc;
@@ -307,7 +328,9 @@ switch (argC) {
     fprintf (fp, "\"...\n");
   }
 
-  if (detect) CDFsetValidate (VALIDATEFILEon);
+  if (!toValidate) CDFsetValidate (VALIDATEFILEoff);
+  else if (detect) CDFsetValidate (VALIDATEFILEon);
+
   /****************************************************************************
   * Open/inquire CDF.
   ****************************************************************************/
@@ -317,7 +340,6 @@ switch (argC) {
   if (status < CDF_OK) return QuitCDF (CDFpath, "1.0", status, CDFpath);
 
   status = CDFlib (SELECT_, CDF_, id,
-                            CDF_READONLY_MODE_, READONLYon,
                    GET_, CDF_VERSION_, &version,
                          CDF_RELEASE_, &release,
                          CDF_INCREMENT_, &increment,
@@ -344,27 +366,40 @@ switch (argC) {
     fprintf (fp, "CDF File:     %s\n",CDFname);
     fprintf (fp, "Version:      %ld.%ld.%ld\n",version,release,
 							  increment);
-    fprintf (fp, "Copyright:    %s\n",copyRight);
-/*    if (dump == ALLDUMP) { */
-      fprintf (fp, "Format:       %s\n",FormatToken(format));
-      fprintf (fp, "Encoding:     %s\n",EncodingToken(encoding));
-      fprintf (fp, "Majority:     %s\n",MajorityToken(majority));
-      fprintf (fp, "NumrVars:     %ld\n",numrVars);
-      fprintf (fp, "NumzVars:     %ld\n",numzVars);
-      fprintf (fp, "NumAttrs:     %ld (%ld global, %ld variable)\n",
-  			  numAttrs, numgAttrs, numvAttrs);
-      fprintf (fp, "Compression:  %s\n",CompressionToken(compression,
-                                        compressParms));
-      if (!PriorTo ("3.2.0", (Int32) version, (Int32) release,
-		    (Int32) increment)) 
+    fprintf (fp, "%s\n",copyRight);
+    fprintf (fp, "Format:       %s\n",FormatToken(format));
+    fprintf (fp, "Encoding:     %s\n",EncodingToken(encoding));
+    fprintf (fp, "Majority:     %s\n",MajorityToken(majority));
+    fprintf (fp, "NumrVars:     %ld\n",numrVars);
+    fprintf (fp, "NumzVars:     %ld\n",numzVars);
+    fprintf (fp, "NumAttrs:     %ld (%ld global, %ld variable)\n",
+  		 numAttrs, numgAttrs, numvAttrs);
+    fprintf (fp, "Compression:  %s\n",CompressionToken(compression,
+                                      compressParms));
+    if (!PriorTo ("3.2.0", (Int32) version, (Int32) release,
+		  (Int32) increment)) 
       fprintf (fp, "Checksum:     %s\n", ChecksumToken(checksum));
-      if (!PriorTo ("3.5.0", version, release, increment)) {
-        status = CDFlib (GET_, CDF_LEAPSECONDLASTUPDATED_, &lastUpdated,
-                         NULL_);
-        if (status < CDF_OK) return QuitCDF (CDFpath, "2.1", status, CDFpath);
-        fprintf (fp, "LeapSecondLastUpdated:     %ld\n", lastUpdated);
-      }
+    if (!PriorTo ("3.5.0", (Int32) version, (Int32) release,
+                  (Int32) increment)) {
+      status = CDFlib (GET_, CDF_LEAPSECONDLASTUPDATED_, &lastUpdated,
+                       NULL_);
+      if (status < CDF_OK) return QuitCDF (CDFpath, "2.1", status, CDFpath);
+      fprintf (fp, "LeapSecondLastUpdated:     %ld\n", lastUpdated);
+    }
+  } else {
+    if (compression != NO_COMPRESSION) 
+      fprintf (fp, "CDF File Compression:  %s\n",CompressionToken(compression,
+                   compressParms));
+
   }
+  
+#if !defined(WIN32)
+  setlocale(LC_CTYPE, "en_US.UTF-8");
+#else
+  setlocale(LC_CTYPE, ".UTF8");
+  oldCodePage = GetConsoleOutputCP();
+  SetConsoleOutputCP(65001); 
+#endif
 
   /**********************************************/
   /* Print out the Global Attribute information */
@@ -395,10 +430,12 @@ switch (argC) {
         if (status < CDF_OK) return QuitCDF (CDFpath, "4.0", status, attrName);
 
         if (numgEntries <= 1) {
-          if (!detect) fprintf (fp, "%s (%ld entry):\n", attrName, numgEntries);
+          if (!detect) fprintf (fp, "%s (num:%d) (%ld entry):\n", attrName, i, 
+                                numgEntries);
         } else
-          if (!detect) fprintf (fp, "%s (%ld entries):\n", attrName, numgEntries);
-
+          if (!detect)
+            fprintf (fp, "%s (num:%d) (%ld entries):\n", attrName, i,
+                     numgEntries);
         j = k = 0;
         if (numgEntries > 0) {
           do {
@@ -410,14 +447,27 @@ switch (argC) {
               j++;
               continue;
             } else {
+              if (STRINGdataType(dataTypeX)) checkPtr = NULL;
               if (STRINGdataType(dataTypeX)) imore = 1;
               else imore = 0;
               nBytes = (size_t) (CDFelemSize(dataTypeX) * (numElemsX + imore));
               value = cdf_AllocateMemory ((size_t) nBytes, FatalError);
               status = CDFlib (GET_, gENTRY_DATA_, value,
                                NULL_);
-              if (status < CDF_OK) return QuitCDF (CDFpath, "5.0", status, attrName);
+              if (status < CDF_OK)
+                return QuitCDF (CDFpath, "5.0", status, attrName);
+              if (STRINGdataType(dataTypeX)) {
+		int cnt;
+		*((char *)value+numElemsX) = '\0';
+		checkPtr = UTF8Check ((unsigned char *)value, (int)numElemsX,
+			              &cnt);
+	      }
 	      if (detect) {
+		if (STRINGdataType(dataTypeX)) {
+	          if (checkPtr != NULL) 
+	            printf("** Attr:%s entry:%d failed UTF-8 check...\n",
+                           attrName, j);
+	        }
 		++k; ++j;
                 cdf_FreeMemory (value, FatalError);
 		continue;
@@ -425,20 +475,25 @@ switch (argC) {
               fprintf (fp, "\t%d (CDF_%s/%ld): \t",
                        j,DataTypeToken(dataTypeX),numElemsX);
               if (STRINGdataType(dataTypeX)) {
-                ((char *)value)[numElemsX] = '\0';
                 fprintf (fp, "\"%s\"\n", (char *)value);
               } else {
                 int ix, style;
                 char tmp[256];
-                if (TT2000dataType(dataTypeX)) style = TT2000_3_STYLE;
-                else style = EPOCH0_STYLE;
+                if (TT2000dataType(dataTypeX)) {
+                  if (!ziso8601) style = TT2000_3_STYLE;
+                  else style = TT2000_4_STYLE;
+                } else {
+                  if (!ziso8601) style = DEFAULTstyleDUMP;
+                  else style = EPOCH3_STYLE;
+                }
                 for (ix = 0; ix < numElemsX; ++ix) {
-                  EncodeValue (dataTypeX, (char *)value+ix*CDFelemSize(dataTypeX),
+                  EncodeValue (dataTypeX,
+                               (char *)value+ix*CDFelemSize(dataTypeX),
                                tmp, style, (size_t) sizeof(tmp));
                   if (ix == 0) strcpyX (tText, tmp, 0);
                   else strcatX (tText, tmp, 255);
                   if (numElemsX > 1 && ix < (numElemsX-1))
-                    strcatX (tText, " ", 255);
+                    strcatX (tText, ", ", 255);
                 }
                 fprintf (fp, "%s\n", tText);
               }
@@ -481,7 +536,7 @@ switch (argC) {
                          NULL_);
         if (status < CDF_OK) return QuitCDF (CDFpath, "7.0", status, attrName);
 
-        if (!detect) fprintf (fp, "%s\n", attrName);
+        if (!detect) fprintf (fp, "%s (num:%d)\n", attrName, i);
       }
     }
   }
@@ -494,8 +549,8 @@ switch (argC) {
     fprintf (fp, "\nVariable Information ");
     if (numVars == 0) {
       if (numrVars <= 1) fprintf (fp, "(%ld rVariable, ", numrVars);
-      else fprintf (fp, "%ld rVariables, ", numrVars);
-      if (numzVars <= 1) fprintf (fp, "(%ld zVariable)\n", numzVars);
+      else fprintf (fp, "(%ld rVariables, ", numrVars);
+      if (numzVars <= 1) fprintf (fp, "%ld zVariable)\n", numzVars);
       else fprintf (fp, "%ld zVariables)\n", numzVars); 
     } else
     fprintf (fp, "\n");
@@ -656,6 +711,8 @@ switch (argC) {
                                &compression, compressParms, &cPct,
 			   BOO(Z,zVAR_MAXREC_,rVAR_MAXREC_), &maxRec,
 			   BOO(Z,zVAR_DATATYPE_,rVAR_DATATYPE_), &dataType,
+			   BOO(Z,zVAR_BLOCKINGFACTOR_,rVAR_BLOCKINGFACTOR_),
+			       &bf,
                      NULL_);
     if (status < CDF_OK) return QuitCDF (CDFpath, "11.0", status, varName);
         
@@ -677,12 +734,13 @@ switch (argC) {
         virtual = FALSE;
       if (virtual) {
         maxRec = GetVirtualVarMaxRecord ((long)ii, Z);
-        fprintf (fp, "%s (Virtual) (No: %d) (Recs: %ld)", varName, ii, (maxRec+1));
+        fprintf (fp, "%s (Virtual) (No: %d) (Recs: %ld)", varName, ii,
+                                                          (maxRec+1));
       } else
         fprintf (fp, "%s (No: %d) (Recs: %ld)", varName, ii, (maxRec+1));
       if (METADATADUMP && compression != NO_COMPRESSION) {
-        fprintf (fp, " (Compression: %s)\n",
-                 CompressionToken(compression, compressParms));
+        fprintf (fp, " (Compression: %s BlockingFactor: %ld)\n",
+                 CompressionToken(compression, compressParms), bf);
         Ncharacters (fp, (int) strlen(varName), '-');
        } else {
         fprintf (fp, "\n");
@@ -691,7 +749,9 @@ switch (argC) {
       fprintf (fp, "\n");
     }
     if (useFormat) {
-      status = GetFormatEntry (Z, (long) ii, &prtFormat);
+      status = GetFORMATEntry (id, Z, (long) ii, &prtFormat);
+      if (status == CDF_OK) RemoveLeadingBlanks (prtFormat);
+      else prtFormat = NULL;
     } else
       prtFormat = NULL;
 
@@ -707,8 +767,6 @@ switch (argC) {
                                &maxAllocRec,
                            BOO(Z,zVAR_NUMallocRECS_,rVAR_NUMallocRECS_), 
                                &numAllocRecs,
-                           BOO(Z,zVAR_BLOCKINGFACTOR_,rVAR_BLOCKINGFACTOR_), 
-                               &bf,
                            BOO(Z,zVAR_SPARSERECORDS_,rVAR_SPARSERECORDS_), 
                                &sp,
                      NULL_);
@@ -727,10 +785,23 @@ switch (argC) {
           ((char *)value)[numElems] = '\0';
           strcpyX (tText, value, numElems);
         } else {
-          int style;
-          if (TT2000dataType(dataType)) style = TT2000_3_STYLE;
-          else style = EPOCH0_STYLE;
-          EncodeValue (dataType, value, tText, style, (size_t) sizeof(tText));
+          int style, toUse;
+          if (TT2000dataType(dataType)) {
+            if (!ziso8601) style = TT2000_3_STYLE;
+            else style = TT2000_4_STYLE;
+          } else {
+            if (!ziso8601) style = DEFAULTstyleDUMP;
+            else style = EPOCH3_STYLE;
+          }
+          if (prtFormat != NULL) {
+            if (IsHexFormat(prtFormat))
+              toUse = 0;
+            else
+              toUse = 1;
+          } else
+            toUse = 0;
+          EncodeValueFormat (dataType, value, tText, (toUse==1?prtFormat:NULL),
+                             -1, 0, style, (size_t) sizeof(tText));
         }
       } else
         pad = FALSE;
@@ -826,45 +897,95 @@ switch (argC) {
           status = CDFlib (GET_, ATTR_NAME_, attrName,
                                  BOO(Z,zENTRY_DATA_,rENTRY_DATA_), value,
                            NULL_);
-          if (status < CDF_OK) return QuitCDF (CDFpath, "15.0", status, attrName);
- 
+          if (status < CDF_OK)
+            return QuitCDF (CDFpath, "15.0", status, attrName);
+          if (STRINGdataType(dataTypeX)) *((char *)value+numElemsX) = '\0';
+	  if (STRINGdataType(dataTypeX)) {
+	    int cnt;
+	    checkPtr = UTF8Check ((unsigned char *)value, (int)numElemsX, &cnt);
+	    if (checkPtr != NULL) 
+	      printf("** Attr:%s entry:%d failed UTF-8 check...\n", attrName,
+                     ii);
+	  }
           fprintf (fp, "     %s", attrName);
           ir = (int) strlen(attrName);
           if (ir < 16) Ncharacters (fp, (16-ir), ' ');
           else fprintf (fp, "%s", " ");
           fprintf (fp, "(CDF_%s/%ld): ", DataTypeToken(dataTypeX),numElemsX);
           if (STRINGdataType(dataTypeX)) {
-            ((char *)value)[numElemsX] = '\0';
-            fprintf (fp, "\"%s\"\n", (char *)value);
+            long numStrings;
+            status = CDFlib (GET_, BOO(Z,zENTRY_NUMSTRINGS_,rENTRY_NUMSTRINGS_),
+                                   &numStrings,
+                             NULL_);
+            if (status < CDF_OK)
+              return QuitCDF (CDFpath, "15.1", status, attrName);
+            if (numStrings < 2) {
+              ((char *)value)[numElemsX] = '\0';
+              if (checkPtr == NULL)
+                fprintf (fp, "\"%s\"\n", (char *)value);
+              else {
+                char *badStr = malloc((int)numElemsX + 1);
+                memset (badStr, '.', (size_t)numElemsX);
+                badStr[(int)numElemsX] = '\0';
+                fprintf (fp, "\"%s\"   (<= non-UTF8)\n", badStr);
+                free (badStr);
+                badStr = NULL;
+              }
+            } else {
+              char **strings; int ij, cnt;
+              status = CDFlib (GET_, BOO(Z,zENTRY_STRINGSDATA_,
+                                           rENTRY_STRINGSDATA_),
+                                     &numStrings, &strings,
+                               NULL_);
+              if (status < CDF_OK)
+                return QuitCDF (CDFpath, "15.3", status, attrName);
+              fprintf (fp, "(strings:%ld)\n", numStrings);
+              for (ij =0; ij < numStrings; ++ij) {
+                checkPtr = UTF8Check ((unsigned char *)strings[ij],
+			              (int)strlen(strings[ij]), &cnt);
+                if (checkPtr == NULL)
+                  fprintf (fp, "\"%s\"\n", strings[ij]);
+                else {
+                  size_t len = strlen(strings[ij]);
+                  char *badStr;
+                  badStr = malloc(len + 1);
+                  memset (badStr, '.', len);
+                  badStr[(int)len] = '\0';
+                  fprintf (fp, "\"%s\"   (<= non-UTF8)\n", badStr);
+                  free (badStr);
+                  badStr = NULL;
+                }
+              }
+              CDF_Free_String (numStrings, strings);
+            }
           } else {
             int style;
-            if (TT2000dataType(dataTypeX)) style = TT2000_3_STYLE;
-            else style = EPOCH0_STYLE;
+            if (TT2000dataType(dataTypeX)) {
+              if (!ziso8601) style = TT2000_3_STYLE;
+              else style = TT2000_4_STYLE;
+            } else {
+              if (!ziso8601) style = DEFAULTstyleDUMP;
+              else style = EPOCH3_STYLE;
+            }
             if (numElemsX > 1) {
               int ip;
               int ix = CDFelemSize(dataTypeX);
               for (ip = 0; ip < numElemsX; ++ip) {
-                EncodeValue (dataTypeX, ((char *) value)+ix*ip, tText, style,
-                             (size_t) sizeof(tText));
+                EncodeValueFormat (dataTypeX, ((char *) value)+ix*ip, tText,
+                                   prtFormat, -1, 0, style,
+                                   (size_t) sizeof(tText));
                 fprintf (fp, "%s", tText);
                 if (ip != (numElemsX -1)) fprintf (fp, ", ");
               }
               fprintf (fp, "\n");
             } else {
-              if (strcmpIgCase(attrName,"fillval") == 1 ||
-                  strcmpIgCase(attrName,"validmin") == 1 ||
-                  strcmpIgCase(attrName,"validmax") == 1) {
-                if (dataType == dataTypeX || (dataType == CDF_EPOCH ||
-                                              dataType == CDF_EPOCH16 ||
-                                              dataType == CDF_TIME_TT2000))
-                  EncodeValue (dataType, value, tText, style,
-                               (size_t) sizeof(tText));
-                else
-                  EncodeValue (dataTypeX, value, tText, style,
-                               (size_t) sizeof(tText));
-              } else
+              if (dataTypeX == CDF_EPOCH || dataTypeX == CDF_EPOCH16 ||
+                  dataTypeX == CDF_TIME_TT2000)
                 EncodeValue (dataTypeX, value, tText, style,
                              (size_t) sizeof(tText));
+              else
+                EncodeValueFormat (dataTypeX, value, tText, prtFormat, -1, 0,
+                                   style, (size_t) sizeof(tText));
               fprintf (fp, "%s\n", tText);
             }
           }
@@ -872,8 +993,13 @@ switch (argC) {
         }
       }
     } 
-
-    if ((dump != METADATADUMP && dump != VARIABLEDUMP) || detect) {
+    if (maxRec == -1) {
+      if (prtFormat != NULL) cdf_FreeMemory (prtFormat, FatalError);
+      if (!detect) fprintf (fp, "\n");
+      continue;
+    }
+    if ((dump != METADATADUMP && dump != VARIABLEDUMP && dump != NODATADUMP) ||
+        detect) {
       if (!detect) fprintf (fp, "Variable Data:\n");
       for (i = 0, nValuesPerRec = 1, nLogValuesPerRec = 1; i < numDims; i++) {
         if (dimVarys[i]) nValuesPerRec *= dimSizes[i];
@@ -894,52 +1020,55 @@ switch (argC) {
       nValueBytes[0] = (size_t) numElems * CDFelemSize(dataType);
       if (CDFepochDataType(dataType) && epochVV && virtual && !detect) {
         int style;
-        if (TT2000dataType(dataType)) style = TT2000_3_STYLE;
-        else style = EPOCH0_STYLE;
+        if (TT2000dataType(dataType)) {
+          if (!ziso8601) style = TT2000_3_STYLE;
+          else style = TT2000_4_STYLE;
+        } else {
+          if (!ziso8601) style = DEFAULTstyleDUMP;
+          else style = EPOCH3_STYLE;
+        }
         buffer = (Byte1 *) HandleVirtual((long)ii, Z, &maxRec);
         if (buffer != NULL) {
           offset = 0;
           for (k = 0, value = buffer; k <= maxRec;
                k += 1, value = (char *) value + CDFelemSize(dataType)) {
-            EncodeValue (dataType, value, tText, style,
-	                 (size_t) sizeof(tText));
+            EncodeValueFormat (dataType, value, tText, prtFormat, -1, 0, style,
+	                       (size_t) sizeof(tText));
             fprintf (fp, "  Record # %d: %s\n", (k+1), tText);
           }
         }
       } else {
+	long toRecs, ist, ien;
+        ist = startingRec - 1;
+        ien = endingRec - 1;
+        if (ist == -1) ist = 0;
+        if (ien == -1) ien = maxRec;
+        toRecs = ien - ist + 1;
+        if (ist > maxRec && detect) continue;
         if (sp == NO_SPARSERECORDS) {
-          long toRecs, ist, ien;
-          ist = startingRec - 1;
-          ien = endingRec - 1;
           handles[0] = &buffer;
-          if (ist != -1 && ist > maxRec) {
-            if (!detect) fprintf (fp, "\n");
-            continue;
-          }
-          if (ien != -1 && ien > maxRec) ien = maxRec;
-          if (ist == -1) 
-            toRecs = maxRec + 1;
-          else
-            toRecs = (ien == -1 ? maxRec : ien) - ist + 1;
           AllocateBuffers (toRecs, numDims, dimSizes, &groups, 0, 1, handles,
-                           nValueBytes, ROWmajor(majority), 5, FatalError);
-          InitHyperParmsFromAnywhere (&hyperX, &groups, numDims, &nHypers, 
-                                      &nValues, (ist == -1 ? 0 : ist));
+                           nValueBytes, ROWmajor(majority), 1, FatalError);
+          InitHyperParmsFromAnywhere (&hyperSt, &groups, numDims, &nHypers, 
+                                      &nValues, ist);
           if (STRINGdataType(dataType)) imore = 1;
           else imore = 0;
           nBytes = (size_t) (CDFelemSize(dataType) * (numElems + imore));
           working = cdf_AllocateMemory ((size_t) nBytes, FatalError);
-          if (toCol2Row) value2 = malloc((size_t)nLogValuesPerRec*nValueBytes[0]);
+          value2 = NULL;
+          if (toCol2Row)
+            value2 = malloc((size_t)nLogValuesPerRec*nValueBytes[0]);
           for (hyperN = 0; hyperN < nHypers; hyperN++) {
             for (j = 0; j < numDims; ++j)
-              if (!dimVarys[j]) hyperX.dimCounts[j] = 1;
-            status = HYPER_READ (id, Z, hyperX, buffer);
-            if (status < CDF_OK) return QuitCDF(CDFpath, "16.0", status, varName);
+              if (!dimVarys[j]) hyperSt.dimCounts[j] = 1;
+            status = HYPER_READ (id, Z, hyperSt, buffer);
+            if (status < CDF_OK)
+              return QuitCDF(CDFpath, "16.0", status, varName);
             if (!detect) {
               offset = 0;
               for (k = 0, value = buffer; k < nValues;
                    k += iskip, offset += (size_t) nValueBytes[0]) {
-                i = hyperX.recNumber + k / nLogValuesPerRec; 
+                i = hyperSt.recNumber + k / nLogValuesPerRec; 
                 j = k - k / nLogValuesPerRec * nLogValuesPerRec;
                 if (j == 0) {
                   icount = 0; 
@@ -958,38 +1087,61 @@ switch (argC) {
                   fprintf (fp, "\"%s\"", (char *)working);
                 } else {
                   int style;
-                  if (TT2000dataType(dataType)) style = TT2000_3_STYLE;
-                  else style = EPOCH0_STYLE;
+                  if (TT2000dataType(dataType)) {
+                    if (!ziso8601) style = TT2000_3_STYLE;
+                    else style = TT2000_4_STYLE;
+                  } else {
+                    if (!ziso8601) style = DEFAULTstyleDUMP;
+                    else style = EPOCH3_STYLE;
+                  }
                   if (prtFormat != NULL) {
-	      	    if ((dataType == CDF_FLOAT) || (dataType == CDF_REAL4)) {
+	      	    if (FLOAT4dataType(dataType)) {
                       if (!isnan((double)*(float *)working) &&
                           *(float *)working <= DEFAULT_FLOAT_PADVALUE) {
                         EncodeValueFormat (dataType, working, tText, NULL, 0, 0,
                                            style, (size_t) sizeof(tText));
                       } else
                         EncodeValueFormat (dataType, working, tText, prtFormat,
-                                           0, 0, style, (size_t) sizeof(tText));
-		    } else if ((dataType == CDF_DOUBLE) ||
-		  	       (dataType == CDF_REAL8)) {
+                                           -1, 0, style,
+                                           (size_t) sizeof(tText));
+		    } else if (DOUBLEdataType(dataType)) {
                       if (!isnan(*(double *)working) &&
                           *(double *)working <= DEFAULT_DOUBLE_PADVALUE) {
                         EncodeValueFormat (dataType, working, tText, NULL, 0, 0,
                                            style, (size_t) sizeof(tText));
+                      } else
+                        EncodeValueFormat (dataType, working, tText, prtFormat, 
+                                           -1, 0, style,
+                                           (size_t) sizeof(tText));
+		    } else if (dataType == CDF_EPOCH) {
+                      if (!isnan(*(double *)working) &&
+                          *(double *)working <= DEFAULT_EPOCH_PADVALUE) {
+                        EncodeValueFormat (dataType, working, tText, NULL, 0, 0,
+                                           style, (size_t) sizeof(tText));
+                      } else
+                        EncodeValueFormat (dataType, working, tText, prtFormat, 
+                                           0, 0, style, (size_t) sizeof(tText));
+		    } else if (dataType == CDF_EPOCH16) {
+                      if ((!isnan(*(double *)working)) &&
+                          (!isnan(*((double *)working)+1)) &&
+                          *(double *)working <= DEFAULT_EPOCH16_PADVALUE &&
+                          *(((double *)working)+1) <= DEFAULT_EPOCH16_PADVALUE){
+                        EncodeValueFormat (dataType, working, tText, NULL, 0, 0,
+                                           style, (size_t) sizeof(tText));
                       } else
                         EncodeValueFormat (dataType, working, tText, prtFormat, 
                                            0, 0, style, (size_t) sizeof(tText));
-
 		    } else
-                      EncodeValueFormat (dataType, working, tText, prtFormat, 0,
-                                         0, style, (size_t) sizeof(tText));
+                      EncodeValueFormat (dataType, working, tText, prtFormat,
+                                         -1, 0, style, (size_t) sizeof(tText));
                   } else {
                     if (useFormat && (precision > 0) && 
-                        (dataType == CDF_FLOAT || dataType == CDF_REAL4 ||
-                         dataType == CDF_DOUBLE || dataType == CDF_REAL8)) {
+                        (FLOAT4dataType(dataType) ||
+                         DOUBLEdataType(dataType))) {
                       char newFormat[20+1];
                       snprintf (newFormat, 20, "%%.%dg", precision);
-                      EncodeValueFormat (dataType, working, tText, newFormat, 0,
-                                         0, style, (size_t) sizeof(tText));
+                      EncodeValueFormat (dataType, working, tText, newFormat,
+                                         -1, 0, style, (size_t) sizeof(tText));
                     } else
 		      EncodeValue (dataType, working, tText, style,
 		 	           (size_t) sizeof(tText));
@@ -997,16 +1149,25 @@ switch (argC) {
                   fprintf (fp, "%s", tText);
                 }
                 if (nValuesPerRec == 1) {
+                  if ((recVary && (status == VIRTUAL_RECORD_DATA &&
+                                   i > maxRec)) ||
+                      (!recVary && i > maxRec)) 
+                    fprintf (fp, "\t\t<== Virtual");
                   fprintf (fp, "\n");
                   continue;
                 }
                 icount++;
-                if (icount == nValuesPerRec) fprintf (fp, "]\n");
-                else fprintf (fp, ",");
-                if (toCol2Row && (icount == nValuesPerRec)) free (value2);
+                if (icount == nValuesPerRec) {
+                  fprintf (fp, "]");
+                  if ((recVary && (status == VIRTUAL_RECORD_DATA &&
+                                   i > maxRec)) ||
+                      (!recVary && i > maxRec)) 
+                    fprintf (fp, "\t\t<== Virtual");
+                  fprintf (fp, "\n");
+                } else fprintf (fp, ",");
               }
 	    }
-            IncrHyperParms (&hyperX, &groups, numDims, ROWmajor(majority),
+            IncrHyperParms (&hyperSt, &groups, numDims, ROWmajor(majority),
                             &nValues);
           }
         } else {
@@ -1027,20 +1188,22 @@ switch (argC) {
                                    1L,
                                    BOO(Z,zVAR_DIMINDICES_,rVARs_DIMINDICES_), 
                                    dimIndices,
-                                   BOO(Z,zVAR_DIMINTERVALS_,rVARs_DIMINTERVALS_), 
+                                   BOO(Z,zVAR_DIMINTERVALS_,
+					 rVARs_DIMINTERVALS_), 
                                    dimIntervals,
                                    BOO(Z,zVAR_DIMCOUNTS_,rVARs_DIMCOUNTS_), 
                                    dimCounts,
                           NULL_);
           if (status < CDF_OK) return QuitCDF(CDFpath, "17.0", status, varName);
-          if (toCol2Row) value2 = malloc((size_t)nLogValuesPerRec*nValueBytes[0]);
-          for (k = 0; k < (int) (maxRec+1); k++) {
+          if (toCol2Row)
+            value2 = malloc((size_t)nLogValuesPerRec*nValueBytes[0]);
+          for (k = (int) ist; k <= (int) ien; k++) {
             status = CDFlib(SELECT_, BOO(Z,zVAR_RECNUMBER_,rVARs_RECNUMBER_), 
                                      (long) k,
                             GET_, BOO(Z,zVAR_HYPERDATA_,rVAR_HYPERDATA_), buffer,
                             NULL_);
-/*          if (status == VIRTUAL_RECORD_DATA) continue; */
-            if (status < CDF_OK) return QuitCDF(CDFpath, "18.0", status, varName);
+            if (status < CDF_OK)
+              return QuitCDF(CDFpath, "18.0", status, varName);
 	    if (!detect) {
               offset = 0;
               for (ir = 0, value = buffer; ir < nLogValuesPerRec;
@@ -1062,37 +1225,61 @@ switch (argC) {
                   fprintf (fp, "\"%s\"", (char *)working);
                 } else {
                   int style;
-                  if (TT2000dataType(dataType)) style = TT2000_3_STYLE;
-                  else style = EPOCH0_STYLE;
+                  if (TT2000dataType(dataType)) {
+                    if (!ziso8601) style = TT2000_3_STYLE;
+                    else style = TT2000_4_STYLE;
+                  } else {
+                    if (!ziso8601) style = DEFAULTstyleDUMP;
+                    else style = EPOCH3_STYLE;
+                  }
                   if (prtFormat != NULL) {
-                    if ((dataType == CDF_FLOAT) || (dataType == CDF_REAL4)) {
+                    if (FLOAT4dataType(dataType)) {
                       if (!isnan((double)*(float *)working) &&
                           *(float *)working <= DEFAULT_FLOAT_PADVALUE) {
                         EncodeValueFormat (dataType, working, tText, NULL, 0, 0,
                                            style, (size_t) sizeof(tText));
                       } else
                         EncodeValueFormat (dataType, working, tText, prtFormat,
-                                           0, 0, style, (size_t) sizeof(tText));
-                    } else if ((dataType == CDF_DOUBLE) ||
-		  	       (dataType == CDF_REAL8)) {
+                                           -1, 0, style,
+                                           (size_t) sizeof(tText));
+                    } else if (DOUBLEdataType(dataType)) {
                       if (!isnan(*(double *)working) &&
                           *(double *)working <= DEFAULT_DOUBLE_PADVALUE) {
                         EncodeValueFormat (dataType, working, tText, NULL, 0, 0,
                                            style, (size_t) sizeof(tText));
+                      } else
+                        EncodeValueFormat (dataType, working, tText, prtFormat,
+                                           -1, 0, style,
+                                           (size_t) sizeof(tText));
+                    } else if (dataType == CDF_EPOCH) {
+                      if (!isnan(*(double *)working) &&
+                          *(double *)working <= DEFAULT_EPOCH_PADVALUE) {
+                        EncodeValueFormat (dataType, working, tText, NULL, 0, 0,
+                                           style, (size_t) sizeof(tText));
+                      } else
+                        EncodeValueFormat (dataType, working, tText, prtFormat,
+                                           0, 0, style, (size_t) sizeof(tText));
+                    } else if (dataType == CDF_EPOCH16) {
+                      if (!isnan(*(double *)working) &&
+                          !isnan(*(((double *)working)+1)) &&
+                          *(double *)working <= DEFAULT_EPOCH16_PADVALUE &&
+                          *(((double *)working)+1) <= DEFAULT_EPOCH16_PADVALUE){
+                        EncodeValueFormat (dataType, working, tText, NULL, 0, 0,
+                                           style, (size_t) sizeof(tText));
                       } else
                         EncodeValueFormat (dataType, working, tText, prtFormat,
                                            0, 0, style, (size_t) sizeof(tText));
                     } else
-                      EncodeValueFormat (dataType, working, tText, prtFormat, 0,
-                                         0, style, (size_t) sizeof(tText));
+                      EncodeValueFormat (dataType, working, tText, prtFormat,
+                                         -1, 0, style, (size_t) sizeof(tText));
                   } else {
                     if (useFormat && (precision > 0) && 
-                        (dataType == CDF_FLOAT || dataType == CDF_REAL4 ||
-                         dataType == CDF_DOUBLE || dataType == CDF_REAL8)) {
+                        (FLOAT4dataType(dataType) ||
+                         DOUBLEdataType(dataType))) {
                       char newFormat[20+1];
                       snprintf (newFormat, 20, "%%.%dg", precision);
-                      EncodeValueFormat (dataType, working, tText, newFormat, 0,
-                                         0, style, (size_t) sizeof(tText));
+                      EncodeValueFormat (dataType, working, tText, newFormat,
+                                         -1, 0, style, (size_t) sizeof(tText));
                     } else
 		      EncodeValue (dataType, working, tText, style,
 		  	           (size_t) sizeof(tText));
@@ -1115,9 +1302,12 @@ switch (argC) {
               }
             }
   	  }
-          if (toCol2Row) free (value2);
         } 
       }
+      if (toCol2Row && value2 != NULL) {
+        free (value2);
+        value2 = NULL;
+      }
       if (!detect) fprintf (fp, "\n");
       if (buffer != NULL) cdf_FreeMemory (buffer, FatalError);
       if (working != NULL) cdf_FreeMemory (working, FatalError);
@@ -1134,8 +1324,8 @@ switch (argC) {
   if (numVars > 0) {
     for (k = 0; k < numVars; ++k) 
        cdf_FreeMemory (vars[k], FatalError);
+    cdf_FreeMemory (vars, FatalError);
   }
-
   Exit;
 
 }
@@ -1151,19 +1341,20 @@ CDFstatus status;
 char *msg;
 {
   char text[CDF_STATUSTEXT_LEN+1];
-  printf ("Program failed for file: %s at %s...\n", file, where);
+  fprintf (fp, "Program failed for file: %s at %s...\n", file, where);
   if (status < CDF_OK) {
     CDFlib (SELECT_, CDF_STATUS_, status,
             GET_, STATUS_TEXT_, text,
             NULL_);
-    printf ("ERROR> %s ", text);
-    if (msg != NULL) printf ("(%s)\n", msg);
-    else printf("\n");
+    fprintf (fp, "ERROR> %s ", text);
+    if (msg != NULL) fprintf (fp, "(%s)\n", msg);
+    else fprintf(fp, "\n");
   }
   CDFlib (CLOSE_, CDF_,
           NULL_);
-  printf ("\n");
-  return FALSE;
+  fprintf (fp, "\n");
+  if (fp != stdout) fclose(fp);
+  ExitBAD;
 }
 
 /******************************************************************************
@@ -1346,7 +1537,7 @@ long *maxRec;
   char varName1[CDF_VAR_NAME_LEN256+1], varName2[CDF_VAR_NAME_LEN256+1];
   long recVary1, recVary2, dimVarys1[CDF_MAX_DIMS], dimVarys2[CDF_MAX_DIMS];
   long numDims1, numDims2;
-  char units1[10], units2[10];
+  char units1[21], units2[21];
   double scale;
   int  i;
 
@@ -1592,4 +1783,3 @@ double EpochScale(long dataType1, char *units) {
   return 1.0;
 }
 
-
