@@ -52,6 +52,10 @@ extern unsigned _stklen = BORLANDC_STACK_SIZE;
 extern unsigned _ovrbuffer = BORLANDC_OVERLAY_SIZE;
 #endif
 
+#ifdef WIN32
+  static unsigned int oldCodePage = 0;
+#endif
+
 /******************************************************************************
 * Global variables local to this source file.
 ******************************************************************************/
@@ -73,10 +77,8 @@ MAIN {
 #else
   success = EditCDFs (argc, argv);
 #endif
-#if defined(DEBUG)
-  if (cdf_FreeMemory(NULL,FatalError) > 0) DisplayWarning ("Abandoned buffers.");
-#else
-  cdf_FreeMemory (NULL, FatalError);
+#ifdef WIN32
+  if (oldCodePage != 0) SetConsoleOutputCP(oldCodePage);
 #endif
   return BOO(success,EXIT_SUCCESS_,EXIT_FAILURE_);
 }
@@ -95,11 +97,11 @@ char *argV[];
      "browse", "nobrowse", "zmode", "format", "noformat", "prompt",
      "noprompt", "report", "neg2posfp0", "noneg2posfp0", "cache",
      "statistics", "nostatistics", "gwithentries", "nogwithentries",
-     "vwithentries", "novwithentries", "about", "ziso8601", NULL
+     "vwithentries", "novwithentries", "about", "ziso8601", "novalidate", NULL
    };
    static int optRequired[] = {
      FALSE, FALSE, TRUE, FALSE, FALSE, FALSE, FALSE, TRUE, FALSE, FALSE, TRUE,
-     FALSE, FALSE, FALSE, FALSE, FALSE, FALSE, FALSE, FALSE, 0
+     FALSE, FALSE, FALSE, FALSE, FALSE, FALSE, FALSE, FALSE, FALSE, 0
    };
    static char *reportTokens[] = { "errors", "warnings", "informationals" };
    char CDFspec[DU_MAX_PATH_LEN+1];
@@ -219,13 +221,21 @@ char *argV[];
 	 report[INFOs] = REPORTinfosDEFAULT;
        }
        /************************************************************************
-       * Check for /ZISO8601,-ziso8601 qualifier.
+       * Check for `ziso8601' qualifier.
        ************************************************************************/
        if (qop->qualEntered[ZISO8601qual]) {
          ziso8601 = TRUE;
        }
        else
          ziso8601 = FALSE;
+       /************************************************************************
+       * Check for `novalidate' qualifier.
+       ************************************************************************/
+       if (qop->qualEntered[NOVALIDATEqual]) {
+         toValidate = FALSE;
+       }
+       else
+         toValidate = TRUE;
        /***********************************************************************
        * Check for missing/conflicting parameters/qualifiers.
        ***********************************************************************/
@@ -240,6 +250,13 @@ char *argV[];
        if (qopError) return FALSE;
        break;
    }
+#ifndef WIN32
+   setlocale(LC_CTYPE, "en_US.UTF-8");
+#else
+   setlocale(LC_CTYPE, ".UTF8");
+   oldCodePage = GetConsoleOutputCP();
+   SetConsoleOutputCP(65001);
+#endif
    /***************************************************************************
    * Initialize.
    ***************************************************************************/
@@ -255,10 +272,10 @@ char *argV[];
    setenv ("TERM", "vt100", 1);
 #endif
    InitializeScreen ();
+
    /***************************************************************************
    * Edit CDF(s).
    ***************************************************************************/
-/*   CDFsetValidate (VALIDATEFILEon); */
    if (promptForSpec)
      status = SpecificationPrompt (CDFspec, useFormat, negToPosFp0, zMode,
 				   workingCache, stageCache, compressCache);
@@ -350,8 +367,8 @@ long compressCache;
     ENTERkey_FSI, EXITkey_FSI, HELPkey_FSI, INFOkey_EDIT, NUL
   };
   static struct ItemWindowStruct IW = {
-     0, 0, 80, label, 1, header, 0, NULL, 0, NULL, NULL, NULL, 0, 0, NULL,
-     NULL, REFRESHkey_FSI, TRUE, NSkey_FSI, PSkey_FSI
+     0, 0, 80, label, 1, header, 0, NULL, 0, NULL, NULL, NULL, NULL, 0, 0,
+     NULL, NULL, REFRESHkey_FSI, TRUE, NSkey_FSI, PSkey_FSI
   };
   AOSs6 (infoLines, BLANKs78, BLANKs78, BLANKs78,
 	 " ", "Enter any key to continue...", "")
@@ -402,7 +419,7 @@ long compressCache;
 	   ProblemWindow (noneMsg, FALSE);
 	 }
 	 else {
-	   CDFid id; int lastItemN;
+	   int lastItemN;
 	   Logical isLFS;
 	   strcpyX (CDFpathT, dirS[IW.itemN], DU_MAX_PATH_LEN);
 	   AppendToDir (CDFpathT, CDFs[IW.itemN]);
@@ -412,6 +429,7 @@ long compressCache;
 	     MessageWindow (openingLines, NULL, LogicalFALSE);
 	     zzzzz (1.0);
 	   }
+	   if (!toValidate) CDFsetValidate (VALIDATEFILEoff);
 	   status = CDFlib (OPEN_, CDF_, CDFfullPath, &id,
 			    NULL_);
 	   if (compressed) MessageWindow (NULL);
@@ -537,7 +555,7 @@ long compressCache;
 	   snprintf (question, (size_t) sizeof(question),
 		     "Delete CDF %c%s%c ?", delim, CDFs[IW.itemN], delim);
 	   if (ConfirmWindow(3,78,question,NULL,FALSE,DELETECDFhelpID)) {
-	     CDFid id; CDFstatus status; int lastItemN;
+	     CDFstatus status; int lastItemN;
 	     Logical isLFS;
 	     strcpyX (CDFpathT, dirS[IW.itemN], DU_MAX_PATH_LEN);
 	     AppendToDir (CDFpathT, CDFs[IW.itemN]);
@@ -547,6 +565,7 @@ long compressCache;
 	       MessageWindow (delLines, NULL, LogicalFALSE);
 	       zzzzz (1.0);
 	     }
+	     if (!toValidate) CDFsetValidate (VALIDATEFILEoff);
 	     status = CDFlib (OPEN_, CDF_, CDFfullPath, &id,
 			      DELETE_, CDF_,
 			      NULL_);
@@ -601,6 +620,7 @@ struct ItemWindowStruct *IW;
        IW->iLineNs[CDFn] = CDFn;
        IW->iCols[CDFn] = 0;
        IW->iLens[CDFn] = (int) MINIMUM(CDFs_NAME_LEN,nameLen);
+       IW->ibLens[CDFn] = (int) MINIMUM(CDFs_NAME_LEN,nameLen);
     }
   }
   else {
@@ -639,7 +659,7 @@ long workingCache;
 long stageCache;
 long compressCache;
 {
-  CDFstatus status; CDFid id;
+  CDFstatus status;
   char CDFdir[DU_MAX_DIR_LEN+1], CDFname[DU_MAX_NAME_LEN+1];
   char CDFfullName[DU_MAX_NAME_LEN+1];
   Logical isLFS;
@@ -653,6 +673,7 @@ long compressCache;
       zzzzz (1.0);
     }
   }
+  if (!toValidate) CDFsetValidate (VALIDATEFILEoff);
   status = CDFlib (OPEN_, CDF_, CDFfullName, &id,
 		   NULL_);
   if (compressed) MessageWindow (NULL);
@@ -714,7 +735,7 @@ struct ItemWindowStruct *IWcdfs;
 			/* Pointer to "CDFs Menu" widget.  If NULL, then this
 			   routine was not called from the "CDFs Menu". */
 {
-  CDFstatus status; CDFid id; static Logical first = TRUE;
+  CDFstatus status; static Logical first = TRUE;
   long numDims, dimSizes[CDF_MAX_DIMS];
   char CDFpathT[DU_MAX_PATH_LEN+1], delim;
   AOSs1 (trailer, "Enter: ________   Exit: ________   Help: ________")
@@ -794,7 +815,7 @@ struct ItemWindowStruct *IWcdfs;
 	 break;
        }
        else
-	 ProblemWindow ("Illegal dimensionality.", FALSE);
+	 ProblemWindow ("Illegal or bad dimensionality.", FALSE);
      }
      else {
        PromptWindow (DELETEpw, &PWdim);
@@ -890,7 +911,7 @@ Logical *closed;
      zMODE0key_EDIT, zMODE1key_EDIT, zMODE2key_EDIT, NUL };
    static char label[6+DU_MAX_NAME_LEN+2+1];
    static struct ItemWindowStruct IW = {
-      0, 0, 80, label, 3, hLines, 0, NULL, 0, NULL, NULL, NULL, 4,
+      0, 0, 80, label, 3, hLines, 0, NULL, 0, NULL, NULL, NULL, NULL, 4,
       1, NULL, NULL, REFRESHkey_FSI, FALSE, NUL, NUL
    };
    /***************************************************************************
@@ -910,6 +931,7 @@ Logical *closed;
      IW.iLineNs = iLineNsBrowse;
      IW.iCols = iColsBrowse;
      IW.iLens = iLensBrowse;
+     IW.ibLens = iLensBrowse;
      IW.tLines = tLinesBrowse;
      IW.exitChars = exitCharsBrowse;
    }
@@ -920,6 +942,7 @@ Logical *closed;
      IW.iLineNs = iLineNsEdit;
      IW.iCols = iColsEdit;
      IW.iLens = iLensEdit;
+     IW.ibLens = iLensEdit;
      IW.tLines = tLinesEdit;
      IW.exitChars = exitCharsEdit;
    }
@@ -930,7 +953,6 @@ Logical *closed;
    status = CDFlib (SELECT_, CDF_CACHESIZE_, workingCache,
 			     STAGE_CACHESIZE_, stageCache,
 			     COMPRESS_CACHESIZE_, compressCache,
-			     CDF_READONLY_MODE_, readonlyMode,
 			     CDF_zMODE_, zMode,
 			     CDF_NEGtoPOSfp0_MODE_, negToPosFp0Mode,
 		    NULL_);
@@ -980,13 +1002,12 @@ Logical *closed;
 	   case BROWSEgAttrsIN:
 	     ItemWindow (UNDISPLAYiw, &IW);
 	     if (gAttrsAndEntries) {
-	       if (!EditAttrs(TRUE,CDFname)) {
+	       if (!EditAttrs(TRUE,CDFname,LogicalFALSE)) {
 		 ItemWindow (DELETEiw, &IW);
 		 return FALSE;
 	       }
-	     }
-	     else {
-	       if (!EditAttrsX(TRUE,CDFname)) {
+	     } else {
+	       if (!EditAttrsX(TRUE,CDFname,LogicalFALSE)) {
 		 ItemWindow (DELETEiw, &IW);
 		 return FALSE;
 	       }
@@ -1005,13 +1026,12 @@ Logical *closed;
 	   case BROWSEvAttrsIN:
 	     ItemWindow (UNDISPLAYiw, &IW);
 	     if (vAttrsAndEntries) {
-	       if (!EditAttrs(FALSE,CDFname)) {
+	       if (!EditAttrs(FALSE,CDFname,useFormat)) {
 		 ItemWindow (DELETEiw, &IW);
 		 return FALSE;
 	       }
-	     }
-	     else {
-	       if (!EditAttrsX(FALSE,CDFname)) {
+	     } else {
+	       if (!EditAttrsX(FALSE,CDFname,useFormat)) {
 		 ItemWindow (DELETEiw, &IW);
 		 return FALSE;
 	       }
@@ -1104,115 +1124,115 @@ Logical *closed;
        * Delete CDF.
        ***********************************************************************/
        case DELETECDFkey_EDIT:
-		 if (!browseOnly) {
-			char delim, question[DU_MAX_NAME_LEN+15+1];
-			delim = PickDelimiter (CDFname, strlen(CDFname));
-	   		snprintf (question, (size_t) sizeof(question),
-				  "Delete CDF %c%s%c ?", delim, CDFname, delim);
-	   		if (ConfirmWindow(12,80,question,NULL,FALSE,DELETECDFhelpID)) {
-	     	  status = CDFlib (DELETE_, CDF_,
-			  			       NULL_);
-	     	  if (ReportStatus(status,FALSE)) {
-				ItemWindow (DELETEiw, &IW);
-	       		*closed = TRUE;
-	       		return TRUE;
-	     	  }
-	     	  if (NoMoreAccess(NULL)) {
-	       		ItemWindow (DELETEiw, &IW);
-	       		return FALSE;
-	     	  }
-	   		}
-	 	  }
-	 	else
-	   	  ProblemWindow ("Deleting a CDF not allowed while browsing.", FALSE);
-	 	break;
+	 if (!browseOnly) {
+	   char delim, question[DU_MAX_NAME_LEN+15+1];
+	   delim = PickDelimiter (CDFname, strlen(CDFname));
+	   snprintf (question, (size_t) sizeof(question),
+		     "Delete CDF %c%s%c ?", delim, CDFname, delim);
+	   if (ConfirmWindow(12,80,question,NULL,FALSE,DELETECDFhelpID)) {
+	     status = CDFlib (DELETE_, CDF_,
+	 		      NULL_);
+	     if (ReportStatus(status,FALSE)) {
+	         ItemWindow (DELETEiw, &IW);
+	       	*closed = TRUE;
+	       	return TRUE;
+	     }
+	     if (NoMoreAccess(NULL)) {
+	       ItemWindow (DELETEiw, &IW);
+	       return FALSE;
+	     }
+	   }
+	 }
+	 else
+	   ProblemWindow ("Deleting a CDF not allowed while browsing.", FALSE);
+	 break;
        /***********************************************************************
        * zMode selection keys.
        ***********************************************************************/
        case zMODE0key_EDIT:
        case zMODE1key_EDIT:
        case zMODE2key_EDIT: {
-	 	 long zMode;
-	 	 switch (IW.key) {
-	   		case zMODE0key_EDIT: zMode = zMODEoff; break;
-	   		case zMODE1key_EDIT: zMode = zMODEon1; break;
-	   		case zMODE2key_EDIT: zMode = zMODEon2; break;
-	 	 }
-	 	 status = CDFlib (SELECT_, CDF_zMODE_, zMode,
-						  NULL_);
-	 	 if (!ReportStatus(status,FALSE)) {
-	   	   ItemWindow (DELETEiw, &IW);
-	   	   return FALSE;
-	 	 }
-	 	 if (!BuildCDFmenu(CDFname,&IW)) {
-	 	   ItemWindow (DELETEiw, &IW);
-	 	   return FALSE;
-	 	 }
-	 	 ItemWindow (UPDATEiw, &IW, IW.itemN);
-	 	 break;
+	 long zMode;
+	 switch (IW.key) {
+	   case zMODE0key_EDIT: zMode = zMODEoff; break;
+	   case zMODE1key_EDIT: zMode = zMODEon1; break;
+	   case zMODE2key_EDIT: zMode = zMODEon2; break;
+	  }
+	  status = CDFlib (SELECT_, CDF_zMODE_, zMode,
+			   NULL_);
+	  if (!ReportStatus(status,FALSE)) {
+	      ItemWindow (DELETEiw, &IW);
+	      return FALSE;
+	 }
+	 if (!BuildCDFmenu(CDFname,&IW)) {
+	   ItemWindow (DELETEiw, &IW);
+	   return FALSE;
+	 }
+	 ItemWindow (UPDATEiw, &IW, IW.itemN);
+	 break;
        }
        /***********************************************************************
        * Display online help.
        ***********************************************************************/
        case HELPkey_FSI:
-		 OnlineHelpWindow (ilhFile, CDFhelpID);
-	 	 break;
+	 OnlineHelpWindow (ilhFile, CDFhelpID);
+	 break;
        /***********************************************************************
        * Exit menu (closing CDF).
        ***********************************************************************/
        case EXITkey_FSI: {
-	 	 if (dumpStatistics) {
-	 	   vSTATS vStatsDotCDF, vStatsStage, vStatsCompress;
-	   	   char temp1[MAX_SCREENLINE_LEN+1],
-				temp2[MAX_SCREENLINE_LEN+1],
-				temp3[MAX_SCREENLINE_LEN+1];
-	   	   if (compressed) {
-	     	 MessageWindow (closingLines, NULL, LogicalFALSE);
-	    	 zzzzz (1.0);
-	  	   }
-	   	   status = CDFlib (CLOSE_, CDFwithSTATS_, &vStatsDotCDF,
-							&vStatsStage,
-							&vStatsCompress,
-						    NULL_);
-	   		if (compressed) MessageWindow (NULL);
-	   		*closed = TRUE;
-	   		if (!ReportStatus(status,FALSE)) {
-	     	  ItemWindow (DELETEiw, &IW);
-	     	  return FALSE;
-	   		}
-	   		ItemWindow (DELETEiw, &IW);
-	   		if (vStatsDotCDF.maxBuffers > 0) {
-	     	  BuildStatistics ("DotCDF file", &vStatsDotCDF,
-			  			       temp1, temp2, temp3);
-	     	  InfoWindow (temp1, temp2, temp3, TRUE, FALSE, 0);
-	   		}
-	   		if (vStatsStage.maxBuffers > 0) {
-	     	  BuildStatistics ("staging file", &vStatsStage,
-			  				    temp1, temp2, temp3);
-	     	  InfoWindow (temp1, temp2, temp3, TRUE, FALSE, 0);
-	   		}
-	   		if (vStatsCompress.maxBuffers > 0) {
-	    	  BuildStatistics ("compression scratch file", &vStatsCompress,
-			 				     temp1, temp2, temp3);
-	    	  InfoWindow (temp1, temp2, temp3, TRUE, FALSE, 0);
-	   		}
-	 	  }
-	 	  else {
-	   		if (compressed) {
-	     		MessageWindow (closingLines, NULL, LogicalFALSE);
-	     		zzzzz (1.0);
-	   		}
-	   		status = CDFlib (CLOSE_, CDF_,
-							 NULL_);
-	   		if (compressed) MessageWindow (NULL);
-	   		*closed = TRUE;
-	   		if (!ReportStatus(status,FALSE)) {
-	   		  ItemWindow (DELETEiw, &IW);
-	   		  return FALSE;
-	   		}
-	 		ItemWindow (DELETEiw, &IW);
-	 	  }
-	 	  return TRUE;
+	 if (dumpStatistics) {
+	   vSTATS vStatsDotCDF, vStatsStage, vStatsCompress;
+	   char temp1[MAX_SCREENLINE_LEN+1],
+		temp2[MAX_SCREENLINE_LEN+1],
+		temp3[MAX_SCREENLINE_LEN+1];
+	   if (compressed) {
+	     MessageWindow (closingLines, NULL, LogicalFALSE);
+	     zzzzz (1.0);
+	   }
+	   status = CDFlib (CLOSE_, CDFwithSTATS_, &vStatsDotCDF,
+						   &vStatsStage,
+						   &vStatsCompress,
+			    NULL_);
+	   if (compressed) MessageWindow (NULL);
+	   *closed = TRUE;
+	   if (!ReportStatus(status,FALSE)) {
+     	   ItemWindow (DELETEiw, &IW);
+     	   return FALSE;
+  	   }
+	   ItemWindow (DELETEiw, &IW);
+   	   if (vStatsDotCDF.maxBuffers > 0) {
+     	     BuildStatistics ("DotCDF file", &vStatsDotCDF,
+	  		      temp1, temp2, temp3);
+     	     InfoWindow (temp1, temp2, temp3, TRUE, FALSE, 0);
+   	   }
+   	   if (vStatsStage.maxBuffers > 0) {
+     	     BuildStatistics ("staging file", &vStatsStage,
+			      temp1, temp2, temp3);
+     	     InfoWindow (temp1, temp2, temp3, TRUE, FALSE, 0);
+   	   }
+   	   if (vStatsCompress.maxBuffers > 0) {
+    	     BuildStatistics ("compression scratch file", &vStatsCompress,
+			      temp1, temp2, temp3);
+    	     InfoWindow (temp1, temp2, temp3, TRUE, FALSE, 0);
+    	   }
+ 	 }
+ 	 else {
+   	   if (compressed) {
+     	     MessageWindow (closingLines, NULL, LogicalFALSE);
+     	     zzzzz (1.0);
+   	   }
+   	   status = CDFlib (CLOSE_, CDF_,
+			    NULL_);
+   	   if (compressed) MessageWindow (NULL);
+   	   *closed = TRUE;
+   	   if (!ReportStatus(status,FALSE)) {
+   	     ItemWindow (DELETEiw, &IW);
+   	     return FALSE;
+   	   }
+ 	   ItemWindow (DELETEiw, &IW);
+ 	 }
+ 	 return TRUE;
        }
      }
    }
@@ -1487,84 +1507,84 @@ char **argV[];
       * Ok.
       ************************************************************************/
     case OKin: {
-		int n;
-		char tempS1[1+1];
-		/**********************************************************************
-		* Get the value of each control.
-		**********************************************************************/
-		GetIText ((Handle) controlHs[CDFTEXTin], CDFtext);
-		GetIText ((Handle) controlHs[CACHEin], cacheText);
-		browseOnly = GetCtlValue (controlHs[BROWSEin]);
-		useFormat = GetCtlValue (controlHs[FORMATin]);
-		negToPos = GetCtlValue (controlHs[NEGZin]);
-		promptFor = GetCtlValue (controlHs[PROMPTin]);
-		reportInfos = GetCtlValue (controlHs[INFOin]);
-		reportWarns = GetCtlValue (controlHs[WARNin]);
-		reportErrors = GetCtlValue (controlHs[ERRORin]);
-		dispStats = GetCtlValue (controlHs[STATSin]);
-		gWith = GetCtlValue (controlHs[gWITHin]);
-		vWith = GetCtlValue (controlHs[vWITHin]);
-		for (zMode = 0; zMode < 3; zMode++) {
-		   if (GetCtlValue(controlHs[ZMODEinBASE+zMode])) break;
-		}
-		/**********************************************************************
-		* Build argc/argv.
-		**********************************************************************/
-		*argC = 12 + BOO(NULpString(CDFtext),0,1) +
-			     BOO(NULpString(cacheText),0,2);
-		*argV = (char **) cdf_AllocateMemory (*argC * sizeof(char *), FatalError);
-		n = 0;
-		MAKEstrARGv (argV, n, pgmName)
-		MAKEbooARGv (argV, n, browseOnly, "-browse", "-nobrowse")
-		MAKEbooARGv (argV, n, useFormat, "-format", "-noformat")
-		MAKEbooARGv (argV, n, negToPos, "-neg2posfp0", "-noneg2posfp0")
-		MAKEbooARGv (argV, n, promptFor, "-prompt", "-noprompt")
-		MAKEbooARGv (argV, n, dispStats, "-statistics", "-nostatistics")
-		MAKEbooARGv (argV, n, gWith, "-gwithentries", "-nogwithentries")
-		MAKEbooARGv (argV, n, vWith, "-vwithentries", "-novwithentries")
-		MAKEstrARGv (argV, n, "-zmode")
-		snprintf (tempS1, (size_t) sizeof(tempS1), "%d", zMode);
-		MAKEstrARGv (argV, n, tempS1)
-		MAKEstrARGv (argV, n, "-report")
-		MAKEstrARGv (argV, n, StatusCodeReportOptions(reportErrors,
-						      reportWarns,
-						      reportInfos))
-		if (!NULpString(cacheText)) {
-	  	  MAKEstrARGv (argV, n, "-cache")
-	  	  PtoCstr (cacheText);
-	  	  MAKEstrARGv (argV, n, (char *) cacheText)
-	  	  CtoPstr ((char *) cacheText);
-		}
-		if (!NULpString(CDFtext)) {
-		  PtoCstr (CDFtext);
-		  MAKEstrARGv (argV, n, (char *) CDFtext)
-		  CtoPstr ((char *) CDFtext);
-		}
-		/**********************************************************************
-		* Close the dialog and return.
-		**********************************************************************/
+	int n;
+	char tempS1[1+1];
+	/**********************************************************************
+	* Get the value of each control.
+	**********************************************************************/
+	GetIText ((Handle) controlHs[CDFTEXTin], CDFtext);
+	GetIText ((Handle) controlHs[CACHEin], cacheText);
+	browseOnly = GetCtlValue (controlHs[BROWSEin]);
+	useFormat = GetCtlValue (controlHs[FORMATin]);
+	negToPos = GetCtlValue (controlHs[NEGZin]);
+	promptFor = GetCtlValue (controlHs[PROMPTin]);
+	reportInfos = GetCtlValue (controlHs[INFOin]);
+	reportWarns = GetCtlValue (controlHs[WARNin]);
+	reportErrors = GetCtlValue (controlHs[ERRORin]);
+	dispStats = GetCtlValue (controlHs[STATSin]);
+	gWith = GetCtlValue (controlHs[gWITHin]);
+	vWith = GetCtlValue (controlHs[vWITHin]);
+	for (zMode = 0; zMode < 3; zMode++) {
+	   if (GetCtlValue(controlHs[ZMODEinBASE+zMode])) break;
+	}
+	/**********************************************************************
+	* Build argc/argv.
+	**********************************************************************/
+	*argC = 12 + BOO(NULpString(CDFtext),0,1) +
+		     BOO(NULpString(cacheText),0,2);
+	*argV = (char **) cdf_AllocateMemory (*argC * sizeof(char *), FatalError);
+	n = 0;
+	MAKEstrARGv (argV, n, pgmName)
+	MAKEbooARGv (argV, n, browseOnly, "-browse", "-nobrowse")
+	MAKEbooARGv (argV, n, useFormat, "-format", "-noformat")
+	MAKEbooARGv (argV, n, negToPos, "-neg2posfp0", "-noneg2posfp0")
+	MAKEbooARGv (argV, n, promptFor, "-prompt", "-noprompt")
+	MAKEbooARGv (argV, n, dispStats, "-statistics", "-nostatistics")
+	MAKEbooARGv (argV, n, gWith, "-gwithentries", "-nogwithentries")
+	MAKEbooARGv (argV, n, vWith, "-vwithentries", "-novwithentries")
+	MAKEstrARGv (argV, n, "-zmode")
+	snprintf (tempS1, (size_t) sizeof(tempS1), "%d", zMode);
+	MAKEstrARGv (argV, n, tempS1)
+	MAKEstrARGv (argV, n, "-report")
+	MAKEstrARGv (argV, n, StatusCodeReportOptions(reportErrors,
+					      reportWarns,
+					      reportInfos))
+	if (!NULpString(cacheText)) {
+  	  MAKEstrARGv (argV, n, "-cache")
+  	  PtoCstr (cacheText);
+  	  MAKEstrARGv (argV, n, (char *) cacheText)
+  	  CtoPstr ((char *) cacheText);
+	}
+	if (!NULpString(CDFtext)) {
+	  PtoCstr (CDFtext);
+	  MAKEstrARGv (argV, n, (char *) CDFtext)
+	  CtoPstr ((char *) CDFtext);
+	}
+	/**********************************************************************
+	* Close the dialog and return.
+	**********************************************************************/
 #ifdef __MWERKS__
-		DisposeRoutineDescriptor(FilterDialogQOPfsiUPP);
-		DisposeRoutineDescriptor(OutlineDefaultButtonUPP);
+	DisposeRoutineDescriptor(FilterDialogQOPfsiUPP);
+	DisposeRoutineDescriptor(OutlineDefaultButtonUPP);
 #endif
-		CloseDialog (dialogP);
-		return TRUE;
+	CloseDialog (dialogP);
+	return TRUE;
     }
     /************************************************************************
     * Help.
     ************************************************************************/
     case HELPin: {
-		int n;
-		*argC = 1;
-		*argV = (char **) cdf_AllocateMemory (*argC * sizeof(char *), FatalError);
-		n = 0;
-		MAKEstrARGv (argV, n, pgmName)
+	int n;
+	*argC = 1;
+	*argV = (char **) cdf_AllocateMemory (*argC * sizeof(char *), FatalError);
+	n = 0;
+	MAKEstrARGv (argV, n, pgmName)
 #ifdef __MWERKS__
-		DisposeRoutineDescriptor(FilterDialogQOPfsiUPP);
-		DisposeRoutineDescriptor(OutlineDefaultButtonUPP);
+	DisposeRoutineDescriptor(FilterDialogQOPfsiUPP);
+	DisposeRoutineDescriptor(OutlineDefaultButtonUPP);
 #endif
-		CloseDialog (dialogP);
-		return TRUE;
+	CloseDialog (dialogP);
+	return TRUE;
     }
     /************************************************************************
     * Cancel.
@@ -1574,28 +1594,28 @@ char **argV[];
     	DisposeRoutineDescriptor(FilterDialogQOPfsiUPP);
     	DisposeRoutineDescriptor(OutlineDefaultButtonUPP);
 #endif
-		CloseDialog (dialogP);
-		return FALSE;
+	CloseDialog (dialogP);
+	return FALSE;
     /************************************************************************
     * Select CDF specification (existing CDF).
     ************************************************************************/
     case CDFSELECTin: {
-		StandardFileReply CDFreply;
-		char CDFpath[DU_MAX_PATH_LEN+1];
+	StandardFileReply CDFreply;
+	char CDFpath[DU_MAX_PATH_LEN+1];
 #ifndef __MWERKS__
-		StandardGetFile (FilterForCDFs, -1, NULL, &CDFreply);
+	StandardGetFile (FilterForCDFs, -1, NULL, &CDFreply);
 #else
-		FilterForCDFsUPP = NewFileFilterProc(FilterForCDFs);
-		StandardGetFile (FilterForCDFsUPP, -1, NULL, &CDFreply);
-		DisposeRoutineDescriptor(FilterForCDFsUPP);
+	FilterForCDFsUPP = NewFileFilterProc(FilterForCDFs);
+	StandardGetFile (FilterForCDFsUPP, -1, NULL, &CDFreply);
+	DisposeRoutineDescriptor(FilterForCDFsUPP);
 #endif
-		if (CDFreply.sfGood && !CDFreply.sfIsFolder && !CDFreply.sfIsVolume) {
-	  	  BuildMacPath (&CDFreply.sfFile, CDFpath, TRUE);
-	  	  CDFtext[0] = strlen (CDFpath);
-	  	  strcpyX ((char *) &CDFtext[1], CDFpath, 255);
-	  	  SetIText ((Handle) controlHs[CDFTEXTin], CDFtext);
-		}
-		break;
+	if (CDFreply.sfGood && !CDFreply.sfIsFolder && !CDFreply.sfIsVolume) {
+  	  BuildMacPath (&CDFreply.sfFile, CDFpath, TRUE);
+  	  CDFtext[0] = strlen (CDFpath);
+  	  strcpyX ((char *) &CDFtext[1], CDFpath, 255);
+  	  SetIText ((Handle) controlHs[CDFTEXTin], CDFtext);
+	}
+	break;
     }
     /************************************************************************
     * Select CDF specification (new CDF).
@@ -1603,18 +1623,18 @@ char **argV[];
     * an iBeam (instead of returning it to what it was).
     ************************************************************************/
     case CDFNEWin: {
-		StandardFileReply CDFreply;
-		char CDFpath[DU_MAX_PATH_LEN+1];
-		char prompt[] = "Enter CDF name:";
-		StandardPutFile (CtoPstr(prompt), CtoPstr(""), &CDFreply);
-		if (CDFreply.sfGood && !CDFreply.sfIsFolder && !CDFreply.sfIsVolume) {
-	  	  BuildMacPath (&CDFreply.sfFile, CDFpath, TRUE);
-	  	  CDFtext[0] = strlen (CDFpath);
-	  	  strcpyX ((char *) &CDFtext[1], CDFpath, 255);
-	  	  SetIText ((Handle) controlHs[CDFTEXTin], CDFtext);
-		}
-		SetCursor (&(qd.arrow));
-		break;
+	StandardFileReply CDFreply;
+	char CDFpath[DU_MAX_PATH_LEN+1];
+	char prompt[] = "Enter CDF name:";
+	StandardPutFile (CtoPstr(prompt), CtoPstr(""), &CDFreply);
+	if (CDFreply.sfGood && !CDFreply.sfIsFolder && !CDFreply.sfIsVolume) {
+  	  BuildMacPath (&CDFreply.sfFile, CDFpath, TRUE);
+  	  CDFtext[0] = strlen (CDFpath);
+  	  strcpyX ((char *) &CDFtext[1], CDFpath, 255);
+  	  SetIText ((Handle) controlHs[CDFTEXTin], CDFtext);
+	}
+	SetCursor (&(qd.arrow));
+	break;
     }
     /************************************************************************
     * Check boxes.
@@ -1629,18 +1649,18 @@ char **argV[];
     case STATSin:
     case gWITHin:
     case vWITHin:
-		SetCtlValue (controlHs[itemN], BOO(GetCtlValue(controlHs[itemN]),0,1));
-		break;
+	SetCtlValue (controlHs[itemN], BOO(GetCtlValue(controlHs[itemN]),0,1));
+	break;
     /************************************************************************
     * Radio buttons.
     ************************************************************************/
     case ZMODEinBASE+0:
     case ZMODEinBASE+1:
     case ZMODEinBASE+2:
-		for (i = 0; i < 3; i++) SetCtlValue (controlHs[ZMODEinBASE+i], 0);
-		  SetCtlValue (controlHs[itemN], 1);
-		break;
-    	}
+	for (i = 0; i < 3; i++) SetCtlValue (controlHs[ZMODEinBASE+i], 0);
+	  SetCtlValue (controlHs[itemN], 1);
+	break;
+    }
   }
 }
 #endif
