@@ -40,6 +40,7 @@
 #define CDFSTATS
 #include "cdfstats.h"
 
+#define MAX_SCREENLINE_LEN_stats 256
 /******************************************************************************
 * Increased stack size for Borland C on IBM PC.
 ******************************************************************************/
@@ -53,6 +54,9 @@ extern unsigned _stklen = BORLANDC_STACK_SIZE;
 ******************************************************************************/
 
 Logical report[3];
+#ifdef WIN32
+  static unsigned int oldCodePage = 0;
+#endif
 
 /******************************************************************************
 * Main.
@@ -67,10 +71,8 @@ MAIN {
 #else
   success = CalculateStatistics (argc, argv);
 #endif
-#if defined(DEBUG)
-  if (cdf_FreeMemory(NULL,FatalError) > 0) DisplayWarning ("Abandoned buffers.");
-#else
-  cdf_FreeMemory (NULL, FatalError);
+#ifdef WIN32
+  if (oldCodePage != 0) SetConsoleOutputCP(oldCodePage);
 #endif
   return BOO(success,EXIT_SUCCESS_,EXIT_FAILURE_);
 }
@@ -88,10 +90,10 @@ char *argV[];
   char oSpec[DU_MAX_PATH_LEN+1], oDir[DU_MAX_DIR_LEN+1];
   char oName[DU_MAX_NAME_LEN+1], CDFpath[DU_MAX_PATH_LEN+1];
   char CDFdir[DU_MAX_DIR_LEN+1], CDFname[DU_MAX_NAME_LEN+1];
-  long numVars, numZvars, varN, zMode;
+  long numVars, numZvars, varN, zMode = DEFAULTzModeSTATS;
   long workingCache, stageCache, compressCache;
   Logical qopError = FALSE;
-  struct VarStruct Var;
+  struct VarStructST Var;
   Logical negToPosFp0, displayStats;
   QOP *qop;
   static char *validQuals[] = {
@@ -99,16 +101,16 @@ char *argV[];
     "page", "nopage", "update_valids", "noupdate_valids", "update_scales",
     "noupdate_scales", "zmode", "neg2posfp0", "noneg2posfp0",
     "update_monotonic", "noupdate_monotonic", "report", "cache", "about",
-    "statistics", "nostatistics", NULL
+    "statistics", "nostatistics", "epoch_monotonic", "ziso8601", "novalidate",
+    NULL
   };
   static int optRequired[] = {
     FALSE, FALSE, TRUE, FALSE, FALSE, FALSE, FALSE, FALSE, FALSE, FALSE,
     FALSE, FALSE, FALSE, TRUE, FALSE, FALSE, FALSE, FALSE, TRUE, TRUE,
-    FALSE, FALSE, FALSE, 0
+    FALSE, FALSE, FALSE, FALSE, FALSE, FALSE, 0
   };
   static char *reportTokens[] = { "errors", "warnings", "informationals" };
 
-  CDFsetValidate (VALIDATEFILEon);
   /****************************************************************************
   * Get qualifiers/options/parameters.
   ****************************************************************************/
@@ -235,6 +237,30 @@ char *argV[];
 	report[WARNs] = REPORTwarningsDEFAULT;
 	report[INFOs] = REPORTinfosDEFAULT;
       }
+      if (qop == NULL) return FALSE;
+      /***********************************************************************
+      * Check for `epoch_monotonic' qualifier.
+      ***********************************************************************/
+      if (qop->qualEntered[EPOCH_MONOTONICqual]) {
+        epochMonotonic = TRUE;
+      } else
+        epochMonotonic = FALSE;
+      /*************************************************************************
+      * Check for `zisO8601' qualifier.
+      *************************************************************************/
+      if (qop->qualEntered[ZISO8601qual]) {
+        ziso8601 = TRUE;
+      }
+      else
+        ziso8601 = FALSE;
+      /*************************************************************************
+      * Check for `novalidate' qualifier.
+      *************************************************************************/
+      if (qop->qualEntered[NOVALIDATEqual]) {
+        toValidate = FALSE;
+      }
+      else
+        toValidate = TRUE;
       /************************************************************************
       * Free QOP memory and check for an error.
       ************************************************************************/
@@ -242,6 +268,15 @@ char *argV[];
       if (qopError) return FALSE;
       break;
   }
+
+#if !defined(WIN32)
+  setlocale(LC_CTYPE, "en_US.UTF-8");
+#else
+  setlocale(LC_CTYPE, ".UTF8");
+  oldCodePage = GetConsoleOutputCP();
+  SetConsoleOutputCP(65001);
+#endif
+
   /****************************************************************************
   * Open output file (if specified).
   ****************************************************************************/
@@ -260,8 +295,13 @@ char *argV[];
   * Display information.
   ****************************************************************************/
   ParsePath (CDFpath, CDFdir, CDFname);
-  WriteOut (OUTfp, "Statistics for \"");
-  if (!NULstring(oSpec)) WriteOut (stdout, "Statistics for \"");
+  if (!epochMonotonic) {
+    WriteOut (OUTfp, "Statistics for \"");
+    if (!NULstring(oSpec)) WriteOut (stdout, "Statistics for \"");
+  } else {
+    WriteOut (OUTfp, "Statistics (for epoch monotonic) for \"");
+    if (!NULstring(oSpec)) WriteOut (stdout, "Statistics (for epoch monotonic) for \"");
+  }
   WriteOut (OUTfp, CDFname);
   if (!NULstring(oSpec)) WriteOut (stdout, CDFname);
   if (!EndsWithIgCase(CDFname, ".cdf"))
@@ -273,12 +313,13 @@ char *argV[];
   }
   WriteOut (OUTfp, "\"...\n");
   if (!NULstring(oSpec)) WriteOut (stdout, "\"...\n");
+
+  if (!toValidate) CDFsetValidate (VALIDATEFILEoff);
   /****************************************************************************
   * Open/inquire CDF.
   ****************************************************************************/
   status = CDFlib (OPEN_, CDF_, CDFpath, &id,
-		   SELECT_, CDF_READONLY_MODE_, READONLYon,
-			    CDF_zMODE_, zMode,
+		   SELECT_, CDF_zMODE_, zMode,
 			    CDF_NEGtoPOSfp0_MODE_, BOO(negToPosFp0,
 						       NEGtoPOSfp0on,
 						       NEGtoPOSfp0off),
@@ -336,7 +377,8 @@ char *argV[];
 			      rVAR_MAXREC_, &Var.varMaxRec,
 			NULL_);
        if (!StatusHandlerStats(status)) return FALSE;
-       if (!SetupVar(&Var)) {
+       if (epochMonotonic && !CDFepochDataType(Var.dataTypeV)) continue;
+       if (!SetupVar(id, &Var)) {
 	 FreeVarMem (&Var);
 	 return FALSE;
        }
@@ -364,7 +406,8 @@ char *argV[];
 			      zVAR_MAXREC_, &Var.varMaxRec,
 			NULL_);
        if (!StatusHandlerStats(status)) return FALSE;
-       if (!SetupVar(&Var)) {
+       if (epochMonotonic && !CDFepochDataType(Var.dataTypeV)) continue;
+       if (!SetupVar(id, &Var)) {
 	 FreeVarMem (&Var);
 	 return FALSE;
        }
@@ -397,16 +440,20 @@ char *argV[];
 * SetupVar.
 ******************************************************************************/
 
-Logical SetupVar (Var)
-struct VarStruct *Var;
+Logical SetupVar (id, Var)
+CDFid id;
+struct VarStructST *Var;
 {
   CDFstatus status;
-  char line[MAX_SCREENLINE_LEN+1];
+  char line[MAX_SCREENLINE_LEN_stats+1];
   char delim;
   long dataTypeE;                 /* data type for attribute entry */
   long numElemsE;                 /* number of elements for attribute entry */
   void *temp;
   int dimN;
+  int virtual;
+  long numElms, dataType;
+
   /****************************************************************************
   * Initialize pointers for memory to be allocated.
   ****************************************************************************/
@@ -420,24 +467,38 @@ struct VarStruct *Var;
   Var->minINrange = NULL;
   Var->maxINrange = NULL;
   Var->buffer = NULL;
+  virtual = 0;
+  if (epochMonotonic) {
+    /**************************************************************************
+    * Check if this a virtual variable. If so, sets it.
+    **************************************************************************/
+    status = CDFlib (SELECT_, ATTR_NAME_, "VIRTUAL",
+			      BOO(Var->Z,zENTRY_,rENTRY_), Var->varN,
+		     GET_, BOO(Var->Z,zENTRY_NUMELEMS_,
+				      rENTRY_NUMELEMS_), &numElms,
+		     NULL_);
+    if (status == CDF_OK) virtual = 1;
+  }
   /****************************************************************************
   * Display heading line for this variable.
   ****************************************************************************/
-  strcpyX (line, "\n", MAX_SCREENLINE_LEN);
+  strcpyX (line, "\n", MAX_SCREENLINE_LEN_stats);
   delim = PickDelimiter (Var->varName, strlen(Var->varName));
   snprintf (EofS(line), (size_t) sizeof(line)-strlen(line),
-	    "%3ld.  %c%s%c", Var->varN + 1, delim, Var->varName, delim);
+  	    "%3ld.  %c%s%c", Var->varN + 1, delim, Var->varName, delim);
+  snprintf (EofS(line), (size_t) sizeof(line)-strlen(line),
+  	    "%s", (virtual==1?" (V)":" "));
   if (Var->Z) {
     snprintf (EofS(line), (size_t) sizeof(line)-strlen(line),
 	      "   %ld:[", Var->numDims);
     if (Var->numDims > 0) {
       for (dimN = 0; dimN < Var->numDims; dimN++) {
-	 if (dimN > 0) strcatX (line, ",", MAX_SCREENLINE_LEN);
+    	 if (dimN > 0) strcatX (line, ",", MAX_SCREENLINE_LEN_stats);
 	 snprintf (EofS(line), (size_t) sizeof(line)-strlen(line),
 		   "%ld", Var->dimSizes[dimN]);
       }
     }
-    strcatX (line, "]", MAX_SCREENLINE_LEN);
+    strcatX (line, "]", MAX_SCREENLINE_LEN_stats);
   }
   snprintf (EofS(line), (size_t) sizeof(line)-strlen(line),
 	    "   %s/", TFvarianceToken(Var->recVary));
@@ -447,14 +508,30 @@ struct VarStruct *Var;
   }
   snprintf (EofS(line), (size_t) sizeof(line)-strlen(line),
 	    "   (CDF_%s/%ld)", DataTypeToken(Var->dataTypeV), Var->numElemsV);
-  strcatX (line, "\n\n", MAX_SCREENLINE_LEN);
+  strcatX (line, "\n\n", MAX_SCREENLINE_LEN_stats);
   WriteOut (OUTfp, line);
   /****************************************************************************
   * Return early if no records written.
   ****************************************************************************/
-  if (Var->varMaxRec == -1) {
-    WriteOut (OUTfp, "          No records.\n");
-    return TRUE;
+  if (Var->varMaxRec == -1 && epochMonotonic) {
+    /**************************************************************************
+    * Check if this a virtual variable. If so, handle it.
+    **************************************************************************/
+    if (virtual == 1) {
+      status = CDFlib (SELECT_, ATTR_NAME_, "COMPONENT_1",
+				BOO(Var->Z,zENTRY_,rENTRY_), Var->varN,
+		       GET_, BOO(Var->Z,zENTRY_NUMELEMS_,
+				  	rENTRY_NUMELEMS_), &numElms,
+			     BOO(Var->Z,zENTRY_DATATYPE_,
+				        rENTRY_DATATYPE_), &dataType,
+		       NULL_);
+      if (status != CDF_OK) return FALSE;
+      if (!HandleVirtual(id, Var)) return FALSE;
+        return TRUE;
+    } else {
+      WriteOut (OUTfp, "          No records.\n");
+      return TRUE;
+    }
   }
   /****************************************************************************
   * If range checking requested, get the valid min/max.
@@ -467,21 +544,25 @@ struct VarStruct *Var;
 			   BOO(Var->Z,zENTRY_NUMELEMS_,
 				      rENTRY_NUMELEMS_), &numElemsE,
 		     NULL_);
-    if (status == NO_SUCH_ENTRY)
+    if (status == NO_SUCH_ENTRY) {
       Var->rangeCheckVar = FALSE;
-    else {
+    } else {
       if (!StatusHandlerStats(status)) return FALSE;
-      temp = cdf_AllocateMemory ((size_t) (CDFelemSize(dataTypeE) * numElemsE),
-			     FatalError);
+      temp = cdf_AllocateMemory ((size_t) (CDFelemSize(dataTypeE) * numElemsE +
+                                           (STRINGdataType(dataTypeE)?1:0)),
+			         FatalError);
       status = CDFlib (GET_, BOO(Var->Z,zENTRY_DATA_,rENTRY_DATA_), temp,
 		       NULL_);
       if (!StatusHandlerStats(status)) {
 	cdf_FreeMemory (temp, FatalError);
 	return FALSE;
       }
-      Var->validmin = cdf_AllocateMemory ((size_t) (CDFelemSize(Var->dataTypeV) *
-						Var->numElemsV),
-				      FatalError);
+      if (STRINGdataType(dataTypeE)) *((char *)temp+numElemsE) = '\0';
+      Var->validmin = cdf_AllocateMemory ((size_t) (
+                                              CDFelemSize(Var->dataTypeV) *
+					      Var->numElemsV +
+					      (STRINGdataType(Var->dataTypeV)?1:0)),
+					  FatalError);
       ConvertDataType (dataTypeE, numElemsE, temp, Var->dataTypeV,
 		       Var->numElemsV, Var->validmin);
       cdf_FreeMemory (temp, FatalError);
@@ -499,17 +580,21 @@ struct VarStruct *Var;
       }
       else {
 	if (!StatusHandlerStats(status)) return FALSE;
-	temp = cdf_AllocateMemory ((size_t) (CDFelemSize(dataTypeE) * numElemsE),
-			       FatalError);
+	temp = cdf_AllocateMemory ((size_t) (CDFelemSize(dataTypeE) * numElemsE
+                                             + (STRINGdataType(dataTypeE)?1:0)),
+				   FatalError);
 	status = CDFlib (GET_, BOO(Var->Z,zENTRY_DATA_,rENTRY_DATA_), temp,
 			 NULL_);
 	if (!StatusHandlerStats(status)) {
 	  cdf_FreeMemory (temp, FatalError);
 	  return FALSE;
 	}
-	Var->validmax = cdf_AllocateMemory ((size_t) (CDFelemSize(Var->dataTypeV) *
-						  Var->numElemsV),
-					FatalError);
+        if (STRINGdataType(dataTypeE)) *((char *)temp+numElemsE) = '\0';
+	Var->validmax = cdf_AllocateMemory ((size_t)
+						(CDFelemSize(Var->dataTypeV) *
+						 Var->numElemsV +
+                                                 (STRINGdataType(Var->dataTypeV)?1:0)),
+					    FatalError);
 	ConvertDataType (dataTypeE, numElemsE, temp, Var->dataTypeV,
 			 Var->numElemsV, Var->validmax);
 	cdf_FreeMemory (temp, FatalError);
@@ -523,6 +608,19 @@ struct VarStruct *Var;
   * If fill values are to be ignored, get the fill value.
   ****************************************************************************/
   if (ignoreFills) {
+    status = GetFILLVALEntry (id, Var->Z, Var->varN, &dataTypeE, &numElemsE,
+			      &(Var->fillval));
+    if (status != CDF_OK) {
+      if (status == BAD_DATA_TYPE) {
+        if (Var->fillval != NULL) {
+          cdf_FreeMemory (Var->fillval, FatalError);
+          Var->fillval = NULL;
+        }
+      }
+      Var->ignoreFillsVar = FALSE;
+    } else
+      Var->ignoreFillsVar = TRUE;
+/*
     status = CDFlib (SELECT_, ATTR_, fillvalAttrN,
 			      BOO(Var->Z,zENTRY_,rENTRY_), Var->varN,
 		     GET_, BOO(Var->Z,zENTRY_DATATYPE_,
@@ -550,6 +648,7 @@ struct VarStruct *Var;
       cdf_FreeMemory (temp, FatalError);
       Var->ignoreFillsVar = TRUE;
     }
+*/
   }
   else
     Var->ignoreFillsVar = FALSE;
@@ -580,7 +679,7 @@ struct VarStruct *Var;
   /****************************************************************************
   * Read each record and calculate statistics.
   ****************************************************************************/
-  if (!CALCstat(Var)) return FALSE;
+  if (!CALCstat(Var, FALSE)) return FALSE;
   /****************************************************************************
   * Return - note that memory allocated will be freed by caller.
   ****************************************************************************/
@@ -592,21 +691,21 @@ struct VarStruct *Var;
 ******************************************************************************/
 
 void FreeVarMem (Var)
-struct VarStruct *Var;
+struct VarStructST *Var;
 {
   if (Var->validmin != NULL) cdf_FreeMemory (Var->validmin,
-					 FatalError);
+					     FatalError);
   if (Var->validmax != NULL) cdf_FreeMemory (Var->validmax,
-					 FatalError);
+					     FatalError);
   if (Var->fillval != NULL) cdf_FreeMemory (Var->fillval, FatalError);
   if (Var->format != NULL) cdf_FreeMemory (Var->format, FatalError);
   if (Var->min != NULL) cdf_FreeMemory (Var->min, FatalError);
   if (Var->max != NULL) cdf_FreeMemory (Var->max, FatalError);
   if (Var->last != NULL) cdf_FreeMemory (Var->last, FatalError);
   if (Var->minINrange != NULL) cdf_FreeMemory (Var->minINrange,
-					   FatalError);
+					       FatalError);
   if (Var->maxINrange != NULL) cdf_FreeMemory (Var->maxINrange,
-					   FatalError);
+					       FatalError);
   if (Var->buffer != NULL) cdf_FreeMemory (Var->buffer, FatalError);
   return;
 }
@@ -758,78 +857,78 @@ char **argV[];
       * Ok.
       ************************************************************************/
       case OKin: {
-		int n;
-		char tempS1[1+1];
-		/**********************************************************************
-		* Get the value of each control.
-		**********************************************************************/
-		GetIText ((Handle) controlHs[CDFTEXTin], CDFtext);
-		GetIText ((Handle) controlHs[OUTTEXTin], outText);
-		GetIText ((Handle) controlHs[CACHEin], cacheText);
-		pageOutput = GetCtlValue (controlHs[PAGEin]);
-		useFormat = GetCtlValue (controlHs[FORMATin]);
-		rangeCheck = GetCtlValue (controlHs[RANGEin]);
-		useFillval = GetCtlValue (controlHs[FILLin]);
-		negToPos = GetCtlValue (controlHs[NEGZin]);
-		reportInfos = GetCtlValue (controlHs[INFOin]);
-		reportWarns = GetCtlValue (controlHs[WARNin]);
-		reportErrors = GetCtlValue (controlHs[ERRORin]);
-		updateValids = GetCtlValue (controlHs[VALIDin]);
-		updateScales = GetCtlValue (controlHs[SCALEin]);
-		updateMono = GetCtlValue (controlHs[MONOin]);
-		dispStats = GetCtlValue (controlHs[STATSin]);
-		for (zMode = 0; zMode < 3; zMode++) {
-		   if (GetCtlValue(controlHs[ZMODEinBASE+zMode])) break;
-		}
-		/**********************************************************************
-		* Build argc/argv.
-		**********************************************************************/
+	int n;
+	char tempS1[1+1];
+	/**********************************************************************
+	* Get the value of each control.
+	**********************************************************************/
+	GetIText ((Handle) controlHs[CDFTEXTin], CDFtext);
+	GetIText ((Handle) controlHs[OUTTEXTin], outText);
+	GetIText ((Handle) controlHs[CACHEin], cacheText);
+	pageOutput = GetCtlValue (controlHs[PAGEin]);
+	useFormat = GetCtlValue (controlHs[FORMATin]);
+	rangeCheck = GetCtlValue (controlHs[RANGEin]);
+	useFillval = GetCtlValue (controlHs[FILLin]);
+	negToPos = GetCtlValue (controlHs[NEGZin]);
+	reportInfos = GetCtlValue (controlHs[INFOin]);
+	reportWarns = GetCtlValue (controlHs[WARNin]);
+	reportErrors = GetCtlValue (controlHs[ERRORin]);
+	updateValids = GetCtlValue (controlHs[VALIDin]);
+	updateScales = GetCtlValue (controlHs[SCALEin]);
+	updateMono = GetCtlValue (controlHs[MONOin]);
+	dispStats = GetCtlValue (controlHs[STATSin]);
+	for (zMode = 0; zMode < 3; zMode++) {
+	   if (GetCtlValue(controlHs[ZMODEinBASE+zMode])) break;
+	}
+	/**********************************************************************
+	* Build argc/argv.
+	**********************************************************************/
         *argC = 14 + BOO(NULpString(CDFtext),0,1) +
 		     BOO(NULpString(outText),0,2) +
 		     BOO(NULpString(cacheText),0,2);
         *argV = (char **) cdf_AllocateMemory ((size_t)*argC * sizeof(char *),
 					  FatalError);
-		n = 0;
-		MAKEstrARGv (argV, n, pgmName)
-		if (!NULpString(CDFtext)) {
-		  PtoCstr (CDFtext);
-		  MAKEstrARGv (argV, n, (char *) CDFtext)
-		  CtoPstr ((char *) CDFtext);
-		}
-		MAKEbooARGv (argV, n, pageOutput, "-page", "-nopage")
-		MAKEbooARGv (argV, n, useFormat, "-format", "-noformat")
-		MAKEbooARGv (argV, n, rangeCheck, "-range", "-norange")
-		MAKEbooARGv (argV, n, useFillval, "-fill", "-nofill")
-		MAKEbooARGv (argV, n, negToPos, "-neg2posfp0", "-noneg2posfp0")
-		MAKEbooARGv (argV, n, updateValids, "-update_valids",
-				     "-noupdate_valids")
-		MAKEbooARGv (argV, n, updateScales, "-update_scales",
-				     "-noupdate_scales")
-		MAKEbooARGv (argV, n, updateMono, "-update_monotonic",
-				     "-noupdate_monotonic")
-		MAKEbooARGv (argV, n, dispStats, "-statistics", "-nostatistics")
-		MAKEstrARGv (argV, n, "-zmode")
-		snprintf (tempS1, (size_t) sizeof(tempS1), "%d", zMode);
-		MAKEstrARGv (argV, n, tempS1)
-		MAKEstrARGv (argV, n, "-report")
-		MAKEstrARGv (argV, n, StatusCodeReportOptions(reportErrors,
-						      reportWarns,
-						      reportInfos))
-		if (!NULpString(outText)) {
-		  MAKEstrARGv (argV, n, "-output")
-		  PtoCstr (outText);
-		  MAKEstrARGv (argV, n, (char *) outText)
-		  CtoPstr ((char *) outText);
-		}
-		if (!NULpString(cacheText)) {
-		  MAKEstrARGv (argV, n, "-cache")
-		  PtoCstr (cacheText);
-		  MAKEstrARGv (argV, n, (char *) cacheText)
-		  CtoPstr ((char *) cacheText);
-		}
-		/**********************************************************************
-		* Close the dialog and return.
-		**********************************************************************/
+	n = 0;
+	MAKEstrARGv (argV, n, pgmName)
+	if (!NULpString(CDFtext)) {
+	  PtoCstr (CDFtext);
+	  MAKEstrARGv (argV, n, (char *) CDFtext)
+	  CtoPstr ((char *) CDFtext);
+	}
+	MAKEbooARGv (argV, n, pageOutput, "-page", "-nopage")
+	MAKEbooARGv (argV, n, useFormat, "-format", "-noformat")
+	MAKEbooARGv (argV, n, rangeCheck, "-range", "-norange")
+	MAKEbooARGv (argV, n, useFillval, "-fill", "-nofill")
+	MAKEbooARGv (argV, n, negToPos, "-neg2posfp0", "-noneg2posfp0")
+	MAKEbooARGv (argV, n, updateValids, "-update_valids",
+			     "-noupdate_valids")
+	MAKEbooARGv (argV, n, updateScales, "-update_scales",
+			     "-noupdate_scales")
+	MAKEbooARGv (argV, n, updateMono, "-update_monotonic",
+			     "-noupdate_monotonic")
+	MAKEbooARGv (argV, n, dispStats, "-statistics", "-nostatistics")
+	MAKEstrARGv (argV, n, "-zmode")
+	snprintf (tempS1, (size_t) sizeof(tempS1), "%d", zMode);
+	MAKEstrARGv (argV, n, tempS1)
+	MAKEstrARGv (argV, n, "-report")
+	MAKEstrARGv (argV, n, StatusCodeReportOptions(reportErrors,
+					      reportWarns,
+					      reportInfos))
+	if (!NULpString(outText)) {
+	  MAKEstrARGv (argV, n, "-output")
+	  PtoCstr (outText);
+	  MAKEstrARGv (argV, n, (char *) outText)
+	  CtoPstr ((char *) outText);
+	}
+	if (!NULpString(cacheText)) {
+	  MAKEstrARGv (argV, n, "-cache")
+	  PtoCstr (cacheText);
+	  MAKEstrARGv (argV, n, (char *) cacheText)
+	  CtoPstr ((char *) cacheText);
+	}
+	/**********************************************************************
+	* Close the dialog and return.
+	**********************************************************************/
         CloseDialog (dialogP);
 #ifdef __MWERKS__
         DisposeRoutineDescriptor (FilterDialogQOPsoUPP);
@@ -843,13 +942,14 @@ char **argV[];
       case HELPin: {
         int n;
         *argC = 1;
-        *argV = (char **) cdf_AllocateMemory ((size_t)*argC * sizeof(char *), FatalError);
-		n = 0;
-		MAKEstrARGv (argV, n, pgmName)
-		CloseDialog (dialogP);
+        *argV = (char **) cdf_AllocateMemory ((size_t)*argC * sizeof(char *),
+					      FatalError);
+	n = 0;
+	MAKEstrARGv (argV, n, pgmName)
+	CloseDialog (dialogP);
 #ifdef __MWERKS__
         DisposeRoutineDescriptor (FilterDialogQOPsoUPP);
-		DisposeRoutineDescriptor (OutlineDefaultButtonUPP);
+	DisposeRoutineDescriptor (OutlineDefaultButtonUPP);
 #endif
         return TRUE;
       }
@@ -860,29 +960,29 @@ char **argV[];
         CloseDialog (dialogP);
 #ifdef __MWERKS__
         DisposeRoutineDescriptor (FilterDialogQOPsoUPP);
-		DisposeRoutineDescriptor (OutlineDefaultButtonUPP);
+	DisposeRoutineDescriptor (OutlineDefaultButtonUPP);
 #endif
         return FALSE;
       /************************************************************************
       * Select CDF specification.
       ************************************************************************/
       case CDFSELECTin: {
-		StandardFileReply CDFreply;
-		char CDFpath[DU_MAX_PATH_LEN+1];
+	StandardFileReply CDFreply;
+	char CDFpath[DU_MAX_PATH_LEN+1];
 #ifndef __MWERKS__
-		StandardGetFile (FilterForCDFs, -1, NULL, &CDFreply);
+	StandardGetFile (FilterForCDFs, -1, NULL, &CDFreply);
 #else
-		FilterForCDFsUPP = NewFileFilterProc((ProcPtr) FilterForCDFs);
-		StandardGetFile (FilterForCDFsUPP, -1, NULL, &CDFreply);
-		DisposeRoutineDescriptor (FilterForCDFsUPP);
+	FilterForCDFsUPP = NewFileFilterProc((ProcPtr) FilterForCDFs);
+	StandardGetFile (FilterForCDFsUPP, -1, NULL, &CDFreply);
+	DisposeRoutineDescriptor (FilterForCDFsUPP);
 #endif
-		if (CDFreply.sfGood && !CDFreply.sfIsFolder && !CDFreply.sfIsVolume) {
-		  BuildMacPath (&CDFreply.sfFile, CDFpath, TRUE);
-		  CDFtext[0] = strlen (CDFpath);
-		  strcpyX ((char *) &CDFtext[1], CDFpath, 255);
-		  SetIText ((Handle) controlHs[CDFTEXTin], CDFtext);
-		}
-		break;
+	if (CDFreply.sfGood && !CDFreply.sfIsFolder && !CDFreply.sfIsVolume) {
+	  BuildMacPath (&CDFreply.sfFile, CDFpath, TRUE);
+	  CDFtext[0] = strlen (CDFpath);
+	  strcpyX ((char *) &CDFtext[1], CDFpath, 255);
+	  SetIText ((Handle) controlHs[CDFTEXTin], CDFtext);
+	}
+	break;
       }
       /************************************************************************
       * Select output file.
@@ -890,17 +990,17 @@ char **argV[];
       * an iBeam (instead of returning it to what it was).
       ************************************************************************/
       case OUTSELECTin: {
-		StandardFileReply outReply;
-		char outPath[DU_MAX_PATH_LEN+1], prompt[] = "Enter skeleton table:";
-		StandardPutFile (CtoPstr(prompt), CtoPstr(""), &outReply);
-		if (outReply.sfGood && !outReply.sfIsFolder && !outReply.sfIsVolume) {
-		  BuildMacPath (&outReply.sfFile, outPath, TRUE);
-		  outText[0] = strlen (outPath);
-		  strcpyX ((char *) &outText[1], outPath, 255);
-		  SetIText ((Handle) controlHs[OUTTEXTin], outText);
-		}
-		SetCursor (&(qd.arrow));
-		break;
+	StandardFileReply outReply;
+	char outPath[DU_MAX_PATH_LEN+1], prompt[] = "Enter skeleton table:";
+	StandardPutFile (CtoPstr(prompt), CtoPstr(""), &outReply);
+	if (outReply.sfGood && !outReply.sfIsFolder && !outReply.sfIsVolume) {
+	  BuildMacPath (&outReply.sfFile, outPath, TRUE);
+	  outText[0] = strlen (outPath);
+	  strcpyX ((char *) &outText[1], outPath, 255);
+	  SetIText ((Handle) controlHs[OUTTEXTin], outText);
+	}
+	SetCursor (&(qd.arrow));
+	break;
       }
       /************************************************************************
       * Check boxes.
