@@ -1,4 +1,9 @@
 /******************************************************************************
+* Copyright 1996-2014 United States Government as represented by the
+* Administrator of the National Aeronautics and Space Administration.
+* All Rights Reserved.
+******************************************************************************/
+/******************************************************************************
 *
 *  NSSDC/CDF                 Toolbox of routines for CDF Toolkit (Macintosh).
 *
@@ -926,7 +931,6 @@ char *key;
        *********************************************************************/
        case keyDown: {
 		 if (ABORTkeyMAC(event)) {
-	   		cdf_FreeMemory (NULL, FatalError);
 	   		Exit;
 	 	 }
          *key = (char) (event.message & charCodeMask);
@@ -998,15 +1002,14 @@ void CheckForAbortSOmac () {
     }
     case keyDown:
       if (ABORTkeyMAC(event)) {
-		cdf_FreeMemory (NULL, FatalError);
-		Exit;
+	Exit;
       }
       break;
     case activateEvt:
       if ((event.modifiers & activeFlag) != 0) {
         TEActivate (soTextH);
         ShowControl (soVscrollH);
-		SetCursor (CDF_CURSOR);
+	SetCursor (CDF_CURSOR);
       }
       else {
         TEDeactivate (soTextH);
