@@ -1,4 +1,9 @@
 /******************************************************************************
+* Copyright 1996-2014 United States Government as represented by the
+* Administrator of the National Aeronautics and Space Administration.
+* All Rights Reserved.
+******************************************************************************/
+/******************************************************************************
 *
 *  NSSDC/CDF                                            Windowing functions.
 *
@@ -61,6 +66,13 @@
 * Include files.
 ******************************************************************************/
 
+#define NCURSES_WIDECHAR 1
+
+#define _XOPEN_SOURCE_EXTENDED
+
+#include <stdio.h>
+#include <wchar.h>
+#include "cdftools.h"
 #include "windoz.h"
 
 #if defined(win32)
@@ -206,18 +218,26 @@ int rendition;
 #if defined(SMGui)
   long rowN = row + 1;
   long colN = col + 1;
-  uLong rend = RenditionMapping (rendition);
+  uLongx rend = RenditionMapping (rendition);
   if (StatusBad(smg$change_rendition(&(wid->id),&rowN,&colN,
 				     &num_rows,&num_cols,&rend))) return FALSE;
   return TRUE;
 #endif
 #if defined(CURSESui)
-  int i,j;
-  wattrset (wid->id, RenditionMapping(rendition));
-  for (i = 0; i < num_rows; i++)
-     for (j = 0; j < num_cols; j++)
-	mvwaddch (wid->id, row+i, col+j,
-		  mvwinch(wid->id,row+i,col+j) & A_CHARTEXT);
+  int i,k; size_t iz;
+  char bstring[401]; wchar_t wstring[101];
+  int rend = RenditionMapping(rendition);
+  for (i = 0; i < num_rows; i++) {
+    mvwinnwstr(wid->id, row+i, col, wstring, num_cols);
+    wstring[num_cols] = L'\0';
+#if !defined(WIN32)
+    k = (int) wcstombs (bstring, wstring, 400);
+/*    bstring[k] = (char) '\0'; */
+#else
+    wcstombs_s (&iz, bstring, 400, wstring, 100);
+#endif
+    put_chars (wid, bstring, num_cols, row+i, col, FALSE, rendition);
+  }
   if (!RefreshWINDs(UPDATE_)) return FALSE;
   return TRUE;
 #endif
@@ -256,6 +276,19 @@ int create_pasteboard () {
   return TRUE;
 #endif
 #if defined(CURSESui)
+  char locale[201];
+  int ia;
+  ia = 0;
+  strncpy (locale, setlocale (LC_CTYPE, NULL), 200);
+  if (CDFstrstrIgCase (locale, "utf8") == NULL &&
+      CDFstrstrIgCase (locale, "utf-8") == NULL) {
+#if !defined(WIN32)
+    setlocale (LC_CTYPE, "en_US.UTF-8");
+#else
+    setlocale (LC_CTYPE, ".UTF8");
+#endif
+    ia = 1;
+  }
   initscr ();
   keypad (stdscr, TRUE);
   noecho ();
@@ -265,6 +298,7 @@ int create_pasteboard () {
   batchCount = 0;
   WINDhead = NULL;
   set_cursor_mode (CURSORon);
+  if (ia == 1) setlocale (LC_CTYPE, locale);
   return TRUE;
 #endif
 #if defined(COWui)
@@ -277,13 +311,13 @@ int create_pasteboard () {
   cursorWIND = NULL;
   cursorVisible = FALSE;
   cursorOn = FALSE;
-  pbCurrentChars = (char *) cdf_AllocateMemory (nChars, FatalError);
-  pbCurrentAttrs = (char *) cdf_AllocateMemory (nChars, FatalError);
-  pbNewChars = (char *) cdf_AllocateMemory (nChars, FatalError);
-  pbNewAttrs = (char *) cdf_AllocateMemory (nChars, FatalError);
-  pbNullChars = (char *) cdf_AllocateMemory (nChars, FatalError);
-  pbBlankChars = (char *) cdf_AllocateMemory (nChars, FatalError);
-  pbNormalAttrs = (char *) cdf_AllocateMemory (nChars, FatalError);
+  pbCurrentChars = (char *) cdf_AllocateMemory ((size_t)nChars, FatalError);
+  pbCurrentAttrs = (char *) cdf_AllocateMemory ((size_t)nChars, FatalError);
+  pbNewChars = (char *) cdf_AllocateMemory ((size_t)nChars, FatalError);
+  pbNewAttrs = (char *) cdf_AllocateMemory ((size_t)nChars, FatalError);
+  pbNullChars = (char *) cdf_AllocateMemory ((size_t)nChars, FatalError);
+  pbBlankChars = (char *) cdf_AllocateMemory ((size_t)nChars, FatalError);
+  pbNormalAttrs = (char *) cdf_AllocateMemory ((size_t)nChars, FatalError);
   for (charN = 0; charN < nChars; charN++) {
      pbNullChars[charN] = 0;
      pbBlankChars[charN] = ' ';
@@ -318,8 +352,8 @@ int rendition;			/* Rendition of border (if bordered). */
 {
 #if defined(SMGui)
   LocalId id;
-  uLong attrs = 0;
-  uLong rend = RenditionMapping (NORMAL);   /* Default for display. */
+  uLongx attrs = 0;
+  uLongx rend = RenditionMapping (NORMAL);   /* Default for display. */
   long nRows = num_rows;
   long nCols = num_cols;
   if (StatusBad(smg$create_virtual_display(&nRows,&nCols,&id,
@@ -344,13 +378,13 @@ int rendition;			/* Rendition of border (if bordered). */
   int nChars = num_rows * num_cols;
   if (num_rows < 1 || num_rows > NUMfsiROWS) return FALSE;
   if (num_cols < 1 || num_cols > NUMfsiCOLS) return FALSE;
-  id = (LocalId) cdf_AllocateMemory (sizeof(COWvd), FatalError);
+  id = (LocalId) cdf_AllocateMemory ((size_t)sizeof(COWvd), FatalError);
   id->nRows = num_rows;
   id->nCols = num_cols;
   id->atRowN = 0;
   id->atColN = 0;
-  id->chars = (char *) cdf_AllocateMemory (nChars, FatalError);
-  id->attrs = (char *) cdf_AllocateMemory (nChars, FatalError);
+  id->chars = (char *) cdf_AllocateMemory ((size_t)nChars, FatalError);
+  id->attrs = (char *) cdf_AllocateMemory ((size_t)nChars, FatalError);
   *wid = AddWIND (id, FALSE, (borderMode == BORDER));
   memmove (id->chars, pbBlankChars, nChars);
   memmove (id->attrs, pbNormalAttrs, nChars);
@@ -368,7 +402,7 @@ int delete_pasteboard (eraseMode)
 int eraseMode;
 {
 #if defined(SMGui)
-  uLong flags = (eraseMode == ERASE ? SMG$M_ERASE_PBD : 0);
+  uLongx flags = (eraseMode == ERASE ? SMG$M_ERASE_PBD : 0);
   Logical status = DeleteWINDs();
   if (StatusBad(smg$delete_pasteboard(&pbid,&flags))) return FALSE;
   return status;
@@ -438,7 +472,7 @@ Logical teeEnds;	/* If TRUE, draw ACS_LTEE and ACS_RTEE at the ends of
   long endRowN = startRowN;
   long startColN = LcolN + 1;
   long endColN = RcolN + 1;
-  uLong rend = RenditionMapping(rendition);
+  uLongx rend = RenditionMapping(rendition);
   if (StatusBad(smg$draw_line(&(wid->id),&startRowN,&startColN,
 			      &endRowN,&endColN,&rend,NULL))) return FALSE;
 #endif
@@ -498,7 +532,7 @@ Logical teeEnds;	/* If TRUE, draw ACS_TTEE and ACS_BTEE at the ends of
   long endRowN = BrowN + 1;
   long startColN = colN + 1;
   long endColN = startColN;
-  uLong rend = RenditionMapping(rendition);
+  uLongx rend = RenditionMapping(rendition);
   if (StatusBad(smg$draw_line(&(wid->id),&startRowN,&startColN,
 			      &endRowN,&endColN,&rend,NULL))) return FALSE;
 #endif
@@ -552,7 +586,7 @@ int rendition;		/* Video attributes. */
   long startCol = Lcol + 1;
   long endRow = Brow + 1;
   long endCol = Rcol + 1;
-  uLong rend = RenditionMapping (rendition);
+  uLongx rend = RenditionMapping (rendition);
   if (StatusBad(smg$draw_rectangle(&(wid->id),&startRow,&startCol,
 				   &endRow,&endCol,&rend,NULL))) return FALSE;
   return TRUE;
@@ -715,20 +749,24 @@ int deleteToEOLkey;		/* Key causing the characters starting at the
 				   cursor to be deleted. */
 int refreshKey;			/* Key causing the screen to be refreshed. */
 {
-  int curLen = (int) strlen(field);
+  int curLen = UTF8StrChars (field, (int)strlen(field));
   int insertMode = FALSE;	/* Initially in insert mode. */
   int firstCol = fieldCol;
   int lastCol = fieldCol + fieldLen - 1;
-  int curChar, colN, key, i;
+  int curChar, colN, i;
+  int len, width;
+  wint_t key;
   /****************************************************************************
   * Write current field contents and position cursor.
   ****************************************************************************/
   if (!put_chars(wid,field,curLen,
 		 fieldRow,fieldCol,FALSE,NORMAL)) return FALSE;
-  for (colN = firstCol + curLen; colN <= lastCol; colN++)
+  width = UTF8StrWidth(field);
+  for (colN = firstCol + width; colN <= lastCol; colN++)
      if (!put_chars(wid," ",1,fieldRow,colN,FALSE,NORMAL)) return FALSE;
   curChar = 0;
   if (!set_cursor_abs(wid,fieldRow,firstCol+curChar)) return FALSE;
+    return FALSE;
   /****************************************************************************
   * Read keystrokes until return/exit.
   ****************************************************************************/
@@ -752,7 +790,9 @@ int refreshKey;			/* Key causing the screen to be refreshed. */
      if (key == KB_LEFTARROW) {
        if (curChar > 0) {
 	 curChar--;
-	 if (!set_cursor_abs(wid,fieldRow,firstCol+curChar)) return FALSE;
+	 if (!set_cursor_abs(wid,fieldRow,firstCol+
+					  UTF8FirstCharsWidth(field, curChar)))
+	   return FALSE;
        }
        else
 	 ring_bell ();
@@ -764,7 +804,9 @@ int refreshKey;			/* Key causing the screen to be refreshed. */
      if (key == KB_RIGHTARROW) {
        if (curChar < curLen && curChar < fieldLen - 1) {
 	 curChar++;
-	 if (!set_cursor_abs(wid,fieldRow,firstCol+curChar)) return FALSE;
+	 if (!set_cursor_abs(wid,fieldRow,firstCol+
+					  UTF8FirstCharsWidth(field, curChar)))
+	   return FALSE;
        }
        else
 	 ring_bell ();
@@ -775,16 +817,30 @@ int refreshKey;			/* Key causing the screen to be refreshed. */
      *************************************************************************/
      if (key == KB_DELETE) {
        if (curChar > 0) {
+	 int charLen; int ix, iy, total;
 	 int charsToEnd = curLen - curChar;
-	 memmove (&field[curChar-1], &field[curChar], charsToEnd + 1);
-	 curLen--;
+         iy = (int) strlen(field);
+	 ix = UTF8StrChars (field, iy);
+	 if (ix == iy) 
+	   memmove (&field[curChar-1], &field[curChar], charsToEnd + 1);
+	 else {
+	   char *ptr = UTF8CharAtString (field, curChar);
+	   int len =  UTF8CharAtBytes (field, curChar-1);
+	   memmove (ptr-len, ptr, strlen(ptr) + 1);
+	 }
 	 curChar--;
+	 curLen--;
+	 total = UTF8StrWidth(field);
 	 if (charsToEnd > 0)
-	   if (!put_chars(wid,&field[curChar],charsToEnd,fieldRow,
-			  firstCol+curChar,FALSE,NORMAL)) return FALSE;
+	   if (!put_chars(wid,UTF8CharAtString(field,curChar),charsToEnd,
+			  fieldRow, firstCol+
+				    UTF8FirstCharsWidth(field, curChar),
+			  FALSE,NORMAL)) return FALSE;
 	 if (!put_chars(wid," ",1,fieldRow,
-			firstCol+curLen,FALSE,NORMAL)) return FALSE;
-	 if (!set_cursor_abs(wid,fieldRow,firstCol+curChar)) return FALSE;
+			firstCol+total,FALSE,NORMAL)) return FALSE;
+	 if (!set_cursor_abs(wid,fieldRow,firstCol+
+					  UTF8FirstCharsWidth(field, curChar)))
+	   return FALSE;
        }
        else
 	 ring_bell ();
@@ -796,7 +852,9 @@ int refreshKey;			/* Key causing the screen to be refreshed. */
      if (key == moveToSOLkey) {
        if (curChar > 0) {
 	 curChar = 0;
-	 if (!set_cursor_abs(wid,fieldRow,firstCol+curChar)) return FALSE;
+	 if (!set_cursor_abs(wid,fieldRow,firstCol+
+					  UTF8FirstCharsWidth(field, curChar)))
+	   return FALSE;
        }
        else
 	 ring_bell ();
@@ -808,12 +866,16 @@ int refreshKey;			/* Key causing the screen to be refreshed. */
      if (key == moveToEOLkey) {
        if (curLen < fieldLen && curChar < curLen) {
 	 curChar = curLen;
-	 if (!set_cursor_abs(wid,fieldRow,firstCol+curChar)) return FALSE;
+	 if (!set_cursor_abs(wid,fieldRow,firstCol+
+					  UTF8FirstCharsWidth(field,curChar)))
+	   return FALSE;
 	 continue;
        }
        if (curLen == fieldLen && curChar < curLen - 1) {
 	 curChar = curLen - 1;
-	 if (!set_cursor_abs(wid,fieldRow,firstCol+curChar)) return FALSE;
+	 if (!set_cursor_abs(wid,fieldRow,firstCol+
+					  UTF8FirstCharsWidth(field,curChar)))
+	   return FALSE;
 	 continue;
        }
        ring_bell ();
@@ -826,15 +888,25 @@ int refreshKey;			/* Key causing the screen to be refreshed. */
        if (curChar > 0) {
 	 int charsToEnd = curLen - curChar;
 	 int charsToStart = curChar + 1;
-	 int colN, i;
-	 memmove (field, &field[curChar], charsToEnd + 1);
+	 int colN, i, ix, iy;
+         iy = (int) strlen(field);
+	 ix = UTF8StrChars (field, iy);
+	 if (ix == iy)
+	   memmove (field, &field[curChar], charsToEnd + 1);
+	 else {
+	   char *ptr = UTF8CharAtString (field, curChar);
+	   memmove (field, ptr, strlen(ptr) + 1);
+	 }
 	 curLen = charsToEnd;
 	 curChar = 0;
 	 if (!put_chars(wid,field,curLen,
 			fieldRow,firstCol,FALSE,NORMAL)) return FALSE;
-	 for (colN = firstCol+curLen, i = 0; i < charsToStart-1; colN++, i++)
+	 width = UTF8StrWidth(field);
+	 for (colN = firstCol+width, i = 0; i < charsToStart-1; colN++, i++)
 	    if (!put_chars(wid," ",1,fieldRow,colN,FALSE,NORMAL)) return FALSE;
-	 if (!set_cursor_abs(wid,fieldRow,firstCol+curChar)) return FALSE;
+	 if (!set_cursor_abs(wid,fieldRow,firstCol+
+					  UTF8FirstCharsWidth(field,curChar)))
+	   return FALSE;
        }
        else
 	 ring_bell ();
@@ -847,11 +919,14 @@ int refreshKey;			/* Key causing the screen to be refreshed. */
        if (curChar < curLen) {
 	 int charsToEnd = curLen - curChar;
 	 int colN, i;
-	 field[curChar] = NUL;
+	 *(UTF8CharAtString(field,curChar)) = NUL;
 	 curLen = curChar;
-	 for (colN = firstCol+curChar, i = 0; i < charsToEnd; colN++, i++)
+	 width = UTF8StrWidth(field);
+	 for (colN = firstCol+width, i = 0; i < charsToEnd; colN++, i++)
 	    if (!put_chars(wid," ",1,fieldRow,colN,FALSE,NORMAL)) return FALSE;
-	 if (!set_cursor_abs(wid,fieldRow,firstCol+curChar)) return FALSE;
+	 if (!set_cursor_abs(wid,fieldRow,firstCol+
+					  UTF8FirstCharsWidth(field,curChar)))
+	   return FALSE;
        }
        else
 	 ring_bell ();
@@ -876,31 +951,40 @@ int refreshKey;			/* Key causing the screen to be refreshed. */
      *************************************************************************/
      if (Printable(key)) {
        if (insertMode) {
+	 char *ptr;
 	 if (curLen < fieldLen) {
 	   int charsToEnd = curLen - curChar;
-	   memmove (&field[curChar+1], &field[curChar], charsToEnd + 1);
-	   field[curChar] = key;
+	   len = UTF8CharBytes ((char *)&key);
+	   ptr = UTF8CharAtString (field, curChar);
+	   memmove (ptr+len, ptr, strlen(ptr) + 1);
+	   memmove (ptr, (char *)&key, (size_t) len);
 	   curLen++;
-	   if (!put_chars(wid,&field[curChar],charsToEnd+1,fieldRow,
-			  firstCol+curChar,FALSE,NORMAL)) return FALSE;
+	   if (!put_chars(wid,ptr,charsToEnd+1,
+			  fieldRow, firstCol+curChar,FALSE,NORMAL))
+	     return FALSE;
 	   if (curChar < fieldLen - 1) curChar++;
 	 }
 	 else
 	   ring_bell ();
-       }
-       else {
-	 field[curChar] = key;
+       } else {
+	 char *ptr, *ptr2;
+	 len = UTF8CharBytes ((char *)&key);
+	 ptr = UTF8CharAtString (field, curChar);
+	 ptr2 = UTF8CharAtString (field, curChar+1);
+	 memmove (ptr+len, ptr2, strlen(ptr2) + 1);
+	 memmove (ptr, (char *)&key, (size_t) len);
 	 if (curChar == curLen) {
 	   curLen++;
-	   field[curLen] = NUL;
+	   *(UTF8CharAtString(field,curLen)) = NUL;
 	 }
-	 if (!put_chars(wid,&field[curChar],1,fieldRow,
+	 if (!put_chars(wid,UTF8CharAtString(field,curChar),1,fieldRow,
 			firstCol+curChar,FALSE,NORMAL)) return FALSE;
 	 if (curChar < fieldLen - 1) curChar++;
        }
-       if (!set_cursor_abs(wid,fieldRow,firstCol+curChar)) return FALSE;
-     }
-     else
+       if (!set_cursor_abs(wid,fieldRow,firstCol+
+					UTF8FirstCharsWidth(field,curChar)))
+	 return FALSE;
+     } else
        ring_bell ();
   }
 }
@@ -928,8 +1012,8 @@ int rendition;
 #if defined(SMGui)
   int len = (int) strlen(label);
   static struct dsc$descriptor_s descr = {0,DSC$K_DTYPE_T,DSC$K_CLASS_S,NULL};
-  uLong position = SMG$K_TOP;
-  uLong rend = RenditionMapping(rendition);
+  uLongx position = SMG$K_TOP;
+  uLongx rend = RenditionMapping(rendition);
   int nRows, nCols;
   long startRow = 1;
   long startCol;
@@ -951,7 +1035,7 @@ int rendition;
   if (len > nCols - 2) return FALSE;
   wattrset (wid->id, RenditionMapping(rendition));
   startCol = (nCols - len) / 2;
-  for (i = 0; i < len; i++) mvwaddch (wid->id, 0, startCol + i, label[i]);
+  mvwaddstr (wid->id, 0, startCol, label);
   if (!RefreshWINDs(UPDATE_)) return FALSE;
   return TRUE;
 #endif
@@ -1019,10 +1103,10 @@ int col;
 * put_chars.
 ******************************************************************************/
 
-int put_chars (wid, string, len, row, col, eraseMode, rendition)
+int put_chars (wid, string, chars, row, col, eraseMode, rendition)
 WINDOWid wid;
 char *string;
-int len;
+int chars;			/* The number of characters - not bytes */
 int row;
 int col;
 int eraseMode;
@@ -1030,7 +1114,7 @@ int rendition;
 {
 #if defined(SMGui)
   static struct dsc$descriptor_s descr = {0,DSC$K_DTYPE_T,DSC$K_CLASS_S,NULL};
-  uLong rend = RenditionMapping (rendition);
+  uLongx rend = RenditionMapping (rendition);
   long rowN = row + 1;
   long colN = col + 1;
   if (eraseMode == ERASE) {
@@ -1042,42 +1126,69 @@ int rendition;
     if (StatusBad(smg$erase_chars(&(wid->id),&nChars,
 				  &rowN,&startCol))) return FALSE;
   }
-  descr.dsc$w_length = len;
+  descr.dsc$w_length = chars;
   descr.dsc$a_pointer = string;
   if (StatusBad(smg$put_chars(&(wid->id),&descr,&rowN,
 			      &colN,NULL,&rend,NULL,NULL))) return FALSE;
   return TRUE;
 #endif
 #if defined(CURSESui)
-  int nRows, nCols, colN, i;
+  /* **** Linux   ****/
+  int nRows, nCols, colN, rend, code; size_t it;
+  int i, j, ix, iy, iz, charLen; wchar_t wch[2], blankw = L' ', *wchars;
+  cchar_t onecc;
+  iy = (int) strlen(string);
+  ix = UTF8StrChars (string, iy);
+  wchars = (wchar_t *) malloc(sizeof(wchar_t) * (ix + 1));
+#if !defined(WIN32)
+  iz = (int) mbstowcs (wchars, string, ix);
+  wchars[iz] = L'\0';
+#else
+  mbstowcs_s (&it, wchars, (size_t)ix, string, (size_t)iy);
+#endif
   WindowSize (wid->id, &nRows, &nCols);
+  rend = RenditionMapping(rendition);
   if (eraseMode == ERASE) {
-    wattrset (wid->id, RenditionMapping(NORMAL));
-    for (colN = 1; colN <= nCols - 2; colN++) mvwaddch (wid->id,row,colN,' ');
+    wch[0] = blankw;
+    wch[1] = 0;
+    for (colN = col; colN <= (nCols - 4); colN++) {
+      setcchar (&onecc, wch, (chtype) RenditionMapping(NORMAL), 0, (void *)0);
+      code = mvwadd_wch (wid->id,row,colN,&onecc);
+    }
   }
-  wattrset (wid->id, RenditionMapping(rendition));
-  for (i = 0; i < len; i++) mvwaddch (wid->id, row, col + i, string[i]);
+  for (i = 0, j = 0; i < chars; ++i) {
+    wch[0] = wchars[i];
+    wch[1] = 0;
+    setcchar (&onecc, wch, (chtype) rend, 0, (void *)0);
+    code = mvwadd_wch (wid->id, row, col + j, &onecc);
+/*
+    j += GetWideCharWidth (wch[0]);
+*/
+    j += UTF8WCharWidth (wch[0]);
+  }
+  free (wchars);
+  wchars = NULL;
   if (!RefreshWINDs(UPDATE_)) return FALSE;
   return TRUE;
 #endif
 #if defined(COWui)
   short nCols = wid->id->nCols;
   char rend;
-  char *chars = wid->id->chars;
+  char *winchars = wid->id->chars;
   char *attrs = wid->id->attrs;
   int colN, i, charN;
   if (eraseMode == ERASE) {
     rend = RenditionMapping (NORMAL);
     for (colN = 1; colN <= nCols - 2; colN++) {
        charN = (row * nCols) + colN;
-       chars[charN] = ' ';
+       winchars[charN] = ' ';
        attrs[charN] = rend;
     }
   }
   rend = RenditionMapping (rendition);
-  for (i = 0; i < len; i++) {
+  for (i = 0; i < chars; i++) {
      charN = (row * nCols) + (col + i);
-     chars[charN] = string[i];
+     winchars[charN] = string[i];
      attrs[charN] = rend;
   }
   if (!RefreshWINDs(UPDATE_)) return FALSE;
@@ -1099,7 +1210,7 @@ char *string;
   static struct dsc$descriptor_s descr = {0,DSC$K_DTYPE_T,DSC$K_CLASS_S,NULL};
   int nRows, nCols; long rowN = row + 1; char *stringT;
   WindowSize (wid->id, &nRows, &nCols);
-  stringT = (char *) cdf_AllocateMemory (nCols, FatalError);
+  stringT = (char *) cdf_AllocateMemory ((size_t)nCols, FatalError);
   descr.dsc$w_length = nCols;
   descr.dsc$a_pointer = stringT;
   if (StatusBad(smg$read_from_display(&(wid->id),&descr,NULL,&rowN))) {
@@ -1164,7 +1275,11 @@ int read_input (
 #if defined(CURSESui)
 WINDOWid wid;
 #endif
+#if defined(WIN32)
 int *tcode;		/* Out: key entered. */
+#else
+wint_t *tcode;		/* Out: key entered (wide character). */
+#endif
 int mode;		/* Case conversion mode. */
 Logical block;		/* Block until key available? */
 {
@@ -1183,7 +1298,6 @@ Logical block;		/* Block until key available? */
   if (ABORTkey(*tcode)) {
     set_cursor_mode (CURSORon);
     delete_pasteboard (ERASE);
-    cdf_FreeMemory (NULL, FatalError);
     Exit;
   }
   switch (mode) {
@@ -1197,8 +1311,9 @@ Logical block;		/* Block until key available? */
   * Curses.
   ****************************************************************************/
 #if defined(CURSESui)
+  int ret;
   if (!block) {
-#if defined(unix)
+#if defined(unix) && !defined(__MINGW32__)
     fd_set rset;
     static struct timeval tv = { 0L, 0L };
     FD_ZERO (&rset);
@@ -1209,15 +1324,18 @@ Logical block;		/* Block until key available? */
 #endif
 				     &rset,NULL,NULL,&tv) == 0) return FALSE;
 #endif
-#if defined(dos)
+#if defined(dos) || defined(__MINGW32__)
     if (!kbhit()) return FALSE;
 #endif
   }
-  *tcode = wgetch (wid->id);
+#if defined(WIN32)
+  *tcode = wgetch (wid->id); 
+#else
+  ret = wget_wch (wid->id, tcode);
+#endif
   if (ABORTkey(*tcode)) {
     set_cursor_mode (CURSORon);
     delete_pasteboard (ERASE);
-    cdf_FreeMemory (NULL, FatalError);
     Exit;
   }
   switch (*tcode) {
@@ -1237,6 +1355,9 @@ Logical block;		/* Block until key available? */
 #if defined(__CYGWIN__)
     case CYGWIN_DELETE: *tcode = KB_DELETE; break;
 #endif
+#if defined(linux)
+    case LINUX_DELETE: *tcode = KB_DELETE; break;
+#endif
 #if defined(__MINGW32__)
     case MINGW32_DELETE: *tcode = KB_DELETE; break;
 #endif
@@ -1381,7 +1502,6 @@ Logical block;		/* Block until key available? */
 	 *tcode = (int) (event.message & charCodeMask);
 	 if (ABORTkeyMAC(event)) {
 	   delete_pasteboard (ERASE);
-	   cdf_FreeMemory (NULL, FatalError);
 	   Exit;
 	 }
 	 /*******************************************************************
@@ -1550,6 +1670,7 @@ int col;
   return TRUE;
 #endif
 #if defined(CURSESui)
+int ir, ic;
   wmove (wid->id, row, col);
   wrefresh (wid->id);
   return TRUE;
@@ -1595,7 +1716,7 @@ int set_cursor_mode (cursorMode)
 int cursorMode;                         /* CURSORon/CURSORoff */
 {
 #if defined(SMGui)
-  uLong mode = (cursorMode == CURSORon ? SMG$M_CURSOR_ON :
+  uLongx mode = (cursorMode == CURSORon ? SMG$M_CURSOR_ON :
 						 SMG$M_CURSOR_OFF);
   if (StatusBad(smg$set_cursor_mode(&pbid,&mode))) return FALSE;
   cursorOn = (cursorMode == CURSORon);
@@ -1688,11 +1809,11 @@ int zzzzz (seconds)
 double seconds;
 {
 #if defined(SMGui)					/* ie. VMS */
-#if defined(alphavmsI)
+#if defined(alphavmsI) || defined(ia64vmsI)
   time_t start_time, current_time;
   time (&start_time);
   time (&current_time);
-  while (current_time - start_time < seconds + 1.0) time (&current_time);
+  while ((current_time - start_time) < (seconds + 1.0)) time (&current_time);
 #else
   float seconds4 = seconds;
   if (StatusBad(lib$wait(&seconds4))) return FALSE;
@@ -1712,7 +1833,7 @@ double seconds;
   time_t start_time, current_time;
   time (&start_time);
   time (&current_time);
-  while (current_time - start_time < seconds + 1.0) time (&current_time);
+  while ((current_time - start_time) < (seconds + 1.0)) time (&current_time);
   return TRUE;
 #endif
 #if defined(BORLANDC)
@@ -1756,7 +1877,7 @@ Logical pasted;
 Logical bordered;
 {
   WINDOWid newWIND, tW;
-  newWIND = (WINDOWid) cdf_AllocateMemory (sizeof(WIND), FatalError);
+  newWIND = (WINDOWid) cdf_AllocateMemory ((size_t)sizeof(WIND), FatalError);
   if (WINDhead == NULL)
     WINDhead = newWIND;
   else {
@@ -1871,25 +1992,23 @@ static Logical DeleteWINDs () {
 static Logical RefreshWINDs (level)
 int level;
 {
-  if (batchCount < 1) {
-    WINDOWid tW;
-    switch (level) {
-      case HARD_:
-      case SOFT_:
-      case UPDATE_:
+  WINDOWid tW;
+  switch (level) {
+    case HARD_:
+    case SOFT_:
+    case UPDATE_:
 	/* Do nothing if CURSES. */
-	break;
-    }
-    tW = WINDhead;
-    while (tW != NULL) {
-      if (tW->pasted) {
-	touchwin (tW->id);
-	wnoutrefresh (tW->id);
-      }
-      tW = tW->next;
-    }
-    doupdate ();
+      break;
   }
+  tW = WINDhead;
+  while (tW != NULL) {
+    if (tW->pasted) {
+      touchwin (tW->id);
+      wnoutrefresh (tW->id);
+    }
+    tW = tW->next;
+  }
+  doupdate ();
   return TRUE;
 }
 #endif
@@ -2318,11 +2437,20 @@ LocalId id;
   werase (id);
 #else
   int nRows, nCols, rowN, colN;
+  int iz;
+  wchar_t star = L'*', blank = L' ', wch[2];
+  cchar_t cc1, cc2;
+  wch[0] = star;
+  wch[1] = 0;
+  setcchar (&cc1, wch, (chtype) RenditionMapping(NORMAL), 0, 0);
+  wch[0] = blank;
+  wch[1] = 0;
+  setcchar (&cc2, wch, (chtype) RenditionMapping(NORMAL), 0, 0);
   WindowSize (id, &nRows, &nCols);
   for (rowN = 0; rowN < nRows; rowN++) {
      for (colN = 0; colN < nCols; colN++) {
-	mvwaddch (id, rowN, colN, '*');
-	mvwaddch (id, rowN, colN, ' ');
+	mvwadd_wch (id, rowN, colN, &cc1);
+	mvwadd_wch (id, rowN, colN, &cc2);
      }
   }
 #endif
@@ -2508,7 +2636,7 @@ int maxL;
     case KB_CTRL_E: return ControlKey('E',maxL);
     case KB_CTRL_F: return ControlKey('F',maxL);
     case KB_CTRL_G: return ControlKey('G',maxL);
-#if !defined(dos) && !defined(mac) && !defined(win32)
+#if !defined(dos) && !defined(mac) && !defined(win32) && !defined(__MINGW32__)
     /**************************************************************************
     * Same as DELETE on IBM PC and Macintosh.
     **************************************************************************/
@@ -2627,10 +2755,10 @@ int maxL;
   static char token[MAX_KEY_TOKEN_LEN+1];
   switch (maxL) {
     case 1: case 2:
-      sprintf (token, "%c", key);
+      snprintf (token, (size_t) sizeof(token), "%c", key);
       break;
     default:
-      sprintf (token, "<%c>", key);
+      snprintf (token, (size_t) sizeof(token), "<%c>", key);
       break;
   }
   return token;
@@ -2647,16 +2775,16 @@ int maxL;
   static char token[MAX_KEY_TOKEN_LEN+1];
   switch (maxL) {
     case 3: case 4:
-      sprintf (token, "C-%c", key);
+      snprintf (token, (size_t) sizeof(token), "C-%c", key);
       break;
     case 5:
-      sprintf (token, "CTL-%c", key);
+      snprintf (token, (size_t) sizeof(token), "CTL-%c", key);
       break;
     case 6: case 7:
-      sprintf (token, "Ctrl-%c", key);
+      snprintf (token, (size_t) sizeof(token), "Ctrl-%c", key);
       break;
     default:
-      sprintf (token, "<Ctrl-%c>", key);
+      snprintf (token, (size_t) sizeof(token), "<Ctrl-%c>", key);
       break;
   }
   return token;
